Package: estadistica
Type: Package
Title: Fundamentos de estadística descriptiva e inferencial
Version: 1.2.2
Authors@R: 
  c(person(given = "Vicente",
           family = "Coll-Serrano",
           role = c("aut", "cre"),
           email = "estadistic@uv.es"),
    person(given = "Rosario",
           family = "Martínez Verdú",
           role = "aut"))
Author: Vicente Coll-Serrano [aut, cre],
  Rosario Martínez Verdú [aut]
Description: Este paquete pretende apoyar el proceso enseñanza-aprendizaje de estadística descriptiva e inferencial. Las funciones contenidas en el paquete 'estadistica' cubren los conceptos básicos estudiados en un curso introductorio. Muchos conceptos son ilustrados con gráficos dinámicos o web apps para facilitar su comprensión. This package aims to help the teaching-learning process of descriptive and inferential statistics. The functions contained in the package 'estadistica' cover the basic concepts studied in a statistics introductory course. Many concepts are illustrated with dynamic graphs or web apps to make the understanding easier. See: Esteban et al. (2005, ISBN: 9788497323741), Newbold et al.(2019, ISBN:9781292315034 ), Murgui et al. (2002, ISBN:9788484424673) .
License: GPL-3
Encoding: UTF-8
Language: es, en
LazyData: true
RoxygenNote: 7.3.3
URL: https://www.uv.es/estadistic/
Depends: R (>= 3.5.0)
Imports: cli, dplyr, tidyr, tibble, plotly, ggplot2, rio, shiny,
        shinydashboard, knitr, forecast, openxlsx, cowplot
NeedsCompilation: no
Packaged: 2026-04-25 03:11:07 UTC; vicen
Maintainer: Vicente Coll-Serrano <estadistic@uv.es>
Repository: CRAN
Date/Publication: 2026-04-25 03:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 02:05:34 UTC; unix

Package: estadistica
Type: Package
Title: Fundamentos de estadística descriptiva e inferencial
Version: 1.2.2
Authors@R: 
  c(person(given = "Vicente",
           family = "Coll-Serrano",
           role = c("aut", "cre"),
           email = "estadistic@uv.es"),
    person(given = "Rosario",
           family = "Martínez Verdú",
           role = "aut"))
Author: Vicente Coll-Serrano [aut, cre],
  Rosario Martínez Verdú [aut]
Description: Este paquete pretende apoyar el proceso enseñanza-aprendizaje de estadística descriptiva e inferencial. Las funciones contenidas en el paquete 'estadistica' cubren los conceptos básicos estudiados en un curso introductorio. Muchos conceptos son ilustrados con gráficos dinámicos o web apps para facilitar su comprensión. This package aims to help the teaching-learning process of descriptive and inferential statistics. The functions contained in the package 'estadistica' cover the basic concepts studied in a statistics introductory course. Many concepts are illustrated with dynamic graphs or web apps to make the understanding easier. See: Esteban et al. (2005, ISBN: 9788497323741), Newbold et al.(2019, ISBN:9781292315034 ), Murgui et al. (2002, ISBN:9788484424673) .
License: GPL-3
Encoding: UTF-8
Language: es, en
LazyData: true
RoxygenNote: 7.3.3
URL: https://www.uv.es/estadistic/
Depends: R (>= 3.5.0)
Imports: cli, dplyr, tidyr, tibble, plotly, ggplot2, rio, shiny,
        shinydashboard, knitr, forecast, openxlsx, cowplot
NeedsCompilation: no
Packaged: 2026-04-25 03:11:07 UTC; vicen
Maintainer: Vicente Coll-Serrano <estadistic@uv.es>
Repository: CRAN
Date/Publication: 2026-04-25 03:30:02 UTC
Built: R 4.5.2; ; 2026-04-25 03:54:15 UTC; unix

# Generated by roxygen2: do not edit by hand

export(calculo.probabilidad)
export(coeficiente.variacion)
export(contraste.correlacion)
export(contraste.diferencia.medias)
export(contraste.diferencia.proporciones)
export(contraste.media)
export(contraste.proporcion)
export(contraste.razon.varianzas)
export(contraste.varianza)
export(contraste_bondad)
export(contraste_bondad_cat)
export(contraste_homogeneidad)
export(contraste_independencia)
export(convergencia.varianza)
export(correlacion)
export(covarianza)
export(cuantiles)
export(desviacion)
export(distribucion.normal)
export(distribuciones.probabilidad)
export(ic.correlacion)
export(ic.diferencia.medias)
export(ic.diferencia.proporciones)
export(ic.media)
export(ic.proporcion)
export(ic.razon.varianzas)
export(ic.varianza)
export(leer.datos)
export(matriz.correlacion)
export(matriz.covar)
export(media)
export(mediana)
export(medidas.forma)
export(moda)
export(muestra)
export(nivel.confianza)
export(regresion.simple)
export(resumen.descriptivos)
export(resumir)
export(series.temporales)
export(tabla.bidimensional)
export(tabla.frecuencias)
export(unir.vectores)
export(varianza)
import(cowplot)
import(dplyr)
import(ggplot2)
import(knitr)
import(openxlsx)
import(shiny)
import(shinydashboard)
import(tibble)
import(tidyr)
importFrom(stats,addmargins)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dbinom)
importFrom(stats,dchisq)
importFrom(stats,df)
importFrom(stats,dnorm)
importFrom(stats,dpois)
importFrom(stats,dt)
importFrom(stats,lm)
importFrom(stats,na.omit)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,pt)
importFrom(stats,qchisq)
importFrom(stats,qf)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(tidyr,drop_na)
importFrom(tidyr,pivot_longer)
importFrom(utils,capture.output)
importFrom(utils,edit)

library(shiny)
library(shinydashboard)

#jscode <- "shinyjs.closeWindow = function() { window.close(); }"


ui <- dashboardPage(

  skin = "yellow",

  dashboardHeader(title = "Distribuciones de probabilidad",

                  titleWidth = 300,

  tags$li(a(href = 'https://www.youtube.com/channel/UCSE44FyVr87BEFshZAi4voQ',
            target = '_blank',
            img(src = 'e_R_logoB_web.jpg',
                title = 'Canal youtube:\nLa magia de estadistica', height = "30px"),
            style = "padding-top:10px; padding-bottom:10px;"),
          class = "dropdown"),

  tags$li(a(href = "window.close()",
            title = "Cerrar",
            icon("power-off")),
          class = "dropdown"),

  dropdownMenuOutput("messageMenu")
),

  # sidebar ----
  dashboardSidebar(
    sidebarMenu(#id = "sidebarid",
                id = "tabName",
                menuItem("Binomial",
                         tabName = "Binomial"),
                startExpanded = FALSE,
                conditionalPanel(
                  'input.tabName == "Binomial"',
                  sliderInput("n", label = "n:",
                              min = 1, max = 10, value = 5, step = 1),

                  sliderInput("p", label = "p:",
                              min = 0.1, max = 0.9, value = 0.5, step = 0.1)),

                menuItem("Poisson", tabName = "Poisson"),
                startExpanded = FALSE,
                conditionalPanel(
                  'input.tabName == "Poisson"',
                  sliderInput("lambda", label = "lambda:",
                              min = 0.1, max = 10, value = 2, step = 0.1)),

                menuItem("Uniforme", tabName = "Uniforme"),
                startExpanded = TRUE,
                conditionalPanel(
                  'input.tabName == "Uniforme"',
                  sliderInput("min", label = "min:",
                              min = 0, max = 10, value = 2, step = 1),

                  sliderInput("max", label = "max:",
                              min = 0, max = 10, value = 6, step = 1)),

                menuItem("Exponencial", tabName = "Exponencial"),
                startExpanded = TRUE,
                conditionalPanel(
                  'input.tabName == "Exponencial"',
                  sliderInput("rate", label = "lambda:",
                              min = 0.1, max = 8, value = 2, step = 0.1)),

                menuItem("Normal", tabName = "Normal"),
                startExpanded = TRUE,
                conditionalPanel(
                  'input.tabName == "Normal"',
                  sliderInput("mean", label = "media:",
                              min = -5, max = 5, value = 0, step = 1),

                  sliderInput("sd", label = "desviacion típica:",
                              min = 0.1, max = 5, value = 1, step = 0.1)),

                menuItem("Acerca de...",
                         tabName = "Acerca de",
                         icon = icon("users"),
                         startExpanded = TRUE,
                         tags$footer(
                           tags$p(strong("Autores:"),br(),
                                  "Vicente Coll-Serrano",br(),
                                  "Cristina Pardo García",br(),
                                  "Rosario Martínez Verdú",br(),
                                  "email: estadisTIC@uv.es"),

                           style = "
      * {
    margin: 0;
  }
  html, body {
    height: 50%;
  }
  .wrapper {
    min-height: 100%;
    height: auto !important; /* This line and the next line are not necessary unless you need IE6 support */
    height: 100%;
    margin: 0 auto -155px; /* the bottom margin is the negative value of the footer's height */
  }
  .footer, .push {
    height: -155px; /* .push must be the same height as .footer */
  }

  /*

  Sticky Footer by Ryan Fait
  http://ryanfait.com/

  */"
                         )
                )

    )
  ),

  # body ----
  dashboardBody(
    tabItems(
      # pagina 1 ----
      tabItem(tabName = "Binomial",
              #br(),
              plotOutput("plot.binomial")),
      # pagina 2 ----
      tabItem(tabName = "Poisson",
              #br(),
              plotOutput("plot.poisson")),
      # pagina 3 ----
      tabItem(tabName = "Uniforme",
              #br(),
              plotOutput("plot.uniforme")),
      # pagina 4 ----
      tabItem(tabName = "Exponencial",
              #br(),
              plotOutput("plot.exponencial")),
      # pagina 5 ----
      tabItem(tabName = "Normal",
              #br(),
              plotOutput("plot.normal"))
    )
  )
)

# server -----------------------------------------------------------------------

server <- function(input, output, session) {

  output$plot.binomial <- renderPlot({
    barplot(dbinom(0:10,input$n,input$p), names.arg = c(0:10),
            xlim = c(0, 15), ylim=c(0, 1),
            main = "Distribuci\u00f3n Binomial. X~Bi(n, p)",
            xlab ="x", ylab = "P(x)",
            col = "orange",
            border = "white")
  })

  output$plot.poisson <- renderPlot({
    barplot(dpois(0:10,input$lambda), names.arg = c(0:10),
            xlim = c(0, 13), ylim=c(0, 1),
            main = "Distribuci\u00f2n de Poisson. X~P(lambda)",
            xlab ="x", ylab = "P(x)",
            col = "darkred",
            border = "white")
  })

  output$plot.uniforme <- renderPlot({
    plot(seq(0,10,0.01),dunif(seq(0,10,0.01),input$min,input$max),
         type="l", xlim = c(0, 10), ylim=c(0, 1), lwd=3,
         main = "Distribuci\u00f3n Uniforme. X~U(min, max)",
         xlab ="x", ylab = "f(x)",
         col = "darkblue")
  })

  output$plot.exponencial <- renderPlot({
    plot(seq(0,15,0.1), dexp(seq(0,15,0.1),input$rate),
         type="l", xlim = c(0, 15), ylim=c(0, 1), lwd=3,
         main = "Distribuci\u00f3n Exponencial. X~Exp(lambda)",
         xlab ="x", ylab = "f(x)",
         col = "darkgreen")
  })

  output$plot.normal <- renderPlot({
    plot(seq(-10,10,0.01), dnorm(seq(-10,10,0.01),input$mean,input$sd),
         type="l", xlim = c(-10, 10), ylim=c(0, 1), lwd=3,
         main = "Distribuci\u00f3n Normal. X~N(media, desviacion típica)",
         xlab ="x", ylab = "f(x)",
         col = "red")
  })

}

# shiny app --------------------------------------------------------------------

shinyApp(ui, server)

library(shiny)
library(ggplot2)
library(dplyr)

library(shiny)
library(ggplot2)
library(dplyr)

ui <- fluidPage(
  # Logo y botón de cierre en la parte superior derecha
  # div(style = "position: absolute; right: 20px; top: 10px; z-index: 1000;",
  #     a(href = 'https://www.youtube.com/channel/UCSE44FyVr87BEFshZAi4voQ',
  #       target = '_blank',
  #       img(src = 'logo.jpg',
  #           title = 'Canal youtube:\nLa magia de estadistica',
  #           height = "60px",
  #           style = "margin-right: 15px;")),
  #     a(
  #       href = "javascript:(function(){
  #   window.location.href='https://www.youtube.com/channel/UCSE44FyVr87BEFshZAi4voQ';
  #   setTimeout(function(){ window.close(); }, 500);
  # })()",
  #       title = "Cerrar aplicación",
  #       style = "text-decoration: none; margin-left: 10px; text-align: center; display: inline-block;",
  #       tagList(
  #         icon("power-off", style = "font-size: 30px; color: #d9534f; vertical-align: middle;"),
  #         tags$div("Cerrar app", style = "font-size: 12px; color: #333; margin-top: 3px;")
  #       )
  #     )
  # ),
  # --- BLOQUE REEMPLAZO CABECERA (poner en ui en lugar del div anterior) ---
  tags$head(
    tags$style(HTML('
    /* header */
    .app-header {
      display: flex;
      align-items: center;
      justify-content: space-between;
      padding: 8px 20px;
      position: relative;
      z-index: 1000;
      background: transparent; /* cambiar si quieres fondo */
    }
    /* marca (logo + titulo) */
    .brand {
      display: flex;
      align-items: center;
      gap: 12px;
    }
    .brand img {
      height: 60px; /* tu logo 60px */
      display: block;
    }
    .app-title {
      font-size: 30px;
      font-weight: 600;
      margin: 0;
      line-height: 1;
    }
    /* boton cerrar a la derecha */
    .close-wrapper {
      display: flex;
      flex-direction: column;
      align-items: center;
      text-decoration: none;
      color: inherit;
    }
    .close-wrapper .icon {
      font-size: 30px; /* tamaño del icono */
      color: #d9534f;
      display: block;
      margin-bottom: 4px; /* separacion con el texto */
      position: relative;
      top: 2px; /* ajusta para bajar/subir el icono */
    }
    .close-wrapper .label {
      font-size: 12px;
      color: #333;
    }

    /* pequeño ajuste respuesta en pantallas pequeñas */
    @media (max-width: 480px) {
      .app-title { font-size: 16px; }
      .brand img { height: 48px; }
    }
  '))
  ),

  div(class = "app-header",
      div(class = "brand",
          a(href = 'https://www.youtube.com/channel/UCSE44FyVr87BEFshZAi4voQ',
            target = '_blank',
            img(src = 'logo.jpg',
                title = 'Canal youtube:\\nLa magia de estadistica',
                height = "60px",
                style = "display:block;")
          ),
          # Título al lado del logo
          tags$div(
            tags$h1("Aprendizaje de Distribuciones de Probabilidad", class = "app-title")
          )
      ),

      # Botón cerrar a la derecha (redirecciona y luego intenta cerrar ventana)
      a(href = "javascript:(function(){
                 window.location.href='https://www.youtube.com/channel/UCSE44FyVr87BEFshZAi4voQ';
                 setTimeout(function(){ window.close(); }, 500);
               })()",
        title = "Cerrar aplicación",
        class = "close-wrapper",
        # icon() produce etiquetas <i>, las envolvemos para aplicar estilo
        tags$span(class = "icon", icon("power-off")),
        tags$div(class = "label", "Cerrar app")
      )
  ),
  # --- FIN BLOQUE REEMPLAZO CABECERA ---

  # titlePanel("Aprendizaje de Distribuciones de Probabilidad"),
  uiOutput("mainUI")
)


server <- function(input, output, session) {
  # Inicializar valores reactivos
  mostrar_ejercicios <- reactiveVal(0)
  contador_ayuda <- reactiveVal(0)

  # Controlar la visualización del sidebar y main panel
  output$mainUI <- renderUI({
    if (mostrar_ejercicios() == 0) {
      sidebarLayout(
        sidebarPanel(
          radioButtons("tipo_dist", "Tipo de distribución:",
                       choices = c("Discreta", "Continua"), selected = "Discreta"),
          selectInput("distribucion", "Seleccione distribución:",
                      choices = c("Binomial", "Poisson", "Normal", "Uniforme Discreta",
                                  "Binomial Negativa", "Hipergeométrica", "t-Student",
                                  "Chi-cuadrado", "F", "Exponencial")),
          uiOutput("parametros"),


          conditionalPanel(
            condition = "input.tipo_calculo != 'cuantil'",
            numericInput("valor_x", "Valor de x:", value = 0)
          ),
          # Mostrar las opciones de tipo de cálculo según si la distribución es discreta o continua
          uiOutput("tipoCalculoUI"),
          conditionalPanel(
            condition = "input.tipo_calculo == 'cuantil'",
            sliderInput("prob_p", "Probabilidad p:", min = 0, max = 1,
                        value = 0.5, step = 0.01),
            numericInput("prob_p_num", "O ingrese p numéricamente:",
                         min = 0, max = 1, value = 0.5, step = 0.01)
          ),
          actionButton("calcular", "Calcular"),
          hr(),
          h4("Resultado:"),
          verbatimTextOutput("resultado"),
          hr(),
          h4("Explicación:"),
          uiOutput("explicacion")
        ),
        mainPanel(
          plotOutput("grafico", height = "400px"),
          conditionalPanel(
            condition = "input.tipo_calculo != 'puntual'",
            checkboxInput("mostrar_area", "Mostrar área de probabilidad", value = TRUE)
          ),
          div(style = "text-align: center;",
              actionButton("mostrar_ejercicios", "EJERCICIOS PARA PRACTICAR",
                           style = "margin-top: 20px; width: 40%;"))
        )
      )
    } else {
      # Pantalla completa para ejercicios
      fluidPage(
        h3("Ejercicio de Práctica"),
        uiOutput("enunciado_ejercicio"),
        div(style = "width: 200px; margin-bottom: 15px;",
            numericInput("respuesta_usuario", "Tu respuesta:",
                         value = NA, min = 0, max = 1, step = 0.001,
                         width = "100%")),
        tags$script(HTML(
          "$(document).on('keyup', '#respuesta_usuario', function() {
            var value = $(this).val().replace(',', '.');
            if(value !== $(this).val()) {
              $(this).val(value);
            }
          });"
        )),
        fluidRow(
          column(6, actionButton("verificar_respuesta", "Comprobar respuesta",
                                 style = "width: 100%; margin-bottom: 15px;")),
          column(6, actionButton("ayuda_grafica", "Ayuda gráfica",
                                 style = "width: 100%; background-color: #d4edda;
                                          margin-bottom: 15px;"))
        ),
        htmlOutput("retroalimentacion"),
        conditionalPanel(
          condition = "output.mostrar_grafico_condicional == true",
          plotOutput("grafico_ejercicio", height = "300px")
        ),
        fluidRow(
          column(6,
                 fluidRow(
                   column(6, actionButton("volver_graficos", "ATRÁS",
                                          style = "width: 100%; background-color: #f8f9fa;
                                                   border: 1px solid #ddd;")),
                   column(6, actionButton("nuevo_ejercicio", "NUEVO EJERCICIO",
                                          style = "width: 100%; background-color: #4CAF50;
                                                   color: white;"))
                 ),
                 column(6, div())
          )
        )
      )
    }
  })

  # Asegurar que el valor máximo sea siempre mayor que el mínimo en la Uniforme Discreta
  observeEvent(input$min_unif, {
    if (!is.null(input$max_unif) && input$max_unif <= input$min_unif) {
      updateNumericInput(session, "max_unif", value = input$min_unif + 1)
    }
  })

  observeEvent(input$max_unif, {
    if (!is.null(input$min_unif) && input$max_unif <= input$min_unif) {
      updateNumericInput(session, "min_unif", value = input$max_unif - 1)
    }
  })


  # Output condicional para mostrar el gráfico
  output$mostrar_grafico_condicional <- reactive({
    contador_ayuda() %% 2 == 1
  })
  outputOptions(output, "mostrar_grafico_condicional", suspendWhenHidden = FALSE)

  # Gráfico por defecto (Binomial)
  output$grafico <- renderPlot({
    datos <- data.frame(
      x = 0:10,
      y = dbinom(0:10, size = 10, prob = 0.5)
    )
    ggplot(datos, aes(x = x, y = y)) +
      geom_bar(stat = "identity", fill = "skyblue", width = 0.7) +
      geom_point(size = 3, color = "navy") +
      theme_minimal() +
      labs(title = "Distribución Binomial (n=10, p=0.5)",
           x = "Valor", y = "Probabilidad")
  })


  # Actualizar automáticamente el gráfico al cambiar de distribución o parámetros
  observe({
    req(input$distribucion)
    isolate({
      resultado_temp <- list(valor = 0, prob = 0, tipo = "puntual", texto = "")
      output$grafico <- renderPlot({
        graficar_distribucion(resultado_temp)
      })
    })
  })

  observeEvent(input$mostrar_ejercicios, {
    mostrar_ejercicios(1)
    contador_ayuda(0)
    output$retroalimentacion <- renderUI({NULL})
    generar_ejercicio()
  })

  observeEvent(input$volver_graficos, {
    mostrar_ejercicios(0)
    contador_ayuda(0)
    output$retroalimentacion <- renderUI({NULL})
  })

  observeEvent(input$nuevo_ejercicio, {
    contador_ayuda(0)
    output$retroalimentacion <- renderUI({NULL})
    updateNumericInput(session, "respuesta_usuario", value = NA)
    generar_ejercicio()
  })

  observeEvent(input$ayuda_grafica, {
    contador_ayuda(contador_ayuda() + 1)
  })

  # Actualizar las distribuciones disponibles según el tipo seleccionado
  observeEvent(input$tipo_dist, {
    if (input$tipo_dist == "Discreta") {
      updateSelectInput(session, "distribucion",
                        choices = c("Binomial", "Poisson", "Uniforme Discreta",
                                    "Binomial Negativa", "Hipergeométrica"))
    } else {
      updateSelectInput(session, "distribucion",
                        choices = c("Normal", "t-Student", "Chi-cuadrado",
                                    "F", "Exponencial"))
    }
  })

  # nuevo bloque
  # Renderizar dinámicamente las opciones del tipo de cálculo según el tipo de distribución
  output$tipoCalculoUI <- renderUI({
    if (input$tipo_dist == "Discreta") {
      radioButtons("tipo_calculo", "Tipo de cálculo:",
                   choices = c("Probabilidad puntual (P(X = x))" = "puntual",
                               "Probabilidad acumulada (P(X ≤ x))" = "acumulada",
                               "Probabilidad mayor que (P(X > x))" = "mayor",
                               "Cuantil (P(X ≤ q) = p)" = "cuantil"),
                   selected = "puntual")
    } else {
      radioButtons("tipo_calculo", "Tipo de cálculo:",
                   choices = c("Probabilidad acumulada (P(X ≤ x))" = "acumulada",
                               "Probabilidad mayor que (P(X > x))" = "mayor",
                               "Cuantil (P(X ≤ q) = p)" = "cuantil"),
                   selected = "acumulada")
    }
  })
  # fin nuevo bloque

  # Sincronizar los inputs de probabilidad para cuantiles
  observeEvent(input$prob_p, {
    updateNumericInput(session, "prob_p_num", value = input$prob_p)
  })

  observeEvent(input$prob_p_num, {
    updateSliderInput(session, "prob_p", value = input$prob_p_num)
  })


  # Actualizar parámetros según la distribución seleccionada
  output$parametros <- renderUI({
    tagList(
      switch(input$distribucion,
             "Binomial" = {
               tagList(
                 numericInput("n", "Número de ensayos (n):", value = 10, min = 1),
                 numericInput("p", "Probabilidad de éxito (p):", value = 0.5, min = 0, max = 1, step=0.1)
               )
             },
             "Poisson" = {
               numericInput("lambda", "Tasa (λ):", value = 1, min = 0, max=10, step = 0.5)
             },
             "Normal" = {
               tagList(
                 numericInput("media", "Media (μ):", value = 0),
                 numericInput("sd", "Desviación estándar (σ):", value = 1, min = 0.01)
               )
             },
             "Uniforme Discreta" = {
               tagList(
                 numericInput("min_unif", "Mínimo (a):", value = 0),
                 numericInput("max_unif", "Máximo (b):", value = 10, min = 1)
               )
             },
             "Binomial Negativa" = {
               tagList(
                 numericInput("r", "Número de éxitos (r):", value = 5, min = 1),
                 numericInput("p_neg", "Probabilidad de éxito (p):", value = 0.5, min = 0, max = 1)
               )
             },
             "Hipergeométrica" = {
               tagList(
                 numericInput("m", "Número de éxitos en población (m):", value = 10, min = 1),
                 numericInput("n_hip", "Número de fracasos en población (n):", value = 10, min = 1),
                 numericInput("k", "Número de extracciones (k):", value = 5, min = 1)
               )
             },
             "t-Student" = {
               numericInput("df_t", "Grados de libertad (df):", value = 5, min = 1)
             },
             "Chi-cuadrado" = {
               numericInput("df_chi", "Grados de libertad (df):", value = 5, min = 1)
             },
             "F" = {
               tagList(
                 numericInput("df1", "Grados de libertad numerador (df1):", value = 5, min = 1),
                 numericInput("df2", "Grados de libertad denominador (df2):", value = 10, min = 1)
               )
             },
             "Exponencial" = {
               numericInput("rate", "Tasa (λ):", value = 1, min = 0.01)
             }
      )
    )
  })

  # Generar explicación teórica
  output$explicacion <- renderUI({
    explicacion <- switch(input$distribucion,
                          "Binomial" = "La distribución binomial modela el número de éxitos en n ensayos independientes con probabilidad p.",
                          "Poisson" = "La distribución Poisson modela eventos raros en un intervalo de tiempo/espacio con tasa λ.",
                          "Normal" = "Distribución continua simétrica alrededor de su media μ con desviación σ.",
                          "Uniforme Discreta" = "Todos los resultados entre a y b tienen igual probabilidad.",
                          "Binomial Negativa" = "Número de ensayos hasta obtener r éxitos con probabilidad p.",
                          "Hipergeométrica" = "Éxitos en k extracciones sin reemplazo de población finita.",
                          "t-Student" = "Similar a la normal pero con colas más pesadas, usada en muestras pequeñas.",
                          "Chi-cuadrado" = "Suma de variables normales estándar al cuadrado. Usada en tests de hipótesis.",
                          "F" = "Razón de dos variables chi-cuadrado. Usada en ANOVA.",
                          "Exponencial" = "Modela tiempos entre eventos en un proceso de Poisson con tasa λ."
    )

    HTML(paste("<p>", explicacion, "</p>"))
  })

  # Calcular y graficar cuando se presiona el botón
  observeEvent(input$calcular, {
    req(input$distribucion)

    # Calcular según el tipo de operación
    resultado <- switch(input$tipo_calculo,
                        "puntual" = calcular_puntual(),
                        "acumulada" = calcular_acumulada(),
                        "mayor" = calcular_mayor(),
                        "cuantil" = calcular_cuantil()
    )

    output$resultado <- renderPrint({
      resultado$texto
    })

    output$grafico <- renderPlot({
      graficar_distribucion(resultado)
    })
  })

  # Funciones de cálculo -----
  calcular_puntual <- function() {
    valor <- input$valor_x
    prob <- switch(input$distribucion,
                   "Binomial" = dbinom(valor, input$n, input$p),
                   "Poisson" = dpois(valor, input$lambda),
                   "Normal" = dnorm(valor, input$media, input$sd),
                   "Uniforme Discreta" = ifelse(valor >= input$min_unif & valor <= input$max_unif,
                                                1/(input$max_unif - input$min_unif + 1), 0),
                   "Binomial Negativa" = dnbinom(valor - input$r, input$r, input$p_neg),
                   "Hipergeométrica" = dhyper(valor, input$m, input$n_hip, input$k),
                   "t-Student" = dt(valor, input$df_t),
                   "Chi-cuadrado" = dchisq(valor, input$df_chi),
                   "F" = df(valor, input$df1, input$df2),
                   "Exponencial" = dexp(valor, input$rate)
    )

    list(
      texto = paste0("P(X = ", valor, ") = ", round(prob, 4)),
      valor = valor,
      prob = prob,
      tipo = "puntual"
    )
  }

  calcular_acumulada <- function() {
    valor <- input$valor_x
    prob <- switch(input$distribucion,
                   "Binomial" = pbinom(valor, input$n, input$p),
                   "Poisson" = ppois(valor, input$lambda),
                   "Normal" = pnorm(valor, input$media, input$sd),
                   "Uniforme Discreta" = punif(valor, input$min_unif, input$max_unif),
                   "Binomial Negativa" = pnbinom(valor - input$r, input$r, input$p_neg),
                   "Hipergeométrica" = phyper(valor, input$m, input$n_hip, input$k),
                   "t-Student" = pt(valor, input$df_t),
                   "Chi-cuadrado" = pchisq(valor, input$df_chi),
                   "F" = pf(valor, input$df1, input$df2),
                   "Exponencial" = pexp(valor, input$rate)
    )

    list(
      texto = paste0("P(X ≤ ", valor, ") = ", round(prob, 4)),
      valor = valor,
      prob = prob,
      tipo = "acumulada"
    )
  }

  calcular_mayor <- function() {
    valor <- input$valor_x
    prob <- switch(input$distribucion,
                   "Binomial" = 1 - pbinom(valor, input$n, input$p),
                   "Poisson" = 1 - ppois(valor, input$lambda),
                   "Normal" = 1 - pnorm(valor, input$media, input$sd),
                   "Uniforme Discreta" = 1 - punif(valor, input$min_unif, input$max_unif),
                   "Binomial Negativa" = 1 - pnbinom(valor - input$r, input$r, input$p_neg),
                   "Hipergeométrica" = 1 - phyper(valor, input$m, input$n_hip, input$k),
                   "t-Student" = 1 - pt(valor, input$df_t),
                   "Chi-cuadrado" = 1 - pchisq(valor, input$df_chi),
                   "F" = 1 - pf(valor, input$df1, input$df2),
                   "Exponencial" = 1 - pexp(valor, input$rate)
    )

    list(
      texto = paste0("P(X > ", valor, ") = ", round(prob, 4)),
      valor = valor,
      prob = prob,
      tipo = "mayor"
    )
  }

  calcular_cuantil <- function() {
    p <- input$prob_p
    cuantil <- switch(input$distribucion,
                      "Binomial" = qbinom(p, input$n, input$p),
                      "Poisson" = qpois(p, input$lambda),
                      "Normal" = qnorm(p, input$media, input$sd),
                      "Uniforme Discreta" = qunif(p, input$min_unif, input$max_unif),
                      "Binomial Negativa" = qnbinom(p, input$r, input$p_neg) + input$r,
                      "Hipergeométrica" = qhyper(p, input$m, input$n_hip, input$k),
                      "t-Student" = qt(p, input$df_t),
                      "Chi-cuadrado" = qchisq(p, input$df_chi),
                      "F" = qf(p, input$df1, input$df2),
                      "Exponencial" = qexp(p, input$rate)
    )

    list(
      texto = paste0("El cuantil q tal que P(X ≤ q) = ", p, " es: ", round(cuantil, 4)),
      valor = cuantil,
      prob = p,
      tipo = "cuantil"
    )
  }

  # Función de graficado -----
  graficar_distribucion <- function(resultado) {
    dist <- input$distribucion
    tipo <- input$tipo_calculo
    es_discreta <- input$tipo_dist == "Discreta"

    # --- Validaciones seguras ---
    if (dist == "Uniforme Discreta") {
      req(!is.null(input$min_unif), !is.null(input$max_unif))
      validate(need(input$min_unif < input$max_unif, "El máximo debe ser mayor que el mínimo."))
    }
    if (dist == "Binomial") req(input$n, input$p)
    if (dist == "Poisson") req(input$lambda)
    if (dist == "Normal") req(input$media, input$sd)
    if (dist == "Exponencial") req(input$rate)

    # --- Rango de valores ---
    rango <- switch(dist,
                    "Binomial" = 0:input$n,
                    "Poisson" = 0:max(20, 3 * input$lambda),
                    "Normal" = seq(input$media - 4 * input$sd, input$media + 4 * input$sd, length.out = 400),
                    "Uniforme Discreta" = seq(input$min_unif, input$max_unif, by = 1),
                    "Binomial Negativa" = input$r:(input$r + 5 * input$r),
                    "t-Student" = seq(-4, 4, length.out = 400),
                    "Chi-cuadrado" = seq(0, qchisq(0.999, input$df_chi), length.out = 400),
                    "F" = seq(0, qf(0.999, input$df1, input$df2), length.out = 400),
                    "Exponencial" = seq(0, qexp(0.999, input$rate), length.out = 400)
    )

    # --- Cálculo de densidad o probabilidad ---
    datos <- if (es_discreta) {
      data.frame(
        x = rango,
        y = switch(dist,
                   "Binomial" = dbinom(rango, input$n, input$p),
                   "Poisson" = dpois(rango, input$lambda),
                   "Uniforme Discreta" = rep(1 / (input$max_unif - input$min_unif + 1), length(rango)),
                   "Binomial Negativa" = dnbinom(rango - input$r, input$r, input$p_neg)
        )
      )
    } else {
      data.frame(
        x = rango,
        y = switch(dist,
                   "Normal" = dnorm(rango, input$media, input$sd),
                   "t-Student" = dt(rango, input$df_t),
                   "Chi-cuadrado" = dchisq(rango, input$df_chi),
                   "F" = df(rango, input$df1, input$df2),
                   "Exponencial" = dexp(rango, input$rate)
        )
      )
    }

    # --- Valor actual (x o q) ---
    valor <- if (tipo == "cuantil") resultado$valor else input$valor_x
    req(!is.null(valor))

    # --- Gráfico base ---
    p <- ggplot(datos, aes(x = x, y = y)) +
      theme_minimal(base_size = 14) +
      labs(title = paste("Distribución", dist),
           x = "Valor",
           y = ifelse(es_discreta, "Probabilidad", "Densidad"))

    # --- Sombreado según tipo ---
    if (tipo %in% c("acumulada", "mayor")) {
      if (es_discreta) {
        if (tipo == "acumulada") {
          datos_sombra <- subset(datos, x <= valor)
        } else {
          datos_sombra <- subset(datos, x > valor)
        }
        p <- p +
          geom_bar(data = datos, stat = "identity", fill = "grey80") +
          geom_bar(data = datos_sombra, stat = "identity", fill = "skyblue") +
          geom_point(size = 3, color = "navy")
      } else {
        if (tipo == "acumulada") {
          datos_sombra <- subset(datos, x <= valor)
        } else {
          datos_sombra <- subset(datos, x > valor)
        }
        p <- p +
          geom_area(data = datos_sombra, aes(x, y), fill = "skyblue", alpha = 0.5) +
          geom_line(color = "navy", size = 1)
      }
    } else if (tipo == "puntual") {
      if (es_discreta) {
        p <- p +
          geom_bar(stat = "identity", fill = "grey80") +
          geom_bar(data = subset(datos, x == valor), stat = "identity", fill = "tomato") +
          geom_point(size = 3, color = "navy")
      } else {
        p <- p +
          geom_line(color = "navy", size = 1) +
          geom_vline(xintercept = valor, color = "tomato", linetype = "dashed", size = 1)
      }
    } else if (tipo == "cuantil") {
      p <- p +
        geom_line(color = "navy", size = 1) +
        geom_vline(xintercept = valor, color = "tomato", linetype = "dashed", size = 1)
    }

    p
  }

  # Variables reactivas para controlar la visualización
  mostrar_ejercicios <- reactiveVal(0)

  observeEvent(input$mostrar_ejercicios, {
    mostrar_ejercicios(1)
    generar_ejercicio()
  })

  observeEvent(input$volver_graficos, {
    mostrar_ejercicios(0)
  })

  # Variables reactivas para los ejercicios
  # Variables reactivas para los ejercicios
  ejercicio_actual <- reactiveValues(
    enunciado = NULL,
    distribucion = NULL,
    parametros = NULL,
    tipo = NULL,
    valor = NULL,
    respuesta_correcta = NULL
  )

  # Función para generar un nuevo ejercicio
  generar_ejercicio <- function() {
    # Seleccionar una distribución aleatoria
    dists <- c("Binomial", "Poisson", "Normal", "Uniforme Discreta",
               "Exponencial", "Binomial Negativa")

    ejercicio_actual$distribucion <- sample(dists, 1)

    # Generar parámetros aleatorios según la distribución
    ejercicio_actual$parametros <- switch(
      ejercicio_actual$distribucion,
      "Binomial" = {
        list(
          n = sample(5:20, 1),
          p = round(runif(1, 0.1, 0.9), 2)
        )
      },
      "Poisson" = {
        list(
          lambda = round(runif(1, 1, 10), 1)
        )
      },
      "Normal" = {
        list(
          media = round(rnorm(1, 0, 5)),
          sd = round(runif(1, 1, 5), 1)
        )
      },
      "Uniforme Discreta" = {
        list(
          min = 0,
          max = sample(5:20, 1))
      },
      "Exponencial" = {
        list(
          rate = round(runif(1, 0.1, 2), 1))
      },
      "Binomial Negativa" = {
        list(
          r = sample(2:5, 1),
          p = round(runif(1, 0.2, 0.8), 1))
      }
    )

    # Seleccionar un tipo de pregunta aleatorio
    tipos <- c("puntual", "acumulada", "mayor")
    ejercicio_actual$tipo <- sample(tipos, 1)

    # Generar un valor x apropiado
    ejercicio_actual$valor <- switch(
      ejercicio_actual$distribucion,
      "Binomial" = sample(0:ejercicio_actual$parametros$n, 1),
      "Poisson" = sample(0:15, 1),
      "Normal" = round(rnorm(1, ejercicio_actual$parametros$media,
                             ejercicio_actual$parametros$sd), 1),
      "Uniforme Discreta" = sample(ejercicio_actual$parametros$min:ejercicio_actual$parametros$max, 1),
      "Exponencial" = round(rexp(1, ejercicio_actual$parametros$rate), 1),
      "Binomial Negativa" = sample(ejercicio_actual$parametros$r:(ejercicio_actual$parametros$r+10), 1)
    )

    # Si la distribución es continua y el tipo es puntual, forzamos la probabilidad a cero
    distribuciones_continuas <- c("Normal", "Exponencial")
    if (ejercicio_actual$distribucion %in% distribuciones_continuas &&
        ejercicio_actual$tipo == "puntual") {

      ejercicio_actual$respuesta_correcta <- 0
    } else {
      # Calcular la respuesta correcta normalmente
      ejercicio_actual$respuesta_correcta <- calcular_respuesta(
        ejercicio_actual$distribucion,
        ejercicio_actual$parametros,
        ejercicio_actual$tipo,
        ejercicio_actual$valor
      )
    }

    # Generar el enunciado
    ejercicio_actual$enunciado <- generar_enunciado(
      ejercicio_actual$distribucion,
      ejercicio_actual$parametros,
      ejercicio_actual$tipo,
      ejercicio_actual$valor
    )
  }

  # Función para calcular la respuesta correcta
  calcular_respuesta <- function(dist, params, tipo, valor) {
    switch(tipo,
           "puntual" = switch(dist,
                              "Binomial" = dbinom(valor, params$n, params$p),
                              "Poisson" = dpois(valor, params$lambda),
                              "Normal" = 0,  # Probabilidad puntual continua = 0
                              "Uniforme Discreta" = 1/(params$max - params$min + 1),
                              "Exponencial" = 0,  # Probabilidad puntual continua = 0
                              "Binomial Negativa" = dnbinom(valor - params$r, params$r, params$p)),
           "acumulada" = switch(dist,
                                "Binomial" = pbinom(valor, params$n, params$p),
                                "Poisson" = ppois(valor, params$lambda),
                                "Normal" = pnorm(valor, params$media, params$sd),
                                "Uniforme Discreta" = punif(valor, params$min, params$max),
                                "Exponencial" = pexp(valor, params$rate),
                                "Binomial Negativa" = pnbinom(valor - params$r, params$r, params$p)),
           "mayor" = switch(dist,
                            "Binomial" = 1 - pbinom(valor, params$n, params$p),
                            "Poisson" = 1 - ppois(valor, params$lambda),
                            "Normal" = 1 - pnorm(valor, params$media, params$sd),
                            "Uniforme Discreta" = 1 - punif(valor, params$min, params$max),
                            "Exponencial" = 1 - pexp(valor, params$rate),
                            "Binomial Negativa" = 1 - pnbinom(valor - params$r, params$r, params$p))
    )
  }

  # Función para generar el enunciado del ejercicio (permite mensaje adicional opcional)
  generar_enunciado <- function(dist, params, tipo, valor, mensaje_extra = NULL) {
    texto_dist <- switch(dist,
                         "Binomial" = paste0("Binomial(n=", params$n, ", p=", params$p, ")"),
                         "Poisson" = paste0("Poisson(λ=", params$lambda, ")"),
                         "Normal" = paste0("Normal(μ=", params$media, ", σ=", params$sd, ")"),
                         "Uniforme Discreta" = paste0("Uniforme Discreta(a=", params$min, ", b=", params$max, ")"),
                         "Exponencial" = paste0("Exponencial(λ=", params$rate, ")"),
                         "Binomial Negativa" = paste0("Binomial Negativa(r=", params$r, ", p=", params$p, ")"))

    pregunta <- switch(tipo,
                       "puntual" = paste0("P(X = ", valor, ")"),
                       "acumulada" = paste0("P(X ≤ ", valor, ")"),
                       "mayor" = paste0("P(X > ", valor, ")"))

    extra <- if (!is.null(mensaje_extra)) {
      paste0("<p style='color: red; font-style: italic;'>", mensaje_extra, "</p>")
    } else {
      ""
    }

    HTML(paste0(
      "<h4>Distribución ", texto_dist, "</h4>",
      "<p>Calcula ", pregunta, "</p>",
      "<p>Redondea tu respuesta a 4 decimales</p>",
      extra
    ))
  }

  # Mostrar el enunciado del ejercicio
  output$enunciado_ejercicio <- renderUI({
    if (is.null(ejercicio_actual$enunciado)) {
      HTML("<p>Presiona 'Verificar Respuesta' para comenzar</p>")
    } else {
      ejercicio_actual$enunciado
    }
  })

  # Verificar la respuesta del usuario
  observeEvent(input$verificar_respuesta, {
    req(ejercicio_actual$respuesta_correcta)

    respuesta_usuario <- input$respuesta_usuario
    respuesta_correcta <- ejercicio_actual$respuesta_correcta

    if (is.na(respuesta_usuario)) {
      output$retroalimentacion <- renderUI({
        HTML("<div style='color: red;'>Por favor ingresa una respuesta</div>")
      })
    } else {
      # Comparar respuestas con cierta tolerancia para decimales
      diferencia <- abs(respuesta_usuario - respuesta_correcta)

      if (diferencia < 0.001) {
        mensaje <- paste0(
          "<div style='color: green; font-weight: bold;'>¡Correcto! ",
          "La respuesta es ", round(respuesta_correcta, 4), "</div>"
        )
        # Generar nuevo ejercicio después de 2 segundos si la respuesta es correcta
        delay(2000, generar_ejercicio())
      } else {
        mensaje <- paste0(
          "<div style='color: red;'>Incorrecto. ",
          "La respuesta correcta es ", round(respuesta_correcta, 4), "</div>"
        )
      }

      # Añadir mensaje explicativo si es una distribución continua puntual
      if (ejercicio_actual$tipo == "puntual" &&
          ejercicio_actual$distribucion %in% c("Normal", "Exponencial")) {
        mensaje <- paste0(
          mensaje,
          "<br><br><div style='color: orange; font-style: italic;'>
        En una distribución continua, la probabilidad en un punto es siempre cero.
        </div>"
        )
      }

      output$retroalimentacion <- renderUI({
        HTML(mensaje)
      })
    }
  })


  # Gráfico para el ejercicio
  output$grafico_ejercicio <- renderPlot({
    req(ejercicio_actual$distribucion)

    dist <- ejercicio_actual$distribucion
    params <- ejercicio_actual$parametros
    valor <- ejercicio_actual$valor
    tipo <- ejercicio_actual$tipo

    # Generar datos para el gráfico
    datos <- switch(dist,
                    "Binomial" = {
                      x <- 0:params$n
                      y <- dbinom(x, params$n, params$p)
                      data.frame(x = x, y = y)
                    },
                    "Poisson" = {
                      x <- 0:max(15, 3*params$lambda)
                      y <- dpois(x, params$lambda)
                      data.frame(x = x, y = y)
                    },
                    "Normal" = {
                      x <- seq(params$media - 4*params$sd, params$media + 4*params$sd, length.out = 200)
                      y <- dnorm(x, params$media, params$sd)
                      data.frame(x = x, y = y)
                    },
                    "Uniforme Discreta" = {
                      x <- params$min:params$max
                      y <- rep(1/(params$max - params$min + 1), length(x))
                      data.frame(x = x, y = y)
                    },
                    "Exponencial" = {
                      x <- seq(0, qexp(0.999, params$rate), length.out = 200)
                      y <- dexp(x, params$rate)
                      data.frame(x = x, y = y)
                    },
                    "Binomial Negativa" = {
                      x <- params$r:(params$r + 20)
                      y <- dnbinom(x - params$r, params$r, params$p)
                      data.frame(x = x, y = y)
                    })

    # Crear gráfico base
    p <- ggplot(datos, aes(x = x, y = y)) +
      theme_minimal() +
      labs(title = paste("Distribución", dist),
           x = "Valor", y = ifelse(dist %in% c("Binomial", "Poisson", "Uniforme Discreta", "Binomial Negativa"),
                                   "Probabilidad", "Densidad"))

    # Añadir geometría según sea discreta o continua
    if (dist %in% c("Binomial", "Poisson", "Uniforme Discreta", "Binomial Negativa")) {
      p <- p +
        geom_bar(stat = "identity", fill = "skyblue", width = 0.7) +
        geom_point(size = 3, color = "navy")

      # Añadir polígono de frecuencias
      datos_poligono <- datos
      primer_punto <- data.frame(x = min(datos_poligono$x) - 1, y = 0)
      ultimo_punto <- data.frame(x = max(datos_poligono$x) + 1, y = 0)
      datos_poligono <- rbind(primer_punto, datos_poligono, ultimo_punto)

      p <- p + geom_line(data = datos_poligono, aes(group = 1),
                         color = "darkblue", size = 1, linetype = "solid")
    } else {
      p <- p + geom_line(color = "navy", size = 1)
    }

    # Resaltar área según el tipo de pregunta
    if (tipo == "puntual") {
      if (dist %in% c("Binomial", "Poisson", "Uniforme Discreta", "Binomial Negativa")) {
        p <- p + geom_bar(data = datos %>% filter(x == valor),
                          stat = "identity", fill = "red", width = 0.7)
      } else {
        p <- p +
          geom_vline(xintercept = valor, color = "red") +
          annotate("point", x = valor, y = calcular_respuesta(dist, params, tipo, valor),
                   color = "red", size = 3)
      }
    } else if (tipo == "acumulada") {
      if (dist %in% c("Binomial", "Poisson", "Uniforme Discreta", "Binomial Negativa")) {
        datos_area <- datos %>% filter(x <= valor)
        if (nrow(datos_area) > 0) {
          datos_area_pol <- rbind(
            data.frame(x = min(datos_area$x) - 1, y = 0),
            datos_area,
            data.frame(x = max(datos_area$x), y = 0)
          )
          p <- p +
            geom_polygon(data = datos_area_pol, aes(x = x, y = y),
                         fill = "red", alpha = 0.3)
        }
      } else {
        datos_area <- datos %>% filter(x <= valor)
        p <- p + geom_area(data = datos_area, fill = "red", alpha = 0.3)
      }
      p <- p + geom_vline(xintercept = valor, linetype = "dashed", color = "red", size = 1)
    } else if (tipo == "mayor") {
      if (dist %in% c("Binomial", "Poisson", "Uniforme Discreta", "Binomial Negativa")) {
        datos_area <- datos %>% filter(x >= valor)
        if (nrow(datos_area) > 0) {
          datos_area_pol <- rbind(
            data.frame(x = min(datos_area$x), y = 0),
            datos_area,
            data.frame(x = max(datos_area$x) + 1, y = 0)
          )
          p <- p +
            geom_polygon(data = datos_area_pol, aes(x = x, y = y),
                         fill = "blue", alpha = 0.3)
        }
      } else {
        datos_area <- datos %>% filter(x >= valor)
        p <- p + geom_area(data = datos_area, fill = "blue", alpha = 0.3)
      }
      p <- p + geom_vline(xintercept = valor, linetype = "dashed", color = "blue", size = 1)
    }

    p
  })
}

shinyApp(ui, server)

library(shiny)

ui <- basicPage(

  plotOutput("plot1"),

  inputPanel(
    sliderInput("mean", label = "Media 1:",
                min = -5, max = 5, value = 0, step = 1),

    sliderInput("sd", label = "Desviaci\u00f3n t\u00edpica 1:",
                min = 0.1, max = 5, value = 1, step = 0.1),

    sliderInput("mean2", label = "Media 2:",
                min = -5, max = 5, value = 0, step = 1),

    sliderInput("sd2", label = "Desviaci\u00f3n t\u00edpica 2:",
                min = 0.1, max = 5, value = 1, step = 0.1)
  )
)

server <- function(input, output) {
  output$plot1 <- renderPlot({

    plot(seq(-10,10,0.01), dnorm(seq(-10,10,0.01),input$mean,input$sd),
         type="l", xlim = c(-8, 8), ylim=c(0, 1), lwd=3, col="deeppink",
         cex.lab=1.5, cex.main=1.5, cex.axis=1.5, axes=FALSE,
         main = "Distribuci\u00f3n Normal. X~N(Media, Desviaci\u00f3n T\u00edpica)",
         xlab ="x", ylab = "f(x)")
    curve(dnorm(x,input$mean2, input$sd2), lwd=3, add=TRUE, col="darkviolet")
    axis(1, at = seq(-8, 8, by = 1), cex.axis=1.5, pos=0)
    axis(2,at = seq(0, 1, by = 0.2), cex.axis=1.5, pos=0)
  })

}

shinyApp(ui, server)

#' @encoding UTF-8
#' @title Nivel de confianza.
#'
#' @description Esta función simula una población de tamaño 100,000 de la que se extraen diversas muestras y construye los correspondientes intervalos de confianzas. El objetivo es transmitir el concepto de nivel de confianza.
#'
#' @param min.pob Es un valor numérico que indica el valor mínimo poblacional. Por defecto \code{min.pob = 2000}
#' @param max.pob Es un valor numérico que indica el valor máximo poblacional. Por defecto \code{max.pob = 45000}
#' @param muestras Es un valor numérico entre 50 y 10000 que indica el número de muestras que se extraen sin reemplazamiento de la población. Por defecto \code{muestras = 200}
#' @param n Es un valor numérico entre 25 y 2000 que indica el tamaño de la muestra. Por defecto \code{n = 100}
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#' @param grafico Si \code{grafico = TRUE} se representan los intervalos de confianza de las muestras seleccionadas y la media poblacional.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#' @param replicar Es un valor lógico. Si \code{replicar = TRUE} el usuario fijará una semilla para que los resultados sean reproducibles. Si \code{replicar = FALSE} los resultados serán aleatorios y cambiarán en cada realización.
#'
#' @return Esta función devuelve un gráfico como un objeto de la clase \code{list}. La lista contiene los valores simulados para las muestras, el porcentaje de intervalos que contienen la media poblacional y su representación gráfica.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @seealso \code{\link{ic.media}}
#'
#' @note Si se seleccionan 10000 muestras de tamaño 2000, el tiempo estimado de ejecución es de 9 minutos.
#'
#' @references
#' Casas José M. () Inferencia estadística. Editoral: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom tidyr pivot_longer
#' @import dplyr ggplot2
#'
#' @export
nivel.confianza <- function(min.pob = 2000,
                            max.pob = 45000,
                            muestras = 200,
                            n = 100,
                            confianza = 0.95,
                            grafico = TRUE,
                            exportar = FALSE,
                            replicar = FALSE){

  if(isTRUE(replicar)){

    semilla <- as.numeric(readline(prompt = "Introducir un valor para fijar la semilla: "))

    set.seed(semilla)

  }

# se define una población de 100000 observaciones que toma valores a y b

if( max.pob <= min.pob) {
  stop("Los valores introducidos no son correctos")
}

poblacion <- sample(min.pob:max.pob,100000,replace=TRUE)
media_poblacional <- mean(poblacion)

# datos muestrales

if(muestras < 50 | muestras > 10000){

  stop("Elige un n\u00famero de muestras entre 50 y 10000")
}

if(n < 25 | n > 2000){

  stop("Elige un tama\u00f1o para la muestra de entre 50 y 2000")
}

muestra_select <- c()
medias_muestra <- c()
varianzas_muestra <- c()

for(i in 1:muestras){
  seleccion <- sample(poblacion,n,replace=TRUE)
  media <- media(seleccion)
  varianza <- varianza(seleccion)

  muestra_select <- cbind(muestra_select,seleccion)
  medias_muestra <- rbind(medias_muestra,media)
  varianzas_muestra <- rbind(varianzas_muestra, varianza)

}

resultados_muestrales <- as.data.frame(cbind(1:muestras,medias_muestra,varianzas_muestra))
colnames(resultados_muestrales) <- c("muestra","media.muestral","varianza.muestral")
row.names(resultados_muestrales) <- NULL

# fijamos el nivel de confianza para calcular el
# intervalo confianza media muestral con var pob desconocida

z <- qnorm((1-confianza)/2, lower.tail = FALSE)

resultados_muestrales <- resultados_muestrales %>% mutate(inferior = media.muestral - z * sqrt(varianza.muestral/n),
                                                          superior = media.muestral + z * sqrt(varianza.muestral/n),
                                                          incluido = case_when(media_poblacional >= inferior & media_poblacional <= superior ~ 1,
                                                                               TRUE ~ 0))
porcentaje.intervalos <- resultados_muestrales %>% summarize(porcentaje = sum(incluido)/n())


if(grafico){

  df <- resultados_muestrales %>%
    select(muestra,inferior,superior,incluido) %>%
    mutate(color = ifelse(incluido == 0, "red", "blue")) %>%
    pivot_longer(cols= c(2,3))

  if(muestras > 200){

    df <- resultados_muestrales[sample(muestras,200),] %>%
      arrange(muestra) %>%
      select(muestra,inferior,superior,incluido) %>%
      mutate(color = ifelse(incluido == 0, "red", "blue")) %>%
      pivot_longer(cols= c(2,3))

  } else{

    df <- resultados_muestrales %>%
      select(muestra,inferior,superior,incluido) %>%
      mutate(color = ifelse(incluido == 0, "red", "blue")) %>%
      pivot_longer(cols= c(2,3))
  }

  interactivo <- as.numeric(readline('\u00bfQuieres hacer el gr\u00e1fico interactivo?: \n 1. "S\u00ed." \n 2. "No." \n'))

    plot <-  ggplot(df, aes(x=value, y=muestra)) +
      geom_line(aes(group = muestra), color = "grey") +
      geom_point(size=1, color = df$color) +
      geom_vline(aes(xintercept = media_poblacional), color = "darkred",size=1) +
      geom_text(aes(x=media_poblacional, label ="media poblaci\u00f3n", y = muestras),
                color = "darkred", hjust = -0.05, vjust=-0.5, size = 3) +
      geom_text(aes(x=media_poblacional, label=round(media_poblacional,2), y=0), hjust= -0.10, vjust = 1.2, size = 3, color="darkred") +
      scale_color_identity() +
      labs(title = paste("Intervalos de confianza de un total de ",muestras," muestras",sep=""),
           y="N\u00famero de muestra",
           x=paste("El ",round(porcentaje.intervalos *100,2),"% de los intervalos de confianza\ncalculados contienen el valor de la media poblacional",sep=""))

    if(interactivo == 1){

      plot <- plotly::ggplotly(plot, tooltip=c("y","x"))

    }

}

  if (exportar) {
    filename <- paste("Intervalos de confianza"," (", Sys.time(), ").xlsx", sep = "")
    filename <- gsub(" ", "_", filename)
    filename <- gsub(":", ".", filename)
    rio::export(resultados_muestrales, row.names = TRUE, file = filename)
  }

  if(grafico){
    if(muestras >200){
      print("Como el n\u00famero de muestras es grande, se seleccionar\u00e1n aleatoriamente 200 muestras para representarlas en el gr\u00e1fico")
    }

    return(list(resultados_muestrales, porcentaje.intervalos,plot))

  } else{

    return(list(resultados_muestrales, porcentaje.intervalos))

  }

}

#' @title Intervalo confianza de una proporción.
#'
#' @description Calcula el intervalo de confianza de una proporción.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qricproporcion.png}{width = 200px}}
#' \if{latex}{\figure{qricproporcion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param irrestricto Es un valor lógico. Por defecto, irrectricto = FALSE. si se considera un muestreo irrectricto (extracción sin reemplazamiento), cambiar el argumento a irrestricto = TRUE.
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return Devuelve el intervalo de confianza de la proporción poblacional en un objeto de tipo \code{data.frame}. Si \code{grafico = T} devuelve una \code{list} con el intervalo de confianza y su representación gráfica.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) Para tamaños muestrales muy grandes (n>100):
#'
#' \if{html}{\figure{icproporcion3.png}{width = 640px}}
#' \if{latex}{\figure{icproporcion3.png}{options: width=12cm}}
#'
#' El usuario puede elegir entre tres estrategias:
#'
#' (1.1) En el error típico aproximar p por su estimación muestral.
#' (1.2) En el error típico considerar el caso: p=q=0.5
#' (1.3) Obtener el valor de p a partir del estadístico.
#'
#' (2) Para cualquier tamaño muestral puede obtenerse el intervalo:
#'
#' \if{html}{\figure{icproporcion1.png}{width = 120px}}
#' \if{latex}{\figure{icproporcion1.png}{options: width=2.5cm}}
#'
#' correspondiendo los valores a las raíces de:
#'
#' \if{html}{\figure{icproporcion2.png}{width = 640px}}
#' \if{latex}{\figure{icproporcion2.png}{options: width=12cm}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm qnorm na.omit
#' @import dplyr ggplot2
#'
#' @export
ic.proporcion <- function(x,
                          variable = NULL,
                          introducir = FALSE,
                          irrestricto = FALSE,
                          confianza = 0.95,
                          grafico = FALSE){


  #print("Intervalo de confianza de una proporci\u00f3n. El tama\u00f1o de la muestra es grande.")

  if(confianza >= 0 & confianza <=1){

    confianza <- confianza
    alfa2 <- (1- confianza)/2
    valor_critico <- qnorm(alfa2,lower.tail = F)

  } else{

    stop("El nivel de confianza debe fijarse entre 0 y 1")

  }


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 1){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

    }

  } else if(length(variable) == 1){

    if(is.numeric(variable)){

      if(variable <= length(x)){

        variable <- variable

      } else{

        stop("Selecci\u00fn err\u00f3nea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es v\u00e1lido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
    stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

  }

  x <- na.omit(x)

  if(!all(x == 0 | x==1)){

    print("Aplica a tus datos la condici\u00f3n que debe cumplir la poblaci\u00f3n para transfomar los datos en ceros (ausencia/no \u00e9xito) y unos (presencia/\u00e9xito)")
    stop("Los valores en la muestra deben ser 0 y 1.")

  }

  clase <- sapply(x, class)

  if (!clase %in% c("numeric","integer")) {

    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")

  }

  # tama\u00f1o de la muestra
  n <- nrow(x)

  if(n < 100){
    print("El tama\u00f1o de la muestra no es suficientemente grande (n<100), la aproximaci\u00f3n a la normal no es buena.")
    print("Se obtendr\u00e1 el intervalo de p a partir del c\u00e1lculo de probabilidad del estad\u00edstico. Este criterio no tiene en cuenta el tama\u00f1o de la muestra." )
  }


  # media muestral

    p_mu <- round(sum(x,na.rm=TRUE)/n,4)

} else{   # aqu\u00ed empieza introducir datos

  n <- readline(prompt = "Introducir el tama\u00f1o de la muestra: ")
  n <- as.numeric(n)

  if(n < 100){
    print("El tama\u00f1o de la muestra no es suficientemente grande (n<100), la aproximaci\u00f3n a la normal no es buena.")
    print("Se obtendr\u00e1 el intervalo de p a partir del c\u00e1lculo de probabilidad del estad\u00edstico. Este criterio no tiene en cuenta el tama\u00f1o de la muestra." )
  }


    p_mu <- readline(prompt = "Introducir el valor de la proporci\u00f3n muestral: ")
    p_mu <- as.numeric(p_mu)

}


  if(isFALSE(irrestricto)){

    if(n < 100){
      print("El intervalo obtenido no es sim\u00e9trico respecto a la proporci\u00f3n muestral")

      aproximacion <- 3

      x <- n + valor_critico^2
      y <- -(2 * p_mu * n + valor_critico^2)
      z <- p_mu^2 * n

      limite_inf <- (-y + sqrt((y^2) - (4 * x * z)))/(2 * x)
      limite_sup <- (-y - sqrt((y^2) - (4 * x * z)))/(2 * x)

      if(limite_inf <= limite_sup){

        limite_inferior <- limite_inf
        limite_superior <- limite_sup

      } else{

        limite_inferior <- limite_sup
        limite_superior <- limite_inf

      }

    } else{

      aproximacion <- as.numeric(readline('\u00bfQue estrategia quieres utilizar para aproximar el valor de p en el error t\u00edpico? \n 1. "Aproximar por la proporci\u00f3n muestral" \n 2. "p=q=0.5" \n 3. "Obtener el valor de p a partir del estad\u00edstico" \n'))

      if(aproximacion == 1){

        print("Como n es suficientemente grande, se aproxima el valor de p poblacional por su estimaci\u00f3n puntual (p muestral)")

        error_tipico <- sqrt((p_mu * (1-p_mu))/n)
        limite_inferior <- p_mu - valor_critico * error_tipico
        limite_superior <- p_mu + valor_critico * error_tipico

      } else if(aproximacion == 2){

        error_tipico <- sqrt(0.25/n)
        limite_inferior <- p_mu - valor_critico * error_tipico
        limite_superior <- p_mu + valor_critico * error_tipico

      } else{

        aproximacion  <- 3

        print("Este criterio no tiene en cuenta el tama\u00f1o de la muestra. Se obtendr\u00e1 el intervalo de p a partir del c\u00e1lculo de probabilidad del estad\u00edstico")
        print("El intervalo obtenido no es sim\u00e9trico respecto a la proporci\u00f3n muestral")
        x <- n + valor_critico^2
        y <- -(2 * p_mu * n + valor_critico^2)
        z <- p_mu^2 * n

        limite_inf <- (-y + sqrt((y^2) - (4 * x * z)))/(2 * x)
        limite_sup <- (-y - sqrt((y^2) - (4 * x * z)))/(2 * x)

        if(limite_inf <= limite_sup){

          limite_inferior <- limite_inf
          limite_superior <- limite_sup

        } else{

          limite_inferior <- limite_sup
          limite_superior <- limite_inf

        }

      }

    }

  } else{

    print("Intervalo para la proporci\u00f3n adecuado si el muestreo es sin reemplazamiento y la poblaci\u00f3n es finita")
    aproximacion <- 0
    N <- readline(prompt= "Introduce el tama\u00f1o (N) de la poblaci\u00f3n: ")
    N <- as.numeric(N)

    factor <- sqrt((N-n)/(N-1))
    error_tipico <- sqrt((p_mu * (1-p_mu))/n) * factor
    limite_inferior <- p_mu - valor_critico * error_tipico
    limite_superior <- p_mu + valor_critico * error_tipico

  }

  if(grafico){

    if(aproximacion == 3){

      intervalo <- data.frame(ic = round(c(inferior=limite_inferior,p_mu = p_mu, superior=limite_superior),4),y=c(0,0,0))

      plot <- ggplot(intervalo,aes(x= ic,y)) +
        geom_line(aes(group = y), color = "grey",size = 3)+
        geom_point(aes(color=ic), size=3,show.legend = FALSE) +
        geom_text(aes(label = ic), size = 2.5, vjust=2) +
        scale_y_continuous(expand=c(0,0)) +
        scale_color_gradientn(colours=c("red","darkgreen","blue"))+
        labs(y="",x="Intervalo de confianza de la proporci\u00f3n") +
        tema_blanco

    } else{

      seq <- seq(-4,4,length=1000) * error_tipico + p_mu
      seq <- as.data.frame(seq)

      plot <- ggplot(seq, aes(seq)) +
        stat_function(fun = dnorm, args = list(mean = p_mu, sd = error_tipico)) +
        geom_area(stat = "function", fun = dnorm, args = list(mean = p_mu, sd = error_tipico), fill = "darkgreen", xlim = c(limite_inferior,limite_superior)) +
        labs(x = "", y = "",title = paste("Intervalo de confianza de la proporci\u00f3n\n(NC=",confianza*100,"%)")) +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = round(c(limite_inferior,p_mu,limite_superior),4)) +
        tema_blanco +
        theme(axis.text.x = element_text(angle = 45)) +
        theme(axis.line.x = element_line(color = "black") )

    }

  }


  IC <- cbind(limite_inferior,limite_superior)
  IC <- as.data.frame(IC)
  row.names(IC) <- NULL

  if(grafico){

    return(list(IC,plot))

  } else{

    return(IC)

  }

}

#' @title Contraste de hipótesis de correlación
#'
#' @description Realiza el contraste de hipótesis sobre el coeficiente de correlación.
#'
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param hipotesis_nula Es un valor numérico. Por defecto el valor está fijado a cero (incorrelación).
#' @param tipo_contraste Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
#'        Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
#'        Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
#'        Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#'
#' @return Esta función devuelve un objeto de la clase \code{data.frame} en el que se incluye la hipótesis nula contrastada, el valor del estadístico de prueba y el p-valor.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' El estadístico del contraste es:
#'
#' \if{html}{\figure{ccorrelacion.png}{width=240px}}
#' \if{latex}{\figure{ccorrelacion.png}{width=4cm}}
#'
#' que se distribuye como una t con n-2 grados de libertad.
#'
#' @seealso \code{\link{ic.correlacion}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats qnorm na.omit pnorm pt qt
#' @import dplyr ggplot2
#'
#' @export
contraste.correlacion <- function(x,
                                  variable = NULL,
                                  introducir = FALSE,
                                  hipotesis_nula = 0,
                                  tipo_contraste = "bilateral",
                                  alfa = 0.05){


  cat("Se asume que la variable bivariante (X,Y) se distribuye conjuntamente normal\n")


  tipo_contraste <- tolower(tipo_contraste)
  tipo_contraste <- match.arg(tipo_contraste)


if(is.null(hipotesis_nula) | !is.numeric(hipotesis_nula)){

  stop("Tienes que introducir un valor para la hip\u00f3tesis nula")

} else{

  H0 <- hipotesis_nula

}

  if(H0 < -1 | H0 > 1){

    stop("El coeficiente de correlaci\u00f3n debe estar comprendido entre -1 y 1.")

  }


if(alfa >= 0 & alfa <=1){

  if(tipo_contraste == "bilateral"){
    valor_critico <- qnorm(alfa/2,lower.tail = F)
  }
  if(tipo_contraste == "cola izquierda"){
    valor_critico <- qnorm(alfa,lower.tail = T)
  }
  if(tipo_contraste == "cola derecha"){
    valor_critico <- qnorm(alfa,lower.tail = F)
  }

  valor_critico <- round(valor_critico,4)

} else{
  stop("El nivel de significaci\u00f3n debe fijarse entre 0 y 1")
}


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 2){
      x <- x
    } else{
      warning("Para calcular el contraste hay que seleccionar 2 variables")
      stop("El conjunto de datos seleccionado no tiene la dimensi\u00f3n adecuada")
    }
  } else{

    if(length(variable) == 2){
        if(is.numeric(variable)){
          if(all(variable <= length(x))){
            variable <- variable
          } else{
            stop("Selecci\u00f3n err\u00f3nea de variable")
          }
        }
      if(is.character(variable)){
        if(all(variable %in% varnames)){
          variable = match(variable,varnames)
          } else {
            stop("El nombre de la variable no es v\u00e1lido")
          }
        }

      x <- x[,variable] %>% as.data.frame()
      names(x) <- varnames[variable]

      } else{
        warning("Para calcular el contraste de correlaci\u00f3 hay que seleccionar dos variables")
        stop("El conjunto de datos seleccionado parece ser no v\u00e1lido")
      }
  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))){
    stop("No puede calcularse el contraste porque la variable seleccionada no es cuantitativa")
  }

  # tama\u00f1o de la muestra
  x <- na.omit(x)
  n <- nrow(x)
  correlacion <- as.numeric(correlacion(x))

} else{   # aqu\u00ed empieza introducir datos

  print("A continuaci\u00f3n, vas a introducir los datos muestrales.")

  n <- readline(prompt = "Introducir el tama\u00f1o de la muestra : ")
  n <- as.numeric(n)

  correlacion <- readline(prompt = "Introducir el valor del coeficiente de correlaci\u00f3n muestral: ")
  correlacion <- as.numeric(correlacion)

  if((correlacion < -1 | correlacion > 1)){

    stop("El coeficiente de correlaci\u00f3n debe estar comprendido entre -1 y 1.")

  }

}


if(hipotesis_nula == 0){

  print("El contraste de independencia es equivalente a contrastar que el coeficiente de correlaci\u00f3n es cero frente a la alternativa de que es distinto de cero")

  estadistico.prueba <- sqrt((correlacion^2 * (n - 2)) / (1 - correlacion^2))

  valor_critico <- round(qt(alfa/2,n-2,lower.tail=FALSE),4)

  pvalor <- 2 * pt(estadistico.prueba, n-2,lower.tail=FALSE)


    if((estadistico.prueba > - valor_critico & estadistico.prueba < valor_critico)){

      cat(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada\npor el intervalo [", -valor_critico," , ",valor_critico,"]\n",sep=""))
      cat("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n\n")


    } else{

      cat(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada\npor el intervalo [", -valor_critico," , ", valor_critico,"]\n",sep=""))
      cat("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n\n")

    }

  } else{

    print("Vas a realizar un contraste bilateal de correlaci\u00f3n")
    warning("Actualmente solo se encuentra implementado el contraste bilateral")

    estadistico.prueba <- 0.5*log((1+correlacion)/(1-correlacion))

    limite_inferior <- round(0.5 * log((1+H0)/(1-H0)) - valor_critico * sqrt(1/(n - 3)),5)
    limite_superior <- round(0.5 * log((1+H0)/(1-H0)) + valor_critico * sqrt(1/(n - 3)),5)

    pvalor <- 2 * pnorm(estadistico.prueba,lower.tail=FALSE)


    if(estadistico.prueba > limite_inferior & estadistico.prueba < limite_superior){

      cat(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada\npor el intervalo [", limite_inferior," , ",limite_superior,"]\n",sep=""))
      cat("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n\n")


    } else{

      cat(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada\npor el intervalo [", limite_inferior," , ",limite_superior,"]\n",sep=""))
      cat("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n\n")

    }

  }


  CH <- cbind(H0,estadistico.prueba,pvalor)
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  return(CH)

}

#' @title Covarianza.
#'
#' @description Calcula la covarianza.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcovarianza.png}{width = 200px}}
#' \if{latex}{\figure{qrcovarianza.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x solo tiene 2 variables (columnas), el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables a seleccionar.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#' @param tipo Es un carácter. Por defecto de calcula la covarianza muestral (tipo = "muestral"). Si tipo = "cuasi", se calcula la cuasi-covarianza muestral.
#'
#' @return Esta función devuelve la covarianza en un objeto de la clase \code{vector}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) La covarianza muestral se obtiene a partir de la siguiente expresión:
#'
#' \if{html}{\figure{covarianzamuestra.png}{width = 400px}}
#' \if{latex}{\figure{covarianzamuestra.png}{options: width=6cm}}
#'
#' (2) Muchos manuales y prácticamente todos los softwares (SPSS, Excel, etc.) calculan la covarianza a partir de la expresión:
#'
#' \if{html}{\figure{covarianzacuasi.png}{width = 400px}}
#' \if{latex}{\figure{covarianzacuasi.png}{options: width=6cm}}
#'
#' Nosotros nos referimos a esta expresión como cuasi-covarianza muestral.
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la covarianza poblacional:
#'
#' \if{html}{\figure{covarianzapob.png}{width = 400px}}
#' \if{latex}{\figure{covarianzapob.png}{options: width=6cm}}

#' @seealso \code{\link{varianza}}, \code{\link{desviacion}},\code{\link{matriz.covar}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @importFrom stats cov
#' @import dplyr
#'
#' @export
covarianza <- function(x, variable = NULL, pesos = NULL, tipo = c("muestral", "cuasi")) {

  tipo <- match.arg(tolower(tipo), c("muestral", "cuasi"))

  if (!is.data.frame(x)) x <- data.frame(x)

  # --- Seleccion de variables ---
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)][1:2]
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables.")
    varnames <- names(x)[variable]
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("Nombre de variable no v\u00e1lido.")
    varnames <- variable
  } else {
    stop("El argumento 'variable' debe ser num\u00e9ico o car\u00e1cter.")
  }

  # --- Caso SIN pesos ---
  if (is.null(pesos)) {
    x_sel <- x[, varnames, drop = FALSE]
    x_sel <- na.omit(x_sel)
    n_eff <- nrow(x_sel)
    if (n_eff < 2) stop("No hay suficientes observaciones completas.")

    cov_val <- stats::cov(x_sel[[1]], x_sel[[2]], use = "complete.obs")
    # R ya devuelve la version cuasi (divisor n-1)
    if (tipo == "muestral") cov_val <- cov_val * ((n_eff - 1) / n_eff)

    result <- as.data.frame(round(as.numeric(cov_val), 4))
    names(result) <- paste0(varnames[1], "_", varnames[2])
    row.names(result) <- NULL

    class(result) <- c("resumen", "data.frame")
    return(result)
  }


  # --- Caso CON pesos ---
  if (length(pesos) != 1) stop("Solo se admite una variable de pesos.")

  # Determinar la columna de pesos desde el data.frame ORIGINAL
  if (is.character(pesos)) {
    if (!pesos %in% names(x)) stop("Nombre de pesos no v\u00e1lido.")
    peso_col <- pesos
  } else if (is.numeric(pesos)) {
    if (pesos > ncol(x)) stop("Indice de columna de pesos fuera de rango del conjunto de datos.")
    peso_col <- names(x)[pesos]
  } else {
    stop("El argumento 'pesos' debe ser numr\u00e9ico o de tipo car\u00e1cter.")
  }

  # Crear nuevo data.frame con las variables y los pesos
  x_sel <- x[, c(varnames, peso_col), drop = FALSE]
  names(x_sel) <- c("var1", "var2", "pesos")

  # Eliminar NA
  x_sel <- na.omit(x_sel)
  n_eff <- nrow(x_sel)
  if (n_eff < 2) stop("No hay suficientes observaciones completas para calcular la covarianza ponderada.")

  # Calcular medias ponderadas
  media1 <- sum(x_sel$var1 * x_sel$pesos, na.rm = TRUE) / sum(x_sel$pesos, na.rm = TRUE)
  media2 <- sum(x_sel$var2 * x_sel$pesos, na.rm = TRUE) / sum(x_sel$pesos, na.rm = TRUE)

  # Covarianza ponderada
  sum_cuad <- sum((x_sel$var1 - media1) * (x_sel$var2 - media2) * x_sel$pesos, na.rm = TRUE)

  if (tipo == "muestral") {
    cov_val <- sum_cuad / sum(x_sel$pesos, na.rm = TRUE)
  } else {
    cov_val <- sum_cuad / (sum(x_sel$pesos, na.rm = TRUE) - 1)
  }

  # Resultado como df
  result <- as.data.frame(round(as.numeric(cov_val), 4))
  names(result) <- paste0(varnames[1], "_", varnames[2])
  row.names(result) <- NULL

  class(result) <- c("resumen", "data.frame")
  return(result)
}

# .onAttach <- function(libname, pkgname) {
#   if (requireNamespace("cli", quietly = TRUE)) {
#     # Mensaje en azul
#     cli::cli_alert_info(col_blue("Este paquete est\u00e1 en desarrollo. Por favor, si detectas errores o quieres hacernos alguna sugerencia, cont\u00e1ctanos."))
#
#     # Enlace al canal de YouTube
#     cli::cli_text(cli::col_blue("Tambi\u00e9n puedes seguirnos en el canal de YouTube: {cli::style_hyperlink('https://go.uv.es/estadistic/youtube', 'https://go.uv.es/estadistic/youtube')}"))
#
#     cli::cli_alert_info(col_blue("Este paquete se desarrolla como parte de un proyecto de innovaci\u00f3n educativa de la Universidad de Valencia."))
#
#
#   } else {
#     # Version alternativa sin cli
#     packageStartupMessage("Este paquete est\u00e1 en desarrollo. Visita nuestro canal: https://go.uv.es/estadistic/youtube")
#   }
# }

.onAttach <- function(libname, pkgname) {
  # No mostrar mensajes en sesiones no interactivas (como R CMD check)
  if (!interactive()) return()

  if (requireNamespace("cli", quietly = TRUE)) {
    # Mensaje en azul usando cli
    cli::cli_alert_info(cli::col_blue(
      "Este paquete est\u00e1 en desarrollo. Por favor, si detectas errores o quieres hacernos alguna sugerencia, cont\u00e1ctanos."
    ))

    cli::cli_text(cli::col_blue(
      "Tambi\u00e9n puedes seguirnos en el canal de YouTube: {cli::style_hyperlink('https://go.uv.es/estadistic/youtube', 'https://go.uv.es/estadistic/youtube')}"
    ))

    cli::cli_alert_info(cli::col_blue(
      "Este paquete se desarrolla como parte de un proyecto de innovaci\u00f3n educativa de la Universidad de Valencia."
    ))
  } else {
    # Version alternativa sin cli
    packageStartupMessage(
      "Este paquete est\u00e1 en desarrollo. Visita nuestro canal: https://go.uv.es/estadistic/youtube"
    )
  }
}


# Funcion auxiliar para color azul
col_blue <- function(x) {
  if (requireNamespace("cli", quietly = TRUE)) {
    cli::col_blue(x)
  } else {
    x
  }
}

#' @title Contraste de hipótesis sobre la diferencia de medias.
#'
#' @description Realiza el contraste de hipótesis sobre la diferencia de medias poblacionales.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcdifmedias.png}{width = 200px}}
#' \if{latex}{\figure{qrcdifmedias.png}{options: width=3cm}}
#'
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere solo a dos variables, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param var_pob Es un carácter. Indica si la varianza poblacional es conocida (por defecto, \code{var_pob = "conocida"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.
#' @param iguales Si las varianzas poblacionales se consideran distintas (por defecto \code{iguales = FALSE}) o iguales (cambiar el argumento a \code{iguales = TRUE}).
#' @param hipotesis_nula Es un valor numérico.
#' @param tipo_contraste Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
#'        Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
#'        Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
#'        Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba, el p-valor y  el intervalo de confianza para la diferencia de medias muestrales supuesta cierta la hipótesis nula. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @seealso \code{\link{ic.diferencia.medias}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm na.omit dnorm pt qt dt
#' @import dplyr ggplot2
#'
#' @export
contraste.diferencia.medias <- function(x,
                                        variable = NULL,
                                        introducir = FALSE,
                                        var_pob = c("conocida","desconocida"),
                                        iguales = FALSE,
                                        hipotesis_nula = 0,
                                        tipo_contraste = c("bilateral","cola derecha","cola izquierda"),
                                        alfa = 0.05,
                                        grafico = FALSE){


  tipo_contraste <- tolower(tipo_contraste)
  tipo_contraste <- match.arg(tipo_contraste)

  var_pob <- tolower(var_pob)
  var_pob <- match.arg(var_pob)

  if(is.null(hipotesis_nula) | !is.numeric(hipotesis_nula)){

    stop("Tienes que introducir un valor para la hipotesis nula")

  } else{

    H0 <- hipotesis_nula

  }


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 2){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variables.")

    }

  } else if(length(variable) == 2){

    if(is.numeric(variable)){

      if(all(variable <= length(x))){

        variable <- variable

      } else{

        stop("Seleccion erronea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es valido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el intervalo de confianza hay que seleccionar dos variables")
    stop("El conjunto de datos seleccionado no es adecuado.")

  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {

    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")

  }

  # tama\u00f1os de la muestras
  n1 <- length(x[1][!is.na(x[1])])
  n2 <- length(x[2][!is.na(x[2])])
  n <- c(n1,n2)

  # medias muestrales
  media1 <- as.numeric(media(x[1]))
  media2 <- as.numeric(media(x[2]))


  if(var_pob == "conocida"){

    if(isTRUE(iguales)){

      varianza_pob1 <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
      varianza_pob1 <- as.numeric(varianza_pob1)

      varianza_pob2 <- varianza_pob1

    } else{

      varianza_pob1 <- readline(prompt = "Introducir el valor de la varianza poblacional 1: ")
      varianza_pob1 <- as.numeric(varianza_pob1)

      varianza_pob2 <- readline(prompt = "Introducir el valor de la varianza poblacional 2: ")
      varianza_pob2 <- as.numeric(varianza_pob2)

    }

  } else{

    var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      var_mu1 = as.numeric(varianza(x[1]))
      var_mu2 = as.numeric(varianza(x[2]))


    } else{

      var_mu1 <- as.numeric(varianza(x[1],tipo="cuasi"))
      var_mu2 <- as.numeric(varianza(x[2],tipo="cuasi"))

    }

  }


} else{   # aqu\uf00ed empieza introducir datos

  n <- c()
  media <- c()

  for(i in 1:2){

    n0 <- readline(prompt = paste("Introducir el tama\u00f1o de la muestra ",i,": ",sep=""))
    n0 <- as.numeric(n0)

    n <- c(n,n0)

    media0 <- readline(prompt = paste("Introducir el valor de la media muestral ",i,": ",sep=""))
    media0 <- as.numeric(media0)

    media <- c(media,media0)

  }

    if(var_pob == "conocida"){

      varianza_pob <- c()

      if(isTRUE(iguales)){

        varianza_pob0 <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
        var_pob0 <- as.numeric(varianza_pob0)

        varianza_pob <- c(var_pob0,var_pob0)


      } else{

        for(i in 1:2){

        varianza_pob0 <- readline(prompt = paste("Introducir el valor de la varianza poblacional ",i,": ",sep=""))
        var_pob0 <- as.numeric(varianza_pob0)

        varianza_pob <- c(varianza_pob,var_pob0)

        }
      }

    } else{ # las varianzas son desconocidas

      var_mu <- c()

      var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

      for(i in 1:2){

        if(var_muestra == 1){

          varianza_muestral0 <- readline(prompt = paste("Introducir el valor de la varianza muestral ",i,": ",sep=""))
          var_mu0 <- as.numeric(varianza_muestral0)

          var_mu <- c(var_mu, var_mu0)

        } else{

          varianza_cuasi0 <- readline(prompt = paste("Introducir el valor de la cuasivarianza muestral ",i,": ",sep=""))
          var_mu0 <- as.numeric(varianza_cuasi0)

          var_mu <- c(var_mu, var_mu0)

        }
      }
    }

  n1 <- n[1]
  n2 <- n[2]
  media1 <- media[1]
  media2 <- media[2]

  if(var_pob == "conocida"){
    varianza_pob1 <- varianza_pob[1]
    varianza_pob2 <- varianza_pob[2]

  } else{
    var_mu1 <- var_mu[1]
    var_mu2 <- var_mu[2]
  }

}


# calculo contrastes

  dif_medias <- media1 - media2

  if (n1 > 30 & n2 > 30){
    tamano <- "grande"
  } else {
    tamano <- "pequeno"
  }

  if(var_pob == "conocida"){

    # caso 1
    error_tipico <- sqrt(varianza_pob1/n1 + varianza_pob2/n2)

  } else{ # var poblacionales desconocidas

    if(tamano == "grande"){

      print("Como los tama\u00f1os muestrales son grandes, se aproximan las varianzas poblacionales por las varianzas (o cuasi-varianzas) muestrales")
      error_tipico <- sqrt(var_mu1/n1 + var_mu2/n2)

    } else{  # tama\u00f1os muestrales peque\u00f1os

      if(isFALSE(iguales)){ # varianzas desconocidas y distintas

        if(var_muestra == 1){

          # varianzas poblacionales desconocidas y distintas (varianza muestral)
          numerador <- (var_mu1/(n1-1) + var_mu2/(n2-1))^2
          denominador <- ((var_mu1/(n1-1))^2/(n1+1))+((var_mu2/(n2-1))^2/(n2+1))
          gl <- (numerador / denominador)-2
          gl <- ceiling(gl)
          error_tipico <- sqrt((var_mu1/(n1-1))+(var_mu2/(n2-1)))

        } else{

          # varianzas poblacionales desconocidas y distintas (cuasivarianza muestral)
          print("Este es el contraste que generalmente calculan los softwares (SPSS, Excel, etc.)")
          numerador <- (var_mu1/n1 + var_mu2/n2)^2
          denominador <- ((var_mu1/n1)^2/(n1-1))+((var_mu2/n2)^2/(n2-1))
          gl <- numerador / denominador
          gl <- ceiling(gl)
          error_tipico <- sqrt((var_mu1/n1)+(var_mu2/n2))

        }

      } else{ # varianzas desconocida pero iguales

        gl <- n1+n2-2

        if(var_muestra == 1){

          # caso 2_1 con varianza muestral
          numerador <- sqrt(n1+n2) * sqrt(n1*var_mu1 + n2*var_mu2)
          denominador <- sqrt(n1*n2) * sqrt(n1+n2-2)

          error_tipico <- numerador/denominador

        } else{   # comprobado con la cuasi

          # caso 2_2 con cuasivarianza muestral

          print("Este es el contraste que generalmente calculan los softwares (SPSS, Excel, etc.)")
          numerador <- var_mu1*(n1-1) + var_mu2*(n2-1)
          denominador <- n1+n2-2

          error_tipico <- sqrt(numerador/denominador)*sqrt((n1+n2)/(n1*n2))

        }

      }
    }
  }

  estadistico.prueba <- (dif_medias - H0)/error_tipico

  if(alfa >= 0 & alfa <=1){

    if(var_pob == "conocida" | (var_pob == "desconocida" & tamano == "grande")){

      if(tipo_contraste == "bilateral"){
        valor_critico <- qnorm(alfa/2,lower.tail = F)
      }
      if(tipo_contraste == "cola izquierda"){
        valor_critico <- qnorm(alfa,lower.tail = T)
      }
      if(tipo_contraste == "cola derecha"){
        valor_critico <- qnorm(alfa,lower.tail = F)
      }

    } else{

      if(tipo_contraste == "bilateral"){
        valor_critico <- qt(alfa/2,gl,lower.tail = F)
      }
      if(tipo_contraste == "cola izquierda"){
        valor_critico <- qt(alfa,gl,lower.tail = T)
      }
      if(tipo_contraste == "cola derecha"){
        valor_critico <- qt(alfa,gl,lower.tail = F)
      }

    }

    valor_critico <- round(valor_critico,4)

  } else{
    stop("El nivel de significacion debe fijarse entre 0 y 1")
  }


  if(tipo_contraste == "bilateral"){

    estadistico.prueba2 <- abs(estadistico.prueba)
    media_inf <- H0 - valor_critico * error_tipico
    media_sup <- H0 + valor_critico * error_tipico

    if(var_pob == "conocida" | (var_pob == "desconocida" & tamano == "grande")){

      pvalor <- 2 * pnorm(estadistico.prueba2,lower.tail=FALSE)

      if(isTRUE(grafico)){

        plot <- ggplot(NULL, aes(c(-4,4))) +
          geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-4, -valor_critico)) +
          geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-valor_critico, valor_critico)) +
          geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 4)) +
          geom_vline(xintercept = -estadistico.prueba2, linetype = "dashed") +
          geom_vline(xintercept = estadistico.prueba2, linetype = "dashed") +
          labs(x = "", y = "") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(estadistico.prueba2,-estadistico.prueba2,-valor_critico,valor_critico)) +
          theme(axis.text.x = element_text(angle = 45))

        df <- data.frame(x = c(H0-4*error_tipico, H0+4*error_tipico))
        plot2 <- ggplot(df, aes(x)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(df[1,1], media_inf)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "darkgreen", xlim = c(media_inf, media_sup)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(media_sup, df[2,1])) +
          geom_vline(xintercept = dif_medias, linetype = "dashed") +
          labs(x = "", y = "",title="Intervalo de la diferencia de medias muestrales\n(supuesta H0 cierta)") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(media_inf,dif_medias,media_sup)) +
          theme(axis.text.x = element_text(angle = 45))


      }


    } else{

      pvalor <- 2 * pt(estadistico.prueba2,gl, lower.tail=FALSE)

      if(isTRUE(grafico)){

        plot <- ggplot(NULL, aes(c(-3,3))) +
          geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "darkgreen", xlim = c(-valor_critico, valor_critico)) +
          geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(-3, -valor_critico)) +
          geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(valor_critico, 3)) +
          geom_vline(xintercept = 0, color = "black") +
          geom_vline(xintercept = -estadistico.prueba2, color = "blue", linetype = "dashed") +
          geom_vline(xintercept = estadistico.prueba2, color = "blue", linetype = "dashed") +
          labs(title = paste("Distribuci\u00f3n t con ", gl, " grados de libertad",sep=""), x = "", y = "") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(-estadistico.prueba2,estadistico.prueba2,valor_critico,-valor_critico)) +
          theme(axis.text.x = element_text(angle = 45))

        plot2 <- NULL


      }

    }

    if(estadistico.prueba >= -valor_critico & estadistico.prueba <=  valor_critico){

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

  }

  if(tipo_contraste == "cola derecha"){

    media_inf <- -Inf
    media_sup <- H0 + valor_critico * error_tipico

    if(var_pob == "conocida" | (var_pob == "desconocida" & tamano == "grande")){

      pvalor <- pnorm(estadistico.prueba,lower.tail=FALSE)

      if(isTRUE(grafico)){

        plot <- ggplot(NULL, aes(c(-3,3))) +
          geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-3L,valor_critico)) +
          geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 3L)) +
          geom_vline(xintercept = estadistico.prueba, linetype = "dashed") +
          labs(x = "", y = "") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(estadistico.prueba,valor_critico)) +
          theme(axis.text.x = element_text(angle = 45))

        df <- data.frame(x = c(H0-4*error_tipico, H0+4*error_tipico))
        plot2 <- ggplot(df, aes(x)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "darkgreen", xlim = c(df[1,1], media_sup)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(media_sup, df[2,1])) +
          geom_vline(xintercept = dif_medias, linetype = "dashed") +
          labs(x = "", y = "",title="Intervalo de la diferencia de medias muestrales\n(supuesta H0 cierta)") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(dif_medias,media_sup)) +
          theme(axis.text.x = element_text(angle = 45))

      }


    } else{

      pvalor <- pt(estadistico.prueba,gl, lower.tail=FALSE)

      if(isTRUE(grafico)){

        plot <- ggplot(NULL, aes(c(-4,4))) +
          geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "darkgreen", xlim = c(-4, valor_critico)) +
          geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(valor_critico, 4)) +
          geom_vline(xintercept = 0, color = "black") +
          geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
          labs(title = paste("Distribuci\u00f3n t con ", gl, " grados de libertad",sep=""), x = "", y = "") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(estadistico.prueba,valor_critico)) +
          theme(axis.text.x = element_text(angle = 45))

        plot2 <- NULL


      }


    }

    if(estadistico.prueba > valor_critico){

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }


  }

  if(tipo_contraste == "cola izquierda"){

    media_inf <- H0 + valor_critico * error_tipico # valor critico negativo
    media_sup <- Inf

    if(var_pob == "conocida" | (var_pob == "desconocida" & tamano == "grande")){

      pvalor <- pnorm(estadistico.prueba,lower.tail=TRUE)

      if(isTRUE(grafico)){

        plot <- ggplot(NULL, aes(c(-4,4))) +
          geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-4, valor_critico)) +
          geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(valor_critico, 4)) +
          geom_vline(xintercept = estadistico.prueba, linetype = "dashed") +
          labs(x = "z", y = "") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(estadistico.prueba,-valor_critico)) +
          theme(axis.text.x = element_text(angle = 45))

        df <- data.frame(x = c(H0-4*error_tipico, H0+4*error_tipico))
        plot2 <- ggplot(df, aes(x)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(df[1,1], media_inf)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "darkgreen", xlim = c(media_inf, df[2,1])) +
          geom_vline(xintercept = dif_medias, linetype = "dashed") +
          labs(x = "", y = "",title="Intervalo de la diferencia de medias muestrales\n(supuesta H0 cierta)") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(media_inf,dif_medias)) +
          theme(axis.text.x = element_text(angle = 45))

      }

    } else{

      pvalor <- pt(estadistico.prueba, gl, lower.tail=TRUE)

      if(isTRUE(grafico)){

        plot <- ggplot(NULL, aes(c(-3,3))) +
          geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "darkgreen", xlim = c(valor_critico,3)) +
          geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(-3,valor_critico)) +
          geom_vline(xintercept = 0, color = "black") +
          geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
          labs(title = paste("Distribuci\u00f3n t con ", gl, " grados de libertad",sep=""), x = "", y = "") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(estadistico.prueba,valor_critico)) +
          theme(axis.text.x = element_text(angle = 45))

        plot2 <- NULL


      }

    }

    if(estadistico.prueba < valor_critico){

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

  }


  CH <- cbind(H0,estadistico.prueba,pvalor)
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  Imedia <- cbind(`limite inferior`=media_inf,`limite superior`=media_sup)

  if(isTRUE(grafico)){

    grafico <- list(plot,plot2)

    return(list(`Estadistico`=CH,`Intervalo de la diferencia de medias muestrales (supuesta H0 cierta)`= Imedia,`Graficos`= grafico))

  } else{

    return(list(`Estadistico`=CH,`Intervalo de la diferencia de medias muestrales (supuesta H0 cierta)`= Imedia))

  }
}

#' @title Intervalo confianza para la diferencia de medias.
#'
#' @description Calcula el intervalo de confianza de la diferencia de medias poblacionales.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qricdiferenciamedias.png}{width = 200px}}
#' \if{latex}{\figure{qricdiferenciamedias.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere a dos variables, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param poblacion Es un carácter. Indica la distribución de probabilidad de la población. Por defecto \code{poblacion = "normal"}. Si la distribución de la población es desconocida, cambiar a \code{poblacion = "desconocida"}.
#' @param var_pob Es un carácter. Indica si la varianza poblacional es conocida (por defecto, \code{var_pob = "conocida"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.
#' @param iguales Por defecto se considera que las varianzas poblacionales son distintas (\code{iguales = FALSE}). En el supuesto de varianzas poblacionales iguales cambiar el argumento a \code{iguales = TRUE}.
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#'
#' @return Devuelve el intervalo de confianza de la diferencia de medias poblacionales en un objeto de tipo \code{data.frame}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' Se obtienen los intervalos según los siguientes casos:
#'
#' Caso 1: Varianzas poblacionales conocidas
#'
#' \if{html}{\figure{icdifmedias1.png}{width = 60px}}
#' \if{latex}{\figure{icdifmedias1.png}{options: width=10cm}}
#'
#' Nota: Si los tamaños muestrales nx y ny son suficientemente grandes, pueden estimarse las varianzas poblacionales
#' por sus correspondientes varianzas (o cuasivarianzas), incluso aunque las distribuciones poblacionales no sean normales
#' (por aplicación del TCL).
#'
#' Caso 2. Varianzas poblacionales desconocidas pero iguales
#'
#' (2.1) con varianza muestral:
#'
#' \if{html}{\figure{icdifmedias2.png}{width = 560px}}
#' \if{latex}{\figure{icdifmedias2.png}{options: width=10cm}}
#'
#' (2.2) con cuasivarianza muestral:
#'
#' \if{html}{\figure{icdifmedias2cuasi.png}{width = 600px}}
#' \if{latex}{\figure{icdifmedias2cuasi.png}{options: width=10cm}}
#'
#' Nota: Tanto en el caso (2.1) como (2.2) la distribución t tiene (nx+ny-2) grados de libertad.
#'
#' Caso 3. Varianzas poblacionales desconocidas y distintas
#'
#' (3.1) con varianza muestral:
#'
#' \if{html}{\figure{icdifmedias3.png}{width = 480px}}
#' \if{latex}{\figure{icdifmedias3.png}{options: width=8cm}}
#'
#'la distribución t con grados de libertad igual al entero más próximo de v.
#'
#' \if{html}{\figure{icdifmedias3gl.png}{width = 520px}}
#' \if{latex}{\figure{icdifmedias3gl.png}{options: width=8cm}}
#'
#' (3.2) con cuasivarianza muestral:
#'
#' \if{html}{\figure{icdifmedias3cuasi.png}{width = 480px}}
#' \if{latex}{\figure{icdifmedias3cuasi.png}{options: width=7cm}}
#'
#' la distribución t con grados de libertad igual a v, donde v = (parte entera de v*) + 1
#'
#' \if{html}{\figure{icdifmedias3cuasigl.png}{width = 520px}}
#' \if{latex}{\figure{icdifmedias3cuasigl.png}{options: width=6cm}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm qnorm pt qt na.omit
#' @import dplyr ggplot2
#'
#' @export
ic.diferencia.medias <- function(x,
                                 variable = NULL,
                                 introducir = FALSE,
                                 poblacion = c("normal","desconocida"),
                                 var_pob = c("conocida","desconocida"),
                                 iguales = FALSE,
                                 confianza = 0.95){


  poblacion <- tolower(poblacion)
  poblacion <- match.arg(poblacion)

  var_pob <- tolower(var_pob)
  var_pob <- match.arg(var_pob)


if(confianza >= 0 & confianza <=1){

  confianza <- confianza
  alfa2 <- (1- confianza)/2

} else{

  stop("El nivel de confianza debe fijarse entre 0 y 1")

}


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 2){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variables.")

    }

  } else if(length(variable) == 2){

    if(is.numeric(variable)){

      if(all(variable <= length(x))){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es v\u00e1lido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el intervalo de confianza hay que seleccionar dos variables")
    stop("El conjunto de datos seleccionado no es adecuado.")

  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {

    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")

  }

  # tama\i00f1os de la muestras
  n1 <- length(x[1][!is.na(x[1])])
  n2 <- length(x[2][!is.na(x[2])])
  n <- c(n1,n2)

  # medias muestrales
  media1 <- as.numeric(media(x[1]))
  media2 <- as.numeric(media(x[2]))


  if(var_pob == "conocida"){

    if(isTRUE(iguales)){

      varianza_pob1 <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
      varianza_pob1 <- as.numeric(varianza_pob1)

      varianza_pob2 <- varianza_pob1

    } else{

      varianza_pob1 <- readline(prompt = "Introducir el valor de la varianza poblacional 1: ")
      varianza_pob1 <- as.numeric(varianza_pob1)

      varianza_pob2 <- readline(prompt = "Introducir el valor de la varianza poblacional 2: ")
      varianza_pob2 <- as.numeric(varianza_pob2)

    }

  } else{

    var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      var_mu1 = as.numeric(varianza(x[1]))
      var_mu2 = as.numeric(varianza(x[2]))


    } else{

      var_mu1 <- as.numeric(varianza(x[1],tipo="cuasi"))
      var_mu2 <- as.numeric(varianza(x[2],tipo="cuasi"))

    }

  }


} else{   # aqu\u00ed empieza introducir datos

  n <- c()
  media <- c()

  for(i in 1:2){

    n0 <- readline(prompt = paste("Introducir el tama\u00f1o de la muestra ",i,": ",sep=""))
    n0 <- as.numeric(n0)

    n <- c(n,n0)

    media0 <- readline(prompt = paste("Introducir el valor de la media muestral ",i,": ",sep=""))
    media0 <- as.numeric(media0)

    media <- c(media,media0)

  }

    if(var_pob == "conocida"){

      varianza_pob <- c()

      if(isTRUE(iguales)){

        varianza_pob0 <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
        var_pob0 <- as.numeric(varianza_pob0)

        varianza_pob <- c(var_pob0,var_pob0)


      } else{

        for(i in 1:2){

        varianza_pob0 <- readline(prompt = paste("Introducir el valor de la varianza poblacional ",i,": ",sep=""))
        var_pob0 <- as.numeric(varianza_pob0)

        varianza_pob <- c(varianza_pob,var_pob0)

        }
      }

    } else{ # las varianzas son desconocidas

      var_mu <- c()

      var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

      for(i in 1:2){

        if(var_muestra == 1){

          varianza_muestral0 <- readline(prompt = paste("Introducir el valor de la varianza muestral ",i,": ",sep=""))
          var_mu0 <- as.numeric(varianza_muestral0)

          var_mu <- c(var_mu, var_mu0)

        } else{

          varianza_cuasi0 <- readline(prompt = paste("Introducir el valor de la cuasivarianza muestral ",i,": ",sep=""))
          var_mu0 <- as.numeric(varianza_cuasi0)

          var_mu <- c(var_mu, var_mu0)

        }
      }
    }

  n1 <- n[1]
  n2 <- n[2]
  media1 <- media[1]
  media2 <- media[2]

  if(var_pob == "conocida"){
    varianza_pob1 <- varianza_pob[1]
    varianza_pob2 <- varianza_pob[2]

  } else{
    var_mu1 <- var_mu[1]
    var_mu2 <- var_mu[2]
  }

}

# calculo intervalos

  dif_medias <- media1 - media2

  if(poblacion == "normal"){

    if(var_pob == "conocida"){

      # caso de varianza poblacionales conocidas (iguales o distintas por la seleccion de varianza poblacional)
      # casos 1 y 2
      valor_critico <- qnorm(alfa2,lower.tail = FALSE)
      error_tipico <- sqrt(varianza_pob1/n1 + varianza_pob2/n2)


    } else{ # varianzas poblacionales desconocidas

      if(isTRUE(iguales)){ # varianzas poblaciones desconocidas e iguales

        if(all(n>=30)){ # muestras grandes (caso 2)

          print("Como los tama\u00f1os muestrales son grandes se estima la varianza poblacional")
          valor_critico <- qnorm(alfa2,lower.tail = FALSE)
          error_tipico <- sqrt(var_mu1/n1 + var_mu2/n2)

        } else{ # muestras peque\u00f1as

          # caso 3. n1 y n2 son peque\u00f1as

          valor_critico <- qt(alfa2,n1+n2-2,lower.tail = FALSE)

          if(var_muestra == 1){

            # caso 3_1 con varianza muestral
            numerador <- sqrt(n1+n2) * sqrt(n1*var_mu1 + n2*var_mu2)
            denominador <- sqrt(n1*n2) * sqrt(n1+n2-2)
            error_tipico <- numerador/denominador

          } else{   # comprobado con la cuasi

            # caso 3_2 con cuasivarianza muestral

            print("Este es el intervalo que generalmente calculan los softwares (SPSS, Excel, etc.)")
            numerador <- var_mu1*(n1-1) + var_mu2*(n2-1)
            denominador <- n1+n2-2

            error_tipico <- sqrt(numerador/denominador)*sqrt((n1+n2)/(n1*n2))

          }

        }

      } else {  # varianzas poblaciones desconocidas y distintas

        if(all(n>=30)){ # muestras grandes (caso 2)

          print("Como los tama\u00f1os muestrales son grandes se estima la varianza poblacional")
          valor_critico <- qnorm(alfa2,lower.tail = FALSE)
          error_tipico <- sqrt(var_mu1/n1 + var_mu2/n2)

        } else{ # muestras pequenas (caso 4)

          if(var_muestra == 1){
          # caso 4.1
          # varianzas poblacionales desconocidas y distintas (varianza muestral)
          numerador <- (var_mu1/(n1-1) + var_mu2/(n2-1))^2
          denominador <- ((var_mu1/(n1-1))^2/(n1+1))+((var_mu2/(n2-1))^2/(n2+1))
          gl <- (numerador / denominador)-2
          gl <- ceiling(gl)
          valor_critico <- qt(alfa2,gl,lower.tail = FALSE)
          error_tipico <- sqrt((var_mu1/(n1-1))+(var_mu2/(n2-1)))

          } else{
          # caso 4.2
          # varianzas poblacionales desconocidas y distintas (cuasivarianza muestral)
          print("Este es el intervalo que generalmente calculan los softwares (SPSS, Excel, etc.)")
          numerador <- (var_mu1/n1 + var_mu2/n2)^2
          denominador <- ((var_mu1/n1)^2/(n1-1))+((var_mu2/n2)^2/(n2-1))
          gl <- numerador / denominador
          gl <- ceiling(gl)
          valor_critico <- qt(alfa2,gl,lower.tail = FALSE)
          error_tipico <- sqrt((var_mu1/n1)+(var_mu2/n2))

          }
        }
      }
    }

  } else{ # distribuci\u00f3n es desconocida

    if(var_pob == "conocida"){

      if(n1 >= 30 & n2 >= 30){ # muestras grandes

        print("Como los tama\u00f1os muestrales son grandes, por aplicaci\u00f3n del TCL se puede suponer que las medias muestrales se distribuyen aproximadamente normales")

        valor_critico <- qnorm(alfa2,lower.tail = FALSE)
        error_tipico <- sqrt(var_mu1/n1 + var_mu2/n2)

      } else {

        print("La distribuci\u00f3n de probabilidad de la poblaci\u00f3n es desconocida y el tama\u00f1o de alguna muestra es peque\u00f1o (n<30)")
        stop("Bajo estas condiciones no es posible estimar el intervalo de confianza")

      }


    } else{ # poblacion desconocida y varianzas poblaciones desconocidas

      if(n1 >= 30 & n2 >= 30){ # muestras grandes

        # se\u00fan libro de casas

        valor_critico <- qnorm(alfa2,lower.tail = FALSE) # valor critico normal

        if(isTRUE(iguales)){

          if(var_muestra == 1){

            # similar caso 3_1 con varianza muestral
            numerador <- sqrt(n1+n2) * sqrt(n1*var_mu1 + n2*var_mu2)
            denominador <- sqrt(n1*n2) * sqrt(n1+n2-2)
            error_tipico <- numerador/denominador

          } else{   # comprobado con la cuasi

            # similar caso 3_2 con cuasivarianza muestral

            print("Este es el intervalo que generalmente calculan los softwares (SPSS, Excel, etc.)")
            numerador <- var_mu1*(n1-1) + var_mu2*(n2-1)
            denominador <- n1+n2-2

            error_tipico <- sqrt(numerador/denominador)*sqrt((n1+n2)/(n1*n2))

          }


        } else {

          if(var_muestra == 1){

            # similar caso 4_1 con varianza muestral (se estima var_pob por varianza muestral)
            error_tipico <- sqrt((var_mu1/n1)+(var_mu2/n2))

          } else{   # comprobado con la cuasi (se estima var_pob por estimador insesgado: cuasivar muestral)

            # similar caso 4_2 con cuasivarianza muestral
            error_tipico <- sqrt((var_mu1/n1)+(var_mu2/n2))

          }

        }

      } else{ #muestras peque\u00f1as

        print("La distribuci\u00f3n de probabilidad de la poblaci\u00f3 y su varianza son desconocidas. Adem\u00e1s, el tama\u00f1o de alguna muestra es peque\u00f1o (n<30)")
        stop("Bajo estas condiciones no es posible estimar el intervalo de confianza")


      }

    }

  }

  limite_inferior <- dif_medias - valor_critico * error_tipico
  limite_superior <- dif_medias + valor_critico * error_tipico


  IC <- cbind(limite_inferior,limite_superior)
  IC <- as.data.frame(IC)
  row.names(IC) <- NULL

  return(IC)

}

#' @title Intervalo confianza para la diferencia de dos proporciones.
#'
#' @description Calcula el intervalo de confianza de la diferencia de dos proporciones.
#'
#' \if{html}{\figure{qricdiferenciaproporciones.png}{width = 200px}}
#' \if{latex}{\figure{qricdiferenciaproporciones.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si introducir = FALSE (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos x y/o variable. Si introducir = TRUE, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, confianza = 0.95 (95 por ciento)
#' @param grafico Es un valor lógico. Por defecto grafico = FALSE. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a grafico = TRUE. Nota: Esta opción no está implementada para todos los casos.#'
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' Se obtiene el intervalo:
#'
#' \if{html}{\figure{icdifproporciones.png}{width = 520px}}
#' \if{latex}{\figure{icdifproporciones.png}{options: width=12cm}}
#'
#' Nota: El usuario puede seguir dos estrategias:
#' (1) Sustituir las proporciones muestrales del error típico por sus estimaciones máximo-verosímiles (proporciones muestrales)
#' (2) Considerar el caso: p=q=0.5
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm qnorm na.omit
#' @import dplyr ggplot2
#'
#' @export
ic.diferencia.proporciones <- function(x,
                                       variable = NULL,
                                       introducir = FALSE,
                                       confianza = 0.95,
                                       grafico = FALSE){


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 2){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar dos variables")
      stop("El conjunto de datos seleccionado tiene mas de 2 variables.")

    }

  } else if(length(variable) == 2){

    if(is.numeric(variable)){

      if(all(variable <= length(x))){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variables")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es v\u00e1lido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el intervalo de confianza hay que seleccionar dos variables")
    stop("El conjunto de datos seleccionado no es adecuado.")

  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {

    stop("No puede calcularse el intervalo de confianza porque las variables seleccionadas no son cuantitativas")

  }

  x1 <- na.omit(x[1])
  x2 <- na.omit(x[2])


  if(!all((x1 == 0) | x1 ==1)){

    print("Aplica a tus datos la condici\u00f3n que debe cumplir la poblaci\u00f3n para transfomar los datos en ceros (ausencia/no \u00e9xito) y unos (presencia/\u00e9xito)")
    stop("Los valores en la muestra deben ser 0 y 1.")

  }

  if(!all((x2 == 0) | x2 ==1)){

    print("Aplica a tus datos la condici\u00f3n que debe cumplir la poblaci\u00f3n para transfomar los datos en ceros (ausencia/no \u00e9xito) y unos (presencia/\u00e9xito)")
    stop("Los valores en la muestra deben ser 0 y 1.")

  }


  # tama\u00f1o de la muestra
  n1 <- nrow(x1)
  n2 <- nrow(x2)

  # media muestral
  p_mu1 <- round(sum(x1)/n1,6)
  p_mu2 <- round(sum(x2)/n2,6)


} else{   # aqu\u00ed empieza introducir datos


  print("Primero vas a introducir los datos de la muestra 1 y a continuaci\u00f3n introducir\u00e1s los datos de la muestra 2")
  print("Si los datos provienen de encuestas realizadas antes y despu\u00e9s de una determinada acci\u00f3n, introduce primero los datos de la encuesta realizada despu\u00e9s de dicha acci\u00f3n.")

  n1 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 1: ")
  n1 <- as.numeric(n1)


  p_mu1 <- readline(prompt = "Introducir el valor de la proporci\u00f3n muestral 1: ")
  p_mu1 <- as.numeric(p_mu1)


  n2 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 2: ")
  n2 <- as.numeric(n2)

  p_mu2 <- readline(prompt = "Introducir el valor de la proporci\u00f3n muestral 2: ")
  p_mu2 <- as.numeric(p_mu2)


}

  if(confianza >= 0 & confianza <=1){

    confianza <- confianza
    alfa_2 <- (1-confianza)/2
    valor_critico <- qnorm(alfa_2,lower.tail=FALSE)

  } else{

    stop("El nivel de confianza debe fijarse entre 0 y 1")

  }

  aproximacion <- as.numeric(readline('\u00bfQue estrategia quieres utilizar para aproximar el valor de p en el error t\u00edpico? \n 1. "Aproximar por las proporciones muestrales" \n 2. "p=q=0.5" \n'))

  if(aproximacion == 1){

    error_tipico <- sqrt((p_mu1 * (1-p_mu1))/n1 + (p_mu2 * (1-p_mu2))/n2)

  } else{

    error_tipico <- sqrt(0.25/n1 + 0.25/n2)

  }

  limite_inf <- (p_mu1 - p_mu2) - valor_critico * error_tipico
  limite_sup <- (p_mu1 - p_mu2) + valor_critico * error_tipico

  if(limite_inf < limite_sup){

    limite_inferior <- limite_inf
    limite_superior <- limite_sup

  } else{

    limite_inferior <- limite_sup
    limite_superior <- limite_inf
  }

  if(grafico){

      seq <- seq(-4,4,length=1000) * error_tipico + (p_mu1 - p_mu2)
      seq <- as.data.frame(seq)

      plot <- ggplot(seq, aes(seq)) +
        stat_function(fun = dnorm, args = list(mean = (p_mu1 - p_mu2), sd = error_tipico)) +
        geom_area(stat = "function", fun = dnorm, args = list(mean = (p_mu1 - p_mu2), sd = error_tipico), fill = "darkgreen", xlim = c(limite_inferior,limite_superior)) +
        labs(x = "", y = "",title = paste("Intervalo de confianza de la diferencia de proporciones\n(NC=",confianza*100,"%)")) +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = round(c(limite_inferior,(p_mu1 - p_mu2),limite_superior),4)) +
        tema_blanco +
        theme(axis.text.x = element_text(angle = 45)) +
        theme(axis.line.x = element_line(color = "black") )

  }

  IC <- cbind(limite_inferior,limite_superior)
  IC <- as.data.frame(IC)
  row.names(IC) <- NULL

  if(grafico){

    return(list(IC,plot))

  } else{

    return(IC)

  }

}

#' @title Contraste de Homogeneidad de Poblaciones.
#'
#' @description Contrasta la hipótesis de que todas las poblaciones constituidas por una variable categórica o criterio de clasificación de las que se han seleccionado las muestras son homogéneas (tienen la misma distribución de probabilidad).
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' @param x Conjunto de datos. Puede ser una matriz o un dataframe. Debe contener sólo 2 columnas.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre el número de muestras, sobre el número de categorías de la variable poblacional, el nombre de cada categoría de la variable o de cada muestra en la posición de fila y el nombre de la categoría de la variable o de cada muestra en la posición de columna. A continuación se abrirá una ventana con un editor de datos y deberá introducir los valores de las frecuencias observadas de la tabla de contingencia.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba y el p-valor. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con el valor crítico.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) El estadístico del contraste de homogeneidad es:
#'
#' \deqn{\chi ^{2} = \sum_{i=1}^{I} \sum_{j=1}^{J}
#' \frac{(O_{ij} - E_{ij})^{2}}{E_{ij}} =
#' \sum_{i=1}^{I} \sum_{j=1}^{J}
#' \frac{(O_{ij} - \frac{O_{i.} \times O_{.j}}{n})^{2}}{\frac{O_{i.} \times O_{.j}}{n}}}
#'
#' donde \eqn{O_{ij}} son las frecuencias conjuntas observadas, \eqn{E_{ij}} son las frecuencias teóricas o esperadas,
#' \eqn{O_{i.}} es el tamaño de cada muestra, \eqn{O_{.j}} es el total de individuos del conjunto de las muestras
#' clasificados en la categoría \eqn{j} de la variable, y \eqn{n} es la suma de los tamaños de todas las muestras.
#'
#' \deqn{\chi_{(I-1)(J-1)}^{2}}
#'
#' donde \eqn{I} es el número de muestras y \eqn{J} es el número de categorías de la variable.
#'
#' Además, se exige que el tamaño de las muestras sea grande y que todas las frecuencias teóricas no estén por debajo de 5.
#' Si alguna no lo cumple, es necesario reagrupar las categorías contiguas hasta conseguir superar esa cota.
#'
#' (2) Si el número de grados de libertad es 1, al estadístico del contraste se le aplica la siguiente corrección de Yates:
#'
#' \deqn{\chi^{2} = \sum_{i=1}^{k} \frac{(\left| O_{i} - E_{i} \right| - 0.5)^{2}}{E_{i}}}
#'
#' @seealso \code{\link{contraste_independencia}},\code{\link{contraste_bondad_cat}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#' @importFrom utils edit
#' @import dplyr ggplot2
#'
#' @export
contraste_homogeneidad <- function(x,
                                   introducir = FALSE,
                                   alfa = 0.05,
                                   grafico = FALSE) {


  # Si el usuario quiere introducir los datos manualmente
  if (introducir == TRUE) {

    # Caso cuando el usuario quiere introducir los datos manualmente
    nfilas <- as.numeric(readline(prompt = "Introduce el n\u00famero de categor\u00edas de la primera variable: "))
    ncolumnas <- as.numeric(readline(prompt = "Introduce el n\u00famero de categor\u00f1as de la segunda variable: "))

    nombre_filas <- nombre_columnas <- c()

    # Introducir nombres de las filas
    for (j in 1:nfilas) {
      nombre_filas <- c(nombre_filas, readline(prompt = paste("Introduce el nombre de la fila n\u00famero ", j, ": ", sep = "")))
    }

    # Introducir nombres de las columnas
    for (k in 1:ncolumnas) {
      nombre_columnas <- c(nombre_columnas, readline(prompt = paste("Introduce el nombre de la columna n\u00famero ", k, ": ", sep = "")))
    }

    # Crear la matriz vacia
    x <- matrix(0, ncol = ncolumnas, nrow = nfilas)
    rownames(x) <- nombre_filas
    colnames(x) <- nombre_columnas

    # Permitir editar la matriz
    matriz_obs <- edit(x)
    print(matriz_obs)

  } else { # Caso cuando se proporciona un data frame

    # Verificar que el input es un data.frame
    if (!is.data.frame(x)) {
      stop("El input debe ser un data frame.")
    }

    # Mostrar los nombres de las columnas
    cat("Las columnas disponibles en el data frame son:\n")
    print(colnames(x))

    # Funcion auxiliar para seleccionar columnas por nombre o posicion
    seleccionar_columna <- function(prompt_msg) {
      seleccion <- readline(prompt = prompt_msg)

      # Verificar si el usuario ingreso un numero (posicion) o un nombre (texto)
      if (suppressWarnings(!is.na(as.numeric(seleccion)))) {
        seleccion <- as.numeric(seleccion)
        if (seleccion < 1 || seleccion > ncol(x)) {
          stop("La posici/u00f3n seleccionada est\u00e1 fuera del rango de las columnas disponibles.")
        }
        return(seleccion)
      } else {
        # Seleccion por nombre
        if (!seleccion %in% colnames(x)) {
          stop("El nombre de columna introducido no existe.")
        }
        return(which(colnames(x) == seleccion))
      }
    }

    # Seleccion de las columnas
    col1 <- seleccionar_columna("Selecciona la primera variable (por nombre o posici\u00f3n): ")
    col2 <- seleccionar_columna("Selecciona la segunda variable (por nombre o posici\u00f3n): ")

    # Verificar que las variables seleccionadas son factores o caracteres
    if (!(is.factor(x[[col1]]) || is.character(x[[col1]])) ||
        !(is.factor(x[[col2]]) || is.character(x[[col2]]))) {
      stop("Ambas variables seleccionadas deben ser factores o caracteres.")
    }

    # Convertir a factor si son caracteres
    if (is.character(x[[col1]])) {
      x[[col1]] <- as.factor(x[[col1]])
      cat("La primera variable se ha convertido a factor.\n")
    }

    if (is.character(x[[col2]])) {
      x[[col2]] <- as.factor(x[[col2]])
      cat("La segunda variable se ha convertido a factor.\n")
    }

    # Extraer los datos de las variables seleccionadas
    data_frame_obs <- data.frame(x[[col1]], x[[col2]])
    colnames(data_frame_obs) <- c(colnames(x)[col1], colnames(x)[col2])

    # Mostrar los niveles de cada factor
    cat("Los niveles actuales de la primera variable son:\n")
    print(levels(data_frame_obs[[1]]))
    cat("Los niveles actuales de la segunda variable son:\n")
    print(levels(data_frame_obs[[2]]))

    # Preguntar al usuario si desea reordenar los niveles de la primera variable
    respuesta_reordenar_1 <- readline(prompt = "\u00bfQuieres reordenar los niveles de la primera variable? \n 1. \"Si\" \n 2. \"No\" \n")

    if (respuesta_reordenar_1 == "1") {
      # Reordenar niveles de la primera variable
      cat("Reordena los niveles de la primera variable (separados por comas):\n")
      nuevo_orden1 <- unlist(strsplit(readline(), ","))
      levels(data_frame_obs[[1]]) <- nuevo_orden1
    }

    # Preguntar al usuario si desea reordenar los niveles de la segunda variable
    respuesta_reordenar_2 <- readline(prompt = "\u00bfQuieres reordenar los niveles de la segunda variable? \n 1. \"Si\" \n 2. \"No\" \n")

    if (respuesta_reordenar_2 == "1") {
      # Reordenar niveles de la segunda variable
      cat("Reordena los niveles de la segunda variable (separados por comas):\n")
      nuevo_orden2 <- unlist(strsplit(readline(), ","))
      levels(data_frame_obs[[2]]) <- nuevo_orden2
    }

    # Crear la tabla de contingencia
    matriz_obs <- table(data_frame_obs[[1]], data_frame_obs[[2]])
    print(matriz_obs)

    # Confirmar la matriz observada
    respuesta <- readline(prompt = '\u00bfEs esta la matriz de datos observados? \n 1. "Si" \n 2. "No" \n')

    if (respuesta == "2") {
      cat("Introduce o modifica la matriz de datos observados:\n")
      matriz_obs <- edit(matriz_obs)
      print(matriz_obs)

    } else if (respuesta == "1") {
      cat("\u00A1Perfecto! Seguimos con estos datos.\n")
    } else {
      stop("El comando introducido no es correcto.")
    }
  }

  # Continuar con el analisis como antes...
  n <- sum(matriz_obs)
  mar_x <- apply(matriz_obs, 1, sum)
  mar_y <- apply(matriz_obs, 2, sum)
  matriz_esp <- crossprod(t(mar_x), mar_y) / n

  .check_min_obs_extended_cols(matriz_obs, matriz_esp)

  g.l <- (nrow(matriz_obs) - 1) * (ncol(matriz_obs) - 1)

  if (g.l > 1) {
    estadistico.prueba <- sum(((matriz_obs - matriz_esp) ^ 2)  / matriz_esp)
  } else {
    estadistico.prueba <- sum(((abs(matriz_obs - matriz_esp) - 0.5) ^ 2)  / matriz_esp)
    warning('Se ha aplicado la correcci\u00f3n de Yates')
  }

  valor_critico <- qchisq(alfa, g.l, lower.tail = FALSE)

  if (estadistico.prueba < valor_critico) {
    print(paste("No se rechaza la hip\u00f3tesis nula. El valor del estad\u00edstico de contraste: ",
                round(estadistico.prueba, 2),
                ", se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n [0, ",
                round(valor_critico, 2), "]", sep=""))
  } else {
    print(paste("Se rechaza la hip\u00f3tesis nula. El valor del estad\u00edstico de contraste: ",
                round(estadistico.prueba, 2),
                ", se encuentra fuera de la regi\u00f3n de aceptaci\u00f3n [0, ",
                round(valor_critico, 2), "]", sep=""))
  }

  pvalor <- pchisq(estadistico.prueba, g.l, lower.tail = FALSE)

  H0 <- "Las variables son homog\u00e9neas"
  CH <- cbind(H0, estadistico.prueba, round(pvalor, 4))
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3esis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  if (isTRUE(grafico)) {

    percentil99 <- qchisq(.9999, g.l)

    data <- data.frame(x=seq(from = 0, to = percentil99, percentil99/200))
    data$y <- dchisq(data$x, g.l)

    name_plot <- ggplot(data, aes(x, y)) +
      geom_area(fill = "#C4961A") +
      geom_area(data = subset(data, x < valor_critico), fill = "#008B8B") +
      geom_vline(xintercept = 0, color = "black") +
      geom_vline(xintercept = estadistico.prueba, color = "#A52A2A", linetype = "dashed", size = 1) +
      labs(title = paste("Distribuci\u00f3n Chi-Cuadrado con ", g.l, " grados de libertad", sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = round(c(0L, estadistico.prueba, valor_critico), 2)) +
      theme(axis.text.x = element_text(angle = 45))

    return(list(CH, name_plot))

  } else {
    return(CH)
  }
}

#' Método de impresión para objetos de clase "resumen"
#'
#' @param x Objeto de clase "resumen"
#' @param ... Argumentos adicionales
#' @method print resumen
#' @keywords internal
print.resumen <- function(x, ...) {
  x <- as.data.frame(x)
  print(format(round(x, 4), scientific = FALSE, nsmall = 4),
        quote = FALSE, ...)
}

#' @title Media (aritmética).
#'
#' @description Calcula la media aritmética.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrposicion.png}{width = 200px}}
#' \if{latex}{\figure{qrposicion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#'
#' @return Si \code{pesos = NULL}, devuelve la media (aritmética) de todas la variables seleccionadas en un \code{vector}. En caso contrario, devuelve únicamente la media de la variable para la que se ha facilitado la distribución de frecuencias.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' Si se obtiene la media (muestral) a partir de los datos brutos, como generalmente hacen los softwares:
#'
#' \if{html}{\figure{media.png}{width = 640px}}
#' \if{latex}{\figure{media.png}{options: width=3.5cm}}
#'
#' Si se desea obtener la media (muestral) a partir de una tabla estadística se utiliza la expresión:
#'
#' \if{html}{\figure{media2.png}{width = 64px}}
#' \if{latex}{\figure{media2.png}{options: width=3.5cm}}
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la media poblacional:
#'
#' \if{html}{\figure{mediapob.png}{width = 640px}}
#' \if{latex}{\figure{mediapob.png}{options: width=3cm}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' media1 <- media(startup[1])
#' media2 <- media(startup,variable=1)
#' media3 <- media(salarios2018,variable=6,pesos=7)
#'
#' @importFrom stats na.omit
#'
#' @import dplyr
#'
#' @export
media <- function(x,
                  variable = NULL,
                  pesos = NULL) {

  # Si es un vector, convertirlo a data.frame
  if (!is.data.frame(x)) {
    x <- data.frame(variable = x)
  }

  # Determinar variables seleccionadas
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)]
  } else if (is.numeric(variable)) {
    varnames <- names(x)[variable]
  } else if (is.character(variable)) {
    varnames <- variable
  } else {
    stop("El argumento 'variable' debe ser num\u00e9rico o de tipo car\u00e1cter")
  }

  # Verificar que las columnas seleccionadas existen
  if (!all(varnames %in% names(x))) {
    stop("Alguna variable seleccionada no existe en el data.frame")
  }

  # Subconjunto con las variables seleccionadas
  x_sel <- x[, varnames, drop = FALSE]

  # Manejo de pesos (si existen)
  if (!is.null(pesos)) {
    if (length(varnames) > 1 || length(pesos) > 1) {
      stop("Para el cl\u00e1culo ponderado solo puedes seleccionar una variable y un vector de pesos")
    }

    if (is.character(pesos)) {
      pesos_name <- pesos
    } else if (is.numeric(pesos)) {
      pesos_name <- names(x)[pesos]
    } else {
      stop("El argumento 'pesos' debe ser num\u00e9rico o de tipo car\u00e1cter")
    }

    if (!pesos_name %in% names(x)) {
      stop("El nombre de los pesos no es v\u00e1lido")
    }

    if (pesos_name == varnames) {
      stop("No puedes usar la misma variable como dato y como peso")
    }

    x_sel <- data.frame(variable = x[[varnames]], pesos = x[[pesos_name]])
  }

  # Comprobacion tipo de variable
  if (!all(sapply(x_sel, is.numeric))) {
    stop("No puede calcularse la media: alguna variable seleccionada no es cuantitativa")
  }

  # Calculo de la media
  if (is.null(pesos)) {
    result <- sapply(x_sel, mean, na.rm = TRUE)
    names(result) <- paste0("media_", names(x_sel))
  } else {
    result <- sum(x_sel[[1]] * x_sel[[2]], na.rm = TRUE) / sum(x_sel[[2]], na.rm = TRUE)
    names(result) <- paste0("media_", varnames)
  }

  # --- Convertir a data.frame con una fila ---
  media_df <- as.data.frame(t(result))
  names(media_df) <- varnames

  class(media_df) <- c("resumen", class(media_df))


  return(media_df)

}

#' @title Leer datos.
#'
#' @description Carga un conjunto de datos.
#'
#' \if{html}{\figure{qrleerdatos.png}{width = 200px}}
#' \if{latex}{\figure{qrleerdatos.png}{options: width=3cm}}
#'
#' @usage leer.datos(introducir = FALSE, pos = 1)
#'
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), se abrirá una ventana para que el usuario seleccione el fichero de datos que quiere cargar. Si \code{introducir = TRUE}, el usuario introducirá él mismo los datos.
#' @param pos Es un valor fijo utilizado para mostrar el dataframe del usuario en el Global Environment.
#'
#' @return Al finalizar el proceso de lectura de datos se mostrará el dataframe cargado en el environment.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @references
#' Esteban García, J. et al. (2005). Estadística descriptiva y nociones de probabilidad. Thomson.
#'
#' @importFrom stats na.omit
#'
#' @import dplyr
#'
#' @export
leer.datos <- function(introducir = FALSE, pos=1){

  if(pos!=1){
    stop("No cambies el valor del argumento pos.")
  }

  if(isFALSE(introducir)){

    print("Selecciona el fichero de datos (excel, csv, txt, etc.) con el que quieres trabajar ")

    x <- rio::import(file.choose())
    name_df <- readline(prompt = "Introduce el nombre que quieres darle a tu dataframe: ")
    assign(name_df,x,envir=as.environment(pos))


  } else{
    print("Esta opci\u00f3n es v\u00e1lida si vas a introducir pocos datos.")

    numero_var <- readline(prompt = "\u00bflos datos de cu\u00e1ntas variables quieres introducir? Indica el n\u00famero: ")
    numero_var <- as.numeric(numero_var)

    print("Cuando termines de introducir tus datos pulsa ENTER")
    print("NO UTILICES ESPACIOS EN BLANCO EN LOS NOMBRES DE VARIABLES O DATAFRAME. SI NECESITAS SEPARAR PALABRAS UTILIZA . o _")


    if(numero_var == 1){

      x <- scan()
      variable <- readline(prompt = "Introduce el nombre que le quieres dar a la variable: ")
      x <- as.data.frame(x)
      names(x) <- variable
      name_df <- readline(prompt = "Introduce el nombre que quieres darle a tu dataframe: ")
      assign(name_df,x,envir=as.environment(pos))


    } else{


      lista <- vector(mode="list", length = numero_var)
      n <- c()

      for(i in 1:numero_var){

        print(paste("Introduce los datos de la variable ",i,sep=""))
        var <- scan()
        variable <- readline(prompt = paste("Introduce el nombre que le quieres dar a la variable ",i," :",sep=""))

        lista[[i]] <- var
        names(lista)[i] <- variable
        n[i] <- length(var)

        n <- c(n,n[i])

      }

      df <- lapply(lista,'length<-',max(n))
      attributes(df) <- list(names = names(df),
                             row.names = 1:max(n),
                             class = "data.frame")
      name_df <- readline(prompt = "Introduce el nombre que quieres darle a tu dataframe: ")
      assign(name_df,df,envir=as.environment(pos))


    }
  }
}

#' @title Matriz de correlación.
#'
#' @description Obtiene la matriz de correlación (de Pearson) entre 2 o más variables cuantitativas.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcorrelacion.png}{width = 200px}}
#' \if{latex}{\figure{qrcorrelacion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} solo tiene 2 variables (columnas), \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables a seleccionar.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return La función devuelve la matriz de correlación lineal de las variables seleccionadas en un \code{data.frame}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' Se obtiene la matriz de correlación muestral:
#'
#' \if{html}{\figure{matrizcorrelacion.png}{width = 400px}}
#' \if{latex}{\figure{matrizcorrelacion.png}{options: width=8cm}}
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la matriz de correlació poblacional:
#'
#' \if{html}{\figure{matrizcorrelacionpob.png}{width = 440px}}
#' \if{latex}{\figure{matrizcorrelacionpob.png}{options: width=8cm}}
#'
#' @seealso \code{\link{correlacion}}, \code{\link{covarianza}},\code{\link{matriz.covar}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' matriz_cor <- matriz.correlacion(startup)
#'
#'
#' @importFrom stats na.omit cor
#' @import dplyr
#'
#' @export
matriz.correlacion <- function(x, variable = NULL, exportar = FALSE){

  varnames <- as.character(names(x))
  x <- data.frame(x)
  names(x) <- varnames

  if(is.null(variable)){

    varcuan <-  names(x[unlist(lapply(x, is.numeric))])
    seleccion = match(varcuan,varnames)
    x <- x[seleccion]
    varnames <- varcuan

  } else{

    if(is.numeric(variable)){

      if(all(variable <= length(x))){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variables")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){
        variable = match(variable,varnames)
      } else {
        stop("El nombre de la variable no es v\u00e1lido")
      }
    }

    x <- x[,variable] %>% as.data.frame()
    varnames <- names(x)

  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {
    stop("No puede calcularse la matriz de correlaci\u00f3, alguna variable que has seleccionado no es cuantitativa")
  }


  matriz_cor <- cor(x) %>%
    as.matrix()
  colnames(matriz_cor) <- varnames
  row.names(matriz_cor) <- varnames

  # Exportar
  if (exportar) {

    filename <- paste0("Matriz_de_correlacion_", format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), ".xlsx")

    wb <- openxlsx::createWorkbook()
    openxlsx::addWorksheet(wb, "Matriz_correlacion")

    # nombres de fila a columna
    resumen_export <- cbind(' ' = row.names(matriz_cor), matriz_cor)
    row.names(resumen_export) <- NULL

    openxlsx::writeData(wb, "Matriz_correlacion", resumen_export)

    # formato numerico decimal en Excel
    addStyle(wb, "Matriz_correlacion",
             style = createStyle(numFmt = "0.0000"),
             rows = 2:(nrow(resumen_export)+1),
             cols = 2:(ncol(resumen_export)+1),
             gridExpand = TRUE)

    saveWorkbook(wb, filename, overwrite = TRUE)
  }

  class(matriz_cor) <- c("resumen", class(matriz_cor))


  return(matriz_cor)

}

#' @encoding UTF-8
#' @title Coeficiente de variación.
#'
#' @description Calcula el coeficiente de variación de Pearson.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrdispersion.png}{options: style="width: 25\%;"}}
#' \if{latex}{\figure{qrdispersion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#' @param tipo Es un carácter. Por defecto calcula la desviación típica muestral (\code{tipo = "muestral"}). Si \code{tipo = "cuasi"}, se calcula la cuasi-desviación típica muestral.
#'
#' @return Esta función devuelve el valor del coeficiente de variación en un objeto de la clase \code{vector}. Por defecto, el coeficiente de variación se calcula utilizando la desviación típica muestral.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' El coeficiente de variación (muestral) se obtiene a partir de la siguiente expresión:
#'
#' \if{html}{\figure{coeficientevariacion.png}{options: style="width: 20\%;"}}
#' \if{latex}{\figure{coeficientevariacion.png}{options: width=2cm}}
#'
#' donde S es la desviación típica muestral. También puede calcularse utilizando la cuasi-desviación típica (Sc).
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N), se obtiene el coeficiente de variación poblacional:
#'
#' \if{html}{\figure{coeficientevariacionpob.png}{options: style="width: 20\%;"}}
#' \if{latex}{\figure{coeficientevariacionpob.png}{options: width=2cm}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' variacion1 <- coeficiente.variacion(startup[1])
#' variacion2 <- coeficiente.variacion(startup)
#'
#' @import dplyr
#'
#' @export
coeficiente.variacion <- function(x,
                                  variable = NULL,
                                  pesos = NULL,
                                  tipo = c("muestral", "cuasi")) {

  tipo <- tolower(tipo)
  tipo <- match.arg(tipo)

  # --- Asegurar que sea data.frame ---
  if (!is.data.frame(x)) {
    x <- data.frame(variable = x)
  }

  # --- Seleccionar variable(s) ---
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)]
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    varnames <- names(x)[variable]
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("El nombre de la variable no es v\u00e1lido")
    varnames <- variable
  } else {
    stop("El argumento 'variable' debe ser num\u00e9rico o de tipo car\u00e1cter")
  }

  # Subconjunto con las variables seleccionadas
  x_sel <- x[, varnames, drop = FALSE]

  # --- Manejo de pesos ---
  if (!is.null(pesos)) {
    if (length(varnames) > 1 || length(pesos) > 1)
      stop("Para el c\u00e1lculo ponderado solo puedes seleccionar una variable y unos pesos")

    if (is.character(pesos)) {
      if (!pesos %in% names(x)) stop("El nombre de los pesos no es v\u00e1lido")
      pesos_name <- pesos
    } else if (is.numeric(pesos)) {
      if (any(pesos > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de pesos")
      pesos_name <- names(x)[pesos]
    } else {
      stop("El argumento 'pesos' debe ser num\u00e9rico o de tipo car\u00e1cter")
    }

    if (pesos_name == varnames)
      stop("No puedes usar la misma variable como dato y como peso")

    x_sel <- data.frame(variable = x[[varnames]], pesos = x[[pesos_name]])
    varnames <- varnames[1]
  }

  # --- Comprobacion tipo de variable ---
  if (!all(sapply(x_sel, is.numeric))) {
    stop("No puede calcularse el coeficiente de variaci\u00f3n, alguna variable que has seleccionado no es cuantitativa")
  }
  # Calculo del coeficiente de variacion
  if (is.null(pesos)) {
    valor_media <- as.numeric(media(x_sel))
    valor_desviacion <- as.numeric(desviacion(x_sel, tipo = tipo))
  } else {
    valor_media <- as.numeric(media(x_sel, variable = 1, pesos = 2))
    valor_desviacion <- as.numeric(desviacion(x_sel, variable = 1, pesos = 2, tipo = tipo))
  }

  coef_var <- round(valor_desviacion / valor_media, 4)

  # Convertir a data.frame consistente
  df <- as.data.frame(t(coef_var))
  names(df) <- varnames

  return(df)
}

#' @title Desviación típica.
#'
#' @description Calcula la desviación típica.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrdispersion.png}{width = 200px}}
#' \if{latex}{\figure{qrdispersion.png}{options: width=3cm}}
#'
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#' @param tipo Es un carácter. Por defecto de calcula la desviación típica muestral (\code{tipo = "muestral"}). Si \code{tipo = "cuasi"}, se calcula la cuasi-desviación típica muestral.
#'
#' @return Esta función devuelve un objeto de la clase \code{vector}. Si \code{tipo="muestral"}, devuelve la desviación típica muestral. Si \code{tipo="cuasi"}, devuelve la cuasi-desviación típica muestral.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) La expresión de la de la desviación típica muestral es:
#'
#' \if{html}{\figure{desviacion.png}{width = 32px}}
#' \if{latex}{\figure{desviacion.png}{options: width=5cm}}
#'
#' La desviación típica muestral así definida es el estimador máximo verosímil de la desviación típica de una población normal
#'
#' (2) Muchos manuales y prácticamente todos los softwares (SPSS, Excel, etc.) calculan la expresión:
#'
#' \if{html}{\figure{cuasidesviacion.png}{width = 320px}}
#' \if{latex}{\figure{cuasidesviacion.png}{options: width=5cm}}
#'
#' Nosotros llamamos a esta medida: cuasi-desviación típica muestral y es un estimador insesgado de la desviación típica poblacional.
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la desviación típica poblacional:
#'
#' \if{html}{\figure{desviacionpob.png}{width = 320px}}
#' \if{latex}{\figure{desviacionpob.png}{options: width=5cm}}
#'
#' @seealso \code{\link{media}}, \code{\link{varianza}}, \code{\link{coeficiente.variacion}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' desviacion1 <- desviacion(startup[1])
#' desviaciona2 <- desviacion(startup,variable=1)
#' desviacion3 <- desviacion(startup,variable=1, tipo="cuasi")
#'
#' @importFrom stats sd na.omit
#'
#' @export
desviacion <- function(x, variable = NULL, pesos = NULL, tipo = c("muestral", "cuasi")) {

  tipo <- match.arg(tolower(tipo), c("muestral", "cuasi"))

  # --- Asegurar data.frame ---
  if (!is.data.frame(x)) x <- data.frame(x)

  # --- Seleccion de variables ---
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)]
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    varnames <- names(x)[variable]
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("Nombre de variable no v\u00e1lido")
    varnames <- variable
  } else {
    stop("El argumento 'variable' debe ser num\u00e9rico o de tipo car\u00e1cter")
  }

  x_sel <- x[, varnames, drop = FALSE]

  # --- Comprobar tipo de variables ---
  if (!all(sapply(x_sel, is.numeric))) {
    stop("No puede calcularse la desviaci\u00f3n: alguna variable no es cuantitativa")
  }

  # --- Si no hay pesos, calculamos desviacion simple ---
  if (is.null(pesos)) {
    calcular_desv <- function(col) {
      n_eff <- sum(!is.na(col))
      if (n_eff < 2) return(NA_real_)
      factor <- if (tipo == "muestral") sqrt((n_eff - 1) / n_eff) else 1
      round(sqrt(stats::var(col, na.rm = TRUE)) * factor, 4)
    }

    desv_val <- sapply(x_sel, calcular_desv)
    names(desv_val) <- names(x_sel)

    desv_df <- as.data.frame(t(desv_val))

    return(desv_df)
  }

  # --- Si hay pesos ---
  if (length(pesos) != 1 || length(varnames) != 1) {
    stop("Para desviaci\u00f3n ponderada solo puedes seleccionar una variable y un vector de pesos")
  }

  # Determinar la columna de pesos
  if (is.character(pesos)) {
    if (!pesos %in% names(x)) stop("Nombre de pesos no v\u00e1lido")
    pesos_col <- x[[pesos]]
  } else if (is.numeric(pesos)) {
    if (pesos > ncol(x)) stop("Indice de pesos inv\u00e1lido")
    pesos_col <- x[[pesos]]
  } else {
    stop("El argumento 'pesos' debe ser num\u00e9rico o de tipo car\u00e1cter")
  }

  datos <- na.omit(data.frame(variable = x_sel[[1]], pesos = pesos_col))
  media_pond <- sum(datos$variable * datos$pesos) / sum(datos$pesos)
  sum_cuad <- sum((datos$variable - media_pond)^2 * datos$pesos)

  desv_val <- if (tipo == "muestral") {
    sqrt(sum_cuad / sum(datos$pesos))
  } else {
    sqrt(sum_cuad / (sum(datos$pesos) - 1))
  }

  # --- Convertir a data.frame con una fila ---
  desv_df <- as.data.frame(t(desv_val))
  names(desv_df) <- varnames


  # desv_val <- round(desv_val, 4)
  # names(desv_val) <- paste0("desviacion_", varnames[1])

  class(desv_df) <- c("resumen", class(desv_df))

  return(desv_df)
}

#' @title Contraste de hipótesis sobre la razón de varianzas.
#'
#' @description Realiza el contraste de hipótesis sobre la razón de dos varianzas poblacionales.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcrazonvarianzas.png}{width = 200px}}
#' \if{latex}{\figure{qrcrazonvarianzas.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere solo a dos variables, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param hipotesis_nula Es un valor numérico. Por defecto el valor está fijado a 1, es decir, igualdad de varianzas.
#' @param tipo_contraste Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
#'        Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
#'        Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
#'        Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba, el p-valor y  el intervalo de confianza para la media muestral supuesta cierta la hipótesis nula. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con los valores críticos.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' La hipótesis nula que se considera en el contraste bilateral es:
#'
#' \if{html}{\figure{crazonvar.png}{width = 240px}}
#' \if{latex}{\figure{crazonvar.png}{options: width=3cm}}
#'
#' El estadístico F es:
#'
#' (1) Si trabajamos con la varianza muestral:
#'
#'
#' \if{html}{\figure{crazonvarmuestra.png}{width = 400px}}
#' \if{latex}{\figure{crazonvarmuestra.png}{options: width=5cm}}
#'
#' (2) si trabajamos con la cuasi-varianza muestral:
#'
#' \if{html}{\figure{crazonvarcuasi.png}{width = 200px}}
#' \if{latex}{\figure{crazonvarcuasi.png}{options: width=3cm}}
#'
#' Tanto en (1) como en (2) el estadístico F se distribuye como una F con (n1-1) grados de libertad en el numerador y (n2-1) grados de libertad en el denominador.
#'
#' @seealso \code{\link{ic.razon.varianzas}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats na.omit df pf qf
#' @import dplyr ggplot2
#'
#' @export
contraste.razon.varianzas <- function(x,
                                      variable = NULL,
                                      introducir = FALSE,
                                      hipotesis_nula = 1,
                                      tipo_contraste = c("bilateral","cola derecha","cola izquierda"),
                                      alfa = 0.05,
                                      grafico = FALSE){


print("Se calcula el contraste para el cociente de varianzas supuestas desconocidas las medias poblacionales")

tipo_contraste <- tolower(tipo_contraste)
tipo_contraste <- match.arg(tipo_contraste)


if(is.null(hipotesis_nula) | !is.numeric(hipotesis_nula)){

  stop("Tienes que introducir un valor para la hip\u00f3tesis nula")

} else{

  H0 <- hipotesis_nula

}


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 2){
      x <- x
    } else{
      warning("Para calcular este tipo de contraste hay que seleccionar 2 variables")
      stop("El conjunto de datos seleccionado no tiene la dimensi\u00f3n adecuada")
    }
  } else{

    if(length(variable) == 2){
        if(is.numeric(variable)){
          if(all(variable <= length(x))){
            variable <- variable
          } else{
            stop("Selecci\u00f3n err\u00f3nea de variables")
          }
        }
      if(is.character(variable)){
        if(all(variable %in% varnames)){
          variable = match(variable,varnames)
          } else {
            stop("El nombre de la variable no es v\u00e1lido")
          }
        }

      x <- x[,variable] %>% as.data.frame()
      names(x) <- varnames[variable]

      } else{
        warning("Para calcular el contraste de la raz\u00f3n de varianzas hay que seleccionar dos variables")
        stop("El conjunto de datos seleccionado parece ser no v\u00e1lido")
      }
  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))){
    stop("No puede calcularse el contraste porque la variable seleccionada no es cuantitativa")
  }

  # tama\u00f1o de la muestra
  n1 <- length(x[1][!is.na(x[1])])
  n2 <- length(x[2][!is.na(x[2])])


  var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

  if(var_muestra == 1){

    var_mu1 <- as.numeric(varianza(x[1]))
    var_mu2 <- as.numeric(varianza(x[2]))

  } else{

    var_mu1 <- as.numeric(varianza(x[1], tipo = "cuasi"))
    var_mu2 <- as.numeric(varianza(x[2], tipo = "cuasi"))

  }

} else{   # aqu\u00ed empieza introducir datos

  print("A continuaci\u00f3n, vas a introducir los datos de las muestras.")

  n1 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 1: ")
  n1 <- as.numeric(n1)

  n2 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 2: ")
  n2 <- as.numeric(n2)

  var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      var_mu1 <- readline("Introduce el valor de la varianza muestral 1: ")
      var_mu1 <- as.numeric(var_mu1)

      var_mu2 <- readline("Introduce el valor de la varianza muestral 2: ")
      var_mu2 <- as.numeric(var_mu2)

    } else{

      var_mu1 <- readline("Introduce el valor de la cuasivarianza muestral 1: ")
      var_mu1 <- as.numeric(var_mu1)

      var_mu2 <- readline("Introduce el valor de la cuasivarianza muestral 2: ")
      var_mu2 <- as.numeric(var_mu2)

    }

}

if(alfa >= 0 & alfa <=1){

  if(tipo_contraste == "bilateral"){

    valor_critico1 <- round(qf(alfa/2, df1= n2-1, df2 = n1-1,lower.tail = T),4)
    valor_critico2 <- round(qf(alfa/2, df1= n2-1, df2 = n1-1,lower.tail = F),4)

  }

  if(tipo_contraste == "cola izquierda"){

    valor_critico <- round(qf(alfa, df1= n2-1, df2 = n1-1,lower.tail = T),4)
  }

  if(tipo_contraste == "cola derecha"){

    valor_critico <- round(qf(alfa, df1= n2-1, df2 = n1-1,lower.tail = F),4)
  }

} else{

  stop("El nivel de significacion debe fijarse entre 0 y 1")

}


# calculo del contraste

# contraste del cociente de varianzas con medias desconocidas


if(var_muestra == 1){

  # caso 1.1
  estadistico.prueba <- (n1/(n1-1))*((n2-1)/n2)*(var_mu1/var_mu2)*(1/hipotesis_nula)

} else {

  # caso 1.2
  print("Este es el intervalo de confianza que generalmente calculan los softwares")

  estadistico.prueba <- (var_mu1 / var_mu2) * (1/hipotesis_nula)

}

percentil99 <- qf(.9999, df1= n1-1, df2 = n2-1)

data <- data.frame(x=seq(from = 0, to = percentil99, percentil99/200))
data$y <-df(data$x, df1= n1-1, df2 = n2-1)

if(tipo_contraste == "bilateral"){

  pvalor <- 2 * min(pf(estadistico.prueba, df1= n1-1, df2 = n2-1,lower.tail = F), pf(estadistico.prueba, df1= n1-1, df2 = n2-1,lower.tail = T))

  if(estadistico.prueba >= valor_critico1 & estadistico.prueba <=  valor_critico2){

    print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico1," , ",valor_critico2,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  } else{

    print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico1," , ",valor_critico2,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  }

  if(isTRUE(grafico)){

    plot <- ggplot(data, aes(x,y)) +
      geom_area(fill="darkgreen") +
      geom_area(data=subset(data,x<valor_critico1), fill = "red") +
      geom_area(data=subset(data,x>valor_critico2),fill = "red") +
      geom_vline(xintercept = 0, color = "black") +
      geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
      labs(title = paste("Distribuci\u00f3n F con ", n1-1, " y ",n2-1," grados de libertad",sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = round(c(0L,estadistico.prueba,valor_critico1,valor_critico2),4)) +
      theme(axis.text.x = element_text(angle = 45))
  }


} else if(tipo_contraste == "cola derecha"){

  pvalor <- pf(estadistico.prueba, df1= n1-1, df2 = n2-1,lower.tail = F)

  if(estadistico.prueba >= valor_critico){

    print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", 0," , ",valor_critico,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  } else{

    print(paste("No Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", 0," , ",valor_critico,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  }

  if(isTRUE(grafico)){

    plot <- ggplot(data, aes(x,y)) +
      geom_area(fill="darkgreen") +
      geom_area(data=subset(data,x>valor_critico),fill = "red") +
      geom_vline(xintercept = 0, color = "black") +
      geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
      labs(title = paste("Distribuci\u00f3n F con ", n1-1, " y ",n2-1," grados de libertad",sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = round(c(0L,estadistico.prueba,valor_critico),4)) +
      theme(axis.text.x = element_text(angle = 45))
  }


} else{

  pvalor <- pf(estadistico.prueba, df1= n1-1, df2 = n2-1,lower.tail = T)

  if(estadistico.prueba <= valor_critico){

    print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico," , ",Inf,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  } else{

    print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico," , ",Inf,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  }

  if(isTRUE(grafico)){

    plot <- ggplot(data, aes(x,y)) +
      geom_area(fill="darkgreen") +
      geom_area(data=subset(data,x<valor_critico),fill = "red") +
      geom_vline(xintercept = 0, color = "black") +
      geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
      labs(title = paste("Distribuci\u00f3n F con ", n1-1, " y ",n2-1," grados de libertad",sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = round(c(0L,estadistico.prueba,valor_critico),4)) +
      theme(axis.text.x = element_text(angle = 45))
  }

}

  CH <- cbind(H0,estadistico.prueba,pvalor)
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  if(isTRUE(grafico)){

    return(list(CH,plot))

  } else{

    return(CH)

  }


}

#' Facilita cálculo de cuantiles
#'
#' @description Para obtener los cuantiles
#' @param x Vector
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias
#' @param corte Punto de corte de la distribución de frecuencias
#' @keywords internal
#' @noRd
#' @importFrom stats setNames
.cuantiles.int <- function(x, pesos = NULL, cortes = 0.5){

  if(is.numeric(x)){
    varnames <- "variable.x"
  }else{
    varnames <- as.character(names(x))
  }

  x <- data.frame(x)

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {
    stop("No pueden calcularse los cuantiles, alguna variable que has seleccionado no es cuantitativa")
  }

  if(is.null(pesos)){

    x <- tidyr::drop_na(x)
    #y <- names(x)
    #names(x) <- varnames

    N <- nrow(x)
    tabla <- x %>% group_by(x) %>%
      count() %>%
      ungroup() %>%
      mutate(Ni = cumsum(n))

  } else{

    N <- sum(pesos)

    tabla <- data.frame(x,pesos) %>%
      na.omit
    #y <- names(tabla)
    names(tabla) <- c("x","pesos")

    tabla <- tabla %>%
      arrange(x) %>%
      mutate(Ni = cumsum(pesos))

  }


  cortes <- sort(cortes)

  cuantiles <- c()

  for(i in 1:length(cortes)){

    posicion <- min(which(tabla$Ni >= cortes[i]* N))

    if(cortes[i]* N == tabla$Ni[posicion]){

      cuantil <- mean(c(tabla$x[posicion],tabla$x[posicion+1]),na.rm=TRUE)

    } else {

      cuantil <- tabla$x[posicion]

    }

    cuantiles <- rbind(cuantiles,cuantil)


  }

  cuantiles <- as.data.frame(cuantiles)
  row.names(cuantiles) <- paste(cortes*100,"%",sep="")

  return(cuantiles)
}


#' Facilita cálculo de la mediana
#'
#' @description Para obtener la mediana
#' @param x Vector
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias
#' @noRd
.mediana.int <- function(x, pesos = NULL){

  if(is.numeric(x)){
    varnames <- "variable.x"
  }else{
    varnames <- as.character(names(x))
  }

  x <- data.frame(x)

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {
    stop("No puede calcularse la median, alguna variable que has seleccionado no es cuantitativa")
  }

  if(is.null(pesos)){

    x <- tidyr::drop_na(x)
    y <- names(x)
    names(x) <- "x"

    N <- nrow(x)
    tabla <- x %>% group_by(x) %>%
      count() %>%
      ungroup() %>%
      mutate(Ni = cumsum(n))

  } else{

    N <- sum(pesos)

    tabla <- data.frame(x,pesos) %>%
      na.omit
    y <- names(tabla)
    names(tabla) <- c("x","pesos")

    tabla <- tabla %>%
      arrange(x) %>%
      mutate(Ni = cumsum(pesos))

  }

  posicion <- min(which(tabla$Ni >= N/2))

  if(N/2 == tabla$Ni[posicion]){

    mediana <- mean(c(tabla$x[posicion],tabla$x[posicion+1]),na.rm=TRUE)

  } else {

    mediana <- tabla$x[posicion]

  }

  return(mediana)
}

#' Facilita cálculo de los momentos
#'
#' @description Para obtener los momentos
#' @param x Vector
#' @param orden Orden del momento central
#' @keywords internal
#' @noRd
.momento.central <- function(x, orden){

  if(is.numeric(x)){
    varnames <- "variable.x"
  }else{
    varnames <- as.character(names(x))
  }

  x <- data.frame(x)
  names(x) <- varnames

  orden <- as.integer(orden)

  if(!is.integer(orden)){

    stop("El orden del momento central debe ser un valor num\u00e9rico entero")

  }

  x <- data.frame(x)

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {
    stop("No pueden calcularse las medidas de forma, alguna variable que has seleccionado no es cuantitativa")
  }

  momento_vacio <- vector("list",length=length(x))

  for(i in 1:length(x)){

    x2 <- x[i] %>% na.omit

    momento <- x2 %>%
      mutate(media_x = media(x2),
             momento = (x2-media_x)^orden) %>%
      summarize(momento = sum(momento)/n()) %>%
      as.numeric()

    momento_vacio[[i]] <- momento

  }

  max_long <-  max(lengths(momento_vacio))
  momento <- sapply(momento_vacio, "[", seq_len(max_long)) %>%
    t() %>%
    as.numeric()
  names(momento) <- varnames

  return(momento)

}

#' Crear hojas excel para exportar resultados de series temporales
#'
#' @description Función para organizar la exportación de resultados de series temporales
#' @param wb Crea el libro de excel
#' @param sheet_name Nombre de la hoja excel
#' @param data Objeto a exportar
#' @keywords internal
#' @noRd
.add_sheet_with_style <- function(wb, sheet_name, data) {
  tryCatch({
    if (is.null(data)) {
      stop(paste("Los datos para la hoja", sheet_name, "son NULL"))
    }

    openxlsx::addWorksheet(wb, sheet_name)

    if (inherits(data, "data.frame")) {
      openxlsx::writeData(wb, sheet_name, data, rowNames = TRUE)

      # Aplicar formato numerico
      numeric_cols <- which(sapply(data, is.numeric))
      if (length(numeric_cols) > 0) {
        openxlsx::addStyle(
          wb, sheet_name,
          style = openxlsx::createStyle(numFmt = "0.0000"),
          rows = 2:(nrow(data) + 1),
          cols = numeric_cols + 1,
          gridExpand = TRUE
        )
      }
    } else if (inherits(data, "lm")) {
      openxlsx::writeData(wb, sheet_name, capture.output(summary(data)))
    } else {
      openxlsx::writeData(wb, sheet_name, as.data.frame(data), rowNames = TRUE)
    }
  }, error = function(e) {
    message("Error al crear la hoja ", sheet_name, ": ", e$message)
  })
}


#' Formatear objetos para visualización en consola
#'
#' @description Elimina notación científica y formatea números para output consistente
#' @param x Objeto a formatear (vector, data.frame, matriz, lista)
#' @param decimales Número de decimales a mostrar (default: 4)
#' @return Objeto formateado para visualización
#' @keywords internal
#' @noRd
.format_consola <- function(x, decimales = 4) {
  if (is.numeric(x)) {
    # Formato fijo sin notacióo científica para todo el vector
    formatted <- format(x, scientific = FALSE, digits = 12, nsmall = decimales)

    # Limpieza de ceros decimales (vectorizado)
    formatted <- ifelse(
      grepl("\\.0+$", formatted),
      sub("0+$", "0", formatted),  # Mantener un cero si es .0
      sub("\\.?0+$", "", formatted) # Eliminar ceros sobrantes
    )

    return(formatted)
  }

  if (is.data.frame(x) || is.matrix(x)) {
    return(as.data.frame(lapply(x, .format_consola, decimales = decimales)))
  }

  if (is.list(x)) {
    return(lapply(x, .format_consola, decimales = decimales))
  }

  return(x)
}

#' Mostrar resultados formateados en consola (para listas)
#'
#' @description Muestra listas en consola con formato consistente
#' @param x Lista a mostrar
#' @param titulo Título opcional para la salida
#' @param decimales Número de decimales a mostrar
#' @keywords internal
#' @noRd
.mostrar_lista_resultados <- function(x, titulo = NULL, decimales = 4) {
  tryCatch({
    # Verificar que sea una lista
    if (!is.list(x) || inherits(x, "data.frame")) {
      stop("El objeto no es una lista")
    }

    # Mostrar titulo si se proporciona
    if (!is.null(titulo)) {
      cat("\n", toupper(titulo), "\n", sep = "")
      cat(paste(rep("=", nchar(titulo)), collapse = ""), "\n\n")
    }

    # Formatear y mostrar cada elemento de la lista
    x_formateado <- lapply(x, .format_consola, decimales = decimales)

    # Mostrar cada elemento con su nombre
    for (nombre in names(x_formateado)) {
      cat("$", nombre, "\n", sep = "")
      print(x_formateado[[nombre]], right = FALSE, quote = FALSE)
      cat("\n")
    }

  }, error = function(e) {
    warning("Error al formatear lista: ", e$message)
    print(x)  # Mostrar version sin formatear como fallback
  })

  invisible(x)
}

#' Comprueba el número el mínimo de frecuencias teóricas en contraste bondad
#'
#' @description Comprueba que las frecuencias teóricas sean mínimo 5
#' @param m Frecuencias teóricas
#' @param n_min_obs Mínimo de frecuencias teóricas para no agrupar
#' @keywords internal
#' @noRd
.check_min_obs <- function(m, n_min_obs = 5) {
  output_matrix <- m
  n <- nrow(output_matrix)

  # Empezamos desde la ultima fila y vamos hacia arriba
  i <- n
  while (i > 1) {
    # Si el valor en la fila actual es menor a n_min_obs
    if (output_matrix$Freq_esp[i] < n_min_obs) {
      acum_freq <- output_matrix$Freq_esp[i]  # Empezamos la acumulacion en Frey_obs
      acum_suma <- output_matrix$Freq_obs[i]      # Empezamos la acumulacion en Freq_obs
      j <- i - 1

      # Seguimos hacia arriba sumando hasta que la acumulacion sea >= n_min_obs
      while (acum_freq < n_min_obs && j > 0) {
        acum_freq <- acum_freq + output_matrix$Freq_esp[j]  # Acumulamos en Frey_obs
        acum_suma <- acum_suma + output_matrix$Freq_obs[j]      # Acumulamos en SUMA
        j <- j - 1
      }

      # Actualizamos la fila donde termina la acumulacion
      output_matrix$Freq_esp[j + 1] <- acum_freq
      output_matrix$Freq_obs[j + 1] <- acum_suma

      # Eliminamos las filas que se sumaron (entre j + 2 e i)
      if ((j + 2) <= i) {
        output_matrix <- output_matrix[-((j + 2):i), ]
      }

      # Ajustamos el valor de i para continuar desde la fila donde se termina la acumulacion
      i <- j + 1
    } else {
      # Si la fila actual ya es >= n_min_obs, seguimos con la fila anterior
      i <- i - 1
    }
  }

  return(output_matrix)
}

#' Comprueba el número el mínimo de frecuencias teóricas en contraste homogeneidad e independencia
#'
#' @description Comprueba que las observaciones teóricas sean mínimo 5
#' @param obs_matrix Matriz de frecuencias observadas
#' @param exp_matrix Matriz de frecuencias esperadas
#' @param n_min_exp Mínimo de frecuencias teóricas para no agrupar
#' @keywords internal
#' @noRd
.check_min_obs_extended_cols <- function(obs_matrix, exp_matrix, n_min_exp = 5) {

  output_obs <- obs_matrix
  output_exp <- exp_matrix
  n_cols <- ncol(output_exp)

  # Empezamos desde la ultima columna de la matriz de frecuencias esperadas
  i <- n_cols
  while (i > 1) {
    # Verificamos si algun valor en la columna actual de la matriz de frecuencias esperadas es menor a 5
    if (any(output_exp[, i] < n_min_exp)) {
      # Acumulamos toda la columna actual en la columna anterior
      output_exp[, i - 1] <- output_exp[, i - 1] + output_exp[, i]
      output_obs[, i - 1] <- output_obs[, i - 1] + output_obs[, i]

      # Eliminamos la columna actual en ambas matrices
      output_exp <- output_exp[, -i]
      output_obs <- output_obs[, -i]

      # Ajustamos la cantidad de columnas
      i <- ncol(output_exp)
    } else {
      # Si ningun valor en la columna actual es menor a 5, pasamos a la columna anterior
      i <- i - 1
    }
  }

  return(list(observadas = output_obs, esperadas = output_exp))
}


#' Función interna que combina o resume data frames estadísticos
#' @keywords internal
#' @noRd
#' @import tibble
.resumir <- function(...,
                     nombres = NULL,
                     exportar = FALSE,
                     archivo = NULL) {

  # Captura robusta de argumentos
  listas <- list(...)
  llamadas <- as.list(substitute(list(...)))[-1L]

  # Nombres automaticos si no se pasan
  if (is.null(nombres)) {
    nombres <- names(llamadas)
    if (is.null(nombres) || any(nombres == "")) {
      nombres <- sapply(llamadas, deparse)
    }
  }

  if (length(nombres) != length(listas)) {
    stop("Error interno: no se pudieron asignar los nombres a todos los objetos.")
  }

  # Preparar dataframes y conservar rownames
  listas <- lapply(listas, function(df) {
    rn <- rownames(df)
    df <- as.data.frame(unclass(df)) # limpiar clases
    df$estadistico <- if (!is.null(rn)) rn else seq_len(nrow(df))
    df
  })

  # Comprobamos si las columnas coinciden (excepto "estadistico")
  nombres_col <- lapply(listas, names)
  nombres_col <- lapply(nombres_col, function(x) setdiff(x, "estadistico"))
  mismas_columnas <- length(unique(nombres_col)) == 1

  # --- CASO 1: columnas coinciden ---
  if (mismas_columnas) {
    tablas_etiquetadas <- Map(function(df, nom) {
      df$medida <- as.character(nom)
      df
    }, listas, nombres)

    tabla_final <- dplyr::bind_rows(tablas_etiquetadas)
    tabla_final <- dplyr::select(tabla_final, "medida", "estadistico", dplyr::everything())

    # Exportar a Excel
    if (isTRUE(exportar)) {
      if (is.null(archivo)) {
        archivo <- paste0("resultados_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".xlsx")
      }
      openxlsx::write.xlsx(tabla_final, file = archivo, rowNames = FALSE)
      message("Resultados exportados a: ", archivo)
    }

    return(tabla_final)
  }

  # --- CASO 2: columnas distintas ---
  estructura <- Map(function(df, nom) {
    df_clean <- as.data.frame(unclass(df))  # limpiar clases

    # Tomamos la ultima columna como 'estadistico' y la ponemos al principio
    n_col <- ncol(df_clean)
    df_clean <- cbind(estadsitico = df_clean[[n_col]], df_clean[, -n_col, drop = FALSE])

    # Mostramos todas las filas
    preview_df <- df_clean

    list(
      medida = nom,
      preview = preview_df
    )
  }, listas, nombres)

  names(estructura) <- nombres
  class(estructura) <- "resumen_resultados"

  # Metodo de impresion que muestra todas las filas
  print.resumen_resultados <- function(x, ...) {
    cat("Resumen de objetos estad\u00edsticos:\n\n")
    for (i in seq_along(x)) {
      obj <- x[[i]]
      cat("-", obj$medida, "\n")
      cat("  - Resultados:\n")
      print(obj$preview, row.names = FALSE)
      cat("\n")
    }
    invisible(x)
  }

  assign("print.resumen_resultados", print.resumen_resultados, envir = parent.frame())

  # Exportar Excel en caso de columnas distintas
  if (isTRUE(exportar)) {
    if (is.null(archivo)) {
      archivo <- paste0("resumen_", format(Sys.time(), "%Y%m%d_%H%M%S"), ".xlsx")
    }
    wb <- openxlsx::createWorkbook()
    for (i in seq_along(estructura)) {
      sheet_name <- substr(names(estructura)[i], 1, 31)
      openxlsx::addWorksheet(wb, sheetName = sheet_name)
      openxlsx::writeData(wb, sheet = i, x = estructura[[i]]$preview)
    }
    openxlsx::saveWorkbook(wb, archivo, overwrite = TRUE)
    message("Resumen exportado a: ", archivo)
  }

  return(estructura)
}

#' Combina o resume data frames para resumen estadístico
#'
#' @param ... Data frames a combinar o resumir.
#' @param nombres Vector de nombres opcionales para los objetos.
#' @param exportar Logical. Si TRUE, exporta a Excel.
#' @param archivo Nombre del archivo Excel; si NULL, se genera automáticamente.
#' @return Data frame combinado si las columnas coinciden, o lista resumen si son distintas.
#' @examples
#' \dontrun{
#' Ejemplo caso 1: resumir varios data frames con las mismas columnas
#' media <- estadistica::media(mtcars)
#' cuantiles <- estadistica::cuantiles(mtcars)
#'
#' resultados <- resumir(media, cuantiles, exportar=TRUE)
#' resultados
#'
#' Ejemplo caso 2: resumir varios data frames con distintas columnas
#' forma <- estadistica::medidas.forma(mtcars[c(1, 3, 4)])
#' resultados <- resumir(media, cuantiles, forma, exportar=TRUE)
#' resultados
#' }

#' @export
resumir <- .resumir

#' @title Contraste de hipótesis sobre la proporción.
#'
#' @description Realiza el contraste de hipótesis sobre la proporción poblacional.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcproporcion.png}{width = 200px}}
#' \if{latex}{\figure{qrcproporcion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param hipotesis_nula Es un valor numérico.
#' @param tipo_contraste Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
#'        Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
#'        Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
#'        Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba, el p-valor y  el intervalo de confianza para la proporción muestral supuesta cierta la hipótesis nula. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con los valores críticos.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' En este caso el estadístico Z del contraste es:
#'
#' \if{html}{\figure{cproporcion.png}{width = 320px}}
#' \if{latex}{\figure{cproporcion.png}{options: width=4cm}}
#'
#' @seealso \code{\link{ic.proporcion}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm na.omit dnorm
#' @import dplyr ggplot2
#'
#' @export
contraste.proporcion <- function(x,
                                 variable = NULL,
                                 introducir = FALSE,
                                 hipotesis_nula = NULL,
                                 tipo_contraste = c("bilateral","cola derecha","cola izquierda"),
                                 alfa = 0.05,
                                 grafico = FALSE){

  tipo_contraste <- tolower(tipo_contraste)
  tipo_contraste <- match.arg(tipo_contraste)

  if(is.null(hipotesis_nula)){

    stop("Tienes que introducir un valor para la hipo\u00f3tesis nula")

  }

  if(alfa >= 0 & alfa <=1){

    if(tipo_contraste == "bilateral"){
      valor_critico <- qnorm(alfa/2,lower.tail = F)
    }
    if(tipo_contraste == "cola izquierda"){
      valor_critico <- qnorm(alfa,lower.tail = T)
    }
    if(tipo_contraste == "cola derecha"){
      valor_critico <- qnorm(alfa,lower.tail = F)
    }

    valor_critico <- round(valor_critico,4)

  } else{
    stop("El nivel de significaci\u00f3n debe fijarse entre 0 y 1")
  }

  if(hipotesis_nula >= 0 & hipotesis_nula <=1){
    H0 <- hipotesis_nula
  }else{
    stop("La hip\u00f3tesis nula es una proporci\u00f3n y por tanto tiene que fijarse entre 0 y 1")
  }


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 1){

      x <- x

    } else{

      warning("Para calcular el contraste hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

    }

  } else if(length(variable) == 1){

    if(is.numeric(variable)){

      if(variable <= length(x)){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es v\u00e1lido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el contraste hay que seleccionar una variable")
    stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

  }

  if(!all(x == 0 | x==1)){

    print("Aplica a tus datos la condici\u00f3n que debe cumplir la poblaci\u00f3n para transfomar los datos en ceros (ausencia/no \u00e9xito) y unos (presencia/\u00e9xito)")
    stop("Los valores en la muestra deben ser 0 y 1.")

  }

  x <- na.omit(x)
  clase <- sapply(x, class)

  if (!clase %in% c("numeric","integer")) {

    stop("No puede calcularse el contraste porque la variable seleccionada no es cuantitativa")

  }

  # tama\u00f1o de la muestra
  n <- nrow(x)

  # media muestral

    p_mu <- sum(x,na.rm=TRUE)/n

} else{   # aqu\u00ed empieza introducir datos

  n <- readline(prompt = "Introducir el tama\u00f1o de la muestra: ")
  n <- as.numeric(n)

  p_mu <- readline(prompt = "Introducir el valor de la proporcion muestral: ")
  p_mu <- as.numeric(p_mu)

}

  # calculo de los contrastes
  # estadistico de prueba

  estadistico.Z <- (p_mu - H0)/sqrt(H0*(1-H0)/n)
  estadistico.Z <- round(estadistico.Z,5)
  error_tipico <- sqrt(H0*(1-H0)/n)

  if(tipo_contraste == "bilateral"){

    estadistico.Z2 <- abs(estadistico.Z)
    pvalor <- 2*pnorm(estadistico.Z2,lower.tail=FALSE)
    media_inf <- H0 - valor_critico * error_tipico
    media_sup <- H0 + valor_critico * error_tipico

    if(estadistico.Z >= -valor_critico & estadistico.Z <=  valor_critico){

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-3, -valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-valor_critico, valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 3)) +
        geom_vline(xintercept = -estadistico.Z2, linetype = "dashed") +
        geom_vline(xintercept = estadistico.Z2, linetype = "dashed") +
        labs(x = "", y = "",title="Regi\u00f3n de aceptaci\u00f3n-rechazo") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.Z2,-estadistico.Z2,-valor_critico,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

    }

  } else if(tipo_contraste == "cola derecha"){

    media_inf <- -Inf
    media_sup <- H0 + valor_critico * error_tipico
    pvalor <- pnorm(estadistico.Z,lower.tail=FALSE)

    if(estadistico.Z > valor_critico){

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-3L,valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 3L)) +
        geom_vline(xintercept = estadistico.Z, linetype = "dashed") +
        labs(x = "", y = "",title="Regi\u00f3n de aceptaci\u00f3n-rechazo") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.Z,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

    }

  } else{

    media_inf <- H0 + valor_critico * error_tipico # valor critico negativo
    media_sup <- Inf
    pvalor <- pnorm(estadistico.Z,lower.tail=TRUE)

    if(estadistico.Z < valor_critico){

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-3L, valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(valor_critico, 3L)) +
        geom_vline(xintercept = estadistico.Z, linetype = "dashed") +
        labs(x = "", y = "",title="Regi\u00f3n de aceptaci\u00f3n-rechazo") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.Z,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

    }

  }

  CH <- cbind(H0,estadistico.Z,pvalor)
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  Iproporcion <- cbind(`limite_inferior`=media_inf,`limite_superior`=media_sup)


  if(grafico){

    return(list(`Estadistico`=CH,`Intervalo de la proporcion muestral (supuesta H0 cierta)`= Iproporcion,`Graficos`= plot))

  } else{

    return(list(`Estadistico`=CH,`Intervalo de la proporcion muestral (supuesta H0 cierta)`= Iproporcion))

  }

}

#' @title Coeficiente de correlación.
#'
#' @description Calcula el coeficiente de correlación de Pearson.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcorrelacion.png}{width = 200px}}
#' \if{latex}{\figure{qrcorrelacion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} solo tiene 2 variables (columnas), \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables a seleccionar.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#'
#' @return Esta función devuelve el valor del coeficiente de correlación lineal en un objeto de la clase \code{vector}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' El coeficiente de correlación muestral se obtiene a partir de la siguiente expresión:
#'
#' \if{html}{\figure{correlacion.png}{width = 400px}}
#' \if{latex}{\figure{correlacion.png}{options: width=5.5cm}}
#'
#' Por su construcción, el valor del coeficiente de correlación muestral es el mismo tanto si se calcula a partir de la covarianza y desviaciones típicas muestrales como si se hace a partir de la cuasi-covarianza y cuasi-desviaciones típicas muestrales.
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene el coeficiente de correlació poblacional:
#'
#' \if{html}{\figure{correlacionpob.png}{width = 240px}}
#' \if{latex}{\figure{correlacionpob.png}{options: width=3.5cm}}
#'
#' @seealso \code{\link{matriz.correlacion}}, \code{\link{covarianza}},\code{\link{matriz.covar}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' correlacion1 <- correlacion(startup[,c(1,3)])
#' correlacion2 <- correlacion(startup,variable=c(1,3))
#'
#' @importFrom stats cor
#' @import dplyr
#'
#' @export
correlacion <- function(x,
                        variable = NULL,
                        pesos = NULL) {

  # Asegurar data.frame
  x <- as.data.frame(x)
  varnames <- names(x)

  # --- Seleccion de variables ---
  if (is.null(variable)) {
    # Si solo hay 2 variables numericas, las selecciona
    varcuan <- names(x)[sapply(x, is.numeric)]
    if (length(varcuan) != 2) {
      stop("Debes seleccionar exactamente 2 variables para calcular la correlaci\u00f3n.")
    }
    variable <- match(varcuan, varnames)
  } else {
    if (is.character(variable)) {
      if (!all(variable %in% varnames)) stop("El nombre de la variable no es v\u00e1lido")
      variable <- match(variable, varnames)
    } else if (is.numeric(variable)) {
      if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    } else {
      stop("El argumento 'variable' debe ser num\u00e9rico o de tipo car\u00e1cter")
    }
    if (length(variable) != 2) stop("Solo se pueden seleccionar 2 variables")
  }

  # Subconjunto de variables
  x_sel <- x[, variable, drop = FALSE]
  varnames <- names(x_sel)

  # Comprobar el tipo de variables
  if (!all(sapply(x_sel, is.numeric))) stop("Las variables deben ser cuantitativas")

  # --- Correlacion ---
  if (is.null(pesos)) {
    # Correlacion simple eliminando NAs
    corr_val <- cor(x_sel[[1]], x_sel[[2]], use = "complete.obs")
  } else {
    # Pesos
    if (is.character(pesos)) {
      if (!pesos %in% names(x)) stop("El nombre de los pesos no es v\u00e1lido")
      pesos <- match(pesos, names(x))
    } else if (is.numeric(pesos)) {
      if (any(pesos > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de pesos")
    } else {
      stop("El argumento 'pesos' debe ser num\u00e9rico o de tipo car\u00e1cter")
    }
    x_sel <- x[, c(variable, pesos)]
    colnames(x_sel) <- c("var1", "var2", "pesos")
    x_sel <- na.omit(x_sel)

    # Correlacion ponderada
    corr_val <- sum((x_sel$var1 - sum(x_sel$var1 * x_sel$pesos) / sum(x_sel$pesos)) *
                      (x_sel$var2 - sum(x_sel$var2 * x_sel$pesos) / sum(x_sel$pesos)) * x_sel$pesos) /
      sqrt(sum((x_sel$var1 - sum(x_sel$var1 * x_sel$pesos) / sum(x_sel$pesos))^2 * x_sel$pesos) *
             sum((x_sel$var2 - sum(x_sel$var2 * x_sel$pesos) / sum(x_sel$pesos))^2 * x_sel$pesos))
  }

  # --- Devolver dataframe ---
  result <- data.frame(corr_val)
  names(result) <- paste0(varnames[1],"_",varnames[2])

  return(result)
}

#' @title Regresión lineal simple.
#'
#' @description Calcula la regresión lineal simple.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrregresion1.png}{width = 200px}}
#' \if{latex}{\figure{qrregresion1.png}{options: width=3cm}}
#'
#' \if{html}{\figure{qrregresion2.png}{width = 200px}}
#' \if{latex}{\figure{qrregresion2.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).
#' @param var_depen Es un vector (numérico o carácter) que indica la variable dependiente.
#' @param var_indepen Es un vector (numérico o carácter) que indica la variable independiente.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante de las variables: vector de medias y matriz de varianzas-covarianzas.
#' @param inferencia Si \code{inferencia = FALSE}, valor por defecto, se obtienen los resultados de la regresión simple que se estudian en un curso básico de estadística descriptiva (ver referencias de la función). Si \code{inferencia = TRUE}, se obtienen los resultas inferenciales de la regresión.
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#' @param grafico Si \code{grafico = TRUE}, se muestran algunos de los principales resultados gráficos de la regresión lineal.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return Si \code{inferencia = FALSE}, la función devuelve los principales resultados de la regresión lineal simple que se estudian en estadística descriptiva en un objeto de la clase \code{data.frame}.
#' Si \code{inferencia = TRUE}, la función devuelve los resultados de inferenciales de la regresión. Estos contenidos son estudiados en cursos de inferencia estadística y en temas introductorios de econometría.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' Se obtiene la recta de regresión minimocuadrática de Y (variable dependiente) en función de X (variable independiente).
#' La recta de regresión puede expresarse como:
#'
#' \if{html}{\figure{regresion1.png}{width = 400px}}
#' \if{latex}{\figure{regresion1.png}{options: width=7cm}}
#'
#' o alternativamente:
#'
#' \if{html}{\figure{regresion2.png}{width = 600px}}
#' \if{latex}{\figure{regresion2.png}{options: width=10cm}}
#'
#' En las representaciones gráficas las observaciones anómals se detectan a partir del punto leverage:
#'
#' \if{html}{\figure{influyente.png}{width = 400px}}
#' \if{latex}{\figure{influyente.png}{options: width=6cm}}
#'
#' de forma que una observación tendrá efecto de apalancamiento si:
#'
#' \if{html}{\figure{obsinfluyente.png}{width = 200px}}
#' \if{latex}{\figure{obsinfluyente.png}{options: width=2.5cm}}
#'
#'  donde p=2 (en el caso de la regresión simple). En general, p es igual al número de variables independientes más la constante.
#'
#' Por otra parte, las observaciones atípicas se identifican a partir de los errores estandarizados (se). Estos errores se obtienen a partir de:
#'
#' \if{html}{\figure{errorstandarizado.png}{width = 360px}}
#' \if{latex}{\figure{errorstandarizado.png}{options: width=6cm}}
#'
#' Una observación será atípica si:
#'
#' \if{html}{\figure{obsatipica.png}{width = 160px}}
#' \if{latex}{\figure{obsatipica.png}{options: width=2cm}}
#'
#'  @seealso \code{\link{matriz.covar}}, \code{\link{matriz.correlacion}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#' \dontrun{
#' ejemplo_regresion <- regresion.simple(
#'   turistas,
#'   var_depen = 2,
#'   var_indepen = 3,
#'   grafico = TRUE
#' )
#' }
#' @importFrom stats cor
#' @importFrom utils capture.output
#' @import dplyr knitr ggplot2 cowplot
#'
#' @export
regresion.simple <- function(x,
                             var_depen = NULL,
                             var_indepen = NULL,
                             introducir = FALSE,
                             inferencia = FALSE,
                             confianza = 0.95,
                             grafico = FALSE,
                             exportar = FALSE){

  old <- options()
#  on.exit(options(old))

  options(scipen=999, digits=4)

  if(isFALSE(introducir)){

    varnames <- as.character(names(x))
    x <- data.frame(x)
    names(x) <- varnames


    if(length(x)<2){
      stop("El conjunto de datos seleccionada solo tiene una variable.")
    }

    if(is.null(var_depen) | is.null(var_indepen)){
     stop("Debes seleccionar la variable dependiente y/o independiente")
    }

    if(is.numeric(var_depen)){
      if(var_depen<=length(x)){
        var_depen <- var_depen}
      else{
        stop("Seleccion erronea de variable")
      }
    }

    if(is.character(var_depen)){
      if(var_depen %in% varnames){
        var_depen = match(var_depen,varnames)
      } else {
        stop("El nombre de la variable no es v\u00e1lido")
      }
    }


    if(is.numeric(var_indepen)){
      if(var_indepen<=length(x)){
        var_indepen <- var_indepen}
      else{
        stop("Selecci\u00f3n err\u00f3nea de variable")
      }
    }

    if(is.character(var_indepen)){
      if(var_indepen %in% varnames){
        var_indepen = match(var_indepen,varnames)
      } else {
        stop("El nombre de la variable no es v\u00e1lido")
      }
    }

    if(var_depen == var_indepen){
      stop("La variable dependiente e independiente son la misma variable")
    }

      variable <- c(var_indepen,var_depen)


      x <- x[,variable] %>% as.matrix
      colnames(x) <- varnames[variable]
      varnames <- colnames(x)

    clase <- sapply(x, class)

    if (!all(clase %in% c("numeric","integer"))){
      stop("No puede calcularse la regresi\u00f3 simple porque las variables seleccionadas no son cuantitativas")
    }


  # tabla de resultados parciales

  n <- nrow(x)
  Y <- matrix(x[,2],ncol=1) # matriz coeficinetes v.depen
  vx <- matrix(x[,1],ncol=1) # matriz coeficientes v.indepen
  vX <- cbind(x0=rep(1,n),vx) # con termino constante

  k = ncol(vX) #columnas de la matriz vX (constante + regresores)


  A <- t(vX)%*%vX
  B <- t(vX)%*%Y

 # C <- A/n # MATRIZ DE MOMENTOS CENTRALES DE ORDEN 2 (RESPECTO AL ORIGEN) ENTRE REGRESORES
 # D <- B/n # MATRIZ DE MOMENTOS CENTRALES DE ORDEN 2 (RESPECTO ORIGEN) ENTRE REGRESANDO Y REGRESORES

  invA <- solve(A)
  coeficientes <- invA%*%B
  colnames(coeficientes) <- "Parametros"
  rownames(coeficientes) <- c("constante",varnames[1])

  mediay <- mean(Y)
  valores.teoricos <- vX%*%coeficientes
  colnames(valores.teoricos) <- "Valores.teoricos"

  residuos <- Y - valores.teoricos # observado - estimado
  colnames(residuos) <- "errores"

  sc.observados = (Y-mediay)^2
  colnames(sc.observados) <- "sc.observados"

  sc.teoricos = (valores.teoricos - mediay)^2
  colnames(sc.teoricos) <- "sc.teoricos"

  errores2 = residuos^2
  colnames(errores2) <- "errores2"

  tabla <- cbind(x,
                 obs = 1:n,
                 vx2 = x[,1]^2,
                 vy2 = x[,2]^2,
                 vxy = x[,1] * x[,2],
                 valores.teoricos,
                 residuos,
                 sc.observados,
                 sc.teoricos,
                 errores2) %>%
    as.data.frame() %>%
    round(4) %>%
    select(obs,everything())

  names(tabla) <- c("id",
                    varnames,
                    paste(varnames[1],"^2",sep=""),
                    paste(varnames[2],"^2",sep=""),
                    paste(varnames[1],"*",varnames[2],sep=""),
                    "valores.teoricos",
                    "errores",
                    "sc.observados",
                    "sc.teoricos",
                    "errores^2")

  tabla2 <- tabla %>%
    kable(caption = "c\u00e1lculos intermedios")

  # SUMA DE CUADRADOS
  SCE <- t(residuos)%*%residuos %>% as.numeric()
  SCT <- sum(Y^2)-n*(mean(Y)^2)
  SCR <- sum(t(coeficientes)%*%B)-n*(mean(Y)^2)


  resumen <- data.frame(observaciones = n,
                        constante = coeficientes[1],
                        coeficiente.regresion = coeficientes[2],
                        suma.cuadrados.total = SCT,
                        suma.cuadrados.regresion = SCR,
                        suma.cuadrados.residuos = SCE,
                        varianza.regresion = SCR/n,
                        varianza.residual = SCE/n,
                        coeficiente.determinacion = SCR/SCT,
                        coeficiente.correlacion = correlacion(as.data.frame(x)) ) %>%
    t() %>%
    round(4) %>%
    as.data.frame()

  names(resumen) <- "Resumen"

  resumen2 <- resumen %>%
    kable(col.names="Valor",
          caption = "Resumen medidas de la regresi\u00f3n")


  # deteccion de outliers
  mediax <- as.numeric(media(x[,1]))

  tablaplot <- tabla %>%
    select(1,2,3,7,8,11) %>%
    rename("X"=varnames[1],"Y"=varnames[2]) %>%
    mutate(momento = (X-mediax)^2,
           leverage = 1/n + (momento/sum(momento)),
           puntos.palanca = ifelse(leverage > 3 * k/n, "palanca", "no palanca"),
           error.norm = errores/(sqrt(sum(errores2)/(n-2))*sqrt(1-leverage)),
           atipico = ifelse(abs(error.norm)>2,"atipico","no atipico"),
           distancia.cook = (error.norm^2 / 2) * (leverage /(1-leverage)),
           grupo = paste(puntos.palanca,"_",atipico,sep=""))

  deteccion.outliers <- tablaplot[c(1,8,10,12)]


  } else{   # aqu\u00ed empieza introducir datos

    message("Esta opci\u00f3n obtiene la regresi\u00f3n simple que se estudia en un curso introductorio de estad\u00edstica descriptiva. Ver Esteban y otros (2005).")
    #message("Se deshabilita el argumento: inferencia")

    grafico = FALSE

    print("A continuaci\u00f3n, vas a introducir los datos.")

    mediax <- as.numeric(readline('Introduce el valor de la media de la variable independiente: \n'))
    varx <- as.numeric(readline('Introduce el valor de la varianza muestral de la variable independiente: \n'))
    mediay <- as.numeric(readline('Introduce el valor de la media de la variable dependiente: \n'))
    vary <- as.numeric(readline('Introduce el valor de la varianza muestral de la variable dependiente: \n'))
    covar_xy <- as.numeric(readline('Introduce el valor de la covarianza muestral entre las variables: \n'))

    varnames <- c("X","Y")

    coeficiente.correlacion <- covar_xy/(sqrt(varx)*sqrt(vary))
    coeficiente.determinacion <- covar_xy^2/(varx*vary)


    coeficiente.regresion = round(covar_xy / varx,5)
    constante = round(mediay - coeficiente.regresion * mediax,5)

    coeficientes <- c(constante,coeficiente.regresion)

    resumen <- data.frame() %>%
      summarize(coeficiente.correlacion = round(coeficiente.correlacion,4),
                coeficiente.determinacion = round(coeficiente.determinacion,4),
                varianza.regresion = coeficiente.determinacion * vary,
                varianza.residual = vary - varianza.regresion,
                constante = constante,
                coeficiente.regresion = coeficiente.regresion) %>%
      t() %>%
      round(4) %>%
      as.data.frame()

    names(resumen) <- "Resumen"


    resumen2 <- resumen %>%
      kable(col.names="Valor",
            caption="Resultados de la regresi\u00f3n simple")

  }


  if(inferencia){

    if(introducir){
      message("Para obtener los resultados inferenciales de la regresi\u00f3n es necesario el tama\u00f1o de la muestra")
      n <-  as.numeric(readline('Introduce el valor del tama\u00f1o de la muestra: \n'))
      k = 2

      SCT = n *vary
      SCE = (1-coeficiente.determinacion)*SCT
      SCR = SCT - SCE
      varianza.residuos.regresion = SCE/(n-k)
      error.estandard.regresion = sqrt(varianza.residuos.regresion)

      resultados.parciales <- data.frame(valor = c(n,
                               coeficiente.correlacion,
                               coeficiente.determinacion,
                               varianza.residuos.regresion,
                               error.estandard.regresion,
                               SCT,
                               SCR,
                               SCE))


    } else{

      resultados.parciales <- resumen %>% slice(1,10,9,7,8,4,5,6)
      resultados.parciales[4,1] <- SCE/(n-k)
      resultados.parciales[5,1] <- sqrt(SCE/(n-k))

    }

      rownames(resultados.parciales) <- c("n",
                                       "Coeficiente.correlacion",
                                       "Coeficiente.determinacion",
                                       "Varianza.residuos.regresion",
                                       "Error.estandar.regresion",
                                       "suma.cuadrados.total",
                                       "suma.cuadrados.regresion",
                                       "suma.cuadrados.residuos")


      resultados.parciales2 <- resultados.parciales %>%
        kable(col.names="Valor",
              caption="Resumen resultados de la regresi\u00f3n")

      estadistico = (SCR/(k-1))/(SCE/(n-k))
      tabla.anova <- data.frame(Medida = c("Regresion","errores","total"),
                                Suma.cuadrados = round(c(SCR,SCE,SCT),5),
                                gl = c(k-1,n-k,n-1),
                                Media.suma.cuadrados = round(c(SCR/(k-1),SCE/(n-k),NA),5),
                                estadistico.F = round(c(estadistico,NA,NA),5),
                                p_valor = round(c(pf(estadistico,k-1,n-k,lower.tail=FALSE),NA,NA),6))

      tabla.anova[is.na(tabla.anova)] <- " "

      tabla.anova2 <- tabla.anova %>%
        kable(caption = "ANOVA", align= "c")

      # errores tipicos de la regresion
        # VARIANZAS DE LOS ESTIMADORES (BETAS)
      varianza.residuos = SCE/(n-k)

      if(introducir){

        error.tipico.intercepto <- sqrt((SCE/(n-2)) * (1/n + (mediax^2/(n*varx))))
        error.tipico.coef.regresion <- sqrt((SCE/(n-2))/(n*varx))
        error_betas <- c(error.tipico.intercepto,error.tipico.coef.regresion)

      } else{

        var_betas <- varianza.residuos * diag(invA)
        # error estandar de betas
        error_betas <- sqrt(var_betas)

      }

      # valor_t
      t <- coeficientes/error_betas %>% data.frame()

      # p-valores
      p_valor <- 2 * apply(abs(t),1,pt,df=n-k,lower.tail=FALSE)
      p_valor <- round(p_valor,6)

      # intervalo de confianza
      alfa2 <- (1-confianza)/2
      Lim.inf <- coeficientes - qt(alfa2,n-k,lower.tail = F) * error_betas
      Lim.sup <- coeficientes + qt(alfa2,n-k,lower.tail = F) * error_betas

      # modelo de regresion
      modelo.regresion <- data.frame(coeficientes,
                                     error_betas,
                                     t,
                                     p_valor,Lim.inf,
                                     Lim.sup)

      names(modelo.regresion) <- c("Coeficientes",
                                   "Error t\u00edpico",
                                   "t",
                                   "p-valor",
                                   paste("Lim.inf_",confianza*100,"%",sep=""),
                                   paste("Lim.sup_",confianza*100,"%",sep=""))
      rownames(modelo.regresion) <- c("constante",varnames[1])


      modelo.regresion2 <- modelo.regresion %>%
        kable(caption = "Resultados de la regresi\u00f3n")

      # kk <- list("Resultados globales"=resultados.parciales,
      #             "ANOVA"= tabla.anova,
      #             "modelo.regresion" = modelo.regresion)
      # pander(kk)
    }


    # REPRESENTACION GRAFICA
    if(grafico){

      tablaplot$grupo <- factor(tablaplot$grupo)
      levels(tablaplot$grupo) <- list(palanca_atipico = "palanca_atipico",
                                      palanca_no.atipico = "palanca_no atipico",
                                      no.palanca_atipico = "no palanca_atipico",
                                      normal = "no palanca_no atipico")

      #miscolores <- brewer.pal(4,"Set1")
      miscolores <- c("red","purple","chocolate","darkgreen")
      names(miscolores) <- levels(tablaplot$grupo)

      mispuntos <- c(15,18,17,19)
      names(mispuntos) <- levels(tablaplot$grupo)

      tablaplot <- droplevels(tablaplot)

      escalaColor <- scale_color_manual(name = "observaciones",
                                      values = miscolores,
                                      drop = TRUE,
                                      limits= levels(tablaplot$grupo))

      escalaForma <- scale_shape_manual(name = "observaciones",
                                        values = mispuntos,
                                        drop = TRUE,
                                        limits= levels(tablaplot$grupo))

      plot1 <- ggplot(tablaplot,aes(x=X,y=Y,label=id)) +
        geom_point(aes(color=grupo,shape=grupo),size=2) +
        escalaColor +
        escalaForma +
        geom_smooth(method = "lm", formula = y ~ x, se = FALSE,color="blue") +
        geom_text(data=subset(tablaplot,grupo!='normal'),
                  vjust = -0.7,
                  size = 2.5) +
        labs(title = "Modelo de regresi\u00f3n estimado",
             subtitle= paste(varnames[2],"=",round(coeficientes[1],5),if_else(coeficientes[2] >=0, "+", ""),round(coeficientes[2],5),"*",varnames[1],sep=""),
             x = varnames[1],
             y = varnames[2]) +
        theme_classic() +
        theme(legend.title = element_blank(),
              legend.key.size = unit(0, 'lines'),
              legend.position = "bottom",
              legend.text = element_text(size = 7))


      #plot12 <- ggplot(tablaplot,aes(x=valores.teoricos,y=errores)) +
      #  geom_point()

      plot21 <- ggplot(tablaplot, aes(x=id,y=valores.teoricos,color=grupo,shape=grupo,label=id)) +
        geom_point() +
        geom_hline(yintercept = mediay) +
      geom_text(data=subset(tablaplot,grupo!='normal'),
                vjust = -0.55,
                size= 2.5) +
        labs(y="valores pronosticados (teoricos)") +
        escalaColor +
        escalaForma +
        theme_classic()+
        theme(legend.position = "none")


      plot22 <- ggplot(tablaplot, aes(x=id,y=error.norm,color=grupo,shape=grupo,label=id)) +
        geom_point() +
        geom_hline(yintercept = 0) +
        geom_hline(yintercept = 2, linetype=2) +
        geom_hline(yintercept = -2, linetype=2) +
        geom_text(data=subset(tablaplot,grupo!='normal'),
                  vjust = -0.7,
                  size = 2.5) +
        labs(y="errores estandarizados") +
        escalaColor +
        escalaForma +
        theme_classic()+
        theme(legend.position = "none")

      #plot <- gridExtra::grid.arrange(plot1,plot12,plot21,plot22, ncol=2, nrow=2)
      #plot <- gridExtra::grid.arrange(plot1,gridExtra::arrangeGrob(plot21,plot22), ncol=2)
      left_plot <- cowplot::plot_grid(plot21, plot22, ncol = 1)  # los apila verticalmente
      plot <- cowplot::plot_grid(plot1, left_plot, ncol = 2)

    }

  if (exportar) {
    # Prepara el nombre del archivo
    filename <- paste("Resultados_regresion_simple_", format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), ".xlsx", sep = "")

    # Crear el workbook
    wb <- openxlsx::createWorkbook()

    # Función auxiliar para añadir data.frames con formato
    add_sheet_with_style <- function(wb, sheet_name, data) {
      openxlsx::addWorksheet(wb, sheet_name)
      openxlsx::writeData(wb, sheet_name, data, rowNames = TRUE)

      # Aplicar formato numérico a columnas numéricas
      numeric_cols <- which(sapply(data, is.numeric))
      if (length(numeric_cols) > 0) {
        openxlsx::addStyle(
          wb, sheet_name,
          style = openxlsx::createStyle(numFmt = "0.0000"),
          rows = 2:(nrow(data) + 1),
          cols = numeric_cols + 1,  # +1 porque rowNames=TRUE añade una columna
          gridExpand = TRUE
        )
      }
    }

    if (isFALSE(introducir)) {
      if (inferencia) {
        names(resultados.parciales) <- "valor"

        # Añadir todas las hojas
        add_sheet_with_style(wb, "Resultados_parciales", tabla)
        add_sheet_with_style(wb, "Resumen_medidas", resultados.parciales)
        add_sheet_with_style(wb, "ANOVA", tabla.anova)

        # Hoja para el modelo (manejo especial)
        openxlsx::addWorksheet(wb, "Modelo_estimado")
        openxlsx::writeData(wb, "Modelo_estimado", capture.output(modelo.regresion))

      } else {
        add_sheet_with_style(wb, "Resumen", resumen)
        add_sheet_with_style(wb, "Resultados_parciales", tabla)
      }

    } else {
      if (inferencia) {
        add_sheet_with_style(wb, "Resumen_medidas", resultados.parciales)
        add_sheet_with_style(wb, "ANOVA", tabla.anova)

        # Hoja para el modelo (manejo especial)
        openxlsx::addWorksheet(wb, "Modelo_estimado")
        openxlsx::writeData(wb, "Modelo_estimado", capture.output(modelo.regresion))

      } else {
        add_sheet_with_style(wb, "Resumen", resumen)
      }
    }

    # Guardar el archivo
    openxlsx::saveWorkbook(wb, filename, overwrite = TRUE)
  }

  if(isFALSE(introducir)){

    if(isFALSE(grafico)){
      plot = NULL
    }

    if(inferencia){

      return(list('Calculos.intermedios' = tabla,
                  'Resultados.parciales' = resultados.parciales,
                  'ANOVA' = tabla.anova,
                  'Modelo.estimado' = modelo.regresion,
                  'Deteccion.outliers' = deteccion.outliers,
                                    'Graficos' = plot))

    } else{

      return(list('Calculos.intermedios' = tabla,
                  'Resumen.regresion' = resumen,
                  'Deteccion.outliers' = deteccion.outliers,
                  'Graficos' = plot))

    }

  } else{

    message("No puede realizarse la representaci\u00f3 gr\u00e1fica si se introducen las medidas num\u00e9ricas")

    if(inferencia){

      return(list('Resultados.parciales' = resultados.parciales,
                  'ANOVA' = tabla.anova2,
                  'Moldelo.estimado' = modelo.regresion))

    } else{

      return('Resumen.regresion' = resumen)

    }

  }

  on.exit(options(old))

}

tema_blanco <- theme(
  panel.background = element_rect(fill = "transparent"), # bg of the panel
  plot.background = element_rect(fill = "transparent", color = NA), # bg of the plot
  panel.grid.major = element_blank(), # get rid of major grid
  panel.grid.minor = element_blank(), # get rid of minor grid
  legend.background = element_blank(), # get rid of legend bg
  legend.box.background = element_blank(), # get rid of legend panel bg
  legend.key = element_blank(), # get rid of key legend fill, and of the surrounding
  axis.line.x = element_blank(),
  axis.ticks.x=element_blank(),
  axis.text.x=element_blank(),
  axis.line.y = element_blank(),
  axis.ticks.y=element_blank(),
  axis.text.y=element_blank(),
  legend.title = element_blank()
)

#' @title Contraste de hipótesis de bondad de ajuste para distribuciones discretas.
#'
#' @description Contrasta si los datos de una muestra proceden de una distribución poblacional determinada.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre el número de filas (se abrirá una ventana con un editor de datos y deberá introducir los valores de la variable poblacional y las frecuencias observadas), valor del parámetro poblacional, etc.
#' @param distribucion Es un carácter. Indica el tipo de distribución poblacional que se quiere contrastar en la hipótesis nula (por defecto, \code{distribucion = "equiprobable"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.
#'        Si \code{distribucion = "equiprobable"}, se contrasta que en la distribución poblacional de la hipótesis nula todos los valores de la población tienen la misma probabilidad.
#'        Si \code{distribucion = "poisson"}, se contrasta que la distribución poblacional de la hipótesis nula se distribuye según una Poisson.
#'        Si \code{distribucion = "binomial"}, se contrasta que la distribución poblacional de la hipótesis nula se distribuye según una Binomial.
#' @param parametro Es un valor lógico. Si no se especifica ningún valor para el parámetro o parámetros de la distribución poblacional \code{parametro = FALSE} (por defecto) o si se especifica un valor para dicho parámetro o parámetros (cambiar el argumento a \code{parametro = TRUE})
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba y el p-valor. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con el valor crítico.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Juan José Vidal Llana}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) El estadístico del contraste de bondad de ajuste es:
#'
#' \deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(O_{i} - E_{i})^{2}}{E_{i}}}
#'
#' donde \eqn{O_{i}} son las frecuencias observadas y \eqn{E_{i}} son las frecuencias teóricas o esperadas, y se distribuye como:
#'
#' \deqn{\chi_{k-m-1}^{2}}
#'
#' donde \eqn{k} es el número de valores distintos de la variable, y \eqn{m} es el número de parámetros de la distribución poblacional de la hipótesis nula no especificados (desconocidos) y que se han tenido que estimar.
#'
#' Además, se exige que todas las frecuencias teóricas no estén por debajo de 5. Si alguna no lo cumple, es necesario reagrupar valores contiguos hasta conseguir superar esa cota.
#'
#' (2) Si el número de grados de libertad es 1, al estadístico del contraste se le aplica la siguiente corrección de Yates:
#'
#' \deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(\left| O_{i} - E_{i} \right| - 0.5)^{2}}{E_{i}}}
#'
#' @seealso \code{\link{contraste_bondad_cat}},\code{\link{contraste_independencia}},
#' \code{\link{contraste_homogeneidad}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats dbinom dpois
#' @importFrom utils edit
#' @import dplyr ggplot2
#'
#' @export
contraste_bondad <- function(x,
                             introducir = FALSE,
                             distribucion = "equiprobable",
                             parametro = FALSE,
                             alfa = 0.05,
                             grafico = FALSE){


  if(isFALSE(introducir)) {

    if(is.numeric(x)){

      matriz <- data.frame("Valores" = unique(x),
                           "Freq_obs" = as.numeric(table(x)));print(matriz)

      respuesta <- readline(prompt ='\u00bfEstas son las frecuencias observadas? \n 1. "Si" \n 2. "No" \n')

      if (respuesta == "2"){
        print("Has marcado que estas no son las frecuencias, introduce o modifica lo que creas necesario.")
        matriz <- edit(matriz)
        matriz

      } else if (respuesta == "1"){
        print("\u00A1Perfecto! Seguimos con estos datos")

      } else {
        stop("El comando introducido no es correcto")

      } # End if respuesta

    } else {

      stop("x debe ser un vector de n\u00fameros")

    } # Fin de is.numeric

  } else {
    # se introducen los datos manualmente
    nfilas <- as.numeric(readline(prompt = "Intoduce el n\u00famero de filas: "))

    x <- matrix(0, nrow = nfilas, ncol=2)

    matriz <- edit(x)
    matriz <- as.data.frame(matriz); matriz
    colnames(matriz) <- c("Valores", "Freq_obs")

  } # Fin de introducir datos

  if (distribucion == "equiprobable"){

    # Equiprobable

    n <- length(matriz$Freq_obs)
    matriz$Freq_esp <-  sum(matriz$Freq_obs) / length(matriz$Freq_obs)

  } else if (distribucion == "poisson"){

    # poisson

    if (parametro == F) {
      n <- length(matriz$Freq_obs) - 1
      lambda <- as.numeric(crossprod(matriz$Valores, matriz$Freq_obs)) / sum(matriz$Freq_obs)
    } else{
      n <- length(matriz$Freq_obs)
      lambda <- as.numeric(readline(prompt = "Intoduce el valor de lambda: "))
    } # End if

    matriz$Freq_esp <-dpois(matriz$Valores,lambda) * sum(matriz$Freq_obs)

  } else if (distribucion == "binomial"){

    # Binomial

    if (parametro == F) {
      n <- length(matriz$Freq_obs) - 1
      media <- as.numeric(crossprod(matriz$Valores, matriz$Freq_obs)) / sum(matriz$Freq_obs)
      p <- media / n
    } else{
      n <- length(matriz$Freq_obs)
      #n <- as.numeric(readline(prompt = "Intoduce el valor de n: "))
      p <- as.numeric(readline(prompt = "Intoduce el valor de p: "))
    } # End if

    matriz$Freq_esp <- dbinom(matriz$Valores, n, p ) * sum(matriz$Freq_obs)

  } else {

    stop("No has introducido una distribuci\u00f3n de probabilidad disponible")

  } # Fin de distribucion

  # estimacion de parametros de binonial y poisson
  if (distribucion == "equiprobable"){

    # Equiprobable

    k <- 0

  } else if (distribucion == "poisson"){

    # poisson

    if (parametro == F) {
      k <- 1
    } else{
      k <- 0
    } # End if

  } else if (distribucion == "binomial"){

    # Binomial

    if (parametro == F) {
      k <- 1
    } else{
      k <- 0
    } # End if

  } else {

    stop("No has introducido una distribuci\u00f3n de probabilidad disponible")

  } # Fin de estimacion de parametros de distribucion

  # reagrupar si frecuencias esperadas es menor a 5
  if(sum(matriz$Freq_esp < 5) > 0){
    message("Aqu\u00ed tienes la tabla recalculkada de frecuencias esperadas porque alguna de las frecuencias te\u00f3ricas era menor a 5. Para llevar a cabo el test es necesario reagrupar las categor\u00edas.")
    matriz <- .check_min_obs(matriz)
    print(matriz)
  }

  # Sumar las frecuencias observadas
  suma_freq_obs <- sum(matriz$Freq_obs)

  # Comprobar si las frecuencias esperadas suman la suma de las frecuencias observadas
  if (sum(matriz$Freq_esp) != suma_freq_obs) {
    diferencia <- suma_freq_obs - sum(matriz$Freq_esp)

    # Ajustar la ultima categoria
    matriz$Freq_esp[nrow(matriz)] <- matriz$Freq_esp[nrow(matriz)] + diferencia

    # Opcional: advertir al usuario del ajuste
    message("Se ha ajustado la \u00faltima categor\u00eda en ", diferencia, " para que las frecuencias esperadas sumen ", suma_freq_obs, ".")
  }


  n <- length(matriz$Freq_obs)

  g.l <- n - k - 1

  if(g.l == 1){

    warning("Los grados de libertad son 1; por tanto, es necesario aplicar la correcci\u00f3n de Yates.")
    estadistico.prueba <- sum((abs(matriz$Freq_obs-matriz$Freq_esp)-0.5)^2/matriz$Freq_esp)

  }else{

    estadistico.prueba <- sum((matriz$Freq_obs - matriz$Freq_esp)^2/ matriz$Freq_esp)

  }

  valor_critico <- qchisq(alfa, g.l, lower.tail = F)


  if(estadistico.prueba < valor_critico ){

    print(paste("No se rechaza la hip\u00f3tesis nula. El valor del estad\u00edstico de contraste: ", round(estadistico.prueba, 2) , ", se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n que toma el intervalo [0, ", round(valor_critico, 2), "]", sep=""))

  } else{

    print(paste("Se rechaza la hip\u00f3tesis nula. El valor del estad\u00edstico de contraste: ", round(estadistico.prueba, 2) , ", se encuentra fuera de la regi\u00f3n de aceptaci\u00f3n que toma el intervalo [0, ", round(valor_critico, 2), "]", sep=""))

  }

  pvalor <- pchisq(estadistico.prueba, g.l, lower.tail = F)

  H0 <- paste("Los datos siguen una distribuci\u00f3n ", distribucion)
  CH <- cbind(H0, estadistico.prueba, round(pvalor, 4))
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  if(isTRUE(grafico)){

    percentil99 <- qchisq(.9999, g.l)

    data <- data.frame(x=seq(from = 0, to = percentil99, percentil99/200))
    data$y <-dchisq(data$x, g.l)

    name_plot <- ggplot(data, aes(x, y)) +
      geom_area(fill = "#C4961A") +
      geom_area(data = subset(data, x < valor_critico), fill = "#008B8B") +
      geom_vline(xintercept = 0, color = "black") +
      geom_vline(xintercept = estadistico.prueba, color = "#A52A2A", linetype = "dashed" , size = 1) +
      labs(title = paste("Distribuci\u00f3n Chi-Cuadrado con ", g.l," grados de libertad", sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = round(c(0L, estadistico.prueba, valor_critico), 2)) +
      theme(axis.text.x = element_text(angle = 45))

    return(list(CH, name_plot))

  } else{

    return(CH)

  }

} # Fin de la funcion

#' @title Tabla de frecuencias.
#'
#' @description Esta función presenta la distribución de frecuencias de una variable cuantitativa o cualitativa.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrtablafrecuencias.png}{width = 200px}}
#' \if{latex}{\figure{qrtablafrecuencias.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector (numérico o factor) o un dataframe. Si el dataframe tiene más de una variable, solicitará al usuario que idenfique el nombre de la variable para la que se quiere calcular la tabla de frecuencias.
#' @param eliminar.na Valor lógico. Por defecto \code{eliminar.na = TRUE}. Si se quiere obtener la tabla de frecuencias con NAs, cambiar el argumento a \code{eliminar.na = FALSE}.
#' @param grafico Si \code{grafico = TRUE}, representa el histograma o el gráfico de barras de la variable seleccionada.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return Devuelve la tabla de frecuencias como una \code{tibble}. Si \code{grafico = TRUE}, se devuelve en una lista la tabla de frecuencias y su representación gráfica.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @importFrom tidyr pivot_longer drop_na
#' @import dplyr
#'
#' @export
tabla.frecuencias <- function(x,
                              eliminar.na = TRUE,
                              grafico = FALSE,
                              exportar = FALSE){

  x <- as.data.frame(x)

  varnames <- colnames(x)

  if(length(x) > 1 ) {

    variable <- readline(prompt = "Intoduce el nombre de la variable: ")

  } else{

    variable <- varnames

  }

  if(is.character(variable)){
    if(variable %in% varnames){
      variable = which(varnames == variable)
    } else {
      stop("El nombre de la variable no es v\u00e1lido")
    }
  }

  x <- as.data.frame(x) %>%
    dplyr::select(all_of(variable))

  y <- varnames[variable] # nombre de la variable seleccionada

  clase <- sapply(x, class)

  if (!clase %in% c("numeric","integer","factor","logic")) {
    stop("No puede construirse la tabla de frecuencias, la variable que has\n
         seleccionado es car\u00e1cter")
  }

  if(length(x) > 1){
    stop("Esta funci\u00f3n solo puede contruir la tabla de frecuencias de una variable")
    print("Para obtener la tabla de frecuencias de mas de una variable utiliza la funci\u00f3n apply")
  }

  valores_distintos <- nrow(unique(x))
  valores_ordenados <- unique(x)
  valores_ordenados <- valores_ordenados[,1]

  if(valores_distintos >= 20){

    # tabla de frecuencias de valores agrupados

    print("La variable presenta muchos valores distintos (20 o mas)")
    agrupar <- as.numeric(readline('\u00bfQuieres agrupar los valores en intervalos?: \n 1. "S\u00ed, agrupar en intervalos." \n 2. "No, usar los valores sin agrupar." \n'))

    if(agrupar == 1){
      clase <- sapply(x, class)

      x <- na.omit(x)

      if(clase %in% c("factor","character","logic")){
        stop("La variable no es cuantitativa, no puede representarse el histograma")
      }

      if(nrow(x)<=100){
        intervalos <- ceiling(sqrt(nrow(x)))
      } else {
        intervalos <- ceiling(log(nrow(x))/log(2) + 1)
      }

      amplitud <- (max(x,na.rm=TRUE)-min(x,na.rm=TRUE))/intervalos

      x$cut <- cut(x[,1], seq(min(x[,1],na.rm=TRUE),max(x[,1],na.rm=TRUE),amplitud),
                   include.lowest = TRUE,
                   dig.lab = 8)

      x <- x[2]
      names(x) <- y

    } else{

      print("Has decidido no agrupar los valores, puede que el diagrama de barras no represente adecuadamente la distribuci\u00f3n de la variable")

      agrupar = 0

    }


  } else{

    print("Has decidido no agrupar los valores.")

    agrupar = 0

    }

  # tabla de frecuencias

  # ordena de menor a mayor los valores
  tabla <- x %>% dplyr::arrange(x) %>%
    dplyr::group_by_at(y) %>%   # en lugar de group_by(.dots=y)
    dplyr::count() %>%
    dplyr::ungroup()

  names(tabla) <- c(y,"ni") # nombres de las columnas de tabla

  # calcula las frecuencias
  tabla <- tabla %>%
    dplyr::mutate(Ni = cumsum(ni),
                  fi = ni / sum(ni),
                  Fi = cumsum(fi))

  if(eliminar.na == TRUE){
    x <- drop_na(x)

    tabla <- x %>% dplyr::arrange(x) %>%
      dplyr::group_by_at(y) %>%
      dplyr::count() %>%
      dplyr::ungroup()

    names(tabla) <- c(y,"ni")

    tabla <- tabla %>%
      dplyr::mutate(Ni = cumsum(ni),
                    fi = ni / sum(ni),
                    Fi = cumsum(fi))

  } else {

    tabla <- x %>% dplyr::arrange(x) %>%
      dplyr::group_by_at(y) %>%
      dplyr::count() %>%
      dplyr::ungroup()

    names(tabla) <- c(y,"ni")

    tabla <- tabla %>%
      dplyr::mutate(Ni = cumsum(ni),
                    fi = ni / sum(ni),
                    Fi = cumsum(fi))

  }

  if (exportar) {
    filename <- paste("Tabla de frecuencias de ", y, " (", Sys.time(), ").xlsx", sep = "")
    filename <- gsub(" ", "_", filename)
    filename <- gsub(":", ".", filename)
    rio::export(tabla, file = filename)
  }

  if(grafico){

    df <- cbind(tabla[1],tabla[2],tabla[4],tabla[3],tabla[5])

    if(agrupar == 1){

      marca_clase <- function(cut_label) {
        mean(as.numeric(unlist(strsplit(gsub("\\(|\\)|\\[|\\]", "", as.character(cut_label)), ","))))
      }

      df$marca <- sapply(df[,1], marca_clase)
      variables <- names(df)

      print("Los valores han sido agrupados en intervalos y se representar\u00e1n mediante un histograma")

      plot <- ggplot(df, aes_string(x=variables[1], y = variables[2])) +
        geom_bar(stat = "identity", width = 1,
                 color = "white", fill = "orange") +
        geom_text(aes_string(label=variables[2]), vjust=1.5, size = 2.5) +
        geom_text(aes(label=paste("(",round(df[,3]*100,2),"%)",sep="")), vjust=2.5, size = 2.5, color = "darkgreen" ) +
        labs(title = paste("Histograma de ",variables[1],sep=""),
               x = variables[1],
               y = "") +
        theme(
          panel.background = element_rect(fill = "transparent"), # bg of the panel
          plot.background = element_rect(fill = "transparent", color = NA), # bg of the plot
          panel.grid.major = element_blank(), # get rid of major grid
          panel.grid.minor = element_blank(), # get rid of minor grid
          legend.background = element_blank(), # get rid of legend bg
          legend.box.background = element_blank(), # get rid of legend panel bg
          legend.key = element_blank(), # get rid of key legend fill, and of the surrounding
          axis.ticks.x=element_blank(),
          axis.line.y = element_blank(),
          axis.ticks.y=element_blank(),
          axis.text.x=element_text(size=6,angle=30),
          axis.text.y=element_blank(),
          legend.title = element_blank()
        )


    } else {

      variables <- names(df)

      if(clase == "factor"){
        x_string <- df[,1]

      } else{
        x_string <- as.factor(round(df[,1],2))
      }

      if(valores_distintos > 10){

        print("El diagrama de barras puede no ser una buena representaci\u00f3n gr\u00e1fica si la variable presenta muchos distintos valores (aconsejable como m\u00e1ximo 10-15)")

        plot <- ggplot(df, aes_string(x=x_string,y=variables[2])) +
          geom_bar(stat = "identity",  fill = "orange") +
          geom_text(aes_string(label=variables[2]), vjust=1.5, size = 2.5) +
          geom_text(aes(label=paste("(",round(df[,3]*100,2),"%)",sep="")), vjust=2.65, size = 2.5, color = "blue" ) +
          labs(title = paste("Diagrama de barras de ", variables[1], sep=""),
               x = variables[1],
               y = "") +
          theme(
            panel.background = element_rect(fill = "transparent"), # bg of the panel
            plot.background = element_rect(fill = "transparent", color = NA), # bg of the plot
            panel.grid.major = element_blank(), # get rid of major grid
            panel.grid.minor = element_blank(), # get rid of minor grid
            legend.background = element_blank(), # get rid of legend bg
            legend.box.background = element_blank(), # get rid of legend panel bg
            legend.key = element_blank(), # get rid of key legend fill, and of the surrounding
            axis.ticks.x=element_blank(),
            axis.line.y = element_blank(),
            axis.ticks.y=element_blank(),
            axis.text.x=element_text(size=6,angle=90),
            axis.text.y=element_blank(),
            legend.title = element_blank()
          )


      } else {

       plot <- ggplot(df, aes_string(x=x_string,y=variables[2])) +
          geom_bar(stat = "identity", width = 0.5, fill = "orange") +
          geom_text(aes_string(label=variables[2]), vjust=1.5, size = 2.5) +
          geom_text(aes(label=paste("(",round(df[,3]*100,2),"%)",sep="")), vjust=2.65, size = 2.5, color = "blue" ) +
          labs(title = paste("Diagrama de barras de ", variables[1], sep=""),
               x = variables[1],
               y = "") +
          scale_x_discrete(breaks = valores_ordenados) +
          theme(
            panel.background = element_rect(fill = "transparent"), # bg of the panel
            plot.background = element_rect(fill = "transparent", color = NA), # bg of the plot
            panel.grid.major = element_blank(), # get rid of major grid
            panel.grid.minor = element_blank(), # get rid of minor grid
            legend.background = element_blank(), # get rid of legend bg
            legend.box.background = element_blank(), # get rid of legend panel bg
            legend.key = element_blank(), # get rid of key legend fill, and of the surrounding
            axis.ticks.x=element_blank(),
            axis.line.y = element_blank(),
            axis.ticks.y=element_blank(),
            axis.text.y=element_blank(),
            legend.title = element_blank()
          )
      }



    }

  } # cierra grafico

  if(grafico){

    return(list(tabla,plot))

  } else{

    return(tabla)

  }

}

#' @title Varianza.
#'
#' @description Calcula la varianza.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrdispersion.png}{width = 200px}}
#' \if{latex}{\figure{qrdispersion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#' @param tipo Es un carácter. Por defecto de calcula la varianza muestral (\code{tipo = "muestral"}). Si \code{tipo = "cuasi"}, se calcula la cuasivarianza muestral.
#'
#' @return Esta función devuelve un objeto de la clase \code{vector}. Si \code{tipo="muestral"}, devuelve la varianza muestral. Si \code{tipo="cuasi"}, devuelve la cuasi-varianza muestral.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) La expresión de la varianza muestral es:
#'
#' \if{html}{\figure{varianza.png}{width = 320px}}
#' \if{latex}{\figure{varianza.png}{options: width=5cm}}
#'
#' La varianza muestral así definida es el estimador máximo verosímil de la varianza de una población normal
#'
#' (2) Muchos manuales y prácticamente todos los softwares (SPSS, Excel, etc.) calculan la expresión:
#'
#' \if{html}{\figure{cuasivarianza.png}{width = 320px}}
#' \if{latex}{\figure{cuasivarianza.png}{options: width=5cm}}
#'
#' Nosotros llamamos a esta medida: cuasi-varianza muestral y es un estimador insesgado de la varianza poblacional.
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la varianza poblacional:
#'
#'
#' \if{html}{\figure{varianzapob.png}{width = 320px}}
#' \if{latex}{\figure{varianzapob.png}{options: width=5cm}}
#'
#' @seealso \code{\link{media}}, \code{\link{desviacion}}, \code{\link{coeficiente.variacion}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' varianza1 <- varianza(startup[1])
#' varianza2 <- varianza(startup,variable=1)
#' varianza3 <- varianza(startup,variable=1, tipo="cuasi")
#'
#' @importFrom stats var
#'
#' @export
varianza <- function(x, variable = NULL, pesos = NULL, tipo = c("muestral", "cuasi")) {

  tipo <- match.arg(tolower(tipo), c("muestral", "cuasi"))

  if (!is.data.frame(x)) x <- data.frame(x)

  # --- Seleccion de variables ---
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)]
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    varnames <- names(x)[variable]
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("Nombre de variable no v\u00e1lido")
    varnames <- variable
  } else {
    stop("El argumento 'variable' debe ser num\u00e9rico o de tipo car\u00e1cter")
  }

  x_sel <- x[, varnames, drop = FALSE]

  # --- Comprobar tipo de variables ---
  if (!all(sapply(x_sel, is.numeric))) {
    stop("No puede calcularse la varianza: alguna variable seleccionada no es cuantitativa")
  }

  # --- Si no hay pesos ---
  if (is.null(pesos)) {

    x_sel <- x[, varnames, drop = FALSE]

    # Comprobacion tipo de variable
    if (!all(sapply(x_sel, is.numeric))) {
      stop("No puede calcularse la varianza: alguna variable seleccionada no es cuantitativa")
    }

    calcular_var <- function(col) {
      n_eff <- sum(!is.na(col))
      if (n_eff < 2) return(NA_real_)
      factor <- if (tipo == "muestral") (n_eff - 1) / n_eff else 1
      stats::var(col, na.rm = TRUE) * factor
    }

    var_val <- sapply(x_sel, calcular_var)
    var_df <- as.data.frame(t(var_val))
    names(var_df) <- varnames

  } else {
    # --- Si hay pesos ---

    # Determinar indices o nombres
    if (is.character(pesos)) {
      if (!(pesos %in% names(x))) stop("El nombre de los pesos no es v\u00e1lido")
      pesos_idx <- match(pesos, names(x))
    } else if (is.numeric(pesos)) {
      if (pesos > ncol(x)) stop("Selecci\u00f3n err\u00f3nea de pesos")
      pesos_idx <- pesos
    } else {
      stop("El argumento 'pesos' debe ser num\u00e9rico o de tipo car\u00e1cter")
    }

    if (length(varnames) > 1) {
      stop("Solo puede calcularse la varianza ponderada para una variable a la vez")
    }

    variable_idx <- match(varnames, names(x))
    x_sel <- x[, c(variable_idx, pesos_idx)]
    names(x_sel) <- c("variable2", "pesos")

    if (!is.numeric(x_sel$variable2) || !is.numeric(x_sel$pesos)) {
      stop("Tanto la variable como los pesos deben ser num\u00e9ricos")
    }

    x_sel <- na.omit(x_sel)

    # Calcular media ponderada
    media_pond <- sum(x_sel$variable2 * x_sel$pesos) / sum(x_sel$pesos)

    # Suma ponderada de cuadrados
    sumatorio <- (x_sel$variable2 - media_pond)^2 * x_sel$pesos

    # Denominador segun tipo
    if (tipo == "muestral") {
      denom <- sum(x_sel$pesos)
    } else {
      denom <- sum(x_sel$pesos) - 1
    }

    var_val <- sum(sumatorio) / denom

    var_df <- data.frame(varianza = var_val)
    names(var_df) <- varnames
  }

  class(var_df) <- c("resumen", class(var_df))
  return(var_df)
}

agrupar <- function(observadas,esperadas){
  
  observadas <- matriz$Freq_obs
  esperadas <- matriz$Freq_esp
  
  df <- data.frame(matrix(ncol = 2, nrow = length(observadas)))
  names(df) <- c("o","t")
  
  df$t <- esperadas
  df$o <- observadas
  
  df2t <- c()
  df2o <- c()
  intermediot <- c()
  intermedioo <- c()
  intermedio2t <- NULL
  intermedio2o <- NULL
  df21t <- 0
  df21o <- 0
  df23t <- 0
  df23o <- 0
  
  
  if(df$t[1]<5){
    i=1
    while(i <= nrow(df)){
      
      nt = df$t[i]
      
      if(nt >= 5){
        
        df2t <- df$t[i]
        df2o <- df$o[i]
        
      } else{
        
        j=1
        
        while((nt < 5)  & ((i+j) <= nrow(df))){ #pq exijo que sea mayor a 5
          
          
          if(sum(df$t[i:(i+j)]) >= 5){ # exijo que sea mayor a 5
            df21t <- sum(df$t[i:(i+j)])
            df21o <- sum(df$o[i:(i+j)])
            
          } 
          
          
          j = j + 1    
          
          if((i+j) > nrow(df)){
            
            df22t <- intermediot[length(intermediot)] + df22t
            intermedio2t <- df22t
            df22o <- intermedioo[length(intermedioo)] + df22o
            intermedio2o <- df22o
            
            break
          }
          
          nt <- sum(df$t[i:(i+j)])
          df22t <- sum(df$t[i:(i+j)])
          df22o <- sum(df$o[i:(i+j)])
          
        }
        
        
        if((df21t >= 5) & (df21t < df22t)){
          df2t <- df21t
          df2o <- df21o
          i <- i + j 
        }else{
          df2t <- df22t
          df2o <- df22o
          i <- i + j + 1
          
        }
        
        j = 0
        
      }
      

      if(length(intermedio2t)!=0){
        
        intermediot[length(intermediot)] <- intermedio2t
        intermedioo[length(intermedioo)] <- intermedio2o
        
      }else{
        
        intermediot <- c(intermediot,df2t)
        intermedioo <- c(intermedioo,df2o)
        
      }
      
      intermedio2t <- c()
      intermedio2o <- c()
      
      
      ult.intermedio <- length(intermediot)
      
      
      # Comprobar si es el penúltimo elemento
      
      if(i == (nrow(df)-1)){
        
        if(df$t[i] < 5){
          
          df23t <- sum(df$t[i:(i+1)])
          df23o <- sum(df$o[i:(i+1)])
          
          i = i + 1
          
        } else{
          
          df2t <- df$t[i]
          intermediot <- c(intermediot,df2t)
          
          
          df2o <- df$o[i]
          intermedioo <- c(intermedioo,df2o)
          
        }
        
      }
      
      # comprobar si es el último elemento del df original
      if(i == nrow(df)){
        
        if(df23t !=0 & df23t < 5){
          
          intermediot[ult.intermedio] <- intermediot[ult.intermedio] + df23t
          intermedioo[ult.intermedio] <- intermedioo[ult.intermedio] + df23o
          
        } else{
          
          if(df$t[i]<5){
            
            intermediot[ult.intermedio] <- intermediot[ult.intermedio] + df$t[i]
            intermedioo[ult.intermedio] <- intermedioo[ult.intermedio] + df$o[i]
            
          } else{
            
            intermediot <- c(intermediot,df$t[i])
            intermedioo <- c(intermedioo,df$o[i]) 
            
          }
          
        }
        
        i = i + 1
      }
      

      if(i > nrow(df)){
        break
      }
      
    }
    
  }else{
    for(i in 1:nrow(df)){
      
      nt = df$t[i]
      
      if(nt >= 5){
        
        df2t <- df$t[i]
        df2o <- df$o[i]
        
      } else{
        
        j=1
        
        while((nt < 5)  & ((i+j) <= nrow(df))){ #pq exijo que sea mayor a 5
          
          
          if(sum(df$t[i:(i+j)]) >= 5){ # exijo que sea mayor a 5
            df21t <- sum(df$t[i:(i+j)])
            df21o <- sum(df$o[i:(i+j)])
            
          } 
          
          
          j = j + 1    
          
          if((i+j) > nrow(df)){
            
            df22t <- intermediot[length(intermediot)] + df22t
            intermedio2t <- df22t
            df22o <- intermedioo[length(intermedioo)] + df22o
            intermedio2o <- df22o
            
            break
          }
          
          nt <- sum(df$t[i:(i+j)])
          df22t <- sum(df$t[i:(i+j)])
          df22o <- sum(df$o[i:(i+j)])
          
        }
        
        
        if((df21t >= 5) & (df21t < df22t)){
          df2t <- df21t
          df2o <- df21o
          i <- i + j 
        }else{
          df2t <- df22t
          df2o <- df22o
          i <- i + j + 1
          
        }
        
        j = 0
        
      }
      

      if(length(intermedio2t)!=0){
        
        intermediot[length(intermediot)] <- intermedio2t
        intermedioo[length(intermedioo)] <- intermedio2o
        
        
      }else{
        
        intermediot <- c(intermediot,df2t)
        intermedioo <- c(intermedioo,df2o)
        
        
      }
      
      intermedio2t <- c()
      intermedio2o <- c()
      
      
      ult.intermedio <- length(intermediot)

      
      # Comprobar si es el penúltimo elemento
      
      if(i == (nrow(df)-1)){
        
        if(df$t[i] < 5){
          
          df23t <- sum(df$t[i:(i+1)])
          df23o <- sum(df$o[i:(i+1)])
          
          i = i + 1
          
        } else{
          
          df2t <- df$t[i]
          intermediot <- c(intermediot,df2t)
          
          
          df2o <- df$o[i]
          intermedioo <- c(intermedioo,df2o)
          
        }
        
      }

      # comprobar si es el último elemento del df original
      if(i == nrow(df)){
        
        if(df23t !=0 & df23t < 5){
          
          intermediot[ult.intermedio] <- intermediot[ult.intermedio] + df23t
          intermedioo[ult.intermedio] <- intermedioo[ult.intermedio] + df23o
          
        } else{
          
          if(df$t[i]<5){
            
            intermediot[ult.intermedio] <- intermediot[ult.intermedio] + df$t[i]
            intermedioo[ult.intermedio] <- intermedioo[ult.intermedio] + df$o[i]
            
          } else{
            
            intermediot <- c(intermediot,df$t[i])
            intermedioo <- c(intermedioo,df$o[i]) 
            
          }
          
        }
        
        i = i + 1
      }
      
      if(i > nrow(df)){
        break
      }
      
    }
    
  }
  
  
  
  
  frecuencias <- data.frame(Freq_obs= intermedioo ,
                            Freq_esp = intermediot )
  
  return(frecuencias)
  
}

#' @title Medidas de forma
#'
#' @description Calcula el coeficiente de asimetría y de curtosis de Fisher.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrforma.png}{width = 200px}}
#' \if{latex}{\figure{qrforma.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos, que puede estar formado por una o más variables.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#' @param alternativa Es un valor lógico. Si alternativa = TRUE el resultado de las medidas de forma muestra el coeficiente de asimetría y curtosis calculado según SPSS y EXCEL. Se facilita también los correspondientes errores típicos. Este argumento no funciona si pesos = NULL.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (exportar = TRUE).
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' El coeficiente de asimetría se obtiene a partir de la expresión:
#'
#' \if{html}{\figure{asimetriamuestra.png}{width = 160px}}
#' \if{latex}{\figure{asimetriamuestra.png}{options: width=3cm}}
#'
#' y el coeficiente de curtosis:
#'
#' \if{html}{\figure{curtosismuestra.png}{width = 280px}}
#' \if{latex}{\figure{curtosismuestra.png}{options: width=4cm}}
#'
#' @note
#' (1) El coeficiente de asimetría poblacional es:
#'
#' \if{html}{\figure{asimetriapob.png}{width = 160px}}
#' \if{latex}{\figure{asimetriapob.png}{options: width=3cm}}
#'
#' (2) El coeficiente de curtosis poblacional es:
#'
#' \if{html}{\figure{curtosispob.png}{width = 2800px}}
#' \if{latex}{\figure{curtosispob.png}{options: width=4cm}}
#'
#' (3) Si el argumento alternativa = TRUE, se obtienen los resultados de asimetría y curtosis que generalmente ofrecen softwares como: SPSS, Stata, SAS, Excel, etc.
#'
#'
#' \if{html}{\figure{asimetriasoft.png}{width = 4800px}}
#' \if{latex}{\figure{asimetriasoft.png}{options: width=8cm}}
#'
#'  \if{html}{\figure{curtosissoft.png}{width = 920px}}
#' \if{latex}{\figure{curtosissoft.png}{options: width=13cm}}
#'
#' @seealso \code{\link{varianza}},\code{\link{desviacion}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @importFrom stats complete.cases
#' @examples
#'
#' forma <- medidas.forma(startup)
#' forma2 <- medidas.forma(startup, alternativa= TRUE)
#'
#' @export
medidas.forma <- function(x,
                          variable = NULL,
                          pesos = NULL,
                          alternativa = FALSE,
                          exportar = FALSE){

  if(is.numeric(x)){
    varnames <- "variable.x"
  }else{
    varnames <- as.character(names(x))
  }

  x <- data.frame(x)
  names(x) <- varnames

  if(is.null(variable)){

    varcuan <-  names(x)[which(sapply(x[varnames], is.numeric))]
    #seleccion = match(varcuan,varnames)
    x <- x[varcuan]
    varnames <- varcuan

  } else{

    if(is.numeric(variable)){

      if(all(variable <= length(x))){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variables")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){
        variable = match(variable,varnames)
      } else {
        stop("El nombre de la variable no es v\u00e1lido")
      }
    }

  }


  if(is.null(pesos) & !is.null(variable)){

    x <- x[,variable] %>% as.data.frame()
    varnames <- varnames[variable]
    names(x) <- varnames

  }

  if(!is.null(pesos) & !is.null(variable)){

    if((length(variable) | length(pesos)) > 1){

      stop("Para calcular la media a partir de la distribuci\u00fn de frecuencias solo puedes seleccionar una variable y unos pesos")

    }

    if(is.numeric(pesos)){

      pesos <- pesos

    }


    if(is.character(pesos)){

      if(pesos %in% varnames){
        pesos = match(pesos,varnames)
      } else {
        stop("El nombre de los pesos no es v\u00e1lido")
      }
    }

    if(pesos == variable){

      stop("Has seleccionado la misma columna del dataframe para la variable y los pesos")

    }


    x <- x[,c(variable,pesos)] %>% as.data.frame()
    varnames <- varnames[c(variable,pesos)]
    names(x) <- varnames

  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))) {
    stop("No pueden calcularse las medidas de forma, alguna variable que has seleccionado no es cuantitativa")
  }


  if(is.null(pesos)){

    #N <- nrow(x)
    momento3 <- .momento.central(x,orden = 3)
    momento4 <- .momento.central(x,orden = 4)
    desv.x <- desviacion(x)

    asimetria <- momento3/desv.x^3
    curtosis <- momento4/desv.x^4 - 3

    forma <- bind_rows(asimetria,curtosis) %>%
      as.data.frame()
    row.names(forma) <- c("asimetria","curtosis")


  } else{

    desv.x <- desviacion(x,variable=1,pesos=2)
    forma <-  x %>%
        na.omit %>%
        rename(variable2 = varnames[1], pesos = varnames[2]) %>%
        mutate(media = media(x,variable=1,pesos=2),
                    sumatorio3 = (variable2-media)^3*pesos,
                    sumatorio4 = (variable2-media)^4*pesos) %>%
        summarize(momento3 = sum(sumatorio3)/sum(pesos),
                  momento4 = sum(sumatorio4)/sum(pesos),
                  asimetria = sum(sumatorio3)/(sum(pesos)*desv.x^3),
                  curtosis = sum(sumatorio4)/(sum(pesos)*desv.x^4) - 3)

    N <- sum(x[2])

    asimetria <- as.numeric(forma[3])
    curtosis <- as.numeric(forma[4])
    momento3 <- as.numeric(forma[1])
    momento4 <- as.numeric(forma[2])

    forma <- forma %>%
      select(asimetria,curtosis) %>%
      t() %>%
      as.data.frame()

    names(forma) <- varnames[1]
    row.names(forma) <- c("asimetria","curtosis")

  }
  if (isTRUE(alternativa) & is.null(pesos)) {

    # Convertir a formato largo
    xalt <- x %>%
      tidyr::gather(key = "var_coef", value = "value") %>%
      dplyr::filter(complete.cases(.)) %>%
      dplyr::group_by(var_coef) %>%
      dplyr::summarize(
        N = dplyr::n(),
        c1 = (N * (N + 1)) / ((N - 1) * (N - 2) * (N - 3)),
        c3 = (3 * (N - 1)^2) / ((N - 2) * (N - 3)),
        error_asimetria = sqrt((6 * N * (N - 1)) / ((N - 2) * (N + 1) * (N + 3))),
        error_curtosis = 2 * sqrt((6 * N * (N - 1)) / ((N - 2) * (N + 1) * (N + 3))) *
          sqrt((N^2 - 1) / ((N - 3) * (N + 5)))
      ) %>%
      dplyr::ungroup()

    # Calcular los momentos y desviaciones
    momento3_df <- .momento.central(x, orden = 3)
    momento4_df <- .momento.central(x, orden = 4)
    desv_df <- desviacion(x, tipo = "cuasi")

    # Asegurar que los resultados son vectores numericos
    momento3_vec <- if (is.data.frame(momento3_df)) as.numeric(momento3_df[1, ]) else as.numeric(momento3_df)
    momento4_vec <- if (is.data.frame(momento4_df)) as.numeric(momento4_df[1, ]) else as.numeric(momento4_df)
    desv_vec     <- if (is.data.frame(desv_df)) as.numeric(desv_df[1, ]) else as.numeric(desv_df)

    # Anadir calculos alternativos
    xalt <- xalt %>%
      dplyr::mutate(
        desv.x.muestra = desv_vec,
        c2 = (N * momento4_vec) / desv.x.muestra^4,
        curtosis_soft = (c1 * c2) - c3,
        A1 = N / ((N - 1) * (N - 2)),
        A2 = (N * momento3_vec) / desv.x.muestra^3,
        asimetria_soft = A1 * A2,
        asimetria = as.numeric(asimetria[1, ]),
        curtosis = as.numeric(curtosis[1, ])
      )

    # Seleccionar columnas finales
    forma <- xalt %>%
      dplyr::select(
        N,
        asimetria,
        curtosis,
        asimetria2 = asimetria_soft,
        error_asimetria2 = error_asimetria,
        curtosis2 = curtosis_soft,
        error_curtosis2 = error_curtosis
      )

    # Convertir a formato "medidas en filas / variables en columnas"
    forma_t <- as.data.frame(t(forma))
    colnames(forma_t) <- varnames
    rownames(forma_t) <- c(
      "N",
      "Asimetr\u00eda (muestral)",
      "Curtosis (muestral)",
      "Asimetr\u00eda (alternativa)",
      "Error asimetr\u00eda (alt)",
      "Curtosis (alternativa)",
      "Error curtosis (alt)"
    )

    forma <- forma_t

  }


  # Exportar
  if (exportar) {

    filename <- paste0("Medidas_de_forma_", format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), ".xlsx")

    wb <- openxlsx::createWorkbook()
    openxlsx::addWorksheet(wb, "Medidas_de_forma")

    # nombres de fila a columna
    resumen_export <- cbind(forma = row.names(forma), forma)
    row.names(resumen_export) <- NULL

    openxlsx::writeData(wb, "Medidas_de_forma", resumen_export)

    # formato numerico decimal en Excel
    addStyle(wb, "Medidas_de_forma",
             style = createStyle(numFmt = "0.0000"),
             rows = 2:(nrow(resumen_export)+1),
             cols = 2:(ncol(resumen_export)+1),
             gridExpand = TRUE)

    saveWorkbook(wb, filename, overwrite = TRUE)
  }

  class(forma) <- c("resumen", class(forma))


  return(forma)

}

#' Data: Encuesta cuatrienal de estructura salarial (2018)
#'
#' Datos del Instituto Nacional de Estadística. Los microdatos originales constan de 216,726 observaciones. A efectos de practicar, se han seleccionado aleatoriamente 1000 observaciones y 7 variables de interés.
#' @usage data("salarios2018")
#' @format Dataframe con 1000 observaciones 7 7 variables.
#' \describe{
#'   \item{SEXO}{Sexo (1=hombre, 6=mujer)}
#'   \item{ESTUDIOS}{Nivel de estudios. 1=Menos que primaria,2=Primaria,3=Primera etapa secundaria,4=Segunda etapa secundaria,5=FP superior o similar,6=Diplomado o similar,7=Licenciados o similares y doctores}
#'   \item{TIPO.JORNADA}{Tipo de jornada laboral. 1=Tiempo completo,2=Tiempo parcial}
#'   \item{TIPO.CONTRATO}{Tipo de contrato laboral. 1=Indefinido,2=Duración determinada}
#'   \item{SALARIO.BRUTO.ANUAL}{Salario bruto anual}
#'   \item{SALARIO.ORDINARIO.ANUAL}{Salario ordinario anual}
#'   \item{FACTOR.ELEVACION}{Factor de elevación}
#' }
#' @source Instituto Nacional de Estadística \url{https://www.ine.es/}
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
"salarios2018"

#' Data: Datos de empresas emergentes (startups)
#'
#' Datos simulados. Muestra de 21 empresas emergentes
#' @usage data("startup")
#' @format Dataframe con 21 observaciones de 4 variables.
#' \describe{
#'   \item{gasto.desarrollo}{Gastos de investigación y desarrollo, en euros.}
#'   \item{gasto.marketing}{Gastos de marketing, en euros.}
#'   \item{gasto.gestion}{Gastos de administración, en euros.}
#'   \item{beneficio}{Beneficios, en euros.}
#' }
#' @source Muestra simulada.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
"startup"

#' Data: Ejemplo de dos variables (ejem_bidi)
#'
#' Datos simulados. Muestra de 100 observaciones
#' @usage data("ejem_bidi")
#' @format Dataframe con 100 observaciones de 2 variables.
#' \describe{
#'   \item{x}{Toma valores de 0 a 5.}
#'   \item{x}{Toma valores de 10 a 15}
#' }
#' @source Muestra simulada.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
"ejem_bidi"

#' Datos simulados de dos muestras tomadas en periodos de tiempo distintos. La muestra 1 es tomada en enero y la muestra 2 en junio.
#' @usage data("diseno1")
#' @format Dataframe en formato ancho con 620 observaciones. La pregunta realizada es: ¿Sabe que Valencia es la capital mundial del diseño 2022?
#' \describe{
#'   \item{muestra1}{0: No sabe, 1: Sí Sabe}
#'   \item{muestra2}{0: No sabe, 1: Sí sabe}
#' }
#' @source Muestra simulada.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
"diseno1"


#' Datos simulados de dos muestras tomadas en periodos de tiempo distintos. La muestra 1 es tomada en enero y la muestra 2 en junio.
#' @usage data("diseno2")
#' @format Dataframe en formato largo con 1085 observaciones. La pregunta realizada es: ¿Sabe que Valencia es la capital mundial del diseño 2022?
#' \describe{
#'   \item{muestra}{Toma dos valores: Muestra1 y Muestra2}
#'   \item{resultado}{0: No sabe, 1: Sí sabe}
#' }
#' @source Muestra simulada.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
"diseno2"

#' Data: Turistas internacionales Comunidad Valenciana
#'
#' @usage data("turistas")
#' @format Dataframe con 80 observaciones de 2 variables.
#' \describe{
#'   \item{perido}{Periodo temporal.}
#'   \item{Turistas.internacionales}{Número de turistas con destino principal la Comunidad Valenciana}
#' }
#' @source Movimientos turísticos en fronteras. Frontur. Instituto de Estudios Turísticos (hasta septiembre de 2015) e INE (a partir de octubre de 2015)
#'
"turistas2"

#' Data: Turistas por paises (WTO)
#'
#' Datos de World Tourism Organization.
#' @usage data("turistas")
#' @format Dataframe con 130 observaciones de 3 variables.
#' \describe{
#'   \item{País}{País de destino.}
#'   \item{Llegadas.turistas}{Número de llegada de turistas en 2017, en miles.}
#'   \item{Gasto.viajes}{Gasto en viajes en 2017, en millones de USD.}
#' }
#' @source World Tourism Organization (2019).
#'
"turistas"

#' Data: Hogares
#'
#' Datos de 10 hogares que se utilizan en los ejemplos de (1) tabla bidimensional, (2) covarianza, (3) matriz de covarianzas, (4) correlación y (5) matriz de correlación.
#' @usage data("hogares")
#' @format Dataframe con 10 observaciones de 3 variables.
#' \describe{
#'   \item{Hogares}{Identificación del hogar.}
#'   \item{ingresos}{Ingresos del hogar}
#'   \item{viajes}{Número de hogares realizado por los hogares.}
#' }
#'
"hogares"

#' Data: Viajes vendidos
#'
#' Datos de 5 observaciones que se utilizan en los ejemplos de (1) media, mediana y moda, (2) cuantiles, (3) varianza, desviación típica y coeficiente de variación, (4) medidas de forma y momento central y (5) resumen de descriptivos
#'
#' @usage data("viajes_vendidos")
#' @format Dataframe con 5 observaciones de 3 variables.
#' \describe{
#'   \item{Número.de.viajes.vendidos}{Número de viajes perdidos.}
#'   \item{Empleados}{Número de empleados}
#'   \item{Ni}{Frecuencia absoluta acumulada del número de empleados}
#' }
#'
"viajes_vendidos"

#' @title Mediana.
#'
#' @description Calcula la mediana.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrposicion.png}{width = 200px}}
#' \if{latex}{\figure{qrposicion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#'
#' @return Si \code{pesos = NULL}, devuelve la mediana de todas la variables seleccionadas en un \code{vector}. En caso contrario, devuelve únicamente la mediana de la variable para la que se ha facilitado la distribución de frecuencias.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' La mediana se obtiene a partir de la siguiente regla de decisión:
#'
#' \if{html}{\figure{mediana.png}{width = 640px}}
#' \if{latex}{\figure{mediana.png}{options: scale=.8}}
#'
#' donde: Ni son las frecuencias acumuladas y n el tamaño de la muestra (o N si es la población).
#'
#' @seealso \code{\link{media}}, \code{\link{cuantiles}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' mediana1 <- mediana(startup[1])
#' mediana2 <- mediana(startup,variable=1)
#' mediana3 <- mediana(salarios2018,variable=6,pesos=7)
#'
#' @import dplyr
#'
#' @export
mediana <- function(x, variable = NULL, pesos = NULL) {

  var_name <- deparse(substitute(x))

  # Convertir a data.frame
  if (!is.data.frame(x)) x <- as.data.frame(x)

  # --- Selecci\u00f3n de variables ---
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)]
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    varnames <- names(x)[variable]
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("Nombre de variable no v\u00e1lido")
    varnames <- variable
  } else {
    stop("El argumento 'variable' debe ser num\u00e9rico o de tipo car\u00e1cter")
  }

  # Subconjunto de variables seleccionadas
  x_sel <- x[, varnames, drop = FALSE]

  # --- Manejo de pesos ---
  if (!is.null(pesos)) {
    if (length(varnames) > 1)
      stop("Solo puedes seleccionar una variable con pesos")

    # Buscar columna de pesos en el data.frame original, no en x_sel
    if (is.character(pesos)) {
      if (!pesos %in% names(x)) stop("Nombre de pesos no v\u00e1lido")
      pesos_col <- pesos
    } else if (is.numeric(pesos)) {
      if (pesos > ncol(x)) stop("Selecci\u00f3n de pesos no v\u00e1lida")
      pesos_col <- names(x)[pesos]
    } else {
      stop("El argumento 'pesos' debe ser num\u00e9rico o de tipo car\u00e1cter")
    }

    datos <- na.omit(data.frame(variable = x[[varnames]], pesos = x[[pesos_col]]))
    if (nrow(datos) == 0) return(data.frame(matrix(NA, ncol = 1, dimnames = list(NULL, varnames))))

    result <- .mediana.int(datos$variable, datos$pesos)
    result_df <- data.frame(result)
    colnames(result_df) <- varnames

    class(result_df) <- c("resumen", class(result_df))
    return(round(result_df, 4))
  }

  # --- Mediana simple ---
  result <- sapply(x_sel, .mediana.int)
  result_df <- as.data.frame(t(result))  # convertimos a data.frame con columnas como variables
  colnames(result_df) <- varnames
  rownames(result_df) <- NULL

  class(result_df) <- c("resumen", class(result_df))
  return(result_df)
}

#' @title Intervalo confianza para la media.
#'
#' @description Calcula el intervalo de confianza de la media poblacional.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qricmedia.png}{width = 200px}}
#' \if{latex}{\figure{qricmedia.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param poblacion Es un carácter. Indica la distribución de probabilidad de la población. Por defecto \code{poblacion = "normal"}. Si la distribución de la población es desconocida, cambiar a \code{poblacion = "desconocida"}.
#' @param var_pob Es un carácter. Indica si la varianza poblacional es conocida (por defecto, \code{var_pob = "conocida"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return Devuelve el intervalo de confianza de la media poblacional en un objeto de tipo \code{data.frame}. Si \code{grafico = T} devuelve una \code{list} con el intervalo de confianza y su representación gráfica.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) Si población desconocida, varianza poblacial conocida y muestra pequeña:
#'
#' \if{html}{\figure{icmediadesconocidavarcon.png}{width = 400px}}
#' \if{latex}{\figure{icmediadesconocidavarcon.png}{options: width=10cm}}
#'
#' (2) Si población normal, varianza poblacional conocida (muestra pequeña y grande)
#'
#' \if{html}{\figure{icmediavarcon.png}{width = 440px}}
#' \if{latex}{\figure{icmediavarcon.png}{options: width=10cm}}
#'
#' (3) Si población normal, varianza poblacional desconocida y muestra pequeña
#'
#' Con la varianza muestral:
#'
#' \if{html}{\figure{icmediavardescmuestral.png}{width = 400px}}
#' \if{latex}{\figure{icmediavardescmuestral.png}{options: width=10cm}}
#'
#' Con la cuasivarianza muestral:
#'
#' \if{html}{\figure{icmediavardesccuasi.png}{width = 400px}}
#' \if{latex}{\figure{icmediavardesccuasi.png}{options: width=10cm}}

#'
#' Nota: En ambos casos, el valor crítico sigue una distribución t con n-1 grados de libertad
#'
#' (4) Si población normal, varianza poblacional desconocida y muestra grande:
#' Puede utilizarse la aproximación a la normal. El intervalo se obtiene a partir de la expresión (2)
#' estimando la varianza poblacional por la varianza (o cuasivarianza) muestral.
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm qnorm pt qt na.omit dt
#' @import dplyr ggplot2 cowplot
#'
#' @export
ic.media <- function(x,
                     variable = NULL,
                     introducir = FALSE,
                     poblacion = c("normal","desconocida"),
                     var_pob = c("conocida","desconocida"),
                     confianza = 0.95,
                     grafico = FALSE){

poblacion <- tolower(poblacion)
poblacion <- match.arg(poblacion)

var_pob <- tolower(var_pob)
var_pob <- match.arg(var_pob)

if(confianza >= 0 & confianza <=1){

  confianza <- confianza
  alfa2 <- (1- confianza)/2

} else{

  stop("El nivel de confianza debe fijarse entre 0 y 1")

}


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 1){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variables.")

    }

  } else if(length(variable) == 1){

    if(is.numeric(variable)){

      if(variable <= length(x)){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es v\u00e1lido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
    stop("El conjunto de datos seleccionado tiene mas de 1 variables.")

  }

  x <- na.omit(x)
  clase <- sapply(x, class)

  if (!clase %in% c("numeric","integer")) {

    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")

  }

  # tama\u00f1o de la muestra
  n <- nrow(x)

  # media muestral
  media <- as.numeric(media(x))

  if(var_pob == "conocida"){

    varianza_pob <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
    varianza_pob <- as.numeric(varianza_pob)
    desv_pob <- sqrt(varianza_pob)

  } else{   # varianza poblacional desconocida

      var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

      if(var_muestra == 1){

        desv_mu = as.numeric(desviacion(x))

      } else{

        desv_mu <- as.numeric(desviacion(x,tipo="cuasi"))
      }

  }


} else{   # aqu\u00ed empieza introducir datos

  n <- readline(prompt = "Introducir el tama\u00f1o de la muestra: ")
  n <- as.numeric(n)

  # caso 1. No puede hacerse

  if(poblacion == "desconocida" & var_pob == "desconocida" & n < 30){

    print("La distribuci\u00f3n de probabilidad de la poblaci\u00f3n y su varianza son desconocidas. Adem\u00e1s, el tama\u00f1o de la muestra es peque\u00f1o (n<30)")
    stop("Bajo estas condiciones no es posible estimar el intervalo de confianza")

  }

  media <- readline(prompt = "Introducir el valor de la media muestral: ")
  media <- as.numeric(media)

  if(var_pob == "conocida"){
    varianza_pob <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
    varianza_pob <- as.numeric(varianza_pob)
    desv_pob <- sqrt(varianza_pob)

  } else{ # varianza poblacional desconocida

    var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      varianza_muestral <- readline(prompt = "Introducir el valor de la varianza muestral: ")
      varianza_muestral <- as.numeric(varianza_muestral)
      desv_mu <- sqrt(varianza_muestral)

    } else{

      varianza_cuasi <- readline(prompt = "Introducir el valor de la cuasi-varianza muestral: ")
      varianza_cuasi <- as.numeric(varianza_cuasi)
      desv_mu <- sqrt(varianza_cuasi)
    }
  }
}

# calculo intervalos

  # caso 2 poblacion desconocida y varianza poblacional desconocida

  if(poblacion == "desconocida" & var_pob == "desconocida" & n >= 30){

    print("La distribuci\u00f3n de probabilidad de la poblaci\u00f3n y su varianza son desconocidaS. Sin embargo, el tama\u00f1o de la muestra es grande (n>30)")
    print("En esta situaci\u00f3n, la distribuci\u00f3in de la media muestral puede considerarse normal como consecuencia del TCL.")

    valor_critico <- qnorm(alfa2,lower.tail = FALSE)
    error_tipico <- desv_mu / sqrt(n)

  }

  # casos 3, 4: poblacion desconocida y varianza poblacional conocida

  if(poblacion == "desconocida" & var_pob == "conocida"){

    if(n < 30){

      print("Intervalo de confianza para la media poblacional supuesta conocida la varianza poblacional. n peque\u00f1a (n<30)")
      print("El intervalo de confianza ser\u00e1 obtenido a partir de Tchebychev")

      valor_critico <- 1
      error_tipico <- desv_pob / sqrt(n * alfa2 * 2)

    } else{

      print("Intervalo de confianza para la media poblacional supuesta conocida la varianza poblacional. n grande (n>30)")
      print("Como n es grande la distribuci\u00f3n de la media muestral puede considerarse normal como consecuencia del TCL.")

      valor_critico <- qnorm(alfa2,lower.tail = FALSE)
      error_tipico <- desv_pob / sqrt(n)

    }
  }


  if(poblacion == "normal"){

    # caso 5. Poblacion normal y varianza conocida

    if(var_pob == "conocida"){

      valor_critico <- qnorm(alfa2,lower.tail = FALSE)
      error_tipico <- desv_pob / sqrt(n)

    }

    if(var_pob == "desconocida"){

      if(n < 30){  # con la t-student

        if(var_muestra == 1){ # caso 6_1. normal con varianza desconocida muestra peque\uu00f1a (cuasivar muestral)
          valor_critico <- qt(alfa2,n-1,lower.tail = FALSE)
          error_tipico <- desv_mu /sqrt(n-1)

        } else{  # caso 6_1. normal con varianza desconocida muestra peque\u00f1a (var_muestral)
          print("Este es el intervalo que generalmente calculan los softwares (SPSS, Excel, etc.)")

          valor_critico <- qt(alfa2,n-1,lower.tail = FALSE)
          error_tipico <- desv_mu /sqrt(n)

        }

      }

      if(n >= 30){

        print("El tama\u00f1o de la muestra es grande (n>30). Se puede estimar la varianza poblacional y considerar el intervalo de varianza poblacional conocida.")
        aproximacion <- as.numeric(readline('\u00bfQuieres utilizar la aproximaci\u00f3n de la t a la normal? \n 1. "S\u00ed" \n 2. "No" \n'))

        if(aproximacion == 1){  # aproximacion normal. Se estima sigma por S
          # caso 7. Se estima sigma por S y se estima el IC del caso 5

          valor_critico <- qnorm(alfa2,lower.tail = FALSE)
          error_tipico <- desv_mu / sqrt(n)

        } else{  # con la t-student

          if(var_muestra == 1){ # caso 8_1 muestra grande, var muestral y t

            valor_critico <- qt(alfa2,n-1,lower.tail = FALSE)
            error_tipico <- desv_mu /sqrt(n-1)

          } else{  # caso 8_2 muestra grande, cuasi muestral y t

            valor_critico <- qt(alfa2,n-1,lower.tail = FALSE)
            error_tipico <- desv_mu /sqrt(n)

          }
        }
      }

    }

  }

  limite_inferior <- media - valor_critico * error_tipico
  limite_superior <- media + valor_critico * error_tipico

  if(isTRUE(grafico)){

    if(n>=30){
      tamano <- "grande"
    } else{
      tamano <- "peque\u00f1a"
    }


  if(poblacion == "normal" & var_pob == "desconocida" & n<30){

    plot11 <- ggplot(NULL, aes(c(-4,4))) +
      geom_area(stat = "function", fun = dt, args = list(df = n-1), fill = "darkgreen", xlim = c(-valor_critico, valor_critico)) +
      geom_area(stat = "function", fun = dt, args = list(df = n-1), fill = "white", xlim = c(-valor_critico - 1, -valor_critico)) +
      geom_area(stat = "function", fun = dt, args = list(df = n-1), fill = "white", xlim = c(valor_critico, valor_critico + 1)) +
      labs(title = paste("Distribuci\u00f3n t con ", n-1, " grados de libertad",sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = c(-valor_critico,valor_critico)) +
      theme(axis.text.x = element_text(angle = 45)) +
      geom_point(aes(x= -valor_critico , y=0), color = "red", size = 3) +
      geom_point(aes(x= valor_critico , y=0), color = "blue", size = 3)


    intervalo <- data.frame(ic = round(c(inferior=limite_inferior,media,superior=limite_superior),4),y=c(0,0,0))

    plot12 <- ggplot(intervalo,aes(x= ic,y)) +
      geom_line(aes(group = y), color = "grey",size = 3)+
      geom_point(aes(color=ic), size=3,show.legend = FALSE) +
      geom_text(aes(label = ic), size = 2.5, vjust=2) +
      scale_y_continuous(expand=c(0,0)) +
      scale_color_gradientn(colours=c("red","darkgreen","blue"))+
      labs(y="",x="Intervalo de confianza") +
      tema_blanco



    # plot <- grid::grid.draw(rbind(ggplotGrob(plot11), ggplotGrob(plot12), size = "first"))
    plot <- cowplot::plot_grid(plot11, plot12, ncol = 1)

  } else if(poblacion == "desconocida" & var_pob == "conocida" & n<30) {

    plot <- print("Este intervalo de confianza no tiene representaci\u00f3n gr\u00e1fica")

  } else{

    seq <- seq(-4,4,length=1000) * error_tipico + media
    seq <- as.data.frame(seq)

    plot <- ggplot(seq, aes(seq)) +
      stat_function(fun = dnorm, args = list(mean = media, sd = error_tipico)) +
      geom_area(stat = "function", fun = dnorm, args = list(mean = media, sd = error_tipico), fill = "darkgreen", xlim = c(limite_inferior,limite_superior)) +
      labs(x = "", y = "",title = paste("Intervalo de confianza de la media\n(poblaci\u00f3n",poblacion,",varianza poblacional",var_pob,",n",tamano,")\n(NC=",confianza*100,"%)")) +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = c(limite_inferior,media,limite_superior)) +
      theme(axis.text.x = element_text(angle = 45))
  }

  }

  IC <- cbind(limite_inferior,limite_superior)
  IC <- as.data.frame(IC)
  row.names(IC) <- NULL

  if(isTRUE(grafico)){

    return(list(IC,plot))

  } else{

    return(IC)

  }

}

#' @title Resumen descriptivos.
#'
#' @description Calcula un resumen de los principales estadísticos descriptivos.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrdescriptivos.png}{width = 200px}}
#' \if{latex}{\figure{qrdescriptivos.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return Esta función devuelve los principales estadísticos descriptivos muestrales en un objeto de tipo \code{data.frame}. Los descriptivos que se obtienen son: media, mínimo, cuartil 1, mediana, cuartil 3, máximo, varianza muestral, desviación típica muestral, coeficiente de variación, recorrido inter-cuartílico, asimetría, curtosis y moda.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' descriptivos <- resumen.descriptivos(startup)
#'
#' @import dplyr openxlsx
#'
#' @export
resumen.descriptivos <- function(x, variable = NULL, pesos = NULL, exportar = FALSE) {
  # extrae nombre del objeto pasado
  get_name_from_expr <- function(expr) {
    if (is.name(expr)) return(as.character(expr))
    if (is.character(expr)) return(expr)
    if (is.call(expr)) {
      fn <- as.character(expr[[1]])
      if (fn %in% c("$", "[[") && length(expr) >= 3) {
        el <- expr[[3]]
        if (is.name(el) || is.character(el)) return(as.character(el))
        return(deparse(el)[1])
      }
      if (fn == "[") {
        for (k in seq_along(expr)[-1]) {
          el <- expr[[k]]
          if (is.character(el)) return(as.character(el))
          if (is.name(el)) return(as.character(el))
        }
      }
      if (fn == "(" && length(expr) >= 2) {
        return(get_name_from_expr(expr[[2]]))
      }
    }
    txt <- paste(deparse(expr), collapse = "")
    txt <- trimws(txt)
    m <- regmatches(txt, regexpr("([A-Za-z0-9_.]+)\\s*$", txt))
    if (length(m) >= 1 && nzchar(m[1])) return(m[1])
    return(txt)
  }

  # Capturar expresion de x para nombrar correctamente si es vector
  expr_x <- substitute(x)
  nombre_x <- get_name_from_expr(expr_x)

  # Si no es data.frame, convertirlo
  if (!is.data.frame(x)) {
    x <- as.data.frame(x)
    if (ncol(x) == 1) {
      current_name <- names(x)[1]
      if (is.null(current_name) || current_name %in% c("", "x", "X", "V1")) {
        names(x)[1] <- nombre_x
      }
    }
  }

  # Seleccion de variables
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)]
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("El nombre de variable no es v\u00e1lido")
    varnames <- variable
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    varnames <- names(x)[variable]
  } else stop("El argumento 'variable' debe ser num\u00e9rico o car\u00e1cter")

  x_sel <- x[, varnames, drop = FALSE]

  # --- Calculos basicos ---
  if (!is.null(pesos)) {
    # Si hay pesos, se pasa el data frame completo a las funciones
    valor_media      <- media(x, variable = varnames, pesos = pesos)
    valor_varianza   <- varianza(x, variable = varnames, pesos = pesos)
    valor_desviacion <- desviacion(x, variable = varnames, pesos = pesos)
    valor_coef       <- coeficiente.variacion(x, variable = varnames, pesos = pesos)
    valor_forma      <- medidas.forma(x, variable = varnames, pesos = pesos)
    valor_moda       <- suppressWarnings(moda(x, variable = varnames, pesos = pesos))
    cuantiles_todos  <- cuantiles(x, variable = varnames, pesos = pesos, cortes = c(0, 0.25, 0.5, 0.75, 1))
  } else {
    # Si no hay pesos
    valor_media      <- media(x_sel)
    valor_varianza   <- varianza(x_sel)
    valor_desviacion <- desviacion(x_sel)
    valor_coef       <- coeficiente.variacion(x_sel)
    valor_forma      <- medidas.forma(x_sel)
    valor_moda       <- suppressWarnings(moda(x_sel))
    cuantiles_todos  <- cuantiles(x_sel, cortes = c(0, 0.25, 0.5, 0.75, 1))
  }

  ric <- cuantiles_todos[4, , drop = FALSE] - cuantiles_todos[2, , drop = FALSE]  # Q3 - Q1

  # --- Convertir a data.frame si no lo son ---
  valor_media      <- as.data.frame(valor_media)
  valor_varianza   <- as.data.frame(valor_varianza)
  valor_desviacion <- as.data.frame(valor_desviacion)
  valor_coef       <- as.data.frame(valor_coef)
  valor_forma      <- as.data.frame(valor_forma)
  valor_moda       <- as.data.frame(valor_moda)
  cuantiles_todos  <- as.data.frame(cuantiles_todos)
  ric              <- as.data.frame(ric)

  # --- Ajustar nombres de fila ---
  rownames(valor_media)      <- "media"
  rownames(valor_varianza)   <- "varianza"
  rownames(valor_desviacion) <- "desviacion"
  rownames(valor_coef)       <- "coef_variacion"
  rownames(cuantiles_todos)  <- c("minimo", "cuartil1", "mediana", "cuartil3", "maximo")
  rownames(ric)              <- "RIC"

  # --- Ajustar filas de moda ---
  if (nrow(valor_moda) < 1) valor_moda[1, ] <- NA
  rownames(valor_moda) <- paste0("moda_", seq_len(nrow(valor_moda)))

  # --- Normalizar nombres de columnas ---
  colnames(valor_media)      <- varnames
  colnames(valor_varianza)   <- varnames
  colnames(valor_desviacion) <- varnames
  colnames(valor_coef)       <- varnames
  colnames(cuantiles_todos)  <- varnames
  colnames(ric)              <- varnames
  colnames(valor_forma)      <- varnames
  colnames(valor_moda)       <- varnames

  # --- Ensamblar resumen final ---
  resumen <- rbind(
    valor_media, cuantiles_todos, ric, valor_varianza, valor_desviacion, valor_coef,
    valor_forma, valor_moda
  )

  resumen <- round(resumen, 4)

  # Exportar
  if (exportar) {
    filename <- paste0("Descriptivos_", format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), ".xlsx")
    wb <- openxlsx::createWorkbook()
    openxlsx::addWorksheet(wb, "Descriptivos")
    resumen_export <- cbind('Estadistico' = rownames(resumen), resumen)
    rownames(resumen_export) <- NULL
    openxlsx::writeData(wb, "Descriptivos", resumen_export)
    openxlsx::addStyle(
      wb, "Descriptivos",
      style = openxlsx::createStyle(numFmt = "0.0000"),
      rows = 2:(nrow(resumen_export) + 1),
      cols = 2:(ncol(resumen_export) + 1),
      gridExpand = TRUE
    )
    openxlsx::saveWorkbook(wb, filename, overwrite = TRUE)
  }

  class(resumen) <- c("resumen", class(resumen))
  return(resumen)
}

#' @title Moda.
#'
#' @description Calcula la moda.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrposicion.png}{width = 200px}}
#' \if{latex}{\figure{qrposicion.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#'
#' @return Si \code{pesos = NULL}, devuelve la moda de todas la variables seleccionadas en un \code{data.frame}. En caso contrario, devuelve únicamente la moda de la variable para la que se ha facilitado la distribución de frecuencias.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @import dplyr
#'
#' @export
moda <- function(x, variable = NULL, pesos = NULL) {

  # Detectar si x es vector o data.frame
  if (is.numeric(x) || is.factor(x) || is.integer(x) || is.logical(x)) {
    varnames <- "variable.x"
  } else {
    varnames <- as.character(names(x))
  }

  x <- data.frame(x)
  names(x) <- varnames

  # Seleccion de variables
  if (is.null(variable)) {

    varcuan <- x %>%
      dplyr::select_if(function(col)
        is.numeric(col) | is.integer(col) | is.factor(col) | is.logical(col)
      ) %>%
      names()

    seleccion <- match(varcuan, varnames)
    x <- x[seleccion]
    varnames <- varcuan

  } else {

    if (is.numeric(variable)) {

      if (all(variable <= length(x))) {
        variable <- variable
      } else {
        stop("Selecci\u00f3n err\u00f3nea de variables")
      }
    }

    if (is.character(variable)) {

      if (all(variable %in% varnames)) {
        variable <- match(variable, varnames)
      } else {
        stop("El nombre de la variable no es v\u00e1lido")
      }
    }
  }

  # Si hay pesos nulos y variable definida
  if (is.null(pesos) & !is.null(variable)) {
    x <- x[, variable, drop = FALSE]
    varnames <- varnames[variable]
  }

  # Si hay pesos definidos
  if (!is.null(pesos) & !is.null(variable)) {

    if ((length(variable) > 1) | (length(pesos) > 1)) {
      stop("Para calcular la moda ponderada solo puedes seleccionar una variable y unos pesos")
    }

    if (is.character(pesos)) {
      if (pesos %in% varnames) {
        pesos <- match(pesos, varnames)
      } else {
        stop("El nombre de los pesos no es v\u00e1lido")
      }
    }

    if (pesos == variable) {
      stop("Has seleccionado la misma columna del dataframe para la variable y los pesos")
    }

    x <- x[, c(variable, pesos), drop = FALSE]
    varnames <- varnames[c(variable, pesos)]
  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric", "integer", "factor", "logical"))) {
    stop("No se puede calcular la moda: comprueba el tipo de variable seleccionada")
  }

  # --- Moda sin pesos ---
  if (is.null(pesos)) {

    moda_vacio <- vector("list", length = length(x))

    for (i in seq_along(x)) {

      x_moda <- na.omit(x[[i]])
      if (length(x_moda) == 0) {
        moda_vacio[[i]] <- NA
        next
      }

      valor_moda <- table(x_moda)
      valor_moda <- names(valor_moda)[valor_moda == max(valor_moda)]
      moda_i <- suppressWarnings(as.numeric(valor_moda))
      if (all(is.na(moda_i))) moda_i <- valor_moda

      if (length(moda_i) == length(unique(x_moda))) {
        warning("Esta variable no tiene moda: todos los valores tienen la misma frecuencia")
        moda_i <- NA
      }

      moda_vacio[[i]] <- moda_i
    }

    max_long <- max(lengths(moda_vacio))
    # Forzar a lista y convertir sin simplificar
    moda <- sapply(moda_vacio, "[", seq_len(max_long), simplify = FALSE)

    # Convertir lista a data.frame columna a columna
    moda <- as.data.frame(moda, stringsAsFactors = FALSE)

    # Asegurar el mismo numero de columnas que varnames
    if (ncol(moda) != length(varnames)) {
      # Rellenar columnas faltantes con NA
      for (faltan in (ncol(moda) + 1):length(varnames)) {
        moda[[varnames[faltan]]] <- NA
      }
    }

    names(moda) <- varnames


  } else {
    # --- Moda con pesos ---
    moda <- x %>%
      na.omit() %>%
      dplyr::rename(variable2 = varnames[1], pesos = varnames[2]) %>%
      dplyr::group_by(variable2) %>%
      dplyr::summarize(frecuencia = sum(pesos), .groups = 'drop') %>%
      dplyr::arrange(desc(frecuencia)) %>%
      as.data.frame()

    valores_distintos <- unique(moda$variable2)

    moda <- moda %>%
      dplyr::summarize(moda = variable2[frecuencia == max(frecuencia)]) %>%
      as.data.frame()

    if (nrow(moda) == length(valores_distintos)) {
      warning("Esta variable no tiene moda: todos los valores tienen la misma frecuencia")
      moda <- NA
    }

    names(moda) <- paste("moda_", varnames[1], sep = "")
  }

  return(moda)
}

#' @title Intervalo confianza para el coeficiente de correlación
#'
#' @description Calcula el intervalo de confianza para el coeficiente de correlación.
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#'
#' @return Devuelve el intervalo de confianza de la correlación lineal en un objeto de tipo \code{data.frame}
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) El intervalo para
#'
#' \if{html}{\figure{iccorrelacion1.png}{width = 160px}}
#' \if{latex}{\figure{iccorrelacion1.png}{options: width=4cm}}
#'
#' (2) es:
#'
#' \if{html}{\figure{iccorrelacion2.png}{width = 6400px}}
#' \if{latex}{\figure{iccorrelacion2.png}{options: width=15cm}}
#'
#' Igualando la expresión en (1) al extremo inferior de (2) y al extremo superior de (2) se obtendrá el intervalo para la correlación.
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm qnorm pt qt na.omit
#' @import dplyr
#'
#' @export
ic.correlacion <- function(x,
                           variable = NULL,
                           introducir = FALSE,
                           confianza = 0.95){

print("Calcula el intervalo de confianza de la correlaci\u00f3n de dos poblaciones con distribuci\u00f3n conjuntamente normal")

if(confianza >= 0 & confianza <=1){

  confianza <- confianza
  alfa_2 <- (1-confianza)/2
  valor_critico <- qnorm(alfa_2,lower.tail = FALSE)

} else{

  stop("El nivel de confianza debe fijarse entre 0 y 1")

}


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 2){
      x <- x
    } else{
      warning("Para calcular el intervalo de confianza hay que seleccionar 2 variables")
      stop("El conjunto de datos seleccionado no tiene la dimensi\u00f3n adecuada")
    }
  } else{

    if(length(variable) == 2){
        if(is.numeric(variable)){
          if(all(variable <= length(x))){
            variable <- variable
          } else{
            stop("Seleccion erronea de variable")
          }
        }
      if(is.character(variable)){
        if(all(variable %in% varnames)){
          variable = match(variable,varnames)
          } else {
            stop("El nombre de la variable no es v\u00e1lido")
          }
        }

      x <- x[,variable] %>% as.data.frame()
      names(x) <- varnames[variable]

      } else{
        warning("Para calcular el intervalo de confianza de la correlaci\u00f3 hay que seleccionar dos variables")
        stop("El conjunto de datos seleccionado parece ser no v\u00e1lido")
      }
  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))){
    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")
  }

  # tama\u00f1o de la muestra
  x <- na.omit(x)
  n <- nrow(x)
  correlacion <- as.numeric(correlacion(x))

} else{   # aqu\u00ed empieza introducir datos

  print("A continuaci\u00f3n, vas a introducir los datos muestrales.")

  n <- readline(prompt = "Introducir el tama\u00f3o de la muestra : ")
  n <- as.numeric(n)

  correlacion <- readline(prompt = "Introducir el valor del coeficiente de correlaci\u00f3n muestral: ")
  correlacion <- as.numeric(correlacion)

  if((correlacion < -1 | correlacion > 1)){

    stop("El coeficiente de correlaci\u00f3n debe estar comprendido entre -1 y 1.")

  }


}

  limite_inferior <- 0.5 * log((1+correlacion)/(1-correlacion)) - valor_critico * sqrt(1/(n - 3))
  limite_superior <- 0.5 * log((1+correlacion)/(1-correlacion)) + valor_critico * sqrt(1/(n - 3))

  limite_inferior <- (exp(2*limite_inferior)-1)/(1+exp(2*limite_inferior))
  limite_superior <- (exp(2*limite_superior)-1)/(1+exp(2*limite_superior))


  IC <- cbind(limite_inferior,limite_superior)
  IC <- as.data.frame(IC)
  row.names(IC) <- NULL

  return(IC)

}

#' @title Series temporales.
#'
#' @description Esta función utiliza el método de las medias móviles (centradas) para extraer la tendencia de una serie temporal.
#' A partir de las medias móviles, también se obtienen los índices de variación estacional (IVE).
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrseriestemporales.png}{width = 200px}}
#' \if{latex}{\figure{qrseriestemporales.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param inicio_anual Año de inicio de la serie. Por defecto \code{inicio_anual = 1}.
#' @param periodo_inicio Periodo de inicio de la serie. Por defecto \code{perido_inicio = 1}, es decir, el primer periodo del año 1.
#' @param frecuencia Periodificación de la serie. Por defecto \code{frecuencia = 4}.
#' Si anual, frecuencia = 1.
#' Si semestral, frecuencia = 2.
#' Si cuatrimestral, frecuencia = 3.
#' Si trimestral, frecuencia = 4.
#' Si bimestral, frecuencia = 6.
#' Si mensual, frecuencia = 12.
#' Si semanal, frecuencia = 52.
#' Si diario, frecuencia = 360.
#' @param orden Orden (o puntos) de cálculo de la media móvil. Por defecto \code{orden = frecuencia}.
#' @param prediccion_tendencia vector de periodo temporal (\code{t=0}, origen de la serie) para el que se quiere obtener una predicción de la tendencia de la serie objeto de estudio.
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica la serie original, las medias móviles y la estimación por regresión de la tendencia, cambiar a \code{grafico = TRUE}.
#' @param exportar Para exportar los principales resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return Esta función devuelve un objeto de la clase \code{list}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @seealso \code{\link{regresion.simple}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#' \dontrun{
#' ejemplo_serie <- series.temporales(
#'   turistas2,
#'   variable = 2,
#'   inicio_anual = 2000,
#'   periodo_inicio = 1
#' )
#' }
#' @import dplyr tidyr
#' @importFrom stats lm
#'
#' @export
series.temporales <- function(x,
                              variable = NULL,
                              inicio_anual = 1,
                              periodo_inicio = 1,
                              frecuencia = 4,
                              orden = frecuencia,
                              prediccion_tendencia = FALSE,
                              grafico = FALSE,
                              exportar = FALSE){

  old <- options()
  #on.exit(options(old))

  options(scipen = 999)

  varnames <- as.character(names(x))
  x <- data.frame(x)
  names(x) <- varnames

  if(is.null(variable)){
    if(length(x) == 1){
      x <- x
    } else{
      stop("El conjunto de datos seleccionado tiene mas de 1 variable.")
    }
  } else{
    if(length(variable) == 1){
      if(is.numeric(variable)){
        if(variable <= length(x)){
          variable <- variable
        } else{
          stop("Selecci\u00f3n err\u00f3nea de variable")
          }
      }
    }

    if(is.character(variable)){
      if(all(variable %in% varnames)){
        variable = match(variable,varnames)
      } else {
        stop("El nombre de la variable no es v\u00e1lido")
        }
    }
  }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  if(frecuencia == 4){
    time2 = "T"
  } else if(frecuencia==3){
    time2 = "C"
  } else if(frecuencia == 12){
    time2 = "M"
  } else if(frecuencia == 360){
    time2 = "D"
  } else if(frecuencia == 2){
    time2 = "S"
  } else if(frecuencia == 52){
    time2 = "s"
  } else if(frecuencia == 6){
    time2 = "B" # bimestral (cada dos meses)
  } else{
    time2 = NULL
  }


orden = orden

n <- nrow(x)

period <- rep(1:frecuencia,ceiling((n+periodo_inicio-1)/frecuencia))
time = rep(1:ceiling((n+periodo_inicio-1)/frecuencia),each=frecuencia)

if(periodo_inicio != 1){
  x$periodo <- period[-c(1:(periodo_inicio-1))][1:n]

  if(inicio_anual==1){
    x$time <- time[-c(1:(periodo_inicio-1))][1:n]
  } else{
    x$time <- inicio_anual + time[-c(1:(periodo_inicio-1))][1:n] -1
  }
}else{

  x$periodo <- period[1:n]

  if(is.null(inicio_anual)){
    x$time <- time[1:n]
  } else{
    x$time <- inicio_anual + time[1:n] -1
  }

}

if(frecuencia !=1){
  x$fecha <- paste(x$periodo,time2,"_",x$time,sep="")
}else{
  x$fecha <- x$time
}

x$id <- 1:n


mediasMoviles <- x %>%
  rename(variable_serie = varnames[variable]) %>%
  mutate(mediamovil = forecast::ma(variable_serie, order = orden, centre = TRUE)) %>%
  select(4,2,3,1,6)

# incluyo la variable t para el ajuste de tendencia
NonNAindex <- which(!is.na(mediasMoviles[5]))
firstNonNA <- min(NonNAindex)
lastNonNA <- max(NonNAindex)
mediasMoviles$t <- c(rep(NA,firstNonNA-1),0:(lastNonNA-firstNonNA),rep(NA,n-lastNonNA))

if(frecuencia != 1){
  ive <- mediasMoviles %>%
    mutate(paso1_ive = variable_serie/mediamovil) %>%
    select(time,periodo,paso1_ive) %>%
    arrange(periodo) %>%
    pivot_wider(names_from=periodo,values_from=paso1_ive) %>%
    summarize_at(vars(-1),mean,na.rm=TRUE)

  ivecorregido <- (ive/sum(ive[1,]))*frecuencia
  ivecorregido <- ivecorregido %>%
    t() %>%
    as.data.frame()
  names(ivecorregido) <- "IVE"
  rownames(ivecorregido) <- paste("IVE_",1:frecuencia,sep="")
  #sum(ivecorregido[1,])
}else{
  ivecorregido <- NULL
}


serie_regresion <- subset(mediasMoviles,!is.na(mediamovil))
serie_regresion <- serie_regresion %>%
  select(1,6,4,5)


#cambio nombre de objeto
names(mediasMoviles) <- c("Fecha","Periodo","Time",varnames[variable],
                          "Media.Movil","t")


media_t <- media(serie_regresion[2])
media_mediamovil <- media(serie_regresion[4])
varianza_t <- varianza(serie_regresion[2])
varianza_mediamovil <- varianza(serie_regresion[4])
covarianza_t_mediamovil <- as.numeric(covarianza(serie_regresion[,c(2,4)]))

modelo_series <- lm(mediamovil ~ t, data = serie_regresion)

constante_regresion <- as.numeric(modelo_series$coefficients[1])
coeficiente_regresion <- as.numeric(modelo_series$coefficients[2])
correlacion_t_mediamovil <- as.numeric(correlacion(serie_regresion[,c(2,4)]))
coeficiente_determinacion <- summary(modelo_series)$r.squared
varianza_explicada <- varianza(modelo_series$fitted.values)
varianza_residual <- varianza_mediamovil - varianza_explicada

resultados_regresion <- data.frame(c(media_t,
                                     media_mediamovil,
                                     varianza_t,
                                     varianza_mediamovil,
                                     covarianza_t_mediamovil,
                                     correlacion_t_mediamovil,
                                     constante_regresion,
                                     coeficiente_regresion,
                                     coeficiente_determinacion,
                                     varianza_explicada,
                                     varianza_residual))
names(resultados_regresion) <- "Resultados"

rownames(resultados_regresion) <- c("media t","media mediamovil","varianza t",
                                 "varianza mediamovil","covarianza t_mediamovil",
                                 "correlacion t_mediamovil","constante regresion",
                                 "coeficiente regresion","coeficiente determinacion",
                                 "varianza explicada","varianza residual")


if(isTRUE(grafico)){
  if(frecuencia!=1){
    p <-  ggplot(serie_regresion) +
      geom_point(aes(x = t, y = variable_serie)) +
      geom_line(aes(x = t, y = variable_serie)) +
      geom_point(aes(x = t, y = mediamovil),size=1,color="red") +
      geom_line(aes(x = t, y = mediamovil),size=1,color="red") +
      geom_smooth(aes(t,mediamovil),
                  method = "lm",
                  formula= 'y ~ x',
                  se = FALSE,
                  color = "blue") +
      scale_x_continuous(breaks = serie_regresion$t, labels = serie_regresion$fecha) +
      labs(title=paste("Serie temporal de ",varnames[variable],sep=""),
           subtitle=paste("Ajuste lineal: ",varnames[variable],"=",round(modelo_series$coefficients[1],5),if_else(modelo_series$coefficients[2] >=0, "+", ""),round(modelo_series$coefficients[2],5)," * t",sep=""),
           x="Periodo",
           y=varnames[variable]) +
  theme(axis.text.x=element_text(size=6,angle=90,vjust=0.2))

  } else{
    p <-  ggplot(serie_regresion) +
      geom_point(aes(x = t, y = variable_serie)) +
      geom_line(aes(x = t, y = variable_serie)) +
      geom_smooth(aes(t,mediamovil),
                  method = "lm",
                  formula= 'y ~ x',
                  se = FALSE,
                  color = "blue") +
      scale_x_continuous(breaks = serie_regresion$t, labels = serie_regresion$fecha) +
      labs(title=paste("Serie temporal de ",varnames[variable],sep=""),
           subtitle=paste("Ajuste lineal: ",varnames[variable],"=",round(modelo_series$coefficients[1],5),if_else(modelo_series$coefficients[2] >=0, "+", ""),round(modelo_series$coefficients[2],5)," * t",sep=""),
           x="Periodo",
           y=varnames[variable]) +
      theme(axis.text.x=element_text(size=6,angle=90,vjust=0.2))

  }
}else{
  p <- NULL
}


if(prediccion_tendencia){
  pronostico <- constante_regresion + coeficiente_regresion *prediccion_tendencia
  pronosticos <- data.frame(valor_t = prediccion_tendencia,pronosticos = pronostico)
} else{
  pronosticos <- NULL
}


# if (exportar) {
#   filename <- paste("Resultado series temporales (",Sys.time(), ").xlsx", sep = "")
#   filename <- gsub(" ", "_", filename)
#   filename <- gsub(":", ".", filename)
#
#   if(frecuencia!=1){
#
#     if(prediccion_tendencia){
#       lista <- list(mediasMoviles,ivecorregido,serie_regresion,resultados_regresion,pronosticos)
#
#       rio::export(lista, rowNames = TRUE, filename, sheetName=c("Medias moviles",
#                                                               "IVE",
#                                                               "Datos ajuste tendencia",
#                                                               "Modelo ajuste",
#                                                               "Pronosticos"))
#     }else{
#       lista <- list(mediasMoviles,ivecorregido,serie_regresion,resultados_regresion)
#
#       rio::export(lista, rowNames = TRUE, filename, sheetName=c("Medias moviles",
#                                                               "IVE",
#                                                               "Datos ajuste tendencia",
#                                                               "Modelo ajuste"))
#     }
#   }else{
#     if(prediccion_tendencia){
#       lista <- list(mediasMoviles,serie_regresion,resultados_regresion,pronosticos)
#
#       rio::export(lista, rowNames = TRUE, filename, sheetName=c("Medias moviles",
#                                                               "Datos ajuste tendencia",
#                                                               "Modelo ajuste",
#                                                               "Pronosticos"))
#     }else{
#       lista <- list(mediasMoviles,serie_regresion,resultados_regresion)
#
#       rio::export(lista, rowNames = TRUE, filename, sheetName=c("Medias moviles",
#                                                               "Datos ajueste tendencia",
#                                                               "Modelo ajuste"))
#     }
#   }
#
# }

if (exportar) {
  # Prepara el nombre del archivo (formato más limpio)
  filename <- paste("Resultado_series_temporales_", format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), ".xlsx", sep = "")

  # Crear el workbook
  wb <- openxlsx::createWorkbook()

  # exportacion segun parametros
  if (frecuencia != 1) {
    if (prediccion_tendencia) {
      .add_sheet_with_style(wb, "Medias_moviles", mediasMoviles)
      .add_sheet_with_style(wb, "IVE", ivecorregido)
      .add_sheet_with_style(wb, "Datos_ajuste_tendencia", serie_regresion)
      .add_sheet_with_style(wb, "Modelo_ajuste", resultados_regresion)
      .add_sheet_with_style(wb, "Pronosticos", pronosticos)
    } else {
      .add_sheet_with_style(wb, "Medias_moviles", mediasMoviles)
      .add_sheet_with_style(wb, "IVE", ivecorregido)
      .add_sheet_with_style(wb, "Datos_ajuste_tendencia", serie_regresion)
      .add_sheet_with_style(wb, "Modelo_ajuste", resultados_regresion)
    }
  } else {
    if (prediccion_tendencia) {
      .add_sheet_with_style(wb, "Medias_moviles", mediasMoviles)
      .add_sheet_with_style(wb, "Datos_ajuste_tendencia", serie_regresion)
      .add_sheet_with_style(wb, "Modelo_ajuste", resultados_regresion)
      .add_sheet_with_style(wb, "Pronosticos", pronosticos)
    } else {
      .add_sheet_with_style(wb, "Medias_moviles", mediasMoviles)
      .add_sheet_with_style(wb, "Datos_ajuste_tendencia", serie_regresion)
      .add_sheet_with_style(wb, "Modelo_ajuste", resultados_regresion)
    }
  }

  # Guardar el archivo
  openxlsx::saveWorkbook(wb, filename, overwrite = TRUE)
}

return(list('Medias_moviles' = mediasMoviles,
            'IVE' = ivecorregido,
            'Datos_ajuste_tendencia'= serie_regresion,
            'Modelo_ajuste' = resultados_regresion,
            'Pronosticos' = pronosticos,
            'Grafico' = p))

on.exit(options(old))

}

#' @title Contraste de hipótesis sobre la diferencia de dos proporciones.
#'
#' @description Realiza el contraste de hipótesis sobre la diferencia de dos proporciones.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcdifproporciones.png}{width = 200px}}
#' \if{latex}{\figure{qrcdifproporciones.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere solo a dos variables, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param hipotesis_nula Es un valor numérico. Por defecto el valor está fijado en cero.
#' @param tipo_contraste Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
#'        Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
#'        Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
#'        Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba, el p-valor  el intervalo de confianza para la diferencia de proporciones muestrales supuesta cierta la hipótesis nula. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' El estadístico Z del contraste, que se distribuye N(0,1), es:
#'
#' (1) Si se consideran las proporciones muestrales:
#'
#' \if{html}{\figure{cdifpromuestra.png}{width = 480px}}
#' \if{latex}{\figure{cdifpromuestra.png}{options: width=8cm}}
#'
#' (2) si se estima p como media ponderada de las proporciones muestrales, la ponderación es:
#'
#' \if{html}{\figure{cproponderacion.png}{width = 400px}}
#' \if{latex}{\figure{cproponderacion.png}{options: width=6cm}}
#'
#' y el estadístico resulta:
#'
#' \if{html}{\figure{cdifpropond.png}{width = 480px}}
#' \if{latex}{\figure{cdifpropond.png}{options: width=7cm}}
#'
#' @seealso \code{\link{ic.diferencia.proporciones}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm na.omit dnorm
#' @import dplyr ggplot2
#'
#' @export
contraste.diferencia.proporciones <- function(x,
                                     variable = NULL,
                                     introducir = FALSE,
                                     hipotesis_nula = 0,
                                     tipo_contraste =  c("bilateral","cola derecha","cola izquierda"),
                                     alfa = 0.05,
                                     grafico = FALSE){


  tipo_contraste <- tolower(tipo_contraste)
  tipo_contraste <- match.arg(tipo_contraste)

  if(alfa >= 0 & alfa <=1){

    if(tipo_contraste == "bilateral"){
      valor_critico <- qnorm(alfa/2,lower.tail = F)
    }
    if(tipo_contraste == "cola izquierda"){
      valor_critico <- qnorm(alfa,lower.tail = T)
    }
    if(tipo_contraste == "cola derecha"){
      valor_critico <- qnorm(alfa,lower.tail = F)
    }

    valor_critico <- round(valor_critico,4)

  } else{
    stop("El nivel de significacion debe fijarse entre 0 y 1")
  }

  if(hipotesis_nula >= 0 & hipotesis_nula <=1){
    H0 <- hipotesis_nula
  }else{
    stop("La hip\u00f3tesis nula es una proporcion y por tanto tiene que fijarse entre 0 y 1")
  }



  if(isFALSE(introducir)) {

    x <- data.frame(x)
    varnames <- names(x)

    if(is.null(variable)){

      if(length(x) == 2){

        x <- x

      } else{

        warning("Para realizar el contraste hay que seleccionar dos variables")
        stop("El conjunto de datos seleccionado tiene mas de 2 variables.")

      }

    } else if(length(variable) == 2){

      if(is.numeric(variable)){

        if(all(variable <= length(x))){

          variable <- variable

        } else{

          stop("Selecci\u00f3n err\u00f3nea de variables")

        }
      }

      if(is.character(variable)){

        if(all(variable %in% varnames)){

          variable = match(variable,varnames)

        } else {

          stop("El nombre de la variable no es v\u00e1lido")

        }

      }

      x <- x[,variable] %>% as.data.frame()
      names(x) <- varnames[variable]

    } else{

      warning("Para realizar el contraste hay que seleccionar dos variables")
      stop("El conjunto de datos seleccionado no es adecuado.")

    }

    clase <- sapply(x, class)

    if (!all(clase %in% c("numeric","integer"))) {

      stop("No puede calcularse el contraste porque las variables seleccionadas no son cuantitativas")

    }

    x1 <- na.omit(x[1])
    x2 <- na.omit(x[2])


    if(!all((x1 == 0) | x1 ==1)){

      print("Aplica a tus datos la condici\u00f3n que debe cumplir la poblaci\u00f3n para transfomar los datos en ceros (ausencia/no \u00e9xito) y unos (presencia/\u00e9xito)")
      stop("Los valores en la muestra deben ser 0 y 1.")

    }

    if(!all((x2 == 0) | x2 ==1)){

      print("Aplica a tus datos la condici\u00f3n que debe cumplir la poblaci\u00f3n para transfomar los datos en ceros (ausencia/no \u00e9xito) y unos (presencia/\u00e9xito)")
      stop("Los valores en la muestra deben ser 0 y 1.")

    }


    # tama\u00f1o de la muestra
    n1 <- nrow(x1)
    n2 <- nrow(x2)

    # media muestral
    p_mu1 <- round(sum(x1)/n1,6)
    p_mu2 <- round(sum(x2)/n2,6)

} else{   # aqu\u00ed empieza introducir datos

  print("Primero vas a introducir los datos de la muestra 1 y a continuaci\u00f3n introducir\u00e1s los datos de la muestra 2")
  print("Si los datos provienen de encuestas realizadas antes y despu\u00e9s de una determinada acci\u00f3n, introduce primero los datos de la encuesta realizada despu\u00e9s de dicha acci\u00f3n")

  n1 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 1: ")
  n1 <- as.numeric(n1)

  p_mu1 <- readline(prompt = "Introducir el valor de la proporcion muestral 1: ")
  p_mu1 <- as.numeric(p_mu1)


  n2 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 2: ")
  n2 <- as.numeric(n2)


  p_mu2 <- readline(prompt = "Introducir el valor de la proporcion muestral 2: ")
  p_mu2 <- as.numeric(p_mu2)


}

  # calculo de los contrastes

  est_proporcion <- as.numeric(readline('Selecciona el valor que quieres utilizar para el error t\u00edpico bajo la H0: \n 1. "Estimar p como media ponderada de las proporciones muestrales" \n 2. "Utilizar las proporciones muestrales" \n'))

  # estadistico de prueba
  dif_p <- p_mu1 - p_mu2

  if(est_proporcion == 1){

    est_p <- ((n1*p_mu1) + (n2*p_mu2))/(n1+n2)
    error_tipico <- sqrt(((n1+n2)/(n1*n2)) * est_p * (1-est_p))

  } else{

    error_tipico <- sqrt((p_mu1 * (1-p_mu1))/n1 + (p_mu2 * (1-p_mu2))/n2)

  }

  estadistico.Z <- (dif_p - H0)/error_tipico
  estadistico.Z <- round(estadistico.Z,5)

  if(tipo_contraste == "bilateral"){

    estadistico.Z2 <- abs(estadistico.Z)
    pvalor <- 2*pnorm(estadistico.Z2,lower.tail=FALSE)
    media_inf <- H0 - valor_critico * error_tipico
    media_sup <- H0 + valor_critico * error_tipico

    if(estadistico.Z >= -valor_critico & estadistico.Z <=  valor_critico){

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-4, -valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-valor_critico, valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 4)) +
        geom_vline(xintercept = -estadistico.Z2, linetype = "dashed") +
        geom_vline(xintercept = estadistico.Z2, linetype = "dashed") +
        labs(x = "", y = "",title="Regi\u00f3n de aceptaci\u00f3n-rechazo para\nla diferencia de proporciones") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.Z2,-estadistico.Z2,-valor_critico,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))
    }

  } else if(tipo_contraste == "cola derecha"){

    media_inf <- -Inf
    media_sup <- H0 + valor_critico * error_tipico
    pvalor <- pnorm(estadistico.Z,lower.tail=FALSE)


    if(estadistico.Z > valor_critico){

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-4,valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 4)) +
        geom_vline(xintercept = estadistico.Z, linetype = "dashed") +
        labs(x = "", y = "",title="Regi\u00f3n de aceptaci\u00f3n-rechazo para\nla diferencia de proporciones") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.Z,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))
    }

  } else{

    media_inf <- H0 + valor_critico * error_tipico # valor critico negativo
    media_sup <- Inf
    pvalor <- pnorm(estadistico.Z,lower.tail=TRUE)


    if(estadistico.Z < valor_critico){

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-4, -valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-valor_critico, 4)) +
        geom_vline(xintercept = estadistico.Z, linetype = "dashed") +
        labs(x = "", y = "",title="Regi\u00f3n de aceptaci\u00f3n-rechazo para\nla diferencia de proporciones") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.Z,-valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))
    }

  }


  CH <- cbind(H0,estadistico.Z,pvalor)
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  Idifpro <- cbind(`limite_inferior`=media_inf,`limite_superior`=media_sup)


  if(grafico){

    return(list(`Estadistico`=CH,`Intervalo de la proporcion muestral (supuesta H0 cierta)`= Idifpro,`Graficos`= plot))

  } else{

    return(list(`Estadistico`=CH,`Intervalo de la proporcion muestral (supuesta H0 cierta)`= Idifpro))

  }

}

utils::globalVariables(c("ni", "fi", "sumatorio", "variable2",
                         "frecuencia", "valor", "cuantiles_int",
                         "filas", "columnas", "name","rbindlist",
                         "y","var_muestra","media_x","sumatorio3","sumatorio4","desv.x.muestra","asimetria_soft",
                         "curtosis_soft","error_curtosis","ic","inferior","superior", "color","varianza.muestral",
                         "media.muestral","incluido","value",
                         "grupo","grupo2","vx","vy","coeficiente.determinacion",
                         "varianza.regresion","valores.teoricos","errores",
                         "obs","id","sc.observados","sc.teoricos","errores2","suma.cuadrados.total",
                         "suma.cuadrados.errores","suma.cuadrados.regresion","varianza.residual",
                         "varianza.residuos", "leverage", "puntos.palanca","error.norm",
                         "X","momento", "atipico", "lm", "mediamovil", "paso1_ive",
                         "periodo", "pivot_wider", "matriz","variable_serie",
                         "A1","A2","c1","c2","c3","error_asimetria","var_coef","prediccion",".",
                         "variable1","variable2","..density.."))

#' @title Contraste de hipótesis sobre la varianza.
#'
#' @description Realiza el contraste de hipótesis sobre la varianza poblacional.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcvarianza.png}{width = 200px}}
#' \if{latex}{\figure{qrcvarianza.png}{options: width=3cm}}
#'
#' @usage contraste.varianza(x,
#'                  variable = NULL,
#'                  introducir = FALSE,
#'                  media_poblacion = c("desconocida","conocida"),
#'                  hipotesis_nula = NULL,
#'                  tipo_contraste =  c("bilateral","cola derecha","cola izquierda"),
#'                  alfa = 0.05,
#'                  grafico = FALSE)
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param media_poblacion Es un carácter. Indica si la media de la población es desconocida (por defecto, \code{media_poblacion = "desconocida"}) o conocida (en este caso, cambiar \code{media_poblacion = "conocida"}).
#' @param hipotesis_nula Es un valor numérico.
#' @param tipo_contraste Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
#'        Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
#'        Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
#'        Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba, el p-valor y  el intervalo de confianza para la media muestral supuesta cierta la hipótesis nula. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con los valores críticos.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) Si la media poblacional es desconocida, el estadístico chi-dos es:
#'
#' (1.1) utilizando la varianza muestral:
#'
#' \if{html}{\figure{cvarmuestra.png}{width = 200px}}
#' \if{latex}{\figure{cvarmuestra.png}{options: width=3cm}}
#'
#' (1.2) utilizando la cuasi-varianza muestral:
#'
#' \if{html}{\figure{cvarcuasi.png}{width = 240px}}
#' \if{latex}{\figure{cvarcuasi.png}{options: width=4cm}}
#'
#' (2) Si la media poblacional es conocida.
#'
#' (2.1) utilizando la varianza muestral:
#'
#' \if{html}{\figure{cvarmediaconmuestra.png}{width = 480px}}
#' \if{latex}{\figure{cvarmediaconmuestra.png}{options: width=5cm}}
#'
#'  Nota: En todos los casos, el estadístico chi-dos se distrubuye con n-1 grados de libertad.
#'
#' @seealso \code{\link{ic.varianza}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats na.omit dchisq pchisq df
#' @import dplyr ggplot2
#'
#' @export
contraste.varianza <- function(x,
                               variable = NULL,
                               introducir = FALSE,
                               media_poblacion = c("desconocida","conocida"),
                               hipotesis_nula = NULL,
                               tipo_contraste = c("bilateral","cola derecha","cola izquierda"),
                               alfa = 0.05,
                               grafico = FALSE){


  media_poblacion <- tolower(media_poblacion)
  media_poblacion <- match.arg(media_poblacion)


tipo_contraste <- tolower(tipo_contraste)
tipo_contraste <- match.arg(tipo_contraste)

if(is.null(hipotesis_nula) | !is.numeric(hipotesis_nula)){

  stop("Tienes que introducir un valor para la hip\u00f3tesis nula")

} else{
  H0 <- hipotesis_nula

}


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 1){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variables.")

    }

  } else if(length(variable) == 1){

    if(is.numeric(variable)){

      if(variable <= length(x)){

        variable <- variable

      } else{

        stop("Seleccion erronea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es valido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
    stop("El conjunto de datos seleccionado tiene mas de 1 variables.")

  }

  x <- na.omit(x)
  clase <- sapply(x, class)

  if (!clase %in% c("numeric","integer")) {

    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")

  }

  # tama\u00f1o de la muestra
  n <- nrow(x)

  if(media_poblacion == "desconocida"){
    gl <- n-1 # grados de libertad
  } else{
    gl <- n # grados de libertad
  }

  if(media_poblacion == "desconocida"){

    var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      var_mu <- as.numeric(varianza(x))

    } else{

      var_mu <- as.numeric(varianza(x, tipo = "cuasi"))
      n <- n-1

    }
  } else{

    print("La media poblacional no suele conocerse, este supuesto es te\u00f3rico")

    media <- readline(prompt = "Introducir el valor de la media poblacional: ")
    media <- as.numeric(media)

    sumatorio <- sum((x - media)^2)

  }


} else{   # aquí empieza introducir datos


  n <- readline(prompt = "Introducir el tama\u00f1o de la muestra: ")
  n <- as.numeric(n)

  if(media_poblacion == "desconocida"){
    gl <- n-1 # grados de libertad
  } else{
    gl <- n # grados de libertad
  }

  if(media_poblacion == "desconocida"){

    var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar? \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      var_mu <- readline("Introduce el valor de la varianza muestral: ")
      var_mu <- as.numeric(var_mu)

    } else{

      var_mu <- readline("Introduce el valor de la cuasivarianza muestral: ")
      var_mu <- as.numeric(var_mu)
      n <- n-1

    }

  } else{

    print("La media poblacional no suele conocerse, este supuesto es te\u00f3rico")

    #media <- readline(prompt = "Introducir el valor de la media poblacional: ")
    #media <- as.numeric(media)
    sumatorio <- readline(prompt = "Introducir el valor de la suma cuadr\u00e1tica de las desviaciones de los\nvalores muestrales respecto a la media poblacional: ")
    sumatorio <- as.numeric(sumatorio)

  }

}


if(alfa >= 0 & alfa <=1){

    if(tipo_contraste == "bilateral"){
      valor_critico_L <- round(qchisq(alfa/2,gl,lower.tail = T),4)
      valor_critico_R <- round(qchisq(alfa/2,gl,lower.tail = F),4)

    }
    if(tipo_contraste == "cola izquierda"){
      valor_critico <- round(qchisq(alfa,gl,lower.tail = T),4)
    }
    if(tipo_contraste == "cola derecha"){
      valor_critico <- round(qchisq(alfa,gl,lower.tail = F),4)
    }

} else{
  stop("El nivel de significacion debe fijarse entre 0 y 1")
}


# caluclo de los contrastes

if(media_poblacion == "desconocida"){

  estadistico.prueba <- n * var_mu / H0

} else{

  var_muestra <- 1

  estadistico.prueba <- sumatorio / H0

}

percentil99 <- qchisq(.9999, gl)

df <- data.frame(x=seq(from = 0, to = percentil99, percentil99/200))
df$y <-dchisq(df$x, gl)

  if(tipo_contraste == "bilateral"){

    limite_inf <- round(H0 /n * valor_critico_L,4)
    limite_sup <- round(H0 /n * valor_critico_R,4)
    pvalor <- 2 *min(pchisq(estadistico.prueba,gl,lower.tail=F),pchisq(estadistico.prueba,gl,lower.tail=T))

    if(estadistico.prueba >= valor_critico_L & estadistico.prueba <= valor_critico_R){

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico_L," , ",valor_critico_R,"]",sep=""))
      print("El valor del estad\u00ed de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico_L," , ",valor_critico_R,"]",sep=""))
      print("El valor del estad\u00ed de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(df) +
        geom_path(aes(x,y))+
        geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "darkgreen", xlim = c(valor_critico_L, valor_critico_R)) +
        geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "red", xlim = c(0, valor_critico_L)) +
        geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "red", xlim = c(valor_critico_R, percentil99)) +
        geom_vline(xintercept = 0, color = "black") +
        geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
        labs(title = paste("Distribuci\u00f3n chi con ", gl, " grados de libertad",sep=""), x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(0L,estadistico.prueba,valor_critico_L,valor_critico_R)) +
        theme(axis.text.x = element_text(angle = 45))
    }


  } else if(tipo_contraste == "cola derecha"){

    limite_inf <- 0
    limite_sup <- round(H0 /n * valor_critico,4)
    pvalor <- pchisq(estadistico.prueba,gl,lower.tail= F)


    if(estadistico.prueba <=  valor_critico){

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", 0 ," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00ed de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", 0," , ",valor_critico,"]",sep=""))
      print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(df) +
        geom_path(aes(x,y))+
        geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "darkgreen", xlim = c(0, valor_critico)) +
        geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "red", xlim = c(valor_critico,percentil99)) +
        geom_vline(xintercept = 0, color = "black") +
        geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
        labs(title = paste("Distribuci\u00f3n chi con ", gl, " grados de libertad",sep=""), x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(0L,estadistico.prueba,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))
    }

  } else{

    limite_inf <- round(H0 /n * valor_critico,4)
    limite_sup <- Inf
    pvalor <- pchisq(estadistico.prueba,gl,lower.tail=T)


    if(estadistico.prueba >= valor_critico){

      print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico," , ",Inf,"[",sep=""))
      print("El valor del estad\u00ed de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    } else{

      print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", valor_critico," , ",Inf,"[",sep=""))
      print("El valor del estad\u00ed de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

    }

    if(isTRUE(grafico)){

      plot <- ggplot(df) +
        geom_path(aes(x,y))+
        geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "darkgreen", xlim = c(valor_critico,percentil99)) +
        geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "red", xlim = c(0,valor_critico)) +
        geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
        geom_vline(xintercept = 0, color = "black") +
        labs(title = paste("Distribuci\u00f3n chi con ", gl, " grados de libertad",sep=""), x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(0L,estadistico.prueba,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))
    }



  }


  CH <- cbind(H0,estadistico.prueba,pvalor)
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p valor")
  row.names(CH) <- NULL

  Ivarianza <- cbind(`limite_inferior`=limite_inf,`limite_superior`=limite_sup)


  if(grafico){

    if(var_muestra == 1){

      return(list(`Estadistico`=CH,`Intervalo de la varianza muestral (supuesta H0 cierta)`= Ivarianza,`Graficos`= plot))

    } else {

      return(list(`Estadistico`=CH,`Intervalo de la cuasi-varianza muestral (supuesta H0 cierta)`= Ivarianza,`Graficos`= plot))

    }

  } else{

    if(var_muestra == 1){

      return(list(`Estadistico`=CH,`Intervalo de la varianza muestral (supuesta H0 cierta)`= Ivarianza))

    } else {

      return(list(`Estadistico`=CH,`Intervalo de la cuasi-varianza muestral (supuesta H0 cierta)`= Ivarianza))

    }

  }


}

#' @title Contraste de hipótesis de bondad de ajuste para datos categóricos.
#'
#' @description Contrasta si las probabilidades de que una observación pertenezca a cada una de las categorías de una variable categórica o criterio de clasificación se ajusta o no a unas probabilidades propuestas.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe. En caso de haber más de una variable, el programa preguntará por la variable a seleccionar (por nombre o por posición) que debe ser un factor o carácter.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre el número de categorías de la variable, el nombre de cada categoría. A continuación se abrirá una ventana con un editor de datos y deberá introducir los valores de las frecuencias observadas.
#' @param distribucion Es un vector numérico. Deberá indicarse las probabilidades teóricas para cada categoría de la variable de la hipótesis nula. Por defecto, *distribucion="equiporobable"*.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba y el p-valor. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con el valor crítico.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#'(1) El estadístico del contraste de bondad de ajuste para datos categóricos es:
#'
#' \deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(O_{i} - E_{i})^{2}}{E_{i}}}
#'
#' donde \eqn{O_{i}} son las frecuencias observadas y \eqn{E_{i}} son las frecuencias teóricas o esperadas, y se distribuye como:
#'
#' \deqn{\chi_{k-1}^{2}}
#'
#' donde \eqn{k} es el número de categorías de la variable.
#'
#'Además, se exige que todas las frecuencias teóricas no estén por debajo de 5. Si alguna no lo cumple es necesario reagrupar valores contiguos hasta conseguir superar esa cota.
#'
#' (2) Si el número de grados de libertad es 1, al estadístico del contraste se le aplica la siguiente corrección de Yates:
#'
#' \deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(\left| O_{i} - E_{i} \right| - 0.5)^{2}}{E_{i}}}
#'
#'
#' @seealso \code{\link{contraste_bondad}}, \code{\link{contraste_independencia}},
#' \code{\link{contraste_homogeneidad}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats dbinom dpois
#' @importFrom utils edit
#' @import dplyr ggplot2
#'
#' @export
contraste_bondad_cat <- function(x,
                                 introducir = FALSE,
                                 distribucion = "equiprobable",
                                 alfa = 0.05,
                                 grafico = FALSE) {


    # Si el usuario quiere introducir los datos manualmente
    if (introducir == TRUE) {

      # Caso cuando el usuario quiere introducir los datos manualmente
      nfilas <- as.numeric(readline(prompt = "Introduce el n\u00famero de categor\u00edas de la variable: "))

      nombre_filas <- c()

      # Introducir nombres de las filas
      for (j in 1:nfilas) {
        nombre_filas <- c(nombre_filas, readline(prompt = paste("Introduce el nombre de la categor\u00eda n\u00famero ", j, ": ", sep = "")))
      }

      # Crear la matriz vacia
      x <- matrix(0, ncol = 1, nrow = nfilas)
      rownames(x) <- nombre_filas
      colnames(x) <- "Frecuencias observadas"

      # Permitir editar la matriz
      matriz_obs <- edit(x)
      print(matriz_obs)

    } else { # Caso cuando se proporciona un data frame

      # Verificar que el input es un data.frame
      if (!is.data.frame(x)) {
        stop("El input debe ser un data frame.")
      }

      # Mostrar los nombres de las columnas
      cat("Las columnas disponibles en el data frame son:\n")
      print(colnames(x))

      # Funcion auxiliar para seleccionar columna por nombre o posicion
      seleccionar_columna <- function(prompt_msg) {
        seleccion <- readline(prompt = prompt_msg)

        # Verificar si el usuario ingreso un numero (posicion) o un nombre (texto)
        if (suppressWarnings(!is.na(as.numeric(seleccion)))) {
          seleccion <- as.numeric(seleccion)
          if (seleccion < 1 || seleccion > ncol(x)) {
            stop("La posici\u00f3n seleccionada est\u00e1 fuera del rango de las columnas disponibles.")
          }
          return(seleccion)
        } else {
          # Seleccion por nombre
          if (!seleccion %in% colnames(x)) {
            stop("El nombre de columna introducido no existe.")
          }
          return(which(colnames(x) == seleccion))
        }
      }

      # Seleccionar la columna
      col <- seleccionar_columna("Selecciona la variable (por nombre o posici\u00f3n): ")

      # Verificar que la variable seleccionada es un factor o caracter
      if (!(is.factor(x[[col]]) || is.character(x[[col]]))) {
        stop("La variable seleccionada debe ser un factor o car\u00e1cter.")
      }

      # Convertir a factor si es un caracter
      if (is.character(x[[col]])) {
        x[[col]] <- as.factor(x[[col]])
        cat("La variable se ha convertido a factor.\n")
      }

      # Extraer los datos de la variable seleccionada
      data_frame_obs <- data.frame(x[[col]])
      colnames(data_frame_obs) <- colnames(x)[col]

      # Mostrar los niveles de la variable seleccionada
      cat("Los niveles actuales de la variable son:\n")
      print(levels(data_frame_obs[[1]]))

      # Preguntar al usuario si desea reordenar los niveles de la variable
      respuesta_reordenar <- readline(prompt = "\u00bfQuieres reordenar los niveles de la variable? \n 1. \"Si\" \n 2. \"No\" \n")

      if (respuesta_reordenar == "1") {
        # Reordenar niveles de la variable
        cat("Reordena los niveles de la variable (separados por comas):\n")
        nuevo_orden <- unlist(strsplit(readline(), ","))
        levels(data_frame_obs[[1]]) <- nuevo_orden
      }

      # Crear la tabla de contingencia
      matriz_obs <- table(data_frame_obs[[1]])
      print(matriz_obs)

      # Confirmar la matriz observada
      respuesta <- readline(prompt = '\u00bfEs esta la matriz de datos observados? \n 1. "Si" \n 2. "No" \n')

      if (respuesta == "2") {
        cat("Introduce o modifica la matriz de datos observados:\n")
        matriz_obs <- edit(matriz_obs)
        print(matriz_obs)

      } else if (respuesta == "1") {
        cat("\u00A1Perfecto! Seguimos con estos datos.\n")
      } else {
        stop("El comando introducido no es correcto.")
      }
    }

    # Obtener las frecuencias observadas
    frecuencias_observadas <- as.numeric(matriz_obs)
    categorias <- length(frecuencias_observadas)

    # Verificar y calcular las frecuencias esperadas segun la distribucion
    if (is.character(distribucion) && distribucion == "equiprobable") {
      # Caso de distribucion equiprobable
      frecuencias_esperadas <- rep(sum(frecuencias_observadas) / categorias, categorias)
      cat("Se ha asumido una distribuci\u00f3n equiprobable.\n")

    } else if (is.numeric(distribucion)) {
      # Verificar que la longitud de la distribucion sea igual al numero de categorias
      if (length(distribucion) != categorias) {
        stop("El vector de distribuci\u00f3n debe tener la misma longitud que el n\u00famero de categor\u00edas.")
      }

      # Verificar que la suma de los valores de la distribucion sea 1 o 100% (con tolerancia)
      suma_distribucion <- sum(distribucion)
      if (!(abs(suma_distribucion - 1) < 1e-10 || abs(suma_distribucion - 100) < 1e-8)) {
        stop("La suma del vector de distribuci\u00f3n debe ser 1 o 100%.")
      }

      # Ajustar si la distribucion se ingreso en formato porcentual (con tolerancia)
      if (abs(suma_distribucion - 100) < 1e-8) {
        distribucion <- distribucion / 100
      }

      # Calcular las frecuencias esperadas en base a la distribucion proporcionada
      frecuencias_esperadas <- sum(frecuencias_observadas) * distribucion
      cat("Se ha utilizado la distribuci\u00f3n proporcionada por el usuario.\n")

    } else {
      stop("El argumento 'distribucion' debe ser 'equiprobable' o un vector num\u00e9rico.")
    }


    # Mostrar las frecuencias esperadas
    cat("Frecuencias esperadas:\n")
    print(frecuencias_esperadas)

    # Introducir las frecuencias esperadas como una nueva columna en la matriz
    matriz_completa <- cbind(Freq_obs = frecuencias_observadas, Freq_esp = frecuencias_esperadas)
    matriz_completa <- as.data.frame(matriz_completa)
    rownames(matriz_completa) <- row.names(matriz_obs)

    # Mostrar la matriz completa con frecuencias observadas y esperadas
    cat("Matriz con frecuencias observadas y esperadas:\n")
    print(matriz_completa)

  if(sum(matriz_completa$Freq_esp < 5) > 0){
    message("Aqu\u00ed tienes la tabla recalcudada de frecuencias esperadas porque alguna de las frecuecias te\u00f3ricas era menor a 5. Para llevar a cabo el test es necesario reagrupar las categor\u00edas.")
    matriz_completa <- .check_min_obs(matriz_completa)
    print(matriz_completa)
  }

  n <- length(matriz_completa$Freq_obs)

  g.l <- n - 1

  if(g.l == 1){

    warning("Los grados de libertad son 1; por tanto, es necesario aplicar la correcci\u00f3n de Yates.")
    estadistico.prueba <- sum((abs(matriz_completa$Freq_obs - matriz_completa$Freq_esp)-0.5)^2/matriz_completa$Freq_esp)

  }else{

    estadistico.prueba <- sum((matriz_completa$Freq_obs - matriz_completa$Freq_esp)^2/ matriz_completa$Freq_esp)

  }

  valor_critico <- qchisq(alfa, g.l, lower.tail = F)


  if(estadistico.prueba < valor_critico ){

    print(paste("No se rechaza la hip\u00f3tesis nula. El valor del estad\u00edstico de contraste: ", round(estadistico.prueba, 2) , ", se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n que toma el intervalo [0, ", round(valor_critico, 2), "]", sep=""))

  } else{

    print(paste("Se rechaza la hip\u00f3tesis nula. El valor del estad\u00edstico de contraste: ", round(estadistico.prueba, 2) , ", se encuentra fuera de la regi\u00f3n de aceptaci\u00f3n que toma el intervalo [0, ", round(valor_critico, 2), "]", sep=""))

  }

  pvalor <- pchisq(estadistico.prueba, g.l, lower.tail = F)

  H0 <- paste("Los datos siguen una distribuci\u00f3n ",  paste(distribucion, collapse = ", "))
  CH <- cbind(H0, estadistico.prueba, round(pvalor, 4))
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  if(isTRUE(grafico)){

    percentil99 <- qchisq(.9999, g.l)

    data <- data.frame(x=seq(from = 0, to = percentil99, percentil99/200))
    data$y <-dchisq(data$x, g.l)

    name_plot <- ggplot(data, aes(x, y)) +
      geom_area(fill = "#C4961A") +
      geom_area(data = subset(data, x < valor_critico), fill = "#008B8B") +
      geom_vline(xintercept = 0, color = "black") +
      geom_vline(xintercept = estadistico.prueba, color = "#A52A2A", linetype = "dashed" , size = 1) +
      labs(title = paste("Distribuci\u00f3n Chi-Cuadrado con ", g.l," grados de libertad", sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = round(c(0L, estadistico.prueba, valor_critico), 2)) +
      theme(axis.text.x = element_text(angle = 45))

    return(list(CH, name_plot))

  } else{

    return(CH)

  }

} # Fin de la funcion

#' @title Tabla doble entrada.
#'
#' @description Calcula la tabla de frecuencias bidimensionales.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrtablabidimensional.png}{width = 200px}}
#' \if{latex}{\figure{qrtablabidimensional.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Tiene que ser un dataframe (al menos dos variables, es decir, dos columnas).
#' @param var_filas Variable fila.Por defecto su valor es NUll y el usuario debe escribir el nombre o posición de la variable cuyos valores quiere representar por filas.
#' @param var_columnas Variable columna. Por defecto su valor es NUll y el usuario debe escribir el nombre o posición de la variable cuyos valores quiere representar por columnas
#' @param distribucion Es un carácter. Por defecto se obtien la tabla cruzada (\code{distribucion = "cruzada"}). Para obtener las distribuciones condicionadas cambiar a \code{distribucion = "condicionada"}.
#' @param frecuencias Es un carácter. Por defecto se obtienen las frecuencias absolutas ordinarias (\code{frecuencias = "absolutas"}). Para obtener las frecuencias relativas ordinarias cambiar a \code{frecuencias = "relativas"}.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return Devuelve la tabla cruzada de las dos variables seleccionadas en un \code{data.frame}
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @importFrom stats addmargins na.omit
#' @import dplyr
#'
#' @export
tabla.bidimensional <- function(x,
                                var_filas = NULL,
                                var_columnas = NULL,
                                distribucion = c("cruzada","condicionada"),
                                frecuencias = c("absolutas","relativas"),
                                exportar = FALSE){

  distribucion <- tolower(distribucion)
  distribucion <- match.arg(distribucion)

  frecuencias <- tolower(frecuencias)
  frecuencias <- match.arg(frecuencias)

  x <- data.frame(x)
  varnames <- names(x)

  if(length(x)<2){
    stop("El conjunto de datos seleccionada solo tiene una variable.")
  }

  if(is.null(var_filas) | is.null(var_columnas)){
    stop("Debes seleccionar la variable fila y columna")
  }

  if(is.numeric(var_filas)){
    if(var_filas<=length(x)){
      var_filas <- var_filas}
    else{
      stop("Selecci\u00f3n err\u00f3nea de variable")
    }
  }

  if(is.character(var_filas)){
    if(var_filas %in% varnames){
      var_filas = match(var_filas,varnames)
    } else {
      stop("El nombre de la variable por filas no es v\u00e1lido")
    }
  }


  if(is.numeric(var_columnas)){
    if(var_columnas<=length(x)){
      var_columnas <- var_columnas}
    else{
      stop("Selecci\u00f3n err\u00f3nea de variable")
    }
  }

  if(is.character(var_columnas)){
    if(var_columnas %in% varnames){
      var_columnas = match(var_columnas,varnames)
    } else {
      stop("El nombre de la variable por columna no es v\u00e1lido")
    }
  }

  if(var_filas == var_columnas){
    stop("La variable por fila y columna es la misma variable")
  }

  variable <- c(var_filas,var_columnas)


  x <- x[,variable] %>% as.data.frame()
  names(x) <- varnames[variable]
  varnames <- names(x)

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer","factor","logic"))){
    stop("No puede construirse la tabla de frecuencias, alguna variable seleccionada es car\u00e1cter")
  }


  if(frecuencias == "absolutas"){

    if(distribucion == "cruzada"){
      tabla <- x %>%
        table()
      tabla <- addmargins(tabla)

    } else{
      print("Si quieres obtener la distribuciones de variable1/variable2 (condici\u00f3n por columnas) introduce el valor 1, en caso contrario variable2/variable1 (condici\u00f3n por filas) introduce el valor 2")
      tipo <- readline(prompt = "Distribuci\u00f3nes condicionadas por columnas (1) o por filas (2): ")
      tipo = as.numeric(tipo)

      tabla2 <- x %>%
        table()
      namesfilas <- row.names(tabla2)
      namescolumnas <- colnames(tabla2)

      if(tipo == 1){
        tabla_aux <- x %>%
          select(tipo) %>%
          group_by_at(vars(varnames[tipo]))  %>%
          count() %>%
          ungroup() %>%
          select(n)
        n = length(namescolumnas)
        tabla_aux <- cbind(tabla_aux, replicate(n-1,tabla_aux$n))

      } else{
        tabla_aux <- x %>%
          select(tipo) %>%
          group_by_at(vars(varnames[tipo])) %>%
          count() %>%
          ungroup() %>%
          select(n)

        n = length(namesfilas)
        tabla_aux <- cbind(tabla_aux, replicate(n-1,tabla_aux$n)) %>%
          t()
      }

      tabla <-  tabla_aux * as.matrix(prop.table(tabla2,tipo))
      row.names(tabla) <- namesfilas
      colnames(tabla) <- namescolumnas

    }

  } else{

    if(distribucion == "cruzada"){
      tabla <- x %>%
        table()
      tabla <- prop.table(tabla)
      tabla <- addmargins(tabla)

    } else{
      print("Si quieres obtener la distribuci\u00f3nes de varible1/variable2 (por filas) introduce el valor 1, en caso contrario variable2/variable1 (por columnas) introduce el valor 2")
      tipo <- readline(prompt = "Distribuci\u00f3nes condicionadas por filas (1) o por columnas (2): ")
      tipo = as.numeric(tipo)

      if(tipo == 1){
        tabla2 <- x %>%
          table()
        tabla <- prop.table(tabla2,2)
      } else{
        tabla2 <- x %>%
          table()
        tabla <- prop.table(tabla2,1)
      }
    }
  }

  tabla <- as.data.frame.matrix(tabla)


  # Exportar
  if (exportar) {

    filename <- paste0("Tabla_cruzada_de_", variable[1],"_y_", variable[2],"_", format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), ".xlsx")

    wb <- openxlsx::createWorkbook()
    openxlsx::addWorksheet(wb, "Tabla_bidimensional")

    # nombres de fila a columna
    resumen_export <- cbind('Var1/Var2' = row.names(tabla), tabla)
    row.names(resumen_export) <- NULL

    openxlsx::writeData(wb, "Tabla_bidimensional", resumen_export)

    # formato numerico decimal en Excel
    addStyle(wb, "Tabla_bidimensional",
             style = createStyle(numFmt = "0.0000"),
             rows = 2:(nrow(resumen_export)+1),
             cols = 2:(ncol(resumen_export)+1),
             gridExpand = TRUE)

    saveWorkbook(wb, filename, overwrite = TRUE)
  }

  return(tabla)

}

#' @title Contraste de hipótesis sobre la media.
#'
#' @description Realiza el contraste de hipótesis sobre la media poblacional.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcmedia.png}{options: style="width: 25\%;"}}
#' \if{latex}{\figure{qrcmedia.png}{options: width=3cm}}
#'
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param var_pob Es un carácter. Indica si la varianza poblacional es conocida (por defecto, \code{var_pob = "conocida"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.
#' @param hipotesis_nula Es un valor numérico.
#' @param tipo_contraste Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
#'        Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
#'        Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
#'        Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.
#' @param alfa Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba, el p-valor y  el intervalo de confianza para la media muestral supuesta cierta la hipótesis nula. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con los valores críticos y otra gráfica con el intervalo para la media muestral (supesta cierta H0).
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) Si la varianza poblacional es conocida, el estadístico Z es:
#'
#' \if{html}{\figure{cmediavarcon.png}{width = 240px}}
#' \if{latex}{\figure{cmediavarcon.png}{options: width=3cm}}
#'
#' y se distribuye como una N(0,1)
#'
#' Si la varianza poblacional es desconocida pero la muesta es grande, puede utilizarse la varianza (o cuasi-varianza) muestral.
#'
#' (2) Si la varianza poblacional es desconocida, el estadístico T es:
#'
#' (2.1) usando la varianza muestral
#'
#' \if{html}{\figure{cmediavardescmuestra.png}{width = 240px}}
#' \if{latex}{\figure{cmediavardescmuestra.png}{options: width=3cm}}
#'
#' (2.2) usando la cuasi-varianza muestral
#'
#' \if{html}{\figure{cmediavardesccuasi.png}{width = 240px}}
#' \if{latex}{\figure{cmediavardesccuasi.png}{options: width=3cm}}
#'
#' Nota: en ambos casos el estadístico T se distrubuye como un t con n-1 grados de libertad.
#'
#' @seealso \code{\link{ic.media}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pnorm na.omit dnorm pt qt dt
#' @import dplyr ggplot2
#'
#' @export
contraste.media <- function(x,
                            variable = NULL,
                            introducir = FALSE,
                            var_pob = c("conocida","desconocida"),
                            hipotesis_nula = NULL,
                            tipo_contraste =  c("bilateral","cola derecha","cola izquierda"),
                            alfa = 0.05,
                            grafico = FALSE){

var_pob <- tolower(var_pob)
var_pob <- match.arg(var_pob)


tipo_contraste <- tolower(tipo_contraste)
tipo_contraste <- match.arg(tipo_contraste)


if(is.null(hipotesis_nula) | !is.numeric(hipotesis_nula)){

  stop("Tienes que introducir un valor para la hip\u00f3tesis nula")

} else{

  H0 <- hipotesis_nula

}


if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 1){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

    }

  } else if(length(variable) == 1){

    if(is.numeric(variable)){

      if(variable <= length(x)){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es v\u00e1lido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular contraste hay que seleccionar una variable")
    stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

  }

  x <- na.omit(x)
  clase <- sapply(x, class)

  if (!clase %in% c("numeric","integer")) {

    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")

  }

  # tama\u00f1o de la muestra
  n1 <- nrow(x)
  n <- n1
  gl <- n-1

  # media muestral
  media <- as.numeric(media(x))

  if(var_pob == "conocida"){

    varianza_pob <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
    varianza_pob <- as.numeric(varianza_pob)
    desv_pob <- as.numeric(sqrt(varianza_pob))

  } else{

    var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      desv_mu = as.numeric(desviacion(x))
      n <- n-1

    } else{

      desv_mu <- as.numeric(desviacion(x,tipo="cuasi"))

      }

  }


} else{   # aqu\u00ed empieza introducir datos

  n <- readline(prompt = "Introducir el tama\u00f1o de la muestra: ")
  n1 <- as.numeric(n)
  n <- n1
  gl <- n-1

  media <- readline(prompt = "Introducir el valor de la media muestral: ")
  media <- as.numeric(media)

  if(var_pob == "conocida"){
    varianza_pob <- readline(prompt = "Introducir el valor de la varianza poblacional: ")
    varianza_pob <- as.numeric(varianza_pob)
    desv_pob <- sqrt(varianza_pob)

  } else{

    var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      varianza_muestral <- readline(prompt = "Introducir el valor de la varianza muestral: ")
      varianza_muestral <- as.numeric(varianza_muestral)
      desv_mu <- sqrt(varianza_muestral)
      n <- n-1

    } else{

      varianza_cuasi <- readline(prompt = "Introducir el valor de la cuasi-varianza muestral: ")
      varianza_cuasi <- as.numeric(varianza_cuasi)
      desv_mu <- sqrt(varianza_cuasi)

    }
  }
}



# calculo contrastes


    if(var_pob == "conocida"){

      aproximacion <- 0

      error_tipico <-  desv_pob / sqrt(n1)

    }

    if(var_pob == "desconocida" & (n>30)) {

      aproximacion <- as.numeric(readline(prompt = 'Quieres utilizar la aproximaci\u00f3n de la t a la normal?: \n 1. "S\u00ed" \n 2. "No, usar la t" \n'))

      if(aproximacion == 1){

        error_tipico <-  desv_mu / sqrt(n1)

      } else {

        error_tipico <-  desv_mu / sqrt(n) #definido como n-1 en var muestra y n en causi

      }
    }

    if(var_pob == "desconocida" & (n<=30)) {

      aproximacion <- 0
      error_tipico <-  desv_mu / sqrt(n) #definido como n-1 en var muestra y n en causi

    }

  estadistico.prueba <- (media - H0) / error_tipico
  estadistico.prueba <- as.numeric(estadistico.prueba)


  if(alfa >= 0 & alfa <=1){

    if(var_pob == "conocida" | aproximacion == 1){

      if(tipo_contraste == "bilateral"){
        valor_critico <- qnorm(alfa/2,lower.tail = F)
      }
      if(tipo_contraste == "cola izquierda"){
        valor_critico <- qnorm(alfa,lower.tail = T)
      }
      if(tipo_contraste == "cola derecha"){
        valor_critico <- qnorm(alfa,lower.tail = F)
      }

    } else{

      if(tipo_contraste == "bilateral"){
        valor_critico <- qt(alfa/2,gl,lower.tail = F)
      }
      if(tipo_contraste == "cola izquierda"){
        valor_critico <- qt(alfa,gl,lower.tail = T)
      }
      if(tipo_contraste == "cola derecha"){
        valor_critico <- qt(alfa,gl,lower.tail = F)
      }

    }

    valor_critico <- round(valor_critico,4)

  } else{

    stop("El nivel de significaci\u00f3n debe fijarse entre 0 y 1")

  }



if(tipo_contraste == "bilateral"){

  estadistico.prueba2 <- abs(estadistico.prueba)
  media_inf <- H0 - valor_critico * error_tipico
  media_sup <- H0 + valor_critico * error_tipico

  if(var_pob == "conocida" | aproximacion == 1){

    pvalor <- 2 * pnorm(estadistico.prueba2,lower.tail=FALSE)

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-4, -valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-valor_critico, valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 4)) +
        geom_vline(xintercept = -estadistico.prueba2, linetype = "dashed") +
        geom_vline(xintercept = estadistico.prueba2, linetype = "dashed") +
        labs(x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.prueba2,-estadistico.prueba2,-valor_critico,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

      df <- data.frame(x = c(H0-4*error_tipico, H0+4*error_tipico))
      plot2 <- ggplot(df, aes(x)) +
        geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(df[1,1], media_inf)) +
        geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "darkgreen", xlim = c(media_inf, media_sup)) +
        geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(media_sup, df[2,1])) +
        geom_vline(xintercept = media, linetype = "dashed") +
        labs(x = "", y = "",title="Intervalo de la media muestral\n(supuesta H0 cierta)") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(media_inf,media,media_sup)) +
        theme(axis.text.x = element_text(angle = 45))


    }


  } else{

    pvalor <- 2 * pt(estadistico.prueba2,gl, lower.tail=FALSE)

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "darkgreen", xlim = c(-valor_critico, valor_critico)) +
        geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(-4, -valor_critico)) +
        geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(valor_critico, 4)) +
        geom_vline(xintercept = 0, color = "black") +
        geom_vline(xintercept = -estadistico.prueba2, color = "blue", linetype = "dashed") +
        geom_vline(xintercept = estadistico.prueba2, color = "blue", linetype = "dashed") +
        labs(title = paste("Distribuci\u00f3n t con ", gl, " grados de libertad",sep=""), x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(-estadistico.prueba2,estadistico.prueba2,valor_critico,-valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

      plot2 <- NULL


    }

  }

  if(estadistico.prueba >= -valor_critico & estadistico.prueba <=  valor_critico){

    print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  } else{

    print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [", -valor_critico," , ",valor_critico,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  }

}

if(tipo_contraste == "cola derecha"){

  media_inf <- -Inf
  media_sup <- H0 + valor_critico * error_tipico

  if(var_pob == "conocida" | aproximacion == 1){

    pvalor <- pnorm(estadistico.prueba,lower.tail=FALSE)

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(-4L,valor_critico)) +
        geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(valor_critico, 4L)) +
        geom_vline(xintercept = estadistico.prueba, linetype = "dashed") +
        labs(x = "z", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.prueba,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

      df <- data.frame(x = c(H0-4*error_tipico, H0+4*error_tipico))
      plot2 <- ggplot(df, aes(x)) +
        geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "darkgreen", xlim = c(df[1,1], media_sup)) +
        geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(media_sup, df[2,1])) +
        geom_vline(xintercept = media, linetype = "dashed") +
        labs(x = "", y = "",title="Intervalo de la media muestral\n(supuesta HO cierta)") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(media,media_sup)) +
        theme(axis.text.x = element_text(angle = 45))

    }


  } else{

    pvalor <- pt(estadistico.prueba,gl, lower.tail=FALSE)

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "darkgreen", xlim = c(-4, valor_critico)) +
        geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(valor_critico, 4)) +
        geom_vline(xintercept = 0, color = "black") +
        geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
        labs(title = paste("Distribuci\u00f3n t con ", gl, " grados de libertad",sep=""), x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.prueba,valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

      plot2 <- NULL


    }


  }

  if(estadistico.prueba > valor_critico){

    print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  } else{

    print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo ]-Inf , ", valor_critico,"]",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  }


}

if(tipo_contraste == "cola izquierda"){

  media_inf <- H0 + valor_critico * error_tipico # valor critico negativo
  media_sup <- Inf

  if(var_pob == "conocida" | aproximacion == 1){

    pvalor <- pnorm(estadistico.prueba,lower.tail=TRUE)

    if(isTRUE(grafico)){

        plot <- ggplot(NULL, aes(c(-4,4))) +
          geom_area(stat = "function", fun = dnorm, fill = "red", xlim = c(-4L, valor_critico)) +
          geom_area(stat = "function", fun = dnorm, fill = "darkgreen", xlim = c(valor_critico, 4L)) +
          geom_vline(xintercept = estadistico.prueba, linetype = "dashed") +
          labs(x = "z", y = "") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(estadistico.prueba,-valor_critico)) +
          theme(axis.text.x = element_text(angle = 45))

        df <- data.frame(x = c(H0-4*error_tipico, H0+4*error_tipico))
        plot2 <- ggplot(df, aes(x)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "red", xlim = c(df[1,1], media_inf)) +
          geom_area(stat = "function", fun = dnorm, args= list(mean = H0, sd = error_tipico), fill = "darkgreen", xlim = c(media_inf, df[2,1])) +
          geom_vline(xintercept = media, linetype = "dashed") +
          labs(x = "", y = "",title="Intervalo de la media muestral\n(supuesta H0 cierta)") +
          scale_y_continuous(breaks = NULL) +
          scale_x_continuous(breaks = c(media_inf,media)) +
          theme(axis.text.x = element_text(angle = 45))

      }

  } else{

    pvalor <- pt(estadistico.prueba,gl, lower.tail=TRUE)

    if(isTRUE(grafico)){

      plot <- ggplot(NULL, aes(c(-4,4))) +
        geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "darkgreen", xlim = c(valor_critico,4)) +
        geom_area(stat = "function", fun = dt, args = list(df = gl), fill = "red", xlim = c(-4,valor_critico)) +
        geom_vline(xintercept = 0, color = "black") +
        geom_vline(xintercept = estadistico.prueba, color = "blue", linetype = "dashed") +
        labs(title = paste("Distribuci\u00f3n t con ", gl, " grados de libertad",sep=""), x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(estadistico.prueba,-valor_critico)) +
        theme(axis.text.x = element_text(angle = 45))

      plot2 <- NULL


    }



  }

  if(estadistico.prueba < valor_critico){

    print(paste("Se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) no se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  } else{

    print(paste("No se rechaza la hip\u00f3tesis nula. La regi\u00f3n de aceptaci\u00f3n viene dada por el intervalo [ ",valor_critico," , inf[",sep=""))
    print("El valor del estad\u00edstico de prueba (o valor experimental) se encuentra dentro de la regi\u00f3n de aceptaci\u00f3n")

  }

  }



  CH <- cbind(H0,estadistico.prueba,pvalor)
  CH <- as.data.frame(CH)
  names(CH) <- c("Hip\u00f3tesis nula", "estad\u00edstico de prueba", "p-valor")
  row.names(CH) <- NULL

  Imedia <- cbind(`limite_inferior`=media_inf,`limite_superior`=media_sup)


  if(isTRUE(grafico)){

    plots <- list(plot,plot2)

    return(list(`Estadistico`=CH,`Intervalo de la media muestral (supuesta H0 cierta)`= Imedia,`Graficos`= plots))

  } else{

    return(list(`Estadistico`=CH,`Intervalo de la media muestral (supuesta H0 cierta)`= Imedia))

  }

}

#' @title Cuantiles.
#'
#' @description Calcula los cuantiles.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcuantiles.png}{width = 200px}}
#' \if{latex}{\figure{qrcuantiles.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param pesos Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.
#' @param cortes Vector con los puntos de corte a calcular. Por defecto se calcula el primer, segundo y tercer cuartil.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return Si \code{pesos = NULL}, la función devuelve los cuantiles de todas las variables seleccionadas en un objeto de tipo \code{data.frame}. En caso contrario, devuelve los cuantiles de la variable para la que se ha facilitado la distribución de frecuencias.
#'
#' @author
#' \strong{Vicente Coll-Serrano} (\email{vicente.coll@@uv.es}).
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú} (\email{rosario.martinez@@uv.es}).
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García} (\email{cristina.pardo-garcia@@uv.es}).
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' @details
#'
#' Los cuantiles se obtienen a partir de la siguiente regla de decisión:
#'
#' \if{html}{\figure{cuantiles.png}{width = 680px}}
#' \if{latex}{\figure{cuantiles.png}{options: scale=.85}}
#'
#' Ni son las frecuencias acumuladas y n el tamaño de la muestra (o N si es la población).
#'
#' cuartiles: s=1,2,3 y k=4
#'
#' deciles: s= 1,2,...,9 y k=10
#'
#' percentiles: s=1,2,...,99 y k=100
#'
#' @seealso \code{\link{media}}, \code{\link{mediana}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' cuantiles1 <- cuantiles(startup[1])
#' cuantiles2 <- cuantiles(startup,variable=1,cortes=seq(0.1,0.9,0.1))
#' cuantiles3 <- cuantiles(salarios2018,variable=6,pesos=7 )
#'
#' @importFrom stats quantile
#'
#' @export
cuantiles <- function(x, variable = NULL, pesos = NULL,
                      cortes = c(0.25, 0.5, 0.75),
                      exportar = FALSE) {

  # Asegurar data.frame
  if (!is.data.frame(x)) x <- as.data.frame(x)

  # --- Seleccion de variables ---
  if (is.null(variable)) {
    varnames <- names(x)[sapply(x, is.numeric)]
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    varnames <- names(x)[variable]
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("Nombre de variable no v\u00e1lido")
    varnames <- variable
  } else {
    stop("El argumento 'variable' debe ser num\u00e9rico o de tipo car\u00e1cter")
  }

  x_sel <- x[, varnames, drop = FALSE]

  # --- Manejo de pesos ---
  if (!is.null(pesos)) {
    if (length(varnames) > 1 || length(pesos) > 1)
      stop("Para calcular cuantiles ponderados solo puedes seleccionar una variable y un peso")

    if (is.character(pesos)) {
      if (!pesos %in% names(x)) stop("Nombre de pesos no vl\u00e1ido")
      pesos_name <- pesos
    } else if (is.numeric(pesos)) {
      if (any(pesos > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de pesos")
      pesos_name <- names(x)[pesos]
    } else {
      stop("El argumento 'pesos' debe ser num\u00e9rico o de tipo car\u00e1cter")
    }

    if (pesos_name == varnames)
      stop("No puedes usar la misma variable como dato y como peso")

    x_sel <- data.frame(variable = x[[varnames]], pesos = x[[pesos_name]])
    varnames <- varnames[1]
  }

  cortes <- sort(cortes)

  # --- Calculo de cuantiles usando tu funcion interna ---
  if (is.null(pesos)) {
    cuantiles_mat <- apply(x_sel, 2, .cuantiles.int, cortes = cortes)
  } else {
    cuantiles_mat <- .cuantiles.int(x_sel$variable, pesos = x_sel$pesos, cortes = cortes)
  }

  # Asegurar que es data.frame y poner nombres
  cuantiles_df <- as.data.frame(cuantiles_mat)
  rownames(cuantiles_df) <- paste0(cortes*100, "%")
  names(cuantiles_df) <- paste0("cuantiles_", varnames)

  # --- Exportar a Excel ---
  if (exportar) {
    filename <- paste0("Cuantiles_", format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), ".xlsx")
    wb <- openxlsx::createWorkbook()
    openxlsx::addWorksheet(wb, "Cuantiles")
    resumen_export <- cbind(Cuantil = rownames(cuantiles_df), cuantiles_df)
    rownames(resumen_export) <- NULL
    openxlsx::writeData(wb, "Cuantiles", resumen_export)
    openxlsx::addStyle(wb, "Cuantiles",
                       style = openxlsx::createStyle(numFmt = "0.0000"),
                       rows = 2:(nrow(resumen_export)+1),
                       cols = 2:(ncol(resumen_export)+1),
                       gridExpand = TRUE)
    openxlsx::saveWorkbook(wb, filename, overwrite = TRUE)
  }

  class(cuantiles_df) <- c("resumen", class(cuantiles_df))
  return(cuantiles_df)
}

#' @title Unir vectores.
#'
#' @description Une dos o más vectores numéricos de igual o distinta longitud.
#'
#' \if{html}{\figure{qrleerdatos.png}{width = 200px}}
#' \if{latex}{\figure{qrleerdatos.png}{options: width=3cm}}
#'
#' @usage unir.vectores(...)
#'
#' @param ... Introducir los nombres de los objetos, vectores, que se quiere unir. Si los vectores tienen distinta longitud se rellenarán los espacios con NAs.
#'
#' @return La función devuelve un dataframe.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @export
unir.vectores <- function(...){

  names <- as.character(substitute(list(...)))[-1L]

  if (length(names)<2)
    warning("No hay vectores que unir")

  lista <- list(...)
  max.length <- max(sapply(lista, length))

  lista <- lapply(lista, function(x) { c(x, rep(NA, max.length-length(x)))})

  df <- do.call(cbind, lista)
  df <- as.data.frame(df)
  names(df) <- names

  return(df)
}

#' @title Distribuciones de probabilidad.
#'
#' @description Aplicación interactiva donde se representa las principales distribuciones de probabilidad unidimensionales: Binomial, Poisson, Uniforme, Exponencial y Normal.
#' @usage distribuciones.probabilidad()
#'
#' @return No devuelve un valor, Es una aplicación shiny.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @import shiny shinydashboard
#'
#' @export
distribuciones.probabilidad <- function() {

  appDir <- system.file("examples/probabilidad", package = "estadistica")

  if (appDir == "") {
    stop("Could not find example directory. Try re-installing `mypackage`.", call. = FALSE)
  }

  shiny::runApp(appDir, display.mode = "normal")

}

#' @title Intervalo confianza para la razón (cociente) de varianzas.
#'
#' @description Calcula el intervalo de confianza para la razón (o cociente) de varianzas.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qricrazonvarianzas.png}{width = 200px}}
#' \if{latex}{\figure{qricrazonvarianzas.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere a dos variables, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param media_pob Es un carácter. Por defecto se supone que la media poblacional es desconocida (\code{media_pob="desconocida"})
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return Devuelve el intervalo del cociente de varianzas poblacionales en un objeto de tipo \code{data.frame}. Si \code{grafico = T} devuelve una \code{list} con el intervalo de confianza y su representación gráfica.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' Esta función calcula el intervalo de confianza para el cociente entre la varianza poblacional de la muestra 1 y la de la muestra 2, es decir:
#'
#' \if{html}{\figure{iccocientevar.png}{width = 80px}}
#' \if{latex}{\figure{iccocientevar.png}{options: width=1cm}}
#'
#' Para obtener los intervalos de confianza se opera sobre el estadístico F que se facilita en la nota y que se utiliza para obtener el intervalo del cociente de la varianza de la muestra 2 y la muestra 1.
#'
#' Los intervalos se obtienen bajo el supuesto de que la media poblacional es desconocida:
#'
#' (1) si se trabaja con las varianzas muestrales
#'
#' \if{html}{\figure{iccocientevarmuestra.png}{width = 880px}}
#' \if{latex}{\figure{iccocientevarmuestra.png}{options: width=14cm}}
#'
#' (2) si se trabaja con las cuasi-varianzas muestrales
#'
#' \if{html}{\figure{iccocientevarcuasi.png}{width = 640px}}
#' \if{latex}{\figure{iccocientevarcuasi.png}{options: width=14cm}}
#'
#'
#' @note
#' En el caso de querer deducir el intervalo recíproco, es decir:
#'
#' \if{html}{\figure{iccocientevar2.png}{width = 80px}}
#' \if{latex}{\figure{iccocientevar2.png}{options: width=1cm}}
#'
#' se parte del estadístico:
#'
#' \if{html}{\figure{Frazonvar.png}{width = 400px}}
#' \if{latex}{\figure{Frazonvar.png}{options: width=8cm}}
#'
#' (3) si se trabaja con las varianzas muestrales
#'
#' \if{html}{\figure{iccocientevarmuestra2.png}{width = 880px}}
#' \if{latex}{\figure{iccocientevarmuestra2.png}{options: width=14cm}}
#'
#' (4) si se trabaja con las cuasi-varianzas muestrales
#'
#' \if{html}{\figure{iccocientevarcuasi2.png}{width = 640px}}
#' \if{latex}{\figure{iccocientevarcuasi2.png}{options: width=13cm}}
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pf qf df na.omit
#' @import dplyr ggplot2 cowplot
#'
#' @export
ic.razon.varianzas <- function(x,
                               variable = NULL,
                               introducir = FALSE,
                               media_pob = c("desconocida","conocida"),
                               confianza = 0.95,
                               grafico = FALSE){


  media_pob <- tolower(media_pob)
  media_pob <- match.arg(media_pob)

if(confianza >= 0 & confianza <=1){

  confianza <- confianza
  alfa_2 <- (1-confianza)/2

} else{

  stop("El nivel de confianza debe fijarse entre 0 y 1")

}

if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 2){
      x <- x
    } else{
      warning("Para calcular el intervalo de confianza hay que seleccionar 2 variables")
      stop("El conjunto de datos seleccionado no tiene la dimensi\u00f3n adecuada")
    }
  } else{

    if(length(variable) == 2){
        if(is.numeric(variable)){
          if(all(variable <= length(x))){
            variable <- variable
          } else{
            stop("Selecci\u00f3n err\u00f3nea de variable")
          }
        }
      if(is.character(variable)){
        if(all(variable %in% varnames)){
          variable = match(variable,varnames)
          } else {
            stop("El nombre de la variable no es v\u00e1lido")
          }
        }

      x <- x[,variable] %>% as.data.frame()
      names(x) <- varnames[variable]

      } else{
        warning("Para calcular el intervalo de confianza de la raz\u00f3n de varianzas hay que seleccionar dos variables")
        stop("El conjunto de datos seleccionado parece ser no v\u00e1lido")
      }
  }

  clase <- sapply(x, class)

  if (!all(clase %in% c("numeric","integer"))){
    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")
  }

  # tama\u00f1o de la muestra
  n1 <- length(x[1][!is.na(x[1])])
  n2 <- length(x[2][!is.na(x[2])])


  var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

  if(var_muestra == 1){

    var_mu1 <- as.numeric(varianza(x[1]))
    var_mu2 <- as.numeric(varianza(x[2]))

  } else{

    var_mu1 <- as.numeric(varianza(x[1], tipo = "cuasi"))
    var_mu2 <- as.numeric(varianza(x[2], tipo = "cuasi"))

  }

} else{   # aqu\u00ed empieza introducir datos

  print("A continuaci\u00f3n, vas a introducir los datos de las muestras.")

  n1 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 1: ")
  n1 <- as.numeric(n1)

  n2 <- readline(prompt = "Introducir el tama\u00f1o de la muestra 2: ")
  n2 <- as.numeric(n2)

  var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(var_muestra == 1){

      var_mu1 <- readline("Introduce el valor de la varianza muestral 1: ")
      var_mu1 <- as.numeric(var_mu1)

      var_mu2 <- readline("Introduce el valor de la varianza muestral 2: ")
      var_mu2 <- as.numeric(var_mu2)

    } else{

      var_mu1 <- readline("Introduce el valor de la cuasivarianza muestral 1: ")
      var_mu1 <- as.numeric(var_mu1)

      var_mu2 <- readline("Introduce el valor de la cuasivarianza muestral 2: ")
      var_mu2 <- as.numeric(var_mu2)

    }

}

# calculo del intervalo de confianza

  # IC del cociente de varianzas con medias desconocidas

  if(media_pob == "desconocida"){

    print("Se calcula el intervalo de confianza para el cociente de varianzas supuestas desconocidas las medias poblacionales")

    valor_critico1 <- qf(1-alfa_2, df1= n1-1, df2 = n2-1, lower.tail = F)
    valor_critico2 <- qf(alfa_2, df1= n1-1, df2 = n2-1, lower.tail = F)

    if(var_muestra == 1){

      # caso 1.1

      limite_inferior <- (n1/(n1-1))*((n2-1)/n2)*(var_mu1/var_mu2) * (1/valor_critico2)
      limite_superior <- (n1/(n1-1))*((n2-1)/n2)*(var_mu1/var_mu2) * (1/valor_critico1)

    } else {

      # caso 1.2
      print("Este es el intervalo de confianza que generalmente calculan los softwares (SPSS, Excel, Stata, etc.)")

      limite_inferior <- (var_mu1/var_mu2) * (1/valor_critico2)
      limite_superior <- (var_mu1/var_mu2) * (1/valor_critico1)

    }

  } else {
    # las medias poblacionales conocidas

    valor_critico1 <- qf(1-alfa_2, df1= n1, df2 = n2, lower.tail = F)
    valor_critico2 <- qf(alfa_2, df1= n1, df2 = n2, lower.tail = F)

    if(var_muestra == 1){

      stop("Lo sentimos, este caso a\u00fan no est\u00e1 implementado")

    } else {

      factor1 <- (n1-1)/n1
      factor2 <- (n2-1)/n2

      limite_inferior <- ((factor1*var_mu1)/(factor2*var_mu2)) * valor_critico1
      limite_superior <- ((factor1*var_mu1)/(factor2*var_mu2))  * valor_critico2

    }

  }

  if(grafico){

      percentil99 <- qf(.9999, df1= n1-1, df2 = n2-1)

      data <- data.frame(x=seq(from = 0, to = percentil99, percentil99/200))
      data$y <-df(data$x, df1= n1-1, df2 = n2-1)

      plot1 <- ggplot(data, aes(x,y)) +
        geom_area(fill="darkgreen") +
        geom_area(data=subset(data,x<valor_critico1), fill = "grey") +
        geom_area(data=subset(data,x>valor_critico2),fill = "grey") +
        geom_vline(xintercept = 0L, color = "black") +
        labs(title = paste("Distribuci\u00f3n F con ", n1-1, " y ",n2-1," grados de libertad",sep=""), x = "", y = "") +
        scale_y_continuous(breaks = NULL) +
        scale_x_continuous(breaks = c(round(0L,0),round(valor_critico1,3),round(valor_critico2,3))) +
        theme(axis.text.x = element_text(angle = 45)) +
        geom_point(aes(x= valor_critico1 , y=0), color = "red", size = 3) +
        geom_point(aes(x= valor_critico2 , y=0), color = "blue", size = 3)

      intervalo <- data.frame(ic = round(c(inferior=limite_inferior,superior=limite_superior),4),y=c(0,0))

      plot2 <- ggplot(intervalo,aes(x= ic,y)) +
        geom_line(aes(group = y), color = "grey",size = 3)+
        geom_point(aes(color=ic), size=3,show.legend = FALSE) +
        geom_text(aes(label = ic), size = 2.5, vjust=2) +
        scale_y_continuous(expand=c(0,0)) +
        scale_color_gradientn(colours=c("red","blue"))+
        labs(y="",x="Intervalo de confianza del cociente de varianzas") +
        tema_blanco

      # plot <- grid::grid.draw(rbind(ggplotGrob(plot1), ggplotGrob(plot2), size = "last"))
      plot <- cowplot::plot_grid(plot1, plot2, ncol = 1, align = "v", rel_heights = c(1, 1))

    }

  IC <- cbind(limite_inferior,limite_superior)
  IC <- as.data.frame(IC)
  row.names(IC) <- NULL

  if(grafico){

    return(list(IC,plot))

  } else{

    return(IC)

  }

}

#' @title Distribución normal.
#'
#' @description Aplicación interactiva para comparar dos distribuciones normales.
#' @usage distribucion.normal()
#'
#' @return No devuelve un valor, es una aplicación shiny.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @import shiny
#'
#' @export
distribucion.normal <- function() {
  appDir <- system.file("examples/normal", package = "estadistica")
  if (appDir == "") {
    stop("Could not find example directory. Try re-installing `mypackage`.", call. = FALSE)
  }

  shiny::runApp(appDir, display.mode = "normal")
}

#' @title Intervalo confianza para la varianza.
#'
#' @description Calcula el intervalo de confianza de la varianza poblacional.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qricvarianza.png}{width = 200px}}
#' \if{latex}{\figure{qricvarianza.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Puede ser un vector o un dataframe.
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.
#' @param introducir Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.
#' @param media_poblacion Es un carácter. Indica si la media de la población es desconocida (por defecto, \code{media_poblacion = "desconocida"}) o conocida (en este caso, cambiar \code{media_poblacion = "conocida"}).
#' @param confianza Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)
#' @param grafico Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.
#'
#' @return Devuelve el intervalo de confianza de la varianza poblacional en un objeto de tipo \code{data.frame}. Si \code{grafico = T} devuelve una \code{list} con el intervalo de confianza y su representación gráfica.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) Si la media poblacional es conocida:
#'
#' \if{html}{\figure{icvarianzamedcon.png}{width = 200px}}
#' \if{latex}{\figure{icvarianzamedcon.png}{options: width=8cm}}
#'
#' (2) Si la media poblacional es desconocida.
#'
#' Con la varianza muestral:
#'
#' \if{html}{\figure{icvarianzameddescmuestra.png}{width = 200px}}
#' \if{latex}{\figure{icvarianzameddescmuestra.png}{options: width=6cm}}
#'
#' Con la cuasivarianza muestral:
#'
#' \if{html}{\figure{icvarianzameddesccuasi.png}{width = 200px}}
#' \if{latex}{\figure{icvarianzameddesccuasi.png}{options: width=8cm}}
#'
#' Nota: En todos los casos se obtiene el valor de la chi-dos con n grados de libertad que deja a su derecha una probabilidad de alfa y 1-alfa.
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats pchisq qchisq na.omit
#' @import dplyr ggplot2 cowplot
#'
#' @export
ic.varianza <- function(x,
                        variable = NULL,
                        introducir = FALSE,
                        media_poblacion = c("desconocida","conocida"),
                        confianza = 0.95,
                        grafico = FALSE){

  media_poblacion <- tolower(media_poblacion)
  media_poblacion <- match.arg(media_poblacion)

if(confianza >= 0 & confianza <=1){

  confianza <- confianza
  alfa_2 <- (1-confianza)/2

} else{

  stop("El nivel de confianza debe fijarse entre 0 y 1")

}

if(isFALSE(introducir)) {

  x <- data.frame(x)
  varnames <- names(x)

  if(is.null(variable)){

    if(length(x) == 1){

      x <- x

    } else{

      warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
      stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

    }

  } else if(length(variable) == 1){

    if(is.numeric(variable)){

      if(variable <= length(x)){

        variable <- variable

      } else{

        stop("Selecci\u00f3n err\u00f3nea de variable")

      }
    }

    if(is.character(variable)){

      if(all(variable %in% varnames)){

        variable = match(variable,varnames)

      } else {

        stop("El nombre de la variable no es v\u00e1lido")

      }

    }

    x <- x[,variable] %>% as.data.frame()
    names(x) <- varnames[variable]

  } else{

    warning("Para calcular el intervalo de confianza hay que seleccionar una variable")
    stop("El conjunto de datos seleccionado tiene mas de 1 variable.")

  }

  x <- na.omit(x)
  clase <- sapply(x, class)

  if (!clase %in% c("numeric","integer")) {

    stop("No puede calcularse el intervalo de confianza porque la variable seleccionada no es cuantitativa")

  }

  # tama\u00f1o de la muestra y grados libertad
  n <- nrow(x)
  gl <- n-1

  if(media_poblacion == "desconocida"){

    varianza_muestral <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(varianza_muestral == 1){

      var_mu <- as.numeric(varianza(x))

    } else{

      var_mu <- as.numeric(varianza(x, tipo = "cuasi"))
      n <- n-1

      print("Este es el intervalo de confianza que generalmente calculan los softwares (SPSS, Excel, Stata, ...)")

    }

  } else{

    print("La media poblacional no suele conocerse, este supuesto es te\u00f3rico")

    media <- readline(prompt = "Introducir el valor de la media poblacional: ")
    media <- as.numeric(media)

    sumatorio <- sum((x - media)^2)

  }

} else{   # aquí empieza introducir datos

  n <- readline(prompt = "Introducir el tama\u00f1o de la muestra: ")
  n <- as.numeric(n)
  gl <- n-1

  if(media_poblacion == "desconocida"){

    varianza_muestral <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

    if(varianza_muestral == 1){

      var_mu <- readline("Introduce el valor de la varianza muestral: ")
      var_mu <- as.numeric(var_mu)
      n <- n

    } else{

      var_mu <- readline("Introduce el valor de la cuasivarianza muestral: ")
      var_mu <- as.numeric(var_mu)
      n <- n-1

      print("Este es el intervalo de confianza que generalmente calculan los softwares (SPSS, Stata, Excel,...)")

    }

  } else{

    print("La media poblacional no suele conocerse, este supuesto es te\u00f3rico")

    media <- readline(prompt = "Introducir el valor de la media poblacional: ")
    media <- as.numeric(media)
    sumatorio <- readline(prompt = "Introducir el valor de la suma cuadratica de las desviaciones de los valores muestrales respecto a la media poblacional: ")
    sumatorio <- as.numeric(sumatorio)

  }

}

# calculo de los intervalos de confianza

if(media_poblacion == "desconocida"){
  # caso 1. Media poblacional desconocida, n peque\u00f1a
  print("Intervalo de confianza para la varianza poblacional, supuesta desconocida la media poblacional.")

  valor_critico1 <- qchisq(alfa_2,lower.tail = F, df= gl)
  valor_critico2 <- qchisq(1-alfa_2, lower.tail = F, df= gl)

  limite_inferior <- (n * var_mu) / valor_critico1
  limite_superior <- (n * var_mu) / valor_critico2

} else{

  # caso 2. Media poblacional conocida, n peque\u00f1a
  print("Intervalo de confianza para la varianza poblacional, supuesta conocida la media poblacional. n peque\u00f1a")
  gl <- gl +1
  valor_critico1 <- qchisq(alfa_2,lower.tail = F, df= gl)
  valor_critico2 <- qchisq(1-alfa_2, lower.tail = F, df= gl)

  limite_inferior <- sumatorio / valor_critico1
  limite_superior <- sumatorio / valor_critico2

}

  if(grafico){

    percentil99 <- qchisq(.9999, gl)

    df <- data.frame(x=seq(from = 0, to = percentil99, percentil99/200))
    df$y <-dchisq(df$x, gl)

    plot1 <- ggplot(df) +
      geom_path(aes(x,y))+
      geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "darkgreen", xlim = c(valor_critico2, valor_critico1)) +
      geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "grey", xlim = c(0, valor_critico2)) +
      geom_area(stat = "function", fun = dchisq, args = list(df = gl), fill = "grey", xlim = c(valor_critico1, percentil99)) +
      geom_vline(xintercept = 0, color = "black") +
      labs(title = paste("Distribuci\u00f3n chi con ", gl, " grados de libertad",sep=""), x = "", y = "") +
      scale_y_continuous(breaks = NULL) +
      scale_x_continuous(breaks = c(0L,round(valor_critico2,4),round(valor_critico1,4))) +
      theme(axis.text.x = element_text(angle = 45))+
      geom_point(aes(x= valor_critico2 , y=0), color = "orange4", size = 3) +
      geom_point(aes(x= valor_critico1 , y=0), color = "lightblue4", size = 3)

    intervalo <- data.frame(ic = round(c(inferior=limite_inferior,superior=limite_superior),4),y=c(0,0))

    plot2 <- ggplot(intervalo,aes(x= ic,y)) +
      geom_line(aes(group = y), color = "grey",size = 3)+
      geom_point(aes(color=ic), size=3,show.legend = FALSE) +
      geom_text(aes(label = ic), size = 2.5, vjust=2) +
      scale_y_continuous(expand=c(0,0)) +
      scale_color_gradientn(colours=c("red","blue"))+
      labs(y="",x="Intervalo de confianza") +
      tema_blanco

    # plot <- grid::grid.draw(rbind(ggplotGrob(plot1), ggplotGrob(plot2), size = "last"))
    plot <- cowplot::plot_grid(plot1, plot2, ncol = 1, align = "v", rel_heights = c(1, 1))

  }


  IC <- cbind(limite_inferior,limite_superior)
  IC <- as.data.frame(IC)
  row.names(IC) <- NULL

  if(grafico){

    return(list(IC,plot))

  } else{

    return(IC)

  }


}

#' @title Tamaño de la muestra.
#'
#' @description Calcula el tamaño muestral para estimar la media de una población normal o la proporcion p de una población.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrmuestra1.png}{width = 200px}}
#' \if{latex}{\figure{qrmuestra1.png}{options: width=3cm}}
#'
#' \if{html}{\figure{qrmuestra2.png}{width = 200px}}
#' \if{latex}{\figure{qrmuestra2.png}{options: width=3cm}}
#'
#' @param poblacion Texto, si \code{poblacion = "normal"} (por defecto), calcula el tamaño muestral que permita estimar la media de una población normal. Si \code{poblacion = "dicotomica"}, para estimar la proporción p de una población.
#' @param error_estimacion Es un valor que establece el error de estimación. Es la semiamplitud (mitad de la precisión) del intervalo de confianza. Esta aproximación solo es válida en distribuciones simétricas (normal o t-student).
#' @param confianza Es un valor entre 0 y 1 que indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento).
#' @param irrestricto Es un valor lógico que indica si se considera un muestreo aleatorio simple (por defecto, \code{irrestricto = FALSE}) o sin reemplazamiento (\code{irrestricto = TRUE}).
#'
#' @return Devuelve el tamaño de la muesta en un objeto de tipo \code{data.frame}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) El tamaño muestral para estimar la media poblacional se obtiene a partir de la siguiente expresión:
#'
#' \if{html}{\figure{tamanomedia.png}{options: style="width: 30\%;"}}
#' \if{latex}{\figure{tamanomedia.png}{options: width=4cm}}
#'
#' y si el muestreo es irrestricto:
#'
#' \if{html}{\figure{tamanomediairrestricto.png}{width = 480px}}
#' \if{latex}{\figure{tamanomediairrestricto.png}{options: width=8cm}}
#'
#' Nota: si la varianza poblacional no es conocida puede estimarse a través de la varianza (o cuasi-varianza) muestral.
#'
#' (2) El tamaño muestral para estimar la proporción de una característica se obtiene a partir de la expresión:
#'
#' \if{html}{\figure{tamanoproporcion.png}{width = 280px}}
#' \if{latex}{\figure{tamanoproporcion.png}{options: width=4.5cm}}
#'
#' y si el muestreo es irrectricto:
#'
#' \if{html}{\figure{tamanoproporcionirrestricto.png}{width = 520px}}
#' \if{latex}{\figure{tamanoproporcionirrestricto.png}{options: width=10cm}}
#'
#' Nota: puede estimarse la proporción poblacional por la proporción muestral o, en caso
#' de no disponer de información, suponer el caso más desfavorable: p=q=0.5

#' @note
#' En el caso del tamaño muestral para la media: si la varianza poblacional no es conocida puede estimarse con la varianza muestral (o cuasivarianza muestral).
#' En el caso del tamaño muestral para la proporción: si la proporción poblacional no es conocida, puede estimarse por la proporción muestral o considerar el caso más desfavorable (p=q=0.5)
#'
#' @references
#' Casas José M. (1997) Inferencia estadística. Editoral: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X
#'
#' Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' @importFrom stats qt pt qnorm pnorm
#'
#' @import dplyr
#'
#' @export
muestra <- function(poblacion = c("normal","dicotomica"),
                    error_estimacion = NULL,
                    confianza = 0.95,
                    irrestricto = FALSE){



  poblacion <- tolower(poblacion)
  poblacion <- match.arg(poblacion)

  if(is.null(error_estimacion)){

    stop("Es necesario indicar el error de estimaci\u00f3n (semiamplitud del intervalo de confianza).")

  }

  if(confianza >= 0 & confianza <=1){

    confianza <- confianza
    alfa_2 <- (1-confianza)/2
    valor_critico <- qnorm(alfa_2,lower.tail=FALSE)

  } else{

    stop("El nivel de confianza debe fijarse entre 0 y 1")

  }


  if(poblacion == "normal"){

    varianza_pob <- as.numeric(readline('\u00bfLa varianza poblacional es conocida? \n 1. "S\u00ed" \n 2. "No" \n'))

    if(varianza_pob == 1){

      varianza <- readline(prompt="Introduce el valor de la varianza poblacional: ")

    } else{

      var_muestra <- as.numeric(readline('Selecciona el valor que quieres utilizar: \n 1. "Varianza muestral" \n 2. "Cuasivarianza muestral" \n'))

      if(var_muestra == 1){

        print("Se utilizar\u00e1 la varianza muestral como estimador de la varianza poblacional")
        varianza <- readline(prompt="Introduce el valor de la varianza muestral: ")

        } else{

        print("Se utilizar\u00e1 la cuasi-varianza muestral como estimador de la varianza poblacional")
        n_piloto <- readline(prompt="Introduce el tama\u00f1o de la muestra piloto: ")
        n_piloto <- as.numeric(n_piloto)
        varianza <- readline(prompt="Introduce el valor de la cuasi-varianza muestral de la muestra piloto: ")

        valor_critico <- qt(alfa_2,n_piloto-1,lower.tail=FALSE)

      }

    }

    varianza <- as.numeric(varianza)

    if(isFALSE(irrestricto)){

      tamano <- valor_critico^2 * (varianza / error_estimacion^2)

      if(varianza_pob != 1){ # normal, var desconocida y varianza muestral

        tamano <- 1 + valor_critico^2 * (varianza / error_estimacion^2)

      }

    } else{

      N <- readline(prompt = "Introduce el tama\u00f1o de la poblaci\u00f3n (N): ")
      N <- as.numeric(N)

      numerador <- N * valor_critico^2 * varianza
      denominador <- error_estimacion^2 * (N-1) + valor_critico^2 * varianza
      tamano <- numerador / denominador
    }

  } else{ #poblacion dicotomica

    p_mu <- as.numeric(readline('\u00bfQuieres aproximar la p poblacional por la proporci\u00f3n muestral? \n 1. "S\u00ed" \n 2. "No" \n'))
    p_mu <- as.numeric(p_mu)

    if(p_mu == 1){

    p <- readline(prompt = "Introduce el valor de la proporci\u00f3n muestral: ")
    p <- as.numeric(p)

    } else{

      print("Aproximaci\u00f3n adecuada cuando no se tiene informaci\u00f3n sobre la proporci\u00f3n. Se considerar\u00e1 p=q=0.5")

      p <- 0.5

    }

    if(isFALSE(irrestricto)){

      tamano <- valor_critico^2 *  (p * (1-p) / error_estimacion^2)


    } else{

      N <- readline(prompt = "Introduce el tama\u00f1o de la poblaci\u00f3n (N): ")
      N <- as.numeric(N)

      numerador <- (valor_critico^2 * p * (1-p)) / error_estimacion^2
      denominador <- 1 + (valor_critico^2 * p * (1-p))/(error_estimacion^2 * N)

      tamano <-  numerador / denominador

      }


    }


  tamano <- ceiling(tamano)
  tamano <- as.data.frame(tamano)
  names(tamano) <- "tama\u00f1o de la muestra"

  return(tamano)

}

#' @title Convergencia de la varianza y cuasivarianza muestral.
#'
#' @description Gráfico dinámico que ilustra la convergencia de la varianza y cuasi-varianza muestral a medida que aumenta el tamaño muestral.
#' @usage convergencia.varianza()
#'
#' @return Devuelve un gráfico que es un objeto de la clase \code{plotly} y \code{htmlwidget}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' \strong{Cristina Pardo-García}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @import dplyr ggplot2
#'
#' @export
convergencia.varianza <- function() {

  cat("Al tratarse de un gr\u00e1fico din\u00e1mico llevar\u00e1 unos segundos generarlo.\n\n")
  cat("Para visualizarlo bien haz clic sobre el Zoom")
  sumatorio <- sample(1000:2000,1)

  df <- data.frame(id = 1:100, n = sumatorio/(2:101),n_1 = sumatorio/(1:100))

  df2 <- data.frame(n = rep(df$id,2),
                    varianza = c(df$n,df$n_1),
                    grupo = rep(c("varianza.muestral","cuasi_varianza.muestral"),each=nrow(df)),
                    grupo2 = rep(c("var","cuasivar"),each=nrow(df)))

  p <- ggplot(df2, aes(x=n, y=varianza, group=grupo, frame = n, color = grupo)) +
    geom_line() +
    geom_segment(aes(xend=max(n), yend = varianza), linetype=2, color="darkgreen") +
    geom_point(size = 2) +
    geom_text(aes(x = max(n)+.1, label = paste(sprintf("%5.2f", varianza), "\n ", grupo2 ,sep="")), size = 3, hjust=0) +
    scale_color_manual(values = c("blue","orange4")) +
    labs(title = 'Convergencia de varianza y cuasivarianza muestral', y = 'Tipo varianza', x="Tama\u00f1o de la muestra") +
    theme_bw() +
    theme(panel.border = element_blank(),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank(),
          axis.line = element_line(colour = "black"),
          plot.margin = margin(5.5, 40, 5.5, 5.5))

  plot <- plotly::ggplotly(p)

  plot <- plot %>%
    plotly::layout(showlegend = FALSE) %>%
    plotly::animation_opts(1000, easing = "elastic", redraw = FALSE)

  return(plot)

}

#' @title Matriz de varianzas y covarianzas.
#'
#' @description Obtiene la matriz de varianzas y covarianzas.
#'
#' Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
#'
#' \if{html}{\figure{qrcovarianza.png}{width = 200px}}
#' \if{latex}{\figure{qrcovarianza.png}{options: width=3cm}}
#'
#' @param x Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).
#' @param variable Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} solo tiene 2 variables (columnas), \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables a seleccionar.
#' @param tipo Es un carácter. Por defecto de calcula la matriz de varianzas y covarianzas muestrales (\code{tipo = "muestral"}). Si \code{tipo = "cuasi"}, se calcula la matriz de cuasi-varianzas y cuasi-covarianzas muestrales.
#' @param exportar Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).
#'
#' @return La función devuelve la matriz de varianzas-covarianzas (muestrales, por defecto) de las variables seleccionadas en un \code{data.frame}.
#'
#' @author
#' \strong{Vicente Coll-Serrano}.
#' \emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
#'
#' \strong{Rosario Martínez Verdú}.
#' \emph{Economía Aplicada.}
#'
#' Facultad de Economía. Universidad de Valencia (España)
#'
#' @details
#'
#' (1) Se obtiene la matriz de varianzas y covarianzas muestrales:
#'
#' \if{html}{\figure{matrizvarcovmuestra.png}{width = 400px}}
#' \if{latex}{\figure{matrizvarcovmuestra.png}{options: width=8cm}}
#'
#' (2) Muchos manuales y prácticamente todos los softwares (SPSS, Excel, etc.) facilitan la matriz de cuasi-varianzas y cuasi-covarianzas muestrales:
#'
#' \if{html}{\figure{matrizvarcovcuasi.png}{width = 440px}}
#' \if{latex}{\figure{matrizvarcovcuasi.png}{options: width=8cm}}
#'
#' Nosotros nos referimos a esta expresión como cuasi-covarianza muestral.
#'
#' @note
#' Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la matriz de varianzas y covarianzas poblacional:
#'
#' \if{html}{\figure{matrizvarcovpob.png}{width = 440px}}
#' \if{latex}{\figure{matrizvarcovpob.png}{options: width=8cm}}
#'
#' @seealso \code{\link{varianza}}, \code{\link{desviacion}}
#'
#' @references
#' Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741
#'
#' Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
#'
#' Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
#'
#' @examples
#'
#' matriz_covarianzas1 <- matriz.covar(startup)
#' matriz_covarianzas2 <- matriz.covar(startup, tipo= "cuasi")
#'
#' @importFrom stats na.omit cov
#' @import dplyr
#'
#' @export
matriz.covar <- function(x,
                         variable = NULL,
                         tipo = c("muestral","cuasi"),
                         exportar = FALSE) {

  tipo <- tolower(tipo)
  tipo <- match.arg(tipo)

  # --- Convertir a data.frame ---
  if (!is.data.frame(x)) x <- data.frame(x)
  varnames <- names(x)

  # --- Seleccion de variables ---
  if (is.null(variable)) {
    varcuan <- names(x)[sapply(x, is.numeric)]
    x <- x[, varcuan, drop = FALSE]
    varnames <- varcuan
  } else if (is.numeric(variable)) {
    if (any(variable > ncol(x))) stop("Selecci\u00f3n err\u00f3nea de variables")
    x <- x[, variable, drop = FALSE]
    varnames <- names(x)
  } else if (is.character(variable)) {
    if (!all(variable %in% names(x))) stop("El nombre de la variable no es v\u00e1lido")
    x <- x[, variable, drop = FALSE]
    varnames <- names(x)
  } else stop("El argumento 'variable' debe ser num\u00e9rico o car\u00e1cter")

  # --- Comprobacion de tipo de variables ---
  if (!all(sapply(x, is.numeric))) {
    stop("No puede calcularse la matriz de varianzas-covarianzas: alguna variable no es cuantitativa")
  }

  # --- Preparar matriz vacia ---
  k <- ncol(x)
  matriz_covar <- matrix(NA, nrow = k, ncol = k)
  colnames(matriz_covar) <- varnames
  rownames(matriz_covar) <- varnames

  # --- Rellenar matriz ---
  for (i in seq_len(k)) {
    for (j in seq_len(k)) {
      if (i == j) {
        matriz_covar[i, j] <- as.numeric(varianza(x[, i, drop = FALSE], variable = 1, tipo = tipo))
      } else {
        matriz_covar[i, j] <- as.numeric(covarianza(x[, c(i, j)], variable = 1:2, tipo = tipo))
      }
    }
  }

  # --- Asegurar simetria numerica ---
  matriz_covar[lower.tri(matriz_covar)] <- t(matriz_covar)[lower.tri(matriz_covar)]

  # --- Redondear valores ---
  matriz_covar <- round(matriz_covar, 4)

  # --- Exportar si se requiere ---
  if (exportar) {
    filename <- paste0("Matriz_de_covarianzas_", format(Sys.time(), "%Y-%m-%d_%H-%M-%S"), ".xlsx")
    rio::export(matriz_covar, rowNames = TRUE, file = filename)
  }

  class(matriz_covar) <- c("resumen", class(matriz_covar))
  return(matriz_covar)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{hogares}
\alias{hogares}
\title{Data: Hogares}
\format{
Dataframe con 10 observaciones de 3 variables.
\describe{
  \item{Hogares}{Identificación del hogar.}
  \item{ingresos}{Ingresos del hogar}
  \item{viajes}{Número de hogares realizado por los hogares.}
}
}
\usage{
data("hogares")
}
\description{
Datos de 10 hogares que se utilizan en los ejemplos de (1) tabla bidimensional, (2) covarianza, (3) matriz de covarianzas, (4) correlación y (5) matriz de correlación.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/medidas.forma.R
\name{medidas.forma}
\alias{medidas.forma}
\title{Medidas de forma}
\usage{
medidas.forma(
  x,
  variable = NULL,
  pesos = NULL,
  alternativa = FALSE,
  exportar = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos, que puede estar formado por una o más variables.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{pesos}{Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.}

\item{alternativa}{Es un valor lógico. Si alternativa = TRUE el resultado de las medidas de forma muestra el coeficiente de asimetría y curtosis calculado según SPSS y EXCEL. Se facilita también los correspondientes errores típicos. Este argumento no funciona si pesos = NULL.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (exportar = TRUE).}
}
\description{
Calcula el coeficiente de asimetría y de curtosis de Fisher.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrforma.png}{width = 200px}}
\if{latex}{\figure{qrforma.png}{options: width=3cm}}
}
\details{
El coeficiente de asimetría se obtiene a partir de la expresión:

\if{html}{\figure{asimetriamuestra.png}{width = 160px}}
\if{latex}{\figure{asimetriamuestra.png}{options: width=3cm}}

y el coeficiente de curtosis:

\if{html}{\figure{curtosismuestra.png}{width = 280px}}
\if{latex}{\figure{curtosismuestra.png}{options: width=4cm}}
}
\note{
(1) El coeficiente de asimetría poblacional es:

\if{html}{\figure{asimetriapob.png}{width = 160px}}
\if{latex}{\figure{asimetriapob.png}{options: width=3cm}}

(2) El coeficiente de curtosis poblacional es:

\if{html}{\figure{curtosispob.png}{width = 2800px}}
\if{latex}{\figure{curtosispob.png}{options: width=4cm}}

(3) Si el argumento alternativa = TRUE, se obtienen los resultados de asimetría y curtosis que generalmente ofrecen softwares como: SPSS, Stata, SAS, Excel, etc.


\if{html}{\figure{asimetriasoft.png}{width = 4800px}}
\if{latex}{\figure{asimetriasoft.png}{options: width=8cm}}

 \if{html}{\figure{curtosissoft.png}{width = 920px}}
\if{latex}{\figure{curtosissoft.png}{options: width=13cm}}
}
\examples{

forma <- medidas.forma(startup)
forma2 <- medidas.forma(startup, alternativa= TRUE)

}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{varianza}},\code{\link{desviacion}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coeficiente.variacion.R
\encoding{UTF-8}
\name{coeficiente.variacion}
\alias{coeficiente.variacion}
\title{Coeficiente de variación.}
\usage{
coeficiente.variacion(
  x,
  variable = NULL,
  pesos = NULL,
  tipo = c("muestral", "cuasi")
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{pesos}{Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.}

\item{tipo}{Es un carácter. Por defecto calcula la desviación típica muestral (\code{tipo = "muestral"}). Si \code{tipo = "cuasi"}, se calcula la cuasi-desviación típica muestral.}
}
\value{
Esta función devuelve el valor del coeficiente de variación en un objeto de la clase \code{vector}. Por defecto, el coeficiente de variación se calcula utilizando la desviación típica muestral.
}
\description{
Calcula el coeficiente de variación de Pearson.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrdispersion.png}{options: style="width: 25\%;"}}
\if{latex}{\figure{qrdispersion.png}{options: width=3cm}}
}
\details{
El coeficiente de variación (muestral) se obtiene a partir de la siguiente expresión:

\if{html}{\figure{coeficientevariacion.png}{options: style="width: 20\%;"}}
\if{latex}{\figure{coeficientevariacion.png}{options: width=2cm}}

donde S es la desviación típica muestral. También puede calcularse utilizando la cuasi-desviación típica (Sc).
}
\note{
Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N), se obtiene el coeficiente de variación poblacional:

\if{html}{\figure{coeficientevariacionpob.png}{options: style="width: 20\%;"}}
\if{latex}{\figure{coeficientevariacionpob.png}{options: width=2cm}}
}
\examples{

variacion1 <- coeficiente.variacion(startup[1])
variacion2 <- coeficiente.variacion(startup)

}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unir.vectores.R
\name{unir.vectores}
\alias{unir.vectores}
\title{Unir vectores.}
\usage{
unir.vectores(...)
}
\arguments{
\item{...}{Introducir los nombres de los objetos, vectores, que se quiere unir. Si los vectores tienen distinta longitud se rellenarán los espacios con NAs.}
}
\value{
La función devuelve un dataframe.
}
\description{
Une dos o más vectores numéricos de igual o distinta longitud.

\if{html}{\figure{qrleerdatos.png}{width = 200px}}
\if{latex}{\figure{qrleerdatos.png}{options: width=3cm}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ic.diferencia.proporciones.R
\name{ic.diferencia.proporciones}
\alias{ic.diferencia.proporciones}
\title{Intervalo confianza para la diferencia de dos proporciones.}
\usage{
ic.diferencia.proporciones(
  x,
  variable = NULL,
  introducir = FALSE,
  confianza = 0.95,
  grafico = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{introducir}{Valor lógico. Si introducir = FALSE (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos x y/o variable. Si introducir = TRUE, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.}

\item{confianza}{Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, confianza = 0.95 (95 por ciento)}

\item{grafico}{Es un valor lógico. Por defecto grafico = FALSE. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a grafico = TRUE. Nota: Esta opción no está implementada para todos los casos.#'}
}
\description{
Calcula el intervalo de confianza de la diferencia de dos proporciones.

\if{html}{\figure{qricdiferenciaproporciones.png}{width = 200px}}
\if{latex}{\figure{qricdiferenciaproporciones.png}{options: width=3cm}}
}
\details{
Se obtiene el intervalo:

\if{html}{\figure{icdifproporciones.png}{width = 520px}}
\if{latex}{\figure{icdifproporciones.png}{options: width=12cm}}

Nota: El usuario puede seguir dos estrategias:
(1) Sustituir las proporciones muestrales del error típico por sus estimaciones máximo-verosímiles (proporciones muestrales)
(2) Considerar el caso: p=q=0.5
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribuciones.probabilidad.R
\name{distribuciones.probabilidad}
\alias{distribuciones.probabilidad}
\title{Distribuciones de probabilidad.}
\usage{
distribuciones.probabilidad()
}
\value{
No devuelve un valor, Es una aplicación shiny.
}
\description{
Aplicación interactiva donde se representa las principales distribuciones de probabilidad unidimensionales: Binomial, Poisson, Uniforme, Exponencial y Normal.
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{viajes_vendidos}
\alias{viajes_vendidos}
\title{Data: Viajes vendidos}
\format{
Dataframe con 5 observaciones de 3 variables.
\describe{
  \item{Número.de.viajes.vendidos}{Número de viajes perdidos.}
  \item{Empleados}{Número de empleados}
  \item{Ni}{Frecuencia absoluta acumulada del número de empleados}
}
}
\usage{
data("viajes_vendidos")
}
\description{
Datos de 5 observaciones que se utilizan en los ejemplos de (1) media, mediana y moda, (2) cuantiles, (3) varianza, desviación típica y coeficiente de variación, (4) medidas de forma y momento central y (5) resumen de descriptivos
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/varianza.R
\name{varianza}
\alias{varianza}
\title{Varianza.}
\usage{
varianza(x, variable = NULL, pesos = NULL, tipo = c("muestral", "cuasi"))
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{pesos}{Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.}

\item{tipo}{Es un carácter. Por defecto de calcula la varianza muestral (\code{tipo = "muestral"}). Si \code{tipo = "cuasi"}, se calcula la cuasivarianza muestral.}
}
\value{
Esta función devuelve un objeto de la clase \code{vector}. Si \code{tipo="muestral"}, devuelve la varianza muestral. Si \code{tipo="cuasi"}, devuelve la cuasi-varianza muestral.
}
\description{
Calcula la varianza.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrdispersion.png}{width = 200px}}
\if{latex}{\figure{qrdispersion.png}{options: width=3cm}}
}
\details{
(1) La expresión de la varianza muestral es:

\if{html}{\figure{varianza.png}{width = 320px}}
\if{latex}{\figure{varianza.png}{options: width=5cm}}

La varianza muestral así definida es el estimador máximo verosímil de la varianza de una población normal

(2) Muchos manuales y prácticamente todos los softwares (SPSS, Excel, etc.) calculan la expresión:

\if{html}{\figure{cuasivarianza.png}{width = 320px}}
\if{latex}{\figure{cuasivarianza.png}{options: width=5cm}}

Nosotros llamamos a esta medida: cuasi-varianza muestral y es un estimador insesgado de la varianza poblacional.
}
\note{
Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la varianza poblacional:


\if{html}{\figure{varianzapob.png}{width = 320px}}
\if{latex}{\figure{varianzapob.png}{options: width=5cm}}
}
\examples{

varianza1 <- varianza(startup[1])
varianza2 <- varianza(startup,variable=1)
varianza3 <- varianza(startup,variable=1, tipo="cuasi")

}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{media}}, \code{\link{desviacion}}, \code{\link{coeficiente.variacion}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contraste_homogeneidad.R
\name{contraste_homogeneidad}
\alias{contraste_homogeneidad}
\title{Contraste de Homogeneidad de Poblaciones.}
\usage{
contraste_homogeneidad(x, introducir = FALSE, alfa = 0.05, grafico = FALSE)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser una matriz o un dataframe. Debe contener sólo 2 columnas.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre el número de muestras, sobre el número de categorías de la variable poblacional, el nombre de cada categoría de la variable o de cada muestra en la posición de fila y el nombre de la categoría de la variable o de cada muestra en la posición de columna. A continuación se abrirá una ventana con un editor de datos y deberá introducir los valores de las frecuencias observadas de la tabla de contingencia.}

\item{alfa}{Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)}

\item{grafico}{Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}.}
}
\value{
La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba y el p-valor. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con el valor crítico.
}
\description{
Contrasta la hipótesis de que todas las poblaciones constituidas por una variable categórica o criterio de clasificación de las que se han seleccionado las muestras son homogéneas (tienen la misma distribución de probabilidad).

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
}
\details{
(1) El estadístico del contraste de homogeneidad es:

\deqn{\chi ^{2} = \sum_{i=1}^{I} \sum_{j=1}^{J}
\frac{(O_{ij} - E_{ij})^{2}}{E_{ij}} =
\sum_{i=1}^{I} \sum_{j=1}^{J}
\frac{(O_{ij} - \frac{O_{i.} \times O_{.j}}{n})^{2}}{\frac{O_{i.} \times O_{.j}}{n}}}

donde \eqn{O_{ij}} son las frecuencias conjuntas observadas, \eqn{E_{ij}} son las frecuencias teóricas o esperadas,
\eqn{O_{i.}} es el tamaño de cada muestra, \eqn{O_{.j}} es el total de individuos del conjunto de las muestras
clasificados en la categoría \eqn{j} de la variable, y \eqn{n} es la suma de los tamaños de todas las muestras.

\deqn{\chi_{(I-1)(J-1)}^{2}}

donde \eqn{I} es el número de muestras y \eqn{J} es el número de categorías de la variable.

Además, se exige que el tamaño de las muestras sea grande y que todas las frecuencias teóricas no estén por debajo de 5.
Si alguna no lo cumple, es necesario reagrupar las categorías contiguas hasta conseguir superar esa cota.

(2) Si el número de grados de libertad es 1, al estadístico del contraste se le aplica la siguiente corrección de Yates:

\deqn{\chi^{2} = \sum_{i=1}^{k} \frac{(\left| O_{i} - E_{i} \right| - 0.5)^{2}}{E_{i}}}
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\seealso{
\code{\link{contraste_independencia}},\code{\link{contraste_bondad_cat}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.resumen.R
\name{print.resumen}
\alias{print.resumen}
\title{Método de impresión para objetos de clase "resumen"}
\usage{
\method{print}{resumen}(x, ...)
}
\arguments{
\item{x}{Objeto de clase "resumen"}

\item{...}{Argumentos adicionales}
}
\description{
Método de impresión para objetos de clase "resumen"
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/leer.datos.R
\name{leer.datos}
\alias{leer.datos}
\title{Leer datos.}
\usage{
leer.datos(introducir = FALSE, pos = 1)
}
\arguments{
\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), se abrirá una ventana para que el usuario seleccione el fichero de datos que quiere cargar. Si \code{introducir = TRUE}, el usuario introducirá él mismo los datos.}

\item{pos}{Es un valor fijo utilizado para mostrar el dataframe del usuario en el Global Environment.}
}
\value{
Al finalizar el proceso de lectura de datos se mostrará el dataframe cargado en el environment.
}
\description{
Carga un conjunto de datos.

\if{html}{\figure{qrleerdatos.png}{width = 200px}}
\if{latex}{\figure{qrleerdatos.png}{options: width=3cm}}
}
\references{
Esteban García, J. et al. (2005). Estadística descriptiva y nociones de probabilidad. Thomson.
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{salarios2018}
\alias{salarios2018}
\title{Data: Encuesta cuatrienal de estructura salarial (2018)}
\format{
Dataframe con 1000 observaciones 7 7 variables.
\describe{
  \item{SEXO}{Sexo (1=hombre, 6=mujer)}
  \item{ESTUDIOS}{Nivel de estudios. 1=Menos que primaria,2=Primaria,3=Primera etapa secundaria,4=Segunda etapa secundaria,5=FP superior o similar,6=Diplomado o similar,7=Licenciados o similares y doctores}
  \item{TIPO.JORNADA}{Tipo de jornada laboral. 1=Tiempo completo,2=Tiempo parcial}
  \item{TIPO.CONTRATO}{Tipo de contrato laboral. 1=Indefinido,2=Duración determinada}
  \item{SALARIO.BRUTO.ANUAL}{Salario bruto anual}
  \item{SALARIO.ORDINARIO.ANUAL}{Salario ordinario anual}
  \item{FACTOR.ELEVACION}{Factor de elevación}
}
}
\source{
Instituto Nacional de Estadística \url{https://www.ine.es/}
}
\usage{
data("salarios2018")
}
\description{
Datos del Instituto Nacional de Estadística. Los microdatos originales constan de 216,726 observaciones. A efectos de practicar, se han seleccionado aleatoriamente 1000 observaciones y 7 variables de interés.
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ic.diferencia.medias.R
\name{ic.diferencia.medias}
\alias{ic.diferencia.medias}
\title{Intervalo confianza para la diferencia de medias.}
\usage{
ic.diferencia.medias(
  x,
  variable = NULL,
  introducir = FALSE,
  poblacion = c("normal", "desconocida"),
  var_pob = c("conocida", "desconocida"),
  iguales = FALSE,
  confianza = 0.95
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere a dos variables, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.}

\item{poblacion}{Es un carácter. Indica la distribución de probabilidad de la población. Por defecto \code{poblacion = "normal"}. Si la distribución de la población es desconocida, cambiar a \code{poblacion = "desconocida"}.}

\item{var_pob}{Es un carácter. Indica si la varianza poblacional es conocida (por defecto, \code{var_pob = "conocida"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.}

\item{iguales}{Por defecto se considera que las varianzas poblacionales son distintas (\code{iguales = FALSE}). En el supuesto de varianzas poblacionales iguales cambiar el argumento a \code{iguales = TRUE}.}

\item{confianza}{Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)}
}
\value{
Devuelve el intervalo de confianza de la diferencia de medias poblacionales en un objeto de tipo \code{data.frame}.
}
\description{
Calcula el intervalo de confianza de la diferencia de medias poblacionales.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qricdiferenciamedias.png}{width = 200px}}
\if{latex}{\figure{qricdiferenciamedias.png}{options: width=3cm}}
}
\details{
Se obtienen los intervalos según los siguientes casos:

Caso 1: Varianzas poblacionales conocidas

\if{html}{\figure{icdifmedias1.png}{width = 60px}}
\if{latex}{\figure{icdifmedias1.png}{options: width=10cm}}

Nota: Si los tamaños muestrales nx y ny son suficientemente grandes, pueden estimarse las varianzas poblacionales
por sus correspondientes varianzas (o cuasivarianzas), incluso aunque las distribuciones poblacionales no sean normales
(por aplicación del TCL).

Caso 2. Varianzas poblacionales desconocidas pero iguales

(2.1) con varianza muestral:

\if{html}{\figure{icdifmedias2.png}{width = 560px}}
\if{latex}{\figure{icdifmedias2.png}{options: width=10cm}}

(2.2) con cuasivarianza muestral:

\if{html}{\figure{icdifmedias2cuasi.png}{width = 600px}}
\if{latex}{\figure{icdifmedias2cuasi.png}{options: width=10cm}}

Nota: Tanto en el caso (2.1) como (2.2) la distribución t tiene (nx+ny-2) grados de libertad.

Caso 3. Varianzas poblacionales desconocidas y distintas

(3.1) con varianza muestral:

\if{html}{\figure{icdifmedias3.png}{width = 480px}}
\if{latex}{\figure{icdifmedias3.png}{options: width=8cm}}

la distribución t con grados de libertad igual al entero más próximo de v.

\if{html}{\figure{icdifmedias3gl.png}{width = 520px}}
\if{latex}{\figure{icdifmedias3gl.png}{options: width=8cm}}

(3.2) con cuasivarianza muestral:

\if{html}{\figure{icdifmedias3cuasi.png}{width = 480px}}
\if{latex}{\figure{icdifmedias3cuasi.png}{options: width=7cm}}

la distribución t con grados de libertad igual a v, donde v = (parte entera de v*) + 1

\if{html}{\figure{icdifmedias3cuasigl.png}{width = 520px}}
\if{latex}{\figure{icdifmedias3cuasigl.png}{options: width=6cm}}
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{ejem_bidi}
\alias{ejem_bidi}
\title{Data: Ejemplo de dos variables (ejem_bidi)}
\format{
Dataframe con 100 observaciones de 2 variables.
\describe{
  \item{x}{Toma valores de 0 a 5.}
  \item{x}{Toma valores de 10 a 15}
}
}
\source{
Muestra simulada.
}
\usage{
data("ejem_bidi")
}
\description{
Datos simulados. Muestra de 100 observaciones
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contraste.razon.varianzas.R
\name{contraste.razon.varianzas}
\alias{contraste.razon.varianzas}
\title{Contraste de hipótesis sobre la razón de varianzas.}
\usage{
contraste.razon.varianzas(
  x,
  variable = NULL,
  introducir = FALSE,
  hipotesis_nula = 1,
  tipo_contraste = c("bilateral", "cola derecha", "cola izquierda"),
  alfa = 0.05,
  grafico = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere solo a dos variables, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.}

\item{hipotesis_nula}{Es un valor numérico. Por defecto el valor está fijado a 1, es decir, igualdad de varianzas.}

\item{tipo_contraste}{Es un carácter. Indica el tipo de contraste a realizar. Por defecto, \code{tipo_contraste = "bilateral"}.
Si \code{tipo_contraste = "bilateral"}, se contraste la hipótesis nula igual un valor frente a la alternativa distinto de dicho valor.
Si \code{tipo_contraste = "cola derecha"}, se contrasta la hipótesis nula menor o igual a un valor frente a la alternativa mayor a dicho valor.
Si \code{tipo_contraste = "cola izquierda"}, se contrasta la hipótesis nula mayor o igual a un valor frente a la alternativa menos a dicho valor.}

\item{alfa}{Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)}

\item{grafico}{Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.}
}
\value{
La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba, el p-valor y  el intervalo de confianza para la media muestral supuesta cierta la hipótesis nula. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con los valores críticos.
}
\description{
Realiza el contraste de hipótesis sobre la razón de dos varianzas poblacionales.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrcrazonvarianzas.png}{width = 200px}}
\if{latex}{\figure{qrcrazonvarianzas.png}{options: width=3cm}}
}
\details{
La hipótesis nula que se considera en el contraste bilateral es:

\if{html}{\figure{crazonvar.png}{width = 240px}}
\if{latex}{\figure{crazonvar.png}{options: width=3cm}}

El estadístico F es:

(1) Si trabajamos con la varianza muestral:


\if{html}{\figure{crazonvarmuestra.png}{width = 400px}}
\if{latex}{\figure{crazonvarmuestra.png}{options: width=5cm}}

(2) si trabajamos con la cuasi-varianza muestral:

\if{html}{\figure{crazonvarcuasi.png}{width = 200px}}
\if{latex}{\figure{crazonvarcuasi.png}{options: width=3cm}}

Tanto en (1) como en (2) el estadístico F se distribuye como una F con (n1-1) grados de libertad en el numerador y (n2-1) grados de libertad en el denominador.
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\seealso{
\code{\link{ic.razon.varianzas}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{diseno2}
\alias{diseno2}
\title{Datos simulados de dos muestras tomadas en periodos de tiempo distintos. La muestra 1 es tomada en enero y la muestra 2 en junio.}
\format{
Dataframe en formato largo con 1085 observaciones. La pregunta realizada es: ¿Sabe que Valencia es la capital mundial del diseño 2022?
\describe{
  \item{muestra}{Toma dos valores: Muestra1 y Muestra2}
  \item{resultado}{0: No sabe, 1: Sí sabe}
}
}
\source{
Muestra simulada.
}
\usage{
data("diseno2")
}
\description{
Datos simulados de dos muestras tomadas en periodos de tiempo distintos. La muestra 1 es tomada en enero y la muestra 2 en junio.
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/series.temporales.R
\name{series.temporales}
\alias{series.temporales}
\title{Series temporales.}
\usage{
series.temporales(
  x,
  variable = NULL,
  inicio_anual = 1,
  periodo_inicio = 1,
  frecuencia = 4,
  orden = frecuencia,
  prediccion_tendencia = FALSE,
  grafico = FALSE,
  exportar = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{inicio_anual}{Año de inicio de la serie. Por defecto \code{inicio_anual = 1}.}

\item{periodo_inicio}{Periodo de inicio de la serie. Por defecto \code{perido_inicio = 1}, es decir, el primer periodo del año 1.}

\item{frecuencia}{Periodificación de la serie. Por defecto \code{frecuencia = 4}.
Si anual, frecuencia = 1.
Si semestral, frecuencia = 2.
Si cuatrimestral, frecuencia = 3.
Si trimestral, frecuencia = 4.
Si bimestral, frecuencia = 6.
Si mensual, frecuencia = 12.
Si semanal, frecuencia = 52.
Si diario, frecuencia = 360.}

\item{orden}{Orden (o puntos) de cálculo de la media móvil. Por defecto \code{orden = frecuencia}.}

\item{prediccion_tendencia}{vector de periodo temporal (\code{t=0}, origen de la serie) para el que se quiere obtener una predicción de la tendencia de la serie objeto de estudio.}

\item{grafico}{Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica la serie original, las medias móviles y la estimación por regresión de la tendencia, cambiar a \code{grafico = TRUE}.}

\item{exportar}{Para exportar los principales resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}
}
\value{
Esta función devuelve un objeto de la clase \code{list}.
}
\description{
Esta función utiliza el método de las medias móviles (centradas) para extraer la tendencia de una serie temporal.
A partir de las medias móviles, también se obtienen los índices de variación estacional (IVE).

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrseriestemporales.png}{width = 200px}}
\if{latex}{\figure{qrseriestemporales.png}{options: width=3cm}}
}
\examples{
\dontrun{
ejemplo_serie <- series.temporales(
  turistas2,
  variable = 2,
  inicio_anual = 2000,
  periodo_inicio = 1
)
}
}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{regresion.simple}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{turistas2}
\alias{turistas2}
\title{Data: Turistas internacionales Comunidad Valenciana}
\format{
Dataframe con 80 observaciones de 2 variables.
\describe{
  \item{perido}{Periodo temporal.}
  \item{Turistas.internacionales}{Número de turistas con destino principal la Comunidad Valenciana}
}
}
\source{
Movimientos turísticos en fronteras. Frontur. Instituto de Estudios Turísticos (hasta septiembre de 2015) e INE (a partir de octubre de 2015)
}
\usage{
data("turistas")
}
\description{
Data: Turistas internacionales Comunidad Valenciana
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ic.media.R
\name{ic.media}
\alias{ic.media}
\title{Intervalo confianza para la media.}
\usage{
ic.media(
  x,
  variable = NULL,
  introducir = FALSE,
  poblacion = c("normal", "desconocida"),
  var_pob = c("conocida", "desconocida"),
  confianza = 0.95,
  grafico = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.}

\item{poblacion}{Es un carácter. Indica la distribución de probabilidad de la población. Por defecto \code{poblacion = "normal"}. Si la distribución de la población es desconocida, cambiar a \code{poblacion = "desconocida"}.}

\item{var_pob}{Es un carácter. Indica si la varianza poblacional es conocida (por defecto, \code{var_pob = "conocida"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.}

\item{confianza}{Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)}

\item{grafico}{Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.}
}
\value{
Devuelve el intervalo de confianza de la media poblacional en un objeto de tipo \code{data.frame}. Si \code{grafico = T} devuelve una \code{list} con el intervalo de confianza y su representación gráfica.
}
\description{
Calcula el intervalo de confianza de la media poblacional.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qricmedia.png}{width = 200px}}
\if{latex}{\figure{qricmedia.png}{options: width=3cm}}
}
\details{
(1) Si población desconocida, varianza poblacial conocida y muestra pequeña:

\if{html}{\figure{icmediadesconocidavarcon.png}{width = 400px}}
\if{latex}{\figure{icmediadesconocidavarcon.png}{options: width=10cm}}

(2) Si población normal, varianza poblacional conocida (muestra pequeña y grande)

\if{html}{\figure{icmediavarcon.png}{width = 440px}}
\if{latex}{\figure{icmediavarcon.png}{options: width=10cm}}

(3) Si población normal, varianza poblacional desconocida y muestra pequeña

Con la varianza muestral:

\if{html}{\figure{icmediavardescmuestral.png}{width = 400px}}
\if{latex}{\figure{icmediavardescmuestral.png}{options: width=10cm}}

Con la cuasivarianza muestral:

\if{html}{\figure{icmediavardesccuasi.png}{width = 400px}}
\if{latex}{\figure{icmediavardesccuasi.png}{options: width=10cm}}

Nota: En ambos casos, el valor crítico sigue una distribución t con n-1 grados de libertad

(4) Si población normal, varianza poblacional desconocida y muestra grande:
Puede utilizarse la aproximación a la normal. El intervalo se obtiene a partir de la expresión (2)
estimando la varianza poblacional por la varianza (o cuasivarianza) muestral.
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convergencia.varianza.R
\name{convergencia.varianza}
\alias{convergencia.varianza}
\title{Convergencia de la varianza y cuasivarianza muestral.}
\usage{
convergencia.varianza()
}
\value{
Devuelve un gráfico que es un objeto de la clase \code{plotly} y \code{htmlwidget}.
}
\description{
Gráfico dinámico que ilustra la convergencia de la varianza y cuasi-varianza muestral a medida que aumenta el tamaño muestral.
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regresion.simple.R
\name{regresion.simple}
\alias{regresion.simple}
\title{Regresión lineal simple.}
\usage{
regresion.simple(
  x,
  var_depen = NULL,
  var_indepen = NULL,
  introducir = FALSE,
  inferencia = FALSE,
  confianza = 0.95,
  grafico = FALSE,
  exportar = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).}

\item{var_depen}{Es un vector (numérico o carácter) que indica la variable dependiente.}

\item{var_indepen}{Es un vector (numérico o carácter) que indica la variable independiente.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante de las variables: vector de medias y matriz de varianzas-covarianzas.}

\item{inferencia}{Si \code{inferencia = FALSE}, valor por defecto, se obtienen los resultados de la regresión simple que se estudian en un curso básico de estadística descriptiva (ver referencias de la función). Si \code{inferencia = TRUE}, se obtienen los resultas inferenciales de la regresión.}

\item{confianza}{Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)}

\item{grafico}{Si \code{grafico = TRUE}, se muestran algunos de los principales resultados gráficos de la regresión lineal.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}
}
\value{
Si \code{inferencia = FALSE}, la función devuelve los principales resultados de la regresión lineal simple que se estudian en estadística descriptiva en un objeto de la clase \code{data.frame}.
Si \code{inferencia = TRUE}, la función devuelve los resultados de inferenciales de la regresión. Estos contenidos son estudiados en cursos de inferencia estadística y en temas introductorios de econometría.
}
\description{
Calcula la regresión lineal simple.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrregresion1.png}{width = 200px}}
\if{latex}{\figure{qrregresion1.png}{options: width=3cm}}

\if{html}{\figure{qrregresion2.png}{width = 200px}}
\if{latex}{\figure{qrregresion2.png}{options: width=3cm}}
}
\details{
Se obtiene la recta de regresión minimocuadrática de Y (variable dependiente) en función de X (variable independiente).
La recta de regresión puede expresarse como:

\if{html}{\figure{regresion1.png}{width = 400px}}
\if{latex}{\figure{regresion1.png}{options: width=7cm}}

o alternativamente:

\if{html}{\figure{regresion2.png}{width = 600px}}
\if{latex}{\figure{regresion2.png}{options: width=10cm}}

En las representaciones gráficas las observaciones anómals se detectan a partir del punto leverage:

\if{html}{\figure{influyente.png}{width = 400px}}
\if{latex}{\figure{influyente.png}{options: width=6cm}}

de forma que una observación tendrá efecto de apalancamiento si:

\if{html}{\figure{obsinfluyente.png}{width = 200px}}
\if{latex}{\figure{obsinfluyente.png}{options: width=2.5cm}}

 donde p=2 (en el caso de la regresión simple). En general, p es igual al número de variables independientes más la constante.

Por otra parte, las observaciones atípicas se identifican a partir de los errores estandarizados (se). Estos errores se obtienen a partir de:

\if{html}{\figure{errorstandarizado.png}{width = 360px}}
\if{latex}{\figure{errorstandarizado.png}{options: width=6cm}}

Una observación será atípica si:

\if{html}{\figure{obsatipica.png}{width = 160px}}
\if{latex}{\figure{obsatipica.png}{options: width=2cm}}

 @seealso \code{\link{matriz.covar}}, \code{\link{matriz.correlacion}}
}
\examples{
\dontrun{
ejemplo_regresion <- regresion.simple(
  turistas,
  var_depen = 2,
  var_indepen = 3,
  grafico = TRUE
)
}
}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mediana.R
\name{mediana}
\alias{mediana}
\title{Mediana.}
\usage{
mediana(x, variable = NULL, pesos = NULL)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{pesos}{Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.}
}
\value{
Si \code{pesos = NULL}, devuelve la mediana de todas la variables seleccionadas en un \code{vector}. En caso contrario, devuelve únicamente la mediana de la variable para la que se ha facilitado la distribución de frecuencias.
}
\description{
Calcula la mediana.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrposicion.png}{width = 200px}}
\if{latex}{\figure{qrposicion.png}{options: width=3cm}}
}
\details{
La mediana se obtiene a partir de la siguiente regla de decisión:

\if{html}{\figure{mediana.png}{width = 640px}}
\if{latex}{\figure{mediana.png}{options: scale=.8}}

donde: Ni son las frecuencias acumuladas y n el tamaño de la muestra (o N si es la población).
}
\examples{

mediana1 <- mediana(startup[1])
mediana2 <- mediana(startup,variable=1)
mediana3 <- mediana(salarios2018,variable=6,pesos=7)

}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{media}}, \code{\link{cuantiles}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contraste_bondad.R
\name{contraste_bondad}
\alias{contraste_bondad}
\title{Contraste de hipótesis de bondad de ajuste para distribuciones discretas.}
\usage{
contraste_bondad(
  x,
  introducir = FALSE,
  distribucion = "equiprobable",
  parametro = FALSE,
  alfa = 0.05,
  grafico = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre el número de filas (se abrirá una ventana con un editor de datos y deberá introducir los valores de la variable poblacional y las frecuencias observadas), valor del parámetro poblacional, etc.}

\item{distribucion}{Es un carácter. Indica el tipo de distribución poblacional que se quiere contrastar en la hipótesis nula (por defecto, \code{distribucion = "equiprobable"}) o desconocida. En este último caso debería cambiarse el argumento a \code{var_pob = "desconocida"}.
Si \code{distribucion = "equiprobable"}, se contrasta que en la distribución poblacional de la hipótesis nula todos los valores de la población tienen la misma probabilidad.
Si \code{distribucion = "poisson"}, se contrasta que la distribución poblacional de la hipótesis nula se distribuye según una Poisson.
Si \code{distribucion = "binomial"}, se contrasta que la distribución poblacional de la hipótesis nula se distribuye según una Binomial.}

\item{parametro}{Es un valor lógico. Si no se especifica ningún valor para el parámetro o parámetros de la distribución poblacional \code{parametro = FALSE} (por defecto) o si se especifica un valor para dicho parámetro o parámetros (cambiar el argumento a \code{parametro = TRUE})}

\item{alfa}{Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)}

\item{grafico}{Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}.}
}
\value{
La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba y el p-valor. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con el valor crítico.
}
\description{
Contrasta si los datos de una muestra proceden de una distribución poblacional determinada.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
}
\details{
(1) El estadístico del contraste de bondad de ajuste es:

\deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(O_{i} - E_{i})^{2}}{E_{i}}}

donde \eqn{O_{i}} son las frecuencias observadas y \eqn{E_{i}} son las frecuencias teóricas o esperadas, y se distribuye como:

\deqn{\chi_{k-m-1}^{2}}

donde \eqn{k} es el número de valores distintos de la variable, y \eqn{m} es el número de parámetros de la distribución poblacional de la hipótesis nula no especificados (desconocidos) y que se han tenido que estimar.

Además, se exige que todas las frecuencias teóricas no estén por debajo de 5. Si alguna no lo cumple, es necesario reagrupar valores contiguos hasta conseguir superar esa cota.

(2) Si el número de grados de libertad es 1, al estadístico del contraste se le aplica la siguiente corrección de Yates:

\deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(\left| O_{i} - E_{i} \right| - 0.5)^{2}}{E_{i}}}
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\seealso{
\code{\link{contraste_bondad_cat}},\code{\link{contraste_independencia}},
\code{\link{contraste_homogeneidad}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Juan José Vidal Llana}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matriz.covarianzas.R
\name{matriz.covar}
\alias{matriz.covar}
\title{Matriz de varianzas y covarianzas.}
\usage{
matriz.covar(
  x,
  variable = NULL,
  tipo = c("muestral", "cuasi"),
  exportar = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} solo tiene 2 variables (columnas), \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables a seleccionar.}

\item{tipo}{Es un carácter. Por defecto de calcula la matriz de varianzas y covarianzas muestrales (\code{tipo = "muestral"}). Si \code{tipo = "cuasi"}, se calcula la matriz de cuasi-varianzas y cuasi-covarianzas muestrales.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}
}
\value{
La función devuelve la matriz de varianzas-covarianzas (muestrales, por defecto) de las variables seleccionadas en un \code{data.frame}.
}
\description{
Obtiene la matriz de varianzas y covarianzas.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrcovarianza.png}{width = 200px}}
\if{latex}{\figure{qrcovarianza.png}{options: width=3cm}}
}
\details{
(1) Se obtiene la matriz de varianzas y covarianzas muestrales:

\if{html}{\figure{matrizvarcovmuestra.png}{width = 400px}}
\if{latex}{\figure{matrizvarcovmuestra.png}{options: width=8cm}}

(2) Muchos manuales y prácticamente todos los softwares (SPSS, Excel, etc.) facilitan la matriz de cuasi-varianzas y cuasi-covarianzas muestrales:

\if{html}{\figure{matrizvarcovcuasi.png}{width = 440px}}
\if{latex}{\figure{matrizvarcovcuasi.png}{options: width=8cm}}

Nosotros nos referimos a esta expresión como cuasi-covarianza muestral.
}
\note{
Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la matriz de varianzas y covarianzas poblacional:

\if{html}{\figure{matrizvarcovpob.png}{width = 440px}}
\if{latex}{\figure{matrizvarcovpob.png}{options: width=8cm}}
}
\examples{

matriz_covarianzas1 <- matriz.covar(startup)
matriz_covarianzas2 <- matriz.covar(startup, tipo= "cuasi")

}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{varianza}}, \code{\link{desviacion}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nivel.confianza.R
\encoding{UTF-8}
\name{nivel.confianza}
\alias{nivel.confianza}
\title{Nivel de confianza.}
\usage{
nivel.confianza(
  min.pob = 2000,
  max.pob = 45000,
  muestras = 200,
  n = 100,
  confianza = 0.95,
  grafico = TRUE,
  exportar = FALSE,
  replicar = FALSE
)
}
\arguments{
\item{min.pob}{Es un valor numérico que indica el valor mínimo poblacional. Por defecto \code{min.pob = 2000}}

\item{max.pob}{Es un valor numérico que indica el valor máximo poblacional. Por defecto \code{max.pob = 45000}}

\item{muestras}{Es un valor numérico entre 50 y 10000 que indica el número de muestras que se extraen sin reemplazamiento de la población. Por defecto \code{muestras = 200}}

\item{n}{Es un valor numérico entre 25 y 2000 que indica el tamaño de la muestra. Por defecto \code{n = 100}}

\item{confianza}{Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)}

\item{grafico}{Si \code{grafico = TRUE} se representan los intervalos de confianza de las muestras seleccionadas y la media poblacional.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}

\item{replicar}{Es un valor lógico. Si \code{replicar = TRUE} el usuario fijará una semilla para que los resultados sean reproducibles. Si \code{replicar = FALSE} los resultados serán aleatorios y cambiarán en cada realización.}
}
\value{
Esta función devuelve un gráfico como un objeto de la clase \code{list}. La lista contiene los valores simulados para las muestras, el porcentaje de intervalos que contienen la media poblacional y su representación gráfica.
}
\description{
Esta función simula una población de tamaño 100,000 de la que se extraen diversas muestras y construye los correspondientes intervalos de confianzas. El objetivo es transmitir el concepto de nivel de confianza.
}
\note{
Si se seleccionan 10000 muestras de tamaño 2000, el tiempo estimado de ejecución es de 9 minutos.
}
\references{
Casas José M. () Inferencia estadística. Editoral: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\seealso{
\code{\link{ic.media}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{turistas}
\alias{turistas}
\title{Data: Turistas por paises (WTO)}
\format{
Dataframe con 130 observaciones de 3 variables.
\describe{
  \item{País}{País de destino.}
  \item{Llegadas.turistas}{Número de llegada de turistas en 2017, en miles.}
  \item{Gasto.viajes}{Gasto en viajes en 2017, en millones de USD.}
}
}
\source{
World Tourism Organization (2019).
}
\usage{
data("turistas")
}
\description{
Datos de World Tourism Organization.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cuantiles.R
\name{cuantiles}
\alias{cuantiles}
\title{Cuantiles.}
\usage{
cuantiles(
  x,
  variable = NULL,
  pesos = NULL,
  cortes = c(0.25, 0.5, 0.75),
  exportar = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{pesos}{Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.}

\item{cortes}{Vector con los puntos de corte a calcular. Por defecto se calcula el primer, segundo y tercer cuartil.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}
}
\value{
Si \code{pesos = NULL}, la función devuelve los cuantiles de todas las variables seleccionadas en un objeto de tipo \code{data.frame}. En caso contrario, devuelve los cuantiles de la variable para la que se ha facilitado la distribución de frecuencias.
}
\description{
Calcula los cuantiles.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrcuantiles.png}{width = 200px}}
\if{latex}{\figure{qrcuantiles.png}{options: width=3cm}}
}
\details{
Los cuantiles se obtienen a partir de la siguiente regla de decisión:

\if{html}{\figure{cuantiles.png}{width = 680px}}
\if{latex}{\figure{cuantiles.png}{options: scale=.85}}

Ni son las frecuencias acumuladas y n el tamaño de la muestra (o N si es la población).

cuartiles: s=1,2,3 y k=4

deciles: s= 1,2,...,9 y k=10

percentiles: s=1,2,...,99 y k=100
}
\examples{

cuantiles1 <- cuantiles(startup[1])
cuantiles2 <- cuantiles(startup,variable=1,cortes=seq(0.1,0.9,0.1))
cuantiles3 <- cuantiles(salarios2018,variable=6,pesos=7 )

}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{media}}, \code{\link{mediana}}
}
\author{
\strong{Vicente Coll-Serrano} (\email{vicente.coll@uv.es}).
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú} (\email{rosario.martinez@uv.es}).
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García} (\email{cristina.pardo-garcia@uv.es}).
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ic.correlacion.R
\name{ic.correlacion}
\alias{ic.correlacion}
\title{Intervalo confianza para el coeficiente de correlación}
\usage{
ic.correlacion(x, variable = NULL, introducir = FALSE, confianza = 0.95)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.}

\item{confianza}{Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)}
}
\value{
Devuelve el intervalo de confianza de la correlación lineal en un objeto de tipo \code{data.frame}
}
\description{
Calcula el intervalo de confianza para el coeficiente de correlación.
}
\details{
(1) El intervalo para

\if{html}{\figure{iccorrelacion1.png}{width = 160px}}
\if{latex}{\figure{iccorrelacion1.png}{options: width=4cm}}

(2) es:

\if{html}{\figure{iccorrelacion2.png}{width = 6400px}}
\if{latex}{\figure{iccorrelacion2.png}{options: width=15cm}}

Igualando la expresión en (1) al extremo inferior de (2) y al extremo superior de (2) se obtendrá el intervalo para la correlación.
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ic.proporcion.R
\name{ic.proporcion}
\alias{ic.proporcion}
\title{Intervalo confianza de una proporción.}
\usage{
ic.proporcion(
  x,
  variable = NULL,
  introducir = FALSE,
  irrestricto = FALSE,
  confianza = 0.95,
  grafico = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe.}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} se refiere una sola variable, \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de la variable.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x} y/o \code{variable}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre tamaño muestral, valor de la media muestral, etc.}

\item{irrestricto}{Es un valor lógico. Por defecto, irrectricto = FALSE. si se considera un muestreo irrectricto (extracción sin reemplazamiento), cambiar el argumento a irrestricto = TRUE.}

\item{confianza}{Es un valor numérico entre 0 y 1. Indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento)}

\item{grafico}{Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del intervalo de confianza obtenido, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.}
}
\value{
Devuelve el intervalo de confianza de la proporción poblacional en un objeto de tipo \code{data.frame}. Si \code{grafico = T} devuelve una \code{list} con el intervalo de confianza y su representación gráfica.
}
\description{
Calcula el intervalo de confianza de una proporción.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qricproporcion.png}{width = 200px}}
\if{latex}{\figure{qricproporcion.png}{options: width=3cm}}
}
\details{
(1) Para tamaños muestrales muy grandes (n>100):

\if{html}{\figure{icproporcion3.png}{width = 640px}}
\if{latex}{\figure{icproporcion3.png}{options: width=12cm}}

El usuario puede elegir entre tres estrategias:

(1.1) En el error típico aproximar p por su estimación muestral.
(1.2) En el error típico considerar el caso: p=q=0.5
(1.3) Obtener el valor de p a partir del estadístico.

(2) Para cualquier tamaño muestral puede obtenerse el intervalo:

\if{html}{\figure{icproporcion1.png}{width = 120px}}
\if{latex}{\figure{icproporcion1.png}{options: width=2.5cm}}

correspondiendo los valores a las raíces de:

\if{html}{\figure{icproporcion2.png}{width = 640px}}
\if{latex}{\figure{icproporcion2.png}{options: width=12cm}}
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabla.bidimensional.R
\name{tabla.bidimensional}
\alias{tabla.bidimensional}
\title{Tabla doble entrada.}
\usage{
tabla.bidimensional(
  x,
  var_filas = NULL,
  var_columnas = NULL,
  distribucion = c("cruzada", "condicionada"),
  frecuencias = c("absolutas", "relativas"),
  exportar = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Tiene que ser un dataframe (al menos dos variables, es decir, dos columnas).}

\item{var_filas}{Variable fila.Por defecto su valor es NUll y el usuario debe escribir el nombre o posición de la variable cuyos valores quiere representar por filas.}

\item{var_columnas}{Variable columna. Por defecto su valor es NUll y el usuario debe escribir el nombre o posición de la variable cuyos valores quiere representar por columnas}

\item{distribucion}{Es un carácter. Por defecto se obtien la tabla cruzada (\code{distribucion = "cruzada"}). Para obtener las distribuciones condicionadas cambiar a \code{distribucion = "condicionada"}.}

\item{frecuencias}{Es un carácter. Por defecto se obtienen las frecuencias absolutas ordinarias (\code{frecuencias = "absolutas"}). Para obtener las frecuencias relativas ordinarias cambiar a \code{frecuencias = "relativas"}.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}
}
\value{
Devuelve la tabla cruzada de las dos variables seleccionadas en un \code{data.frame}
}
\description{
Calcula la tabla de frecuencias bidimensionales.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrtablabidimensional.png}{width = 200px}}
\if{latex}{\figure{qrtablabidimensional.png}{options: width=3cm}}
}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matriz.correlacion.R
\name{matriz.correlacion}
\alias{matriz.correlacion}
\title{Matriz de correlación.}
\usage{
matriz.correlacion(x, variable = NULL, exportar = FALSE)
}
\arguments{
\item{x}{Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de \code{x}. Si \code{x} solo tiene 2 variables (columnas), \code{variable = NULL}. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables a seleccionar.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}
}
\value{
La función devuelve la matriz de correlación lineal de las variables seleccionadas en un \code{data.frame}.
}
\description{
Obtiene la matriz de correlación (de Pearson) entre 2 o más variables cuantitativas.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrcorrelacion.png}{width = 200px}}
\if{latex}{\figure{qrcorrelacion.png}{options: width=3cm}}
}
\details{
Se obtiene la matriz de correlación muestral:

\if{html}{\figure{matrizcorrelacion.png}{width = 400px}}
\if{latex}{\figure{matrizcorrelacion.png}{options: width=8cm}}
}
\note{
Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la matriz de correlació poblacional:

\if{html}{\figure{matrizcorrelacionpob.png}{width = 440px}}
\if{latex}{\figure{matrizcorrelacionpob.png}{options: width=8cm}}
}
\examples{

matriz_cor <- matriz.correlacion(startup)


}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{correlacion}}, \code{\link{covarianza}},\code{\link{matriz.covar}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/muestra.R
\name{muestra}
\alias{muestra}
\title{Tamaño de la muestra.}
\usage{
muestra(
  poblacion = c("normal", "dicotomica"),
  error_estimacion = NULL,
  confianza = 0.95,
  irrestricto = FALSE
)
}
\arguments{
\item{poblacion}{Texto, si \code{poblacion = "normal"} (por defecto), calcula el tamaño muestral que permita estimar la media de una población normal. Si \code{poblacion = "dicotomica"}, para estimar la proporción p de una población.}

\item{error_estimacion}{Es un valor que establece el error de estimación. Es la semiamplitud (mitad de la precisión) del intervalo de confianza. Esta aproximación solo es válida en distribuciones simétricas (normal o t-student).}

\item{confianza}{Es un valor entre 0 y 1 que indica el nivel de confianza. Por defecto, \code{confianza = 0.95} (95 por ciento).}

\item{irrestricto}{Es un valor lógico que indica si se considera un muestreo aleatorio simple (por defecto, \code{irrestricto = FALSE}) o sin reemplazamiento (\code{irrestricto = TRUE}).}
}
\value{
Devuelve el tamaño de la muesta en un objeto de tipo \code{data.frame}.
}
\description{
Calcula el tamaño muestral para estimar la media de una población normal o la proporcion p de una población.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrmuestra1.png}{width = 200px}}
\if{latex}{\figure{qrmuestra1.png}{options: width=3cm}}

\if{html}{\figure{qrmuestra2.png}{width = 200px}}
\if{latex}{\figure{qrmuestra2.png}{options: width=3cm}}
}
\details{
(1) El tamaño muestral para estimar la media poblacional se obtiene a partir de la siguiente expresión:

\if{html}{\figure{tamanomedia.png}{options: style="width: 30\%;"}}
\if{latex}{\figure{tamanomedia.png}{options: width=4cm}}

y si el muestreo es irrestricto:

\if{html}{\figure{tamanomediairrestricto.png}{width = 480px}}
\if{latex}{\figure{tamanomediairrestricto.png}{options: width=8cm}}

Nota: si la varianza poblacional no es conocida puede estimarse a través de la varianza (o cuasi-varianza) muestral.

(2) El tamaño muestral para estimar la proporción de una característica se obtiene a partir de la expresión:

\if{html}{\figure{tamanoproporcion.png}{width = 280px}}
\if{latex}{\figure{tamanoproporcion.png}{options: width=4.5cm}}

y si el muestreo es irrectricto:

\if{html}{\figure{tamanoproporcionirrestricto.png}{width = 520px}}
\if{latex}{\figure{tamanoproporcionirrestricto.png}{options: width=10cm}}

Nota: puede estimarse la proporción poblacional por la proporción muestral o, en caso
de no disponer de información, suponer el caso más desfavorable: p=q=0.5
}
\note{
En el caso del tamaño muestral para la media: si la varianza poblacional no es conocida puede estimarse con la varianza muestral (o cuasivarianza muestral).
En el caso del tamaño muestral para la proporción: si la proporción poblacional no es conocida, puede estimarse por la proporción muestral o considerar el caso más desfavorable (p=q=0.5)
}
\references{
Casas José M. (1997) Inferencia estadística. Editoral: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{resumir}
\alias{resumir}
\title{Combina o resume data frames para resumen estadístico}
\usage{
resumir(..., nombres = NULL, exportar = FALSE, archivo = NULL)
}
\arguments{
\item{...}{Data frames a combinar o resumir.}

\item{nombres}{Vector de nombres opcionales para los objetos.}

\item{exportar}{Logical. Si TRUE, exporta a Excel.}

\item{archivo}{Nombre del archivo Excel; si NULL, se genera automáticamente.}
}
\value{
Data frame combinado si las columnas coinciden, o lista resumen si son distintas.
}
\description{
Combina o resume data frames para resumen estadístico
}
\examples{
\dontrun{
Ejemplo caso 1: resumir varios data frames con las mismas columnas
media <- estadistica::media(mtcars)
cuantiles <- estadistica::cuantiles(mtcars)

resultados <- resumir(media, cuantiles, exportar=TRUE)
resultados

Ejemplo caso 2: resumir varios data frames con distintas columnas
forma <- estadistica::medidas.forma(mtcars[c(1, 3, 4)])
resultados <- resumir(media, cuantiles, forma, exportar=TRUE)
resultados
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/covarianza.R
\name{covarianza}
\alias{covarianza}
\title{Covarianza.}
\usage{
covarianza(x, variable = NULL, pesos = NULL, tipo = c("muestral", "cuasi"))
}
\arguments{
\item{x}{Conjunto de datos. Es un dataframe con al menos 2 variables (2 columnas).}

\item{variable}{Es un vector (numérico o carácter) que indica las variables a seleccionar de x. Si x solo tiene 2 variables (columnas), el argumento variable es NULL. En caso contrario, es necesario indicar el nombre o posición (número de columna) de las variables a seleccionar.}

\item{pesos}{Si los datos de la variable están resumidos en una distribución de frecuencias, debe indicarse la columna que representa los valores de la variable y la columna con las frecuencias o pesos.}

\item{tipo}{Es un carácter. Por defecto de calcula la covarianza muestral (tipo = "muestral"). Si tipo = "cuasi", se calcula la cuasi-covarianza muestral.}
}
\value{
Esta función devuelve la covarianza en un objeto de la clase \code{vector}.
}
\description{
Calcula la covarianza.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrcovarianza.png}{width = 200px}}
\if{latex}{\figure{qrcovarianza.png}{options: width=3cm}}
}
\details{
(1) La covarianza muestral se obtiene a partir de la siguiente expresión:

\if{html}{\figure{covarianzamuestra.png}{width = 400px}}
\if{latex}{\figure{covarianzamuestra.png}{options: width=6cm}}

(2) Muchos manuales y prácticamente todos los softwares (SPSS, Excel, etc.) calculan la covarianza a partir de la expresión:

\if{html}{\figure{covarianzacuasi.png}{width = 400px}}
\if{latex}{\figure{covarianzacuasi.png}{options: width=6cm}}

Nosotros nos referimos a esta expresión como cuasi-covarianza muestral.
}
\note{
Si en lugar del tamaño muestral (n) se utiliza el tamaño de la población (N) se obtiene la covarianza poblacional:

\if{html}{\figure{covarianzapob.png}{width = 400px}}
\if{latex}{\figure{covarianzapob.png}{options: width=6cm}}
}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\seealso{
\code{\link{varianza}}, \code{\link{desviacion}},\code{\link{matriz.covar}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distribucion.normal.R
\name{distribucion.normal}
\alias{distribucion.normal}
\title{Distribución normal.}
\usage{
distribucion.normal()
}
\value{
No devuelve un valor, es una aplicación shiny.
}
\description{
Aplicación interactiva para comparar dos distribuciones normales.
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.sets.R
\docType{data}
\name{startup}
\alias{startup}
\title{Data: Datos de empresas emergentes (startups)}
\format{
Dataframe con 21 observaciones de 4 variables.
\describe{
  \item{gasto.desarrollo}{Gastos de investigación y desarrollo, en euros.}
  \item{gasto.marketing}{Gastos de marketing, en euros.}
  \item{gasto.gestion}{Gastos de administración, en euros.}
  \item{beneficio}{Beneficios, en euros.}
}
}
\source{
Muestra simulada.
}
\usage{
data("startup")
}
\description{
Datos simulados. Muestra de 21 empresas emergentes
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Quantitative Methods for Measuring Culture (MC2). Applied Economics.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

\strong{Cristina Pardo-García}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabla.frecuencias.R
\name{tabla.frecuencias}
\alias{tabla.frecuencias}
\title{Tabla de frecuencias.}
\usage{
tabla.frecuencias(x, eliminar.na = TRUE, grafico = FALSE, exportar = FALSE)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector (numérico o factor) o un dataframe. Si el dataframe tiene más de una variable, solicitará al usuario que idenfique el nombre de la variable para la que se quiere calcular la tabla de frecuencias.}

\item{eliminar.na}{Valor lógico. Por defecto \code{eliminar.na = TRUE}. Si se quiere obtener la tabla de frecuencias con NAs, cambiar el argumento a \code{eliminar.na = FALSE}.}

\item{grafico}{Si \code{grafico = TRUE}, representa el histograma o el gráfico de barras de la variable seleccionada.}

\item{exportar}{Para exportar los resultados a una hoja de cálculo Excel (\code{exportar = TRUE}).}
}
\value{
Devuelve la tabla de frecuencias como una \code{tibble}. Si \code{grafico = TRUE}, se devuelve en una lista la tabla de frecuencias y su representación gráfica.
}
\description{
Esta función presenta la distribución de frecuencias de una variable cuantitativa o cualitativa.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.

\if{html}{\figure{qrtablafrecuencias.png}{width = 200px}}
\if{latex}{\figure{qrtablafrecuencias.png}{options: width=3cm}}
}
\references{
Esteban García, J. y otros. (2005). Estadística descriptiva y nociones de probabilidad. Paraninfo. ISBN: 9788497323741

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contraste_bondad_cat.R
\name{contraste_bondad_cat}
\alias{contraste_bondad_cat}
\title{Contraste de hipótesis de bondad de ajuste para datos categóricos.}
\usage{
contraste_bondad_cat(
  x,
  introducir = FALSE,
  distribucion = "equiprobable",
  alfa = 0.05,
  grafico = FALSE
)
}
\arguments{
\item{x}{Conjunto de datos. Puede ser un vector o un dataframe. En caso de haber más de una variable, el programa preguntará por la variable a seleccionar (por nombre o por posición) que debe ser un factor o carácter.}

\item{introducir}{Valor lógico. Si \code{introducir = FALSE} (por defecto), el usuario debe indicar el conjunto de datos que desea analizar usando los argumentos \code{x}. Si \code{introducir = TRUE}, se le solicitará al ususario que introduzca la información relevante sobre el número de categorías de la variable, el nombre de cada categoría. A continuación se abrirá una ventana con un editor de datos y deberá introducir los valores de las frecuencias observadas.}

\item{distribucion}{Es un vector numérico. Deberá indicarse las probabilidades teóricas para cada categoría de la variable de la hipótesis nula. Por defecto, *distribucion="equiporobable"*.}

\item{alfa}{Es un valor numérico entre 0 y 1. Indica el nivel de significación. Por defecto, \code{alfa = 0.05} (5 por ciento)}

\item{grafico}{Es un valor lógico. Por defecto \code{grafico = FALSE}. Si se quiere obtener una representación gráfica del contraste realizado, cambiar el argumento a \code{grafico = TRUE}. Nota: Esta opción no está implementada para todos los casos.}
}
\value{
La función devuelve un objeto de la clase \code{list}. La lista contendrá información sobre: la hipótesis nula contrastada, el estadístico de prueba y el p-valor. Si \code{grafico=TRUE} se incluirá una representación gráfica de la región de aceptación-rechazo con el valor crítico.
}
\description{
Contrasta si las probabilidades de que una observación pertenezca a cada una de las categorías de una variable categórica o criterio de clasificación se ajusta o no a unas probabilidades propuestas.

Lee el código QR para video-tutorial sobre el uso de la función con un ejemplo.
}
\details{
(1) El estadístico del contraste de bondad de ajuste para datos categóricos es:

\deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(O_{i} - E_{i})^{2}}{E_{i}}}

donde \eqn{O_{i}} son las frecuencias observadas y \eqn{E_{i}} son las frecuencias teóricas o esperadas, y se distribuye como:

\deqn{\chi_{k-1}^{2}}

donde \eqn{k} es el número de categorías de la variable.

Además, se exige que todas las frecuencias teóricas no estén por debajo de 5. Si alguna no lo cumple es necesario reagrupar valores contiguos hasta conseguir superar esa cota.

(2) Si el número de grados de libertad es 1, al estadístico del contraste se le aplica la siguiente corrección de Yates:

\deqn{\chi ^{2} = \sum_{i=1}^{k} \frac{(\left| O_{i} - E_{i} \right| - 0.5)^{2}}{E_{i}}}
}
\references{
Casas José M. (1997) Inferencia estadística. Editorial: Centro de estudios Ramón Areces, S.A. ISBN: 848004263-X

Esteban García, J. et al. (2008). Curso básico de inferencia estadística. ReproExprés, SL. ISBN: 8493036595.

Murgui, J.S. y otros. (2002). Ejercicios de estadística Economía y Ciencias sociales. tirant lo blanch. ISBN: 9788484424673

Newbold, P, Carlson, W. y Thorne, B. (2019). Statistics for Business and Economics, Global Edition. Pearson. ISBN: 9781292315034
}
\seealso{
\code{\link{contraste_bondad}}, \code{\link{contraste_independencia}},
\code{\link{contraste_homogeneidad}}
}
\author{
\strong{Vicente Coll-Serrano}.
\emph{Métodos Cuantitativos para la Medición de la Cultura (MC2). Economía Aplicada.}

\strong{Rosario Martínez Verdú}.
\emph{Economía Aplicada.}

Facultad de Economía. Universidad de Valencia (España)
}
